/* GAUSS code
   File: ZIT.g

PROCEDURE FOR CALCULATING BIVARIATE ADDITIVE NONPARAMETRIC REGRESSION 

Reference:
Sperlich, S., Linton, O.B., and H\"{a}rdle, W. (1999).
   Integration and backfitting methods in additive models - finite sample
   properties and comparison.
   Test, 8(2), 419-458.

*/

/* dos cd c:\docs\progs\zit\new ;*/
load DATA2 ;
data=data2 ;
data[.,1]=exp(data[.,1]) ;
n=rows(data) ;

data=sortc(data,1) ; 
smooth=zeros(n,n);


/* Initialize arrays */


smooth0=zeros(n,n);
smooth1=zeros(n,n);
smoothx=zeros(n,1);
smoothz=zeros(n,1);
g0hat=zeros(n,1);
g1hat=zeros(n,1);
g2hat=zeros(n,1);
a0hat=zeros(n,1);
a1hat=zeros(n,1);
a2hat=zeros(n,1);
countj = n*ones(n,1);
counti = n*ones(n,1);



y = data[.,1] ;
x = data[.,2] ;
z = data[.,3] ;

/* Bandwidth selection by rule of thumb */ 


ind = ones(n,1)~x~x.^2~z~z.^2 ;
b = inv(ind'ind)*ind'y ; 
ress = y - ind*b ;

th0 = 2*b[3,.] ;
th1 = 2*b[5,.] ;
sigsq = meanc(ress.^2) ;
sup1 = maxc(x) - minc(x) ;
sup2 = maxc(z) - minc(z) ;

/* k1 = 0.1429 ;*/
/* k2 = 0.7143 ; */
k1 = 1;
k2 =0.282 ; 


h1= ( ( sup1*k2*sigsq/( (k1^2)*(th0+th1)^2) )^(1/5) )*n^(-1/5) ;

h2 = ( ( sup2*k2*sigsq/( (k1^2)*(th0+th1)^2) )^(1/5) )*n^(-1/5)  ;

print "Bandwidth 1 is..." ;; print h1 ;
print "Bandwidth 2 is..." ;; print h2 ;


/* Estimate the bivariate regression functions */


j = 1 ;

do while j<= n ;

print "Iteration..." ;; print j;

i = 1 ;

do while i<= n ;

/* print "Second Iteration..." ;; print i;*/
   

      w1 = fi( (x[j,.]*ones(n,1) - x)./h1)./h1  ;
      w2 = fi( (z[i,.]*ones(n,1) - z)./h1)./h1  ;
      w = w1.*w2 ;


      smooth0[j,i] = meanc(w) ; 
      smoothx[j,.] = meanc(w1) ; 
      smoothz[i,.] = meanc(w2) ; 
      
      
      
if sumc(w) > 0 ;

      yy = sqrt(w).*data[.,1] ;
      xx1 = ones(n,1)~( x - x[j,.]*ones(n,1) ) ;
      xx = xx1~( z - z[i,.]*ones(n,1) ) ;
      xx = sqrt(w).*xx ;


if det(xx'xx) < 0.0000001 ;

      smooth1[j,i] = w'y/sumc(w) ;



else ;
      
      bb = inv(xx'xx)*xx'yy ;
      smooth1[j,i] = bb[1,.] ;



endif ;
      
else ;       

      smooth1[j,i] = 0 ;

countj[j,.] = countj[j,.] - 1;
counti[i,.] = counti[i,.] - 1;
      endif ;
   
   
   i=i+1;
endo;

j=j+ 1 ;
endo ;



a1hat = sumc(smooth1')./countj ;
a2hat = sumc(smooth1)./counti ;
ghat = diag(smooth1) ;
cc1 = meanc(a1hat) ;
cc2 = meanc(a2hat) ;

/* Confidence Intervals */



v1 = (1/(n*h1))*sigsq*k2.*meanc(( smoothz./smooth0') ) ;
v2 = (1/(n*h1))*sigsq*k2.*meanc(( smoothx./smooth0) ) ;

v1i = (1/(n*h1))*sigsq*k2./smoothx ;
v2i = (1/(n*h1))*sigsq*k2./smoothz ;


se1 = sqrt(v1) ;
se2 = sqrt(v2) ;

se1i = sqrt(v1i) ;
se2i = sqrt(v2i) ;



/* Calculate residuals */ 

gres = data[.,1] - ghat ;

ares = data[.,1] - (a1hat + a2hat - cc1*ones(n,1)) ;
mres = data[.,1] - (a1hat.*a2hat/cc1)  ;

/* Calculate sum of squared residuals */

gres = gres - meanc(gres) ;
ares = ares - meanc(ares) ;
mres = mres - meanc(mres) ;


sumgres = sumc(gres.^2) ;
sumares = sumc(ares.^2) ;
summres = sumc(mres.^2) ;
sumres = sumc( (y - meanc(y)).^2 ) ;

print "Full model fit .." ;; print sumgres ;
print "Additive model fit .." ;; print sumares ;
print "Multiplicative model fit .." ;; print summres ;
print "Total sum of squares around mean" ;; print sumres ;




/* fn fi(zz) = (15/16)*(abs(zz).<1).*((ones(n,1)-(zz.^2)).^2) ; */



fn fi(zz) = (1/(sqrt(2*pi)))*exp(-0.5*(zz.^2)) ;            


