/* GAUSS code
   File: NewGaussMax6.g  
   Coded by: Zheng-Feng Guo
   
References:
Guo, Z.-F. and Shintani, M. (2011).
  Nonparametric lag selection for nonlinear additive autoregressive models.
  Economics Letters, 111(2), 131-134.
  DOI: 10.1016/j.econlet.2011.01.014
  
Guo, Z.-F., Cao, L.Y., and He, Y. (2011).
  Nonparametric lag selection for additive models based on the smooth
  backfitting estimator.
  Theoretical Economics Letters, 1, 15-17.

Note, the MC simulations can be time consuming: 10 iterations about 2 minutes.  
   
*/   
new;
cls;
clear all;
outwidth 132;
output file = GStry.out reset;
itr     =  {1};     /* number of replications  =  itr */
rndseed 9;
h       = {1,1.2,1.4,1.8,2.0,2.2,2.4};
h11     = {0.2,0.6,0.8,1,1.4,1.8,2};
nc1     = rows(h11);
nc      = rows(h);
T       = 100;
para1   = 1;
para2   = 2;
phi1    = 0.5;
phi2    = 0.4;
AFPE    = zeros(itr,6);
lag     = zeros(itr,6);
Ahat1   =  zeros(itr,nc);
Ahat2   =  zeros(itr,nc);
Ahat3   =  zeros(itr,nc);
Ahat4   =  zeros(itr,nc);
Ahat5   =  zeros(itr,nc);
Ahat6   =  zeros(itr,nc);
Ahat7   =  zeros(itr,nc);
Ahat8   =  zeros(itr,nc);
Ahat9   =  zeros(itr,nc);
Ahat10  =  zeros(itr,nc);
Ahat11  =  zeros(itr,nc);
Ahat12  =  zeros(itr,nc);
Ahat13  =  zeros(itr,nc);
Bhat1   =  zeros(itr,nc);
Bhat2   =  zeros(itr,nc);
Bhat3   =  zeros(itr,nc);
Bhat4   =  zeros(itr,nc);
Bhat5   =  zeros(itr,nc);
Bhat6   =  zeros(itr,nc);
Bhat7   =  zeros(itr,nc);
Bhat8   =  zeros(itr,nc);
Bhat9   =  zeros(itr,nc);
Bhat10  =  zeros(itr,nc);
Bhat11  =  zeros(itr,nc);
Bhat12  =  zeros(itr,nc);
Bhat13  =  zeros(itr,nc);
FPE1    =  zeros(itr,nc);
FPE2    =  zeros(itr,nc);
FPE3    =  zeros(itr,nc);
FPE4    =  zeros(itr,nc);
FPE5    =  zeros(itr,nc);
FPE6    =  zeros(itr,nc);
FPE7    =  zeros(itr,nc);
FPE8    =  zeros(itr,nc);
FPE9    =  zeros(itr,nc);
FPE10   =  zeros(itr,nc);
FPE11   =  zeros(itr,nc);
FPE12   =  zeros(itr,nc);
FPE13   =  zeros(itr,nc);
CFPE1   =  zeros(itr,nc);
CFPE2   =  zeros(itr,nc);
CFPE3   =  zeros(itr,nc);
CFPE4   =  zeros(itr,nc);
CFPE5   =  zeros(itr,nc);
CFPE6   =  zeros(itr,nc);
CFPE7   =  zeros(itr,nc);
CFPE8   =  zeros(itr,nc);
CFPE9   =  zeros(itr,nc);
CFPE10  =  zeros(itr,nc);
CFPE11  =  zeros(itr,nc);
CFPE12  =  zeros(itr,nc);
CFPE13  =  zeros(itr,nc);
epsln   =  1e-7;  /* double precision factor */
epsln2  =  0.01;  /*oversmoothing parameter*/
/*********************************************/
/* (0) Simulation design                     */
/*********************************************/
tstart  =  time;
tvec    =  { 3600 60 1 0 };
tt      =  tvec*tstart;
output on;
screen on;

k  =  1;
do until k > itr;
/*********************************************/
/* (1) Iteration starts here                 */
/*********************************************/
        e    =  0.1*rndn(T+33,1);
        f    = zeros(T+33,1);
        f[1] =  rndn(1,1);
        f[2] = rndn(1,1);
/*Linear processes AR1*/
        j = 3;
        do until j > T+33;
        f[j] =  phi1*f[j-1]+phi2*f[j-2] +e[j];
        j    =  j + 1;
        endo;
/*Linear processes AR2
        j  =  3;
        do until j > T+33;
        f[j] =  -phi1*f[j-1]+phi2*f[j-2] +e[j];
        j    =  j + 1;
         endo; */
/*Linear processes AR3 
        f[3]  = rndn(1,1);
        f[4]  = rndn(1,1);
        f[5]  = rndn(1,1);
        f[6]  = rndn(1,1);
        f[7]  = rndn(1,1);
        f[8]  = rndn(1,1);
        f[9]  = rndn(1,1);
        f[10] = rndn(1,1);
        j  =  11;
        do until j > T+33;
        f[j] =  -0.5*f[j-6]+0.5*f[j-10] +e[j];
        j    =  j + 1;
         endo;*/
/*nonlinear processes NLAR1 
        j = 3;
         do until j > T+33;
            f[j]  = -0.4*(3-f[j-1]^2)/(1+f[j-1]^2)+0.6*(3-(f[j-2]-0.5)^3)/(1+(f[j-2]-0.5)^4)+ e[j];
               j  =  j + 1;
         endo;*/

/*nonlinear processes NLAR2  
        f[3]  = rndn(1,1);
        f[4]  = rndn(1,1);
        f[5]  = rndn(1,1);
        f[6]  = rndn(1,1);
        f[7]  = rndn(1,1);
        f[8]  = rndn(1,1);
        f[9]  = rndn(1,1);
        f[10] = rndn(1,1);
        j  =  11;
        do until j > T+33;
        f[j] =  (0.4-2*exp(-50*f[j-6]^2))*f[j-6]+ (0.5-0.5*exp(-50*f[j-10]^2))*f[j-10]+e[j];
        j    =  j + 1;
         endo;*/ 

/*nonlinear processes NLAR3  
        f[3]  = rndn(1,1);
        f[4]  = rndn(1,1);
        f[5]  = rndn(1,1);
        f[6]  = rndn(1,1);
        f[7]  = rndn(1,1);
        f[8]  = rndn(1,1);
        f[9]  = rndn(1,1);
        f[10] = rndn(1,1);
        j  =  11;
         do until j > T+33;
        f[j] =  (0.4-2*cos(40*f[j-6])*exp(-30*f[j-6]^2))*f[j-6]+ (0.55-0.55*sin(40*f[j-10])*exp(-10*f[j-10]^2))*f[j-10]+e[j];
        j    =  j + 1;
         endo; */
               
         f    =  trimr(f,20,0);
         x0   =  trimr(f,13,0); /* dependent variable x(t) */
         x1   =  trimr(f,12,1);/* dependent variable x(t-1) */
         x2   =  trimr(f,11,2);/* dependent variable x(t-2) */
         x3   =  trimr(f,10,3);/* dependent variable x(t-3) */
         x4   =  trimr(f,9,4);/* dependent variable x(t-4) */
         x5   =  trimr(f,8,5);/* dependent variable x(t-5) */
         x6   =  trimr(f,7,6);/* dependent variable x(t-6) */
         x7   =  trimr(f,6,7); /* dependent variable x(t-7) */
         x8   =  trimr(f,5,8);/* dependent variable x(t-8) */
         x9   =  trimr(f,4,9);/* dependent variable x(t-9) */
         x10  =  trimr(f,3,10);/* dependent variable x(t-10) */
         x11  =  trimr(f,2,11);/* dependent variable x(t-11) */
         x12  =  trimr(f,1,12);/* dependent variable x(t-12) */
         x13  =  trimr(f,0,13);/* dependent variable x(t-13) */
         data = x1~x2~x3~x4~x5~x6~x7~x8~x9~x10~x11~x12~x13;
         y    = x0;
         sdy  = stdc(y);

/*rule of thumb from Linton's paper */
/*
xx1d     = ones(T,1)~x1~x1.^2;
xx2d     = ones(T,1)~x1~x2~x1.*x1/2~x2.*x2/2~x1.*x2;
xx3d     = ones(T,1)~x1~x2~x3~x1.*x1/2~x2.*x2/2~x3.*x3/2~x1.*x2~x1.*x3~x2.*x3;
xx4d     = ones(T,1)~x1~x2~x3~x4~x1.*x1/2~x2.*x2/2~x3.*x3/2~x4.*x4/2~x1.*x2~x1.*x3~x1.*x4~x2.*x3~x2.*x4~x3.*x4;
beta1d   = inv(xx1d'xx1d)*xx1d'y;
rres1    = y-xx1d*beta1d;
sigmasq1 = meanc(rres1.^2);
beta2d   = inv(xx2d'xx2d)*xx2d'y;
rres2    = y-xx2d*beta2d;
sigmasq2 = meanc(rres2.^2);
beta3d   = inv(xx3d'xx3d)*xx3d'y;
rres3    = y-xx3d*beta3d;
sigmasq3 = meanc(rres3.^2);
beta4d   = inv(xx4d'xx4d)*xx4d'y;
rres4    = y-xx4d*beta4d;
sigmasq4 = meanc(rres4.^2);
maxy     = maxc(y);
miny     = minc(y);
sup      = maxy-miny;
k1       =  1;
k2       = 0.282 ; 

*/

/*
h1 = (4/3)^(1/5)*sdy*T^(-1/5);
h2 = ((sigmasq2*k2*sup/(beta2d[4,1]+beta2d[5,1])^2)^(1/5))*T^(-1/5+epsln2);
h3 = ((sigmasq3*k2*sup/(beta3d[5,1]+beta3d[6,1]+beta3d[7,1])^2)^(1/5))*T^(-1/5+2*epsln2);
h4 = (sigmasq4*k2*sup/(beta4d[6,1]+beta4d[7,1]+beta4d[8,1]+beta4d[9,1])^2)^(1/5)*T^(-1/5+3*epsln2);
*/

/*use rule of thumb to compute the bandwidth */
h1 = (4/3)^(1/5)*sdy*T^(-1/5); 
h2 = (4/3)^(1/5)*sdy*T^(-1/5+epsln2);
h3 = (4/3)^(1/5)*sdy*T^(-1/5+2*epsln2);
h4 = (4/3)^(1/5)*sdy*T^(-1/5+3*epsln2);
h5 = (4/3)^(1/5)*sdy*T^(-1/5+4*epsln2);
h6 = (4/3)^(1/5)*sdy*T^(-1/5+5*epsln2);

/*
h1 = (4/3)^(1/5)*sdy*T^(-1/5); 
h2 = (4/3)^(1/5)*sdy*T^(-1/5+epsln2)*(1+1/T);
h3 = (4/3)^(1/5)*sdy*T^(-1/5+2*epsln2)*(1+2/T);
h4 = (4/3)^(1/5)*sdy*T^(-1/5+3*epsln2)*(1+3/T);
h5 = (4/3)^(1/5)*sdy*T^(-1/5+4*epsln2)*(1+4/T);
h6 = (4/3)^(1/5)*sdy*T^(-1/5+5*epsln2)*(1+5/T);
*/

/*
h1 = sdy*T^(-1/5); 
h2 = sdy*T^(-1/5+epsln2);
h3 = sdy*T^(-1/5+2*epsln2);
h4 = sdy*T^(-1/5+3*epsln2);
h5 = sdy*T^(-1/5+4*epsln2);
h6 = sdy*T^(-1/5+5*epsln2);
*/

/* The bandwidth for B part in FPE by Tschernig and Yang called GFPE(general FPE) */      
         hs1 = sdy*(4/3)^(1/5)*T^(-1/5);
         hs2 = sdy*(4/4)^(1/6)*T^(-1/6);
         hs3 = sdy*(4/5)^(1/7)*T^(-1/7);
         hs4 = sdy*(4/6)^(1/8)*T^(-1/8);
         hs5 = sdy*(4/7)^(1/9)*T^(-1/9);
         hs6 = sdy*(4/8)^(1/10)*T^(-1/10);
     

/****************************One Lag Included ******************************************/

       
/****************************One Lag Included Estimation of B part *******************************************/

i  =  1 ;
Bm0  = zeros(T,13);
phat = zeros(T,13);
/* Estimation of B part */
      do until i>T;
         u1i  =  (data[i,1]*ones(T,1) - data[.,1]);
         u2i  =  (data[i,2]*ones(T,1) - data[.,2]);
         u3i  =  (data[i,3]*ones(T,1) - data[.,3]);
         u4i  =  (data[i,4]*ones(T,1) - data[.,4]);
         u5i  =  (data[i,5]*ones(T,1) - data[.,5]);
         u6i  =  (data[i,6]*ones(T,1) - data[.,6]);
         u7i  =  (data[i,7]*ones(T,1) - data[.,7]);
         u8i  =  (data[i,8]*ones(T,1) - data[.,8]);
         u9i  =  (data[i,9]*ones(T,1) - data[.,9]);
         u10i =  (data[i,10]*ones(T,1) - data[.,10]);
         u11i =  (data[i,11]*ones(T,1) - data[.,11]);
         u12i =  (data[i,12]*ones(T,1) - data[.,12]);
         u13i =  (data[i,13]*ones(T,1) - data[.,13]);
         xx1  = ones(T,1)~u1i;
         xx2  = ones(T,1)~u2i;
         xx3  = ones(T,1)~u3i;
         xx4  = ones(T,1)~u4i;
         xx5  = ones(T,1)~u5i;
         xx6  = ones(T,1)~u6i;
         xx7  = ones(T,1)~u7i;
         xx8  = ones(T,1)~u8i;
         xx9  = ones(T,1)~u9i;
         xx10 = ones(T,1)~u10i;
         xx11 = ones(T,1)~u11i;
         xx12 = ones(T,1)~u12i;
         xx13 = ones(T,1)~u13i;
         
         /* 
         xx1  = ones(T,1);
         xx2  = ones(T,1);
         xx3  = ones(T,1);
         xx4  = ones(T,1);
         xx5  = ones(T,1);
         xx6  = ones(T,1);
         xx7  = ones(T,1);
         xx8  = ones(T,1);
         xx9  = ones(T,1);
         xx10 = ones(T,1);
         xx11 = ones(T,1);
         xx12 = ones(T,1);
         xx13 = ones(T,1);*/
  /*weight for GLS in the conditional mean*/
          w1    =  fi(u1i/hs1)/hs1;
          w2    =  fi(u2i/hs1)/hs1;
          w3    =  fi(u3i/hs1)/hs1;
          w4    =  fi(u4i/hs1)/hs1;
          w5    =  fi(u5i/hs1)/hs1;
          w6    =  fi(u6i/hs1)/hs1;
          w7    =  fi(u7i/hs1)/hs1;
          w8    =  fi(u8i/hs1)/hs1;
          w9    =  fi(u9i/hs1)/hs1;
          w10   =  fi(u10i/hs1)/hs1;
          w11   =  fi(u11i/hs1)/hs1;
          w12   =  fi(u12i/hs1)/hs1;
          w13   =  fi(u13i/hs1)/hs1;
          w1[i] = 0; 
          w2[i] = 0;
          w3[i] = 0;
          w4[i] = 0;
          w5[i] = 0;
          w6[i] = 0;
          w7[i] = 0; 
          w8[i] = 0;
          w9[i] = 0;
          w10[i] = 0;
          w11[i] = 0;
          w12[i] = 0;
          w13[i] = 0;
       wmat1  =  diagrv((eye(T),w1));
       wmat2  = diagrv((eye(T),w2));
       wmat3  = diagrv((eye(T),w3));
       wmat4  = diagrv((eye(T),w4));
       wmat5  = diagrv((eye(T),w5));
       wmat6  = diagrv((eye(T),w6));
       wmat7  = diagrv((eye(T),w7));
       wmat8  = diagrv((eye(T),w8));
       wmat9  = diagrv((eye(T),w9));
       wmat10 = diagrv((eye(T),w10));
       wmat11 = diagrv((eye(T),w11));
       wmat12 = diagrv((eye(T),w12));
       wmat13 = diagrv((eye(T),w13));
       
if sumc(w1)>0;
if det(xx1'wmat1*xx1) < epsln;
  Bm0[i,1]  =  w1'y/sumc(w1) ;
else;
  beta      =  pinv(xx1'wmat1*xx1)*xx1'wmat1'y ;
  Bm0[i,1]  =  beta[1,.];
endif;
endif;

if sumc(w2)>0;
if det(xx2'wmat2*xx2) < epsln;
  Bm0[i,2] =  w2'y/sumc(w2) ;
else;
  beta     =  pinv(xx2'wmat2*xx2)*xx2'wmat2'y ;
  Bm0[i,2] =  beta[1,.];
endif;
endif;

if sumc(w3)>0;
if det(xx3'wmat3*xx3) < epsln;
  Bm0[i,3]  =  w1'y/sumc(w3) ;
else;
  beta      =  pinv(xx3'wmat3*xx3)*xx3'wmat3'y ;
  Bm0[i,3]  =  beta[1,.];
endif;
endif;

if sumc(w4)>0;
if det(xx4'wmat4*xx4) < epsln;
  Bm0[i,4] =  w4'y/sumc(w4) ;
else;
  beta     =  pinv(xx4'wmat4*xx4)*xx4'wmat4'y ;
  Bm0[i,4] =  beta[1,.];
endif;
endif;

if sumc(w5)>0;
if det(xx5'wmat5*xx5) < epsln;
  Bm0[i,5] =  w5'y/sumc(w5) ;
else;
  beta     =  pinv(xx5'wmat5*xx5)*xx5'wmat5'y ;
  Bm0[i,5] =  beta[1,.];
endif;
endif;

if sumc(w6)>0;
if det(xx6'wmat6*xx6) < epsln;
  Bm0[i,6]  =  w6'y/sumc(w6) ;
else;
  beta      =  pinv(xx6'wmat6*xx6)*xx6'wmat6'y ;
  Bm0[i,6]  =  beta[1,.];
endif;
endif;

if sumc(w7)>0;
if det(xx7'wmat7*xx7) < epsln;
      Bm0[i,7]  =  w7'y/sumc(w7) ;
else;
 beta =  pinv(xx7'wmat7*xx7)*xx7'wmat7'y ;
 Bm0[i,7]  =  beta[1,.];
endif;
endif;

if sumc(w8)>0;
if det(xx8'wmat8*xx8) < epsln;
      Bm0[i,8]  =  w8'y/sumc(w8) ;
else;
 beta =  pinv(xx8'wmat8*xx8)*xx8'wmat8'y ;
 Bm0[i,8]  =  beta[1,.];
endif;
endif;

if sumc(w9)>0;
if det(xx9'wmat9*xx9) < epsln;
      Bm0[i,9]  =  w9'y/sumc(w9) ;
else;
 beta =  pinv(xx9'wmat9*xx9)*xx9'wmat9'y ;
 Bm0[i,9]  =  beta[1,.];
endif;
endif;

if sumc(w10)>0;
if det(xx10'wmat10*xx10) < epsln;
      Bm0[i,10]  =  w10'y/sumc(w10) ;
else;
 beta =  pinv(xx10'wmat10*xx10)*xx10'wmat10'y ;
 Bm0[i,10]  =  beta[1,.];
endif;
endif;

if sumc(w11)>0;
if det(xx11'wmat11*xx11) < epsln;
      Bm0[i,11]  =  w11'y/sumc(w11) ;
else;
 beta =  pinv(xx11'wmat11*xx11)*xx11'wmat11'y ;
 Bm0[i,11]  =  beta[1,.];
endif;
endif;

if sumc(w12)>0;
if det(xx12'wmat12*xx12) < epsln;
  Bm0[i,12]  =  w12'y/sumc(w12) ;
else;
  beta =  pinv(xx12'wmat12*xx12)*xx12'wmat12'y ;
  Bm0[i,12]  =  beta[1,.];
endif;
endif;

if sumc(w13)>0;
if det(xx13'wmat13*xx13) < epsln;
      Bm0[i,13]  =  w13'y/sumc(w13) ;
else;
 beta =  pinv(xx13'wmat13*xx13)*xx13'wmat13'y ;
 Bm0[i,13]  =  beta[1,.];
endif;
endif;

/*kernel estimator of the density function*/
  LVO        = ones(T,1);
  LVO[i]     = 0;
  phat[i,1]  = sumc(LVO.*fi(u1i/hs1))/((T-1)*hs1);
  phat[i,2]  = sumc(LVO.*fi(u2i/hs1))/((T-1)*hs1);
  phat[i,3]  = sumc(LVO.*fi(u3i/hs1))/((T-1)*hs1);
  phat[i,4]  = sumc(LVO.*fi(u4i/hs1))/((T-1)*hs1);
  phat[i,5]  = sumc(LVO.*fi(u5i/hs1))/((T-1)*hs1);
  phat[i,6]  = sumc(LVO.*fi(u6i/hs1))/((T-1)*hs1);
  phat[i,7]  = sumc(LVO.*fi(u7i/hs1))/((T-1)*hs1);
  phat[i,8]  = sumc(LVO.*fi(u8i/hs1))/((T-1)*hs1);
  phat[i,9]  = sumc(LVO.*fi(u9i/hs1))/((T-1)*hs1);
  phat[i,10] = sumc(LVO.*fi(u10i/hs1))/((T-1)*hs1);
  phat[i,11] = sumc(LVO.*fi(u11i/hs1))/((T-1)*hs1);
  phat[i,12] = sumc(LVO.*fi(u12i/hs1))/((T-1)*hs1);
  phat[i,13] = sumc(LVO.*fi(u13i/hs1))/((T-1)*hs1);
i = i+1;
endo;
BR1   = (Bm0[.,1]-y).*(Bm0[.,1]-y);
BR2   = (Bm0[.,2]-y).*(Bm0[.,2]-y);
BR3   = (Bm0[.,3]-y).*(Bm0[.,3]-y);
BR4   = (Bm0[.,4]-y).*(Bm0[.,4]-y);
BR5   = (Bm0[.,5]-y).*(Bm0[.,5]-y);
BR6   = (Bm0[.,6]-y).*(Bm0[.,6]-y);
BR7   = (Bm0[.,7]-y).*(Bm0[.,7]-y);
BR8   = (Bm0[.,8]-y).*(Bm0[.,8]-y);
BR9   = (Bm0[.,9]-y).*(Bm0[.,9]-y);
BR10  = (Bm0[.,10]-y).*(Bm0[.,10]-y);
BR11  = (Bm0[.,11]-y).*(Bm0[.,11]-y);
BR12  = (Bm0[.,12]-y).*(Bm0[.,12]-y);
BR13  = (Bm0[.,13]-y).*(Bm0[.,13]-y);
quan  = quantile(phat,0.05);
quant = ones(T,1)*quan;
index = phat.< = quant;
cls;
print phat;
phat = index*10^5+phat;

/****************************One Lag Included ******************************************/

       
/****************************One Lag Included Estimation of A part *******************************************/
kc = 1;
do until kc>nc1;
/*bandwidth for additive models */
b1 = h1*h11[kc,1];
m0 = zeros(T,13);  
 i  =  1 ;
      do until i>T;
         u1i   =  (data[i,1]*ones(T,1) - data[.,1]);
         u2i   =  (data[i,2]*ones(T,1) - data[.,2]);
         u3i   =  (data[i,3]*ones(T,1) - data[.,3]);
         u4i   =  (data[i,4]*ones(T,1) - data[.,4]);
         u5i   =  (data[i,5]*ones(T,1) - data[.,5]);
         u6i   =  (data[i,6]*ones(T,1) - data[.,6]);
         u7i   =  (data[i,7]*ones(T,1) - data[.,7]);
         u8i   =  (data[i,8]*ones(T,1) - data[.,8]);
         u9i   =  (data[i,9]*ones(T,1) - data[.,9]);
         u10i  =  (data[i,10]*ones(T,1) - data[.,10]);
         u11i  =  (data[i,11]*ones(T,1) - data[.,11]);
         u12i  =  (data[i,12]*ones(T,1) - data[.,12]);
         u13i  =  (data[i,13]*ones(T,1) - data[.,13]);
         xx1   = ones(T,1)~u1i;
         xx2   = ones(T,1)~u2i;
         xx3   = ones(T,1)~u3i;
         xx4   = ones(T,1)~u4i;
         xx5   = ones(T,1)~u5i;
         xx6   = ones(T,1)~u6i;
         xx7   = ones(T,1)~u7i;
         xx8   = ones(T,1)~u8i;
         xx09  = ones(T,1)~u9i;
         xx10  = ones(T,1)~u10i;
         xx11  = ones(T,1)~u11i;
         xx12  = ones(T,1)~u12i;
         xx13  = ones(T,1)~u13i;
         /*xx1 = ones(T,1);
         xx2   = ones(T,1);
         xx3   = ones(T,1);
         xx4   = ones(T,1);
         xx5   = ones(T,1);
         xx6   = ones(T,1);
         xx7   = ones(T,1);
         xx8   = ones(T,1);
         xx9   = ones(T,1);
         xx10  = ones(T,1);
         xx11  = ones(T,1);
         xx12  = ones(T,1);
         xx13  = ones(T,1);*/
         
  /*weight for GLS in the conditional mean*/
          w1  =  fi(u1i/b1)/b1;
          w2  =  fi(u2i/b1)/b1;
          w3  =  fi(u3i/b1)/b1;
          w4  =  fi(u4i/b1)/b1;
          w5  =  fi(u5i/b1)/b1;
          w6  =  fi(u6i/b1)/b1;
          w7  =  fi(u7i/b1)/b1;
          w8  =  fi(u8i/b1)/b1;
          w9  =  fi(u9i/b1)/b1;
          w10 =  fi(u10i/b1)/b1;
          w11 =  fi(u11i/b1)/b1;
          w12 =  fi(u12i/b1)/b1;
          w13 =  fi(u13i/b1)/b1;
          
          /*w1 =  fi(u1i/b1);
          w2 =  fi(u2i/b1);
          w3 =  fi(u3i/b1);
          w4 =  fi(u4i/b1);
          w5 =  fi(u5i/b1);
          w6 =  fi(u6i/b1);
          w7 =  fi(u7i/b1);
          w8 =  fi(u8i/b1);
          w9 =  fi(u9i/b1);
          w10 =  fi(u10i/b1);
          w11 =  fi(u11i/b1);
          w12 =  fi(u12i/b1);
          w13 =  fi(u13i/b1);*/
       wmat1  =  diagrv((eye(T),w1));
       wmat2  = diagrv((eye(T),w2));
       wmat3  = diagrv((eye(T),w3));
       wmat4  = diagrv((eye(T),w4));
       wmat5  = diagrv((eye(T),w5));
       wmat6  = diagrv((eye(T),w6));
       wmat7  =  diagrv((eye(T),w7));
       wmat8  = diagrv((eye(T),w8));
       wmat9  = diagrv((eye(T),w9));
       wmat10 = diagrv((eye(T),w10));
       wmat11 = diagrv((eye(T),w11));
       wmat12 = diagrv((eye(T),w12));
       wmat13 = diagrv((eye(T),w13));
if sumc(w1)>0;
if det(xx1'wmat1*xx1) < epsln;
  m0[i,1] =  w1'y/sumc(w1) ;
else;
 beta     =  pinv(xx1'wmat1*xx1)*xx1'wmat1'y ;
 m0[i,1]  =  beta[1,.];
endif;
endif;

if sumc(w2)>0;
if det(xx2'wmat2*xx2) < epsln;
      m0[i,2]  =  w2'y/sumc(w2) ;
else;
 beta =  pinv(xx2'wmat2*xx2)*xx2'wmat2'y ;
 m0[i,2]  =  beta[1,.];
endif;
endif;

if sumc(w3)>0;
if det(xx3'wmat3*xx3) < epsln;
      m0[i,3]  =  w3'y/sumc(w3) ;
else;
 beta =  pinv(xx3'wmat3*xx3)*xx3'wmat3'y ;
 m0[i,3]  =  beta[1,.];
endif;
endif;

if sumc(w4)>0;
if det(xx4'wmat4*xx4) < epsln;
      m0[i,4]  =  w4'y/sumc(w4) ;
else;
 beta =  pinv(xx4'wmat4*xx4)*xx4'wmat4'y ;
 m0[i,4]  =  beta[1,.];
endif;
endif;

if sumc(w5)>0;
if det(xx5'wmat5*xx5) < epsln;
      m0[i,5]  =  w5'y/sumc(w5) ;
else;
 beta =  pinv(xx5'wmat5*xx5)*xx5'wmat5'y ;
 m0[i,5]  =  beta[1,.];
endif;
endif;

if sumc(w6)>0;
if det(xx6'wmat6*xx6) < epsln;
      m0[i,6]  =  w6'y/sumc(w6) ;
else;
 beta =  pinv(xx6'wmat6*xx6)*xx6'wmat6'y ;
 m0[i,6]  =  beta[1,.];
endif;
endif;

if sumc(w7)>0;
if det(xx7'wmat7*xx7) < epsln;
      m0[i,7]  =  w7'y/sumc(w7) ;
else;
 beta =  pinv(xx7'wmat7*xx7)*xx7'wmat7'y ;
 m0[i,7]  =  beta[1,.];
endif;
endif;

if sumc(w8)>0;
if det(xx8'wmat8*xx8) < epsln;
      m0[i,8]  =  w8'y/sumc(w8) ;
else;
 beta =  pinv(xx8'wmat8*xx8)*xx8'wmat8'y ;
 m0[i,8]  =  beta[1,.];
endif;
endif;


if sumc(w9)>0;
if det(xx9'wmat9*xx9) < epsln;
      m0[i,9]  =  w9'y/sumc(w9) ;
else;
 beta =  pinv(xx9'wmat9*xx9)*xx9'wmat9'y ;
 m0[i,9]  =  beta[1,.];
endif;
endif;

if sumc(w10)>0;
if det(xx10'wmat10*xx10) < epsln;
      m0[i,10]  =  w10'y/sumc(w10) ;
else;
 beta =  pinv(xx10'wmat10*xx10)*xx10'wmat10'y ;
 m0[i,10]  =  beta[1,.];
endif;
endif;

if sumc(w11)>0;
if det(xx11'wmat11*xx11) < epsln;
      m0[i,11]  =  w11'y/sumc(w11) ;
else;
 beta =  pinv(xx11'wmat11*xx11)*xx11'wmat11'y ;
 m0[i,11]  =  beta[1,.];
endif;
endif;


if sumc(w12)>0;
if det(xx12'wmat12*xx12) < epsln;
      m0[i,12]  =  w12'y/sumc(w12) ;
else;
 beta =  pinv(xx12'wmat12*xx12)*xx12'wmat12'y ;
 m0[i,12]  =  beta[1,.];
endif;
endif;

if sumc(w13)>0;
if det(xx13'wmat13*xx13) < epsln;
      m0[i,13]  =  w13'y/sumc(w13) ;
else;
 beta =  pinv(xx13'wmat13*xx13)*xx13'wmat13'y ;
 m0[i,13]  =  beta[1,.];
endif;
endif;

i = i+1;
endo;
/* FPE in an additive models */
R1  = (m0[.,1]-y).*(m0[.,1]-y);
R2  = (m0[.,2]-y).*(m0[.,2]-y);
R3  = (m0[.,3]-y).*(m0[.,3]-y);
R4  = (m0[.,4]-y).*(m0[.,4]-y);
R5  = (m0[.,5]-y).*(m0[.,5]-y);
R6  = (m0[.,6]-y).*(m0[.,6]-y);
R7  = (m0[.,7]-y).*(m0[.,7]-y);
R8  = (m0[.,8]-y).*(m0[.,8]-y);
R9  = (m0[.,9]-y).*(m0[.,9]-y);
R10 = (m0[.,10]-y).*(m0[.,10]-y);
R11 = (m0[.,11]-y).*(m0[.,11]-y);
R12 = (m0[.,12]-y).*(m0[.,12]-y);
R13 = (m0[.,13]-y).*(m0[.,13]-y);

Ahat1[k,kc]  =  1/T*sumc(R1);
Ahat2[k,kc]  =  1/T*sumc(R2);
Ahat3[k,kc]  =  1/T*sumc(R3);
Ahat4[k,kc]  =  1/T*sumc(R4);
Ahat5[k,kc]  =  1/T*sumc(R5);
Ahat6[k,kc]  =  1/T*sumc(R6);
Ahat7[k,kc]  =  1/T*sumc(R7);
Ahat8[k,kc]  =  1/T*sumc(R8);
Ahat9[k,kc]  =  1/T*sumc(R9);
Ahat10[k,kc] =  1/T*sumc(R10);
Ahat11[k,kc] =  1/T*sumc(R11);
Ahat12[k,kc] =  1/T*sumc(R12);
Ahat13[k,kc] =  1/T*sumc(R13);
Bhat1[k,kc]  = 2/b1*1/sqrt(2*pi)*1/T*sumc(BR1./phat[.,1])*1/T;
Bhat2[k,kc]  = 2/b1*1/sqrt(2*pi)*1/T*sumc(BR2./phat[.,2])*1/T;
Bhat3[k,kc]  = 2/b1*1/sqrt(2*pi)*1/T*sumc(BR3./phat[.,3])*1/T;
Bhat4[k,kc]  = 2/b1*1/sqrt(2*pi)*1/T*sumc(BR4./phat[.,4])*1/T;
Bhat5[k,kc]  = 2/b1*1/sqrt(2*pi)*1/T*sumc(BR5./phat[.,5])*1/T;
Bhat6[k,kc]  = 2/b1*1/sqrt(2*pi)*1/T*sumc(BR6./phat[.,6])*1/T;
Bhat7[k,kc]  = 2/b1*1/sqrt(2*pi)*1/T*sumc(BR7./phat[.,7])*1/T;
Bhat8[k,kc]  = 2/b1*1/sqrt(2*pi)*1/T*sumc(BR8./phat[.,8])*1/T;
Bhat9[k,kc]  = 2/b1*1/sqrt(2*pi)*1/T*sumc(BR9./phat[.,9])*1/T;
Bhat10[k,kc] = 2/b1*1/sqrt(2*pi)*1/T*sumc(BR10./phat[.,10])*1/T;
Bhat11[k,kc] = 2/b1*1/sqrt(2*pi)*1/T*sumc(BR11./phat[.,11])*1/T;
Bhat12[k,kc] = 2/b1*1/sqrt(2*pi)*1/T*sumc(BR12./phat[.,12])*1/T;
Bhat13[k,kc] = 2/b1*1/sqrt(2*pi)*1/T*sumc(BR13./phat[.,13])*1/T;

FPE1[k,kc]  = Ahat1[k,kc]+Bhat1[k,kc];
FPE2[k,kc]  = Ahat2[k,kc]+Bhat2[k,kc];
FPE3[k,kc]  = Ahat3[k,kc]+Bhat3[k,kc];
FPE4[k,kc]  = Ahat4[k,kc]+Bhat4[k,kc];
FPE5[k,kc]  = Ahat5[k,kc]+Bhat5[k,kc];
FPE6[k,kc]  = Ahat6[k,kc]+Bhat6[k,kc];
FPE7[k,kc]  = Ahat7[k,kc]+Bhat7[k,kc];
FPE8[k,kc]  = Ahat8[k,kc]+Bhat8[k,kc];
FPE9[k,kc]  = Ahat9[k,kc]+Bhat9[k,kc];
FPE10[k,kc] = Ahat10[k,kc]+Bhat10[k,kc];
FPE11[k,kc] = Ahat11[k,kc]+Bhat11[k,kc];
FPE12[k,kc] = Ahat12[k,kc]+Bhat12[k,kc];
FPE13[k,kc] = Ahat13[k,kc]+Bhat13[k,kc];
kc = kc+1;
endo;

min1   = minc(FPE1[k,.]');
min2   = minc(FPE2[k,.]');
min3   = minc(FPE3[k,.]');
min4   = minc(FPE4[k,.]');
min5   = minc(FPE5[k,.]');
min6   = minc(FPE6[k,.]');
min7   = minc(FPE7[k,.]');
min8   = minc(FPE8[k,.]');
min9   = minc(FPE9[k,.]');
min10  = minc(FPE10[k,.]');
min11  = minc(FPE11[k,.]');
min12  = minc(FPE12[k,.]');
min13  = minc(FPE13[k,.]');
minFPE = min1~min2~min3~min4~min5~min6~min7~min8~min9~min10~min11~min12~min13;
lag[k,1]  = minindc(minFPE');
AFPE[k,1] = minc(minFPE');
lag_1 = lag[k,1];
/****************************Two Lag Included ******************************************/

       
/****************************Two Lag Included Estimation of B part *******************************************/
i = 1;
a11  = 0;
a12  = 0;
a21   = 0;
a22  = 0;
a31  = 0;
a32  = 0;
a41  = 0;
a42  = 0;
a51  = 0;
a52  = 0;
a61  = 0;
a62  = 0;
a71  = 0;
a72  = 0;
a81  = 0;
a82  = 0;
a91  = 0;
a92  = 0;
a101 = 0;
a102 = 0;
a111 = 0;
a112 = 0;
a121 = 0;
a122 = 0;
a131 = 0;
a132 = 0;
m0 = zeros(T,26);
m2 = zeros(T,26);
m2d = zeros(T,26);
qz2 = zeros(T,14);

i = 1;
do until i>T;
u0i  =  (data[i,lag_1]*ones(T,1) - data[.,lag_1]);
u1i  =  (data[i,1]*ones(T,1) - data[.,1]);
u2i  =  (data[i,2]*ones(T,1) - data[.,2]);
u3i  =  (data[i,3]*ones(T,1) - data[.,3]);
u4i  =  (data[i,4]*ones(T,1) - data[.,4]);
u5i  =  (data[i,5]*ones(T,1) - data[.,5]);
u6i  =  (data[i,6]*ones(T,1) - data[.,6]);
u7i  =  (data[i,7]*ones(T,1) - data[.,7]);
u8i  =  (data[i,8]*ones(T,1) - data[.,8]);
u9i  =  (data[i,9]*ones(T,1) - data[.,9]);
u10i =  (data[i,10]*ones(T,1) - data[.,10]);
u11i =  (data[i,11]*ones(T,1) - data[.,11]);
u12i =  (data[i,12]*ones(T,1) - data[.,12]);
u13i =  (data[i,13]*ones(T,1) - data[.,13]);

j = 1;
do until j>T;
 u0j  =  (data[j,lag_1]*ones(T,1) - data[.,lag_1]);
 u1j  =  (data[j,1]*ones(T,1) - data[.,1]);
 u2j  =  (data[j,2]*ones(T,1) - data[.,2]);
 u3j  =  (data[j,3]*ones(T,1) - data[.,3]);
 u4j  =  (data[j,4]*ones(T,1) - data[.,4]);
 u5j  =  (data[j,5]*ones(T,1) - data[.,5]);
 u6j  =  (data[j,5]*ones(T,1) - data[.,6]);
 u7j  =  (data[j,7]*ones(T,1) - data[.,7]);
 u8j  =  (data[j,8]*ones(T,1) - data[.,8]);
 u9j  =  (data[j,9]*ones(T,1) - data[.,9]);
 u10j =  (data[j,10]*ones(T,1) - data[.,10]);
 u11j =  (data[j,11]*ones(T,1) - data[.,11]);
 u12j =  (data[j,12]*ones(T,1) - data[.,12]);
 u13j =  (data[j,13]*ones(T,1) - data[.,13]);
 
 w11  = (fi(u0i/h1)/h1).*(fi(u1j/h1)/h1);
 w12  = (fi(u0j/h1)/h1).*(fi(u1i/h1)/h1);
 w21  = (fi(u0i/h1)/h1).*(fi(u2j/h1)/h1);
 w22  = (fi(u0j/h1)/h1).*(fi(u2i/h1)/h1);
 w31  = (fi(u0i/h1)/h1).*(fi(u3j/h1)/h1);
 w32  = (fi(u0j/h1)/h1).*(fi(u3i/h1)/h1);
 w41  = (fi(u0i/h1)/h1).*(fi(u4j/h1)/h1);
 w42  = (fi(u0j/h1)/h1).*(fi(u4i/h1)/h1);
 w51  = (fi(u0i/h1)/h1).*(fi(u5j/h1)/h1);
 w52  = (fi(u0j/h1)/h1).*(fi(u5i/h1)/h1);
 w61  = (fi(u0i/h1)/h1).*(fi(u6j/h1)/h1);
 w62  = (fi(u0j/h1)/h1).*(fi(u6i/h1)/h1);
 w71  = (fi(u0i/h1)/h1).*(fi(u7j/h1)/h1);
 w72  = (fi(u0j/h1)/h1).*(fi(u7i/h1)/h1);
 w81  = (fi(u0i/h1)/h1).*(fi(u8j/h1)/h1);
 w82  = (fi(u0j/h1)/h1).*(fi(u8i/h1)/h1);
 w91  = (fi(u0i/h1)/h1).*(fi(u9j/h1)/h1);
 w92  = (fi(u0j/h1)/h1).*(fi(u9i/h1)/h1);
 w101 = (fi(u0i/h1)/h1).*(fi(u10j/h1)/h1);
 w102 = (fi(u0j/h1)/h1).*(fi(u10i/h1)/h1);
 w111 = (fi(u0i/h1)/h1).*(fi(u11j/h1)/h1);
 w112 = (fi(u0j/h1)/h1).*(fi(u11i/h1)/h1);
 w121 = (fi(u0i/h1)/h1).*(fi(u12j/h1)/h1);
 w122 = (fi(u0j/h1)/h1).*(fi(u12i/h1)/h1);
 w131 = (fi(u0i/h1)/h1).*(fi(u13j/h1)/h1);
 w132 = (fi(u0j/h1)/h1).*(fi(u13i/h1)/h1);
 
 w11[j]  = 0;
 w12[j]  = 0;
 w21[j]  = 0;
 w22[j]  = 0;
 w31[j]  = 0;
 w32[j]  = 0;
 w41[j]  = 0;
 w42[j]  = 0;
 w51[j]  = 0;
 w52[j]  = 0;
 w61[j]  = 0;
 w62[j]  = 0;
 w71[j]  = 0;
 w72[j]  = 0;
 w81[j]  = 0;
 w82[j]  = 0;
 w91[j]  = 0;
 w92[j]  = 0;
 w101[j] = 0;
 w102[j] = 0;
 w111[j] = 0;
 w112[j] = 0;
 w121[j] = 0;
 w122[j] = 0;
 w131[j] = 0;
 w132[j] = 0;
 
 wmat11  = diagrv((eye(T),w11));
 wmat12  = diagrv((eye(T),w12));
 wmat21  = diagrv((eye(T),w21));
 wmat22  = diagrv((eye(T),w22));
 wmat31  = diagrv((eye(T),w31));
 wmat32  = diagrv((eye(T),w32));
 wmat41  = diagrv((eye(T),w41));
 wmat42  = diagrv((eye(T),w42));
 wmat51  = diagrv((eye(T),w51));
 wmat52  = diagrv((eye(T),w52));
 wmat61  = diagrv((eye(T),w61));
 wmat62  = diagrv((eye(T),w62));
 wmat71  = diagrv((eye(T),w71));
 wmat72  = diagrv((eye(T),w72));
 wmat81  = diagrv((eye(T),w81));
 wmat82  = diagrv((eye(T),w82));
 wmat91  = diagrv((eye(T),w91));
 wmat92  = diagrv((eye(T),w92));
 wmat101 = diagrv((eye(T),w101));
 wmat102 = diagrv((eye(T),w102));
 wmat111 = diagrv((eye(T),w111));
 wmat112 = diagrv((eye(T),w112));
 wmat121 = diagrv((eye(T),w121));
 wmat122 = diagrv((eye(T),w122));
 wmat131 = diagrv((eye(T),w131));
 wmat132 = diagrv((eye(T),w132));
 
 xx11  =  ones(T,1)~u0i~u1j ;
 xx12  =  ones(T,1)~u0j~u1i ;
 xx21  =  ones(T,1)~u0i~u2j;
 xx22  =  ones(T,1)~u0j~u2i;
 xx31  =  ones(T,1)~u0i~u3j;
 xx32  =  ones(T,1)~u0j~u3i;
 xx41  =  ones(T,1)~u0i~u4j;
 xx42  =  ones(T,1)~u0j~u4i;
 xx51  =  ones(T,1)~u0i~u5j;
 xx52  =  ones(T,1)~u0j~u5i;
 xx61  =  ones(T,1)~u0i~u6j;
 xx62  =  ones(T,1)~u0j~u6i;
 xx71  =  ones(T,1)~u0i~u7j ;
 xx72  =  ones(T,1)~u0j~u7i ;
 xx81  =  ones(T,1)~u0i~u8j;
 xx82  =  ones(T,1)~u0j~u8i;
 xx91  =  ones(T,1)~u0i~u9j;
 xx92  =  ones(T,1)~u0j~u9i;
 xx101 =  ones(T,1)~u0i~u10j;
 xx102 =  ones(T,1)~u0j~u10i;
 xx111 =  ones(T,1)~u0i~u11j;
 xx112 =  ones(T,1)~u0j~u11i;
 xx121 =  ones(T,1)~u0i~u12j;
 xx122 =  ones(T,1)~u0j~u12i;
 xx131 =  ones(T,1)~u0i~u13j;
 xx132 =  ones(T,1)~u0j~u13i;
 
 /*xx11  =  ones(T,1);
 xx12  =  ones(T,1);
 xx21  =  ones(T,1);
 xx22  =  ones(T,1);
 xx31  =  ones(T,1);
 xx32  =  ones(T,1);
 xx41  =  ones(T,1);
 xx42  =  ones(T,1);
 xx51  =  ones(T,1);
 xx52  =  ones(T,1);
 xx61  =  ones(T,1);
 xx62  =  ones(T,1);
 xx71  =  ones(T,1) ;
 xx72  =  ones(T,1) ;
 xx81  =  ones(T,1);
 xx82  =  ones(T,1);
 xx91  =  ones(T,1);
 xx92  =  ones(T,1);
 xx101  =  ones(T,1);
 xx102  =  ones(T,1);
 xx111  =  ones(T,1);
 xx112  =  ones(T,1);
 xx121  =  ones(T,1);
 xx122  =  ones(T,1);
 xx131  =  ones(T,1);
 xx132  =  ones(T,1);*/


/* for lag = 1 and the first significant lag */
if sumc(w11)> 0;
if det(xx11'wmat11*xx11) < epsln ;
      m2[j,1]  =  w11'y/sumc(w11) ;
else;
 beta =  pinv(xx11'wmat11*xx11)*xx11'wmat11'y ;
 m2[j,1]  =  beta[1,.] ;
endif;
else;
m2[j,1] = 0;
a11 = a11+1;
endif;

if sumc(w12)> 0;
if det(xx12'wmat12*xx12) < epsln ;
  m2[j,2]  =  w12'y/sumc(w12) ;
else;
  beta     =  pinv(xx12'wmat12*xx12)*xx12'wmat12'y ;
  m2[j,2]  =  beta[1,.] ;
endif;
else;
m2[j,2] = 0;
a12     = a12+1;
endif;

/* for lag = 2 and the first significant lag */
if sumc(w21)> 0;
if det(xx21'wmat21*xx21) < epsln ;
      m2[j,3]  =  w21'y/sumc(w21) ;
else;
 beta =  pinv(xx21'wmat21*xx21)*xx21'wmat21'y ;
 m2[j,3]  =  beta[1,.] ;
endif;
else;
m2[j,3] = 0;
a21 = a21+1;
endif;

if sumc(w22)> 0;
if det(xx22'wmat22*xx22) < epsln ;
      m2[j,4]  =  w22'y/sumc(w22) ;
else;
 beta =  pinv(xx22'wmat22*xx22)*xx22'wmat22'y ;
 m2[j,4]  =  beta[1,.] ;
endif;
else;
m2[j,4] = 0;
a22 = a22+1;
endif;

/* for lag = 3 and the first significant lag */
if sumc(w31)> 0;
if det(xx31'wmat31*xx31) < epsln ;
      m2[j,5]  =  w31'y/sumc(w31) ;
else;
 beta =  pinv(xx31'wmat31*xx31)*xx31'wmat31'y ;
 m2[j,5]  =  beta[1,.] ;
endif;
else;
m2[j,5] = 0;
a31 = a31+1;
endif;

if sumc(w32)> 0;
if det(xx32'wmat32*xx32) < epsln ;
      m2[j,6]  =  w32'y/sumc(w32) ;
else;
 beta =  pinv(xx32'wmat32*xx32)*xx32'wmat32'y ;
 m2[j,6]  =  beta[1,.] ;
endif;
else;
m2[j,6] = 0;
a32 = a32+1;
endif;

/* for lag = 4 and the first significant lag */
if sumc(w41)> 0;
if det(xx41'wmat41*xx41) < epsln ;
      m2[j,7]  =  w41'y/sumc(w41) ;
else;
 beta =  pinv(xx41'wmat41*xx41)*xx41'wmat41'y ;
 m2[j,7]  =  beta[1,.] ;
endif;
else;
m2[j,7] = 0;
a41 = a41+1;
endif;

if sumc(w42)> 0;
if det(xx42'wmat42*xx42) < epsln ;
      m2[j,8]  =  w42'y/sumc(w42) ;
else;
 beta =  pinv(xx42'wmat42*xx42)*xx42'wmat42'y ;
 m2[j,8]  =  beta[1,.] ;
endif;
else;
m2[j,8] = 0;
a42 = a42+1;
endif;

/* for lag = 5 and the first significant lag */
if sumc(w51)> 0;
if det(xx51'wmat51*xx51) < epsln ;
      m2[j,9]  =  w51'y/sumc(w51) ;
else;
 beta =  pinv(xx51'wmat51*xx51)*xx51'wmat51'y ;
 m2[j,9]  =  beta[1,.] ;
endif;
else;
m2[j,9] = 0;
a51 = a51+1;
endif;

if sumc(w52)> 0;
if det(xx52'wmat52*xx52) < epsln ;
      m2[j,10]  =  w52'y/sumc(w52) ;
else;
 beta =  pinv(xx52'wmat52*xx52)*xx52'wmat52'y ;
 m2[j,10]  =  beta[1,.] ;
endif;
else;
m2[j,10] = 0;
a52 = a52+1;
endif;
/* for lag = 6 and the first significant lag */
if sumc(w61)> 0;
if det(xx61'wmat61*xx61) < epsln ;
      m2[j,11]  =  w61'y/sumc(w61) ;
else;
 beta =  pinv(xx61'wmat61*xx61)*xx61'wmat61'y ;
 m2[j,11]  =  beta[1,.] ;
endif;
else;
m2[j,11] = 0;
a61 = a61+1;
endif;

if sumc(w62)> 0;
if det(xx62'wmat62*xx62) < epsln ;
      m2[j,12]  =  w62'y/sumc(w62) ;
else;
 beta =  pinv(xx62'wmat62*xx62)*xx62'wmat62'y ;
 m2[j,12]  =  beta[1,.] ;
endif;
else;
m2[j,12] = 0;
a62 = a62+1;
endif;

if sumc(w71)> 0;
if det(xx71'wmat71*xx71) < epsln ;
      m2[j,13]  =  w71'y/sumc(w71) ;
else;
 beta =  pinv(xx71'wmat71*xx71)*xx71'wmat71'y ;
 m2[j,13]  =  beta[1,.] ;
endif;
else;
m2[j,13] = 0;
a71 = a71+1;
endif;

if sumc(w72)> 0;
if det(xx72'wmat72*xx72) < epsln ;
      m2[j,14]  =  w72'y/sumc(w72) ;
else;
 beta =  pinv(xx72'wmat72*xx72)*xx72'wmat72'y ;
 m2[j,14]  =  beta[1,.] ;
endif;
else;
m2[j,14] = 0;
a72 = a72+1;
endif;

if sumc(w81)> 0;
if det(xx81'wmat81*xx81) < epsln ;
      m2[j,15]  =  w81'y/sumc(w81) ;
else;
 beta =  pinv(xx81'wmat81*xx81)*xx81'wmat81'y ;
 m2[j,15]  =  beta[1,.] ;
endif;
else;
m2[j,15] = 0;
a81 = a81+1;
endif;

if sumc(w82)> 0;
if det(xx82'wmat82*xx82) < epsln ;
      m2[j,16]  =  w82'y/sumc(w82) ;
else;
 beta =  pinv(xx82'wmat82*xx82)*xx82'wmat82'y ;
 m2[j,16]  =  beta[1,.] ;
endif;
else;
m2[j,16] = 0;
a82 = a82+1;
endif;

if sumc(w91)> 0;
if det(xx91'wmat91*xx91) < epsln ;
      m2[j,17]  =  w91'y/sumc(w91) ;
else;
 beta =  pinv(xx91'wmat91*xx91)*xx91'wmat91'y ;
 m2[j,17]  =  beta[1,.] ;
endif;
else;
m2[j,17] = 0;
a91 = a91+1;
endif;

if sumc(w92)> 0;
if det(xx92'wmat92*xx92) < epsln ;
      m2[j,18]  =  w92'y/sumc(w92) ;
else;
 beta =  pinv(xx92'wmat92*xx92)*xx92'wmat92'y ;
 m2[j,18]  =  beta[1,.] ;
endif;
else;
m2[j,18] = 0;
a92 = a92+1;
endif;

if sumc(w101)> 0;
if det(xx101'wmat101*xx101) < epsln ;
      m2[j,19]  =  w101'y/sumc(w101) ;
else;
 beta =  pinv(xx101'wmat101*xx101)*xx101'wmat101'y ;
 m2[j,19]  =  beta[1,.] ;
endif;
else;
m2[j,19] = 0;
a101 = a101+1;
endif;

if sumc(w102)> 0;
if det(xx102'wmat102*xx102) < epsln ;
      m2[j,20]  =  w102'y/sumc(w102) ;
else;
 beta =  pinv(xx102'wmat102*xx102)*xx102'wmat102'y ;
 m2[j,20]  =  beta[1,.] ;
endif;
else;
m2[j,20] = 0;
a102 = a102+1;
endif;


if sumc(w111)> 0;
if det(xx111'wmat111*xx111) < epsln ;
      m2[j,21]  =  w111'y/sumc(w111) ;
else;
 beta =  pinv(xx111'wmat111*xx111)*xx111'wmat111'y ;
 m2[j,21]  =  beta[1,.] ;
endif;
else;
m2[j,21] = 0;
a111 = a111+1;
endif;

if sumc(w112)> 0;
if det(xx112'wmat112*xx112) < epsln ;
      m2[j,22]  =  w112'y/sumc(w112) ;
else;
 beta =  pinv(xx112'wmat112*xx112)*xx112'wmat112'y ;
 m2[j,22]  =  beta[1,.] ;
endif;
else;
m2[j,22] = 0;
a112 = a112+1;
endif;


if sumc(w121)> 0;
if det(xx121'wmat121*xx121) < epsln ;
      m2[j,23]  =  w121'y/sumc(w121) ;
else;
 beta =  pinv(xx121'wmat121*xx121)*xx121'wmat121'y ;
 m2[j,23]  =  beta[1,.] ;
endif;
else;
m2[j,23] = 0;
a121 = a121+1;
endif;

if sumc(w122)> 0;
if det(xx122'wmat122*xx122) < epsln ;
      m2[j,24]  =  w122'y/sumc(w122) ;
else;
 beta =  pinv(xx122'wmat122*xx122)*xx122'wmat122'y ;
 m2[j,24]  =  beta[1,.] ;
endif;
else;
m2[j,24] = 0;
a122 = a122+1;
endif;

if sumc(w131)> 0;
if det(xx131'wmat131*xx131) < epsln ;
      m2[j,25]  =  w131'y/sumc(w131) ;
else;
 beta =  pinv(xx131'wmat131*xx131)*xx131'wmat131'y ;
 m2[j,25]  =  beta[1,.] ;
endif;
else;
m2[j,25] = 0;
a131 = a131+1;
endif;

if sumc(w132)> 0;
if det(xx132'wmat132*xx132) < epsln ;
      m2[j,26]  =  w132'y/sumc(w132) ;
else;
 beta =  pinv(xx132'wmat132*xx132)*xx132'wmat132'y ;
 m2[j,26]  =  beta[1,.] ;
endif;
else;
m2[j,26] = 0;
a132 = a132+1;
endif;

j = j+1;
endo;

m2d[i,1]  = 1/(T-a11)*sumc(m2[.,1]);
m2d[i,2]  = 1/(T-a12)*sumc(m2[.,2]);
m2d[i,3]  = 1/(T-a21)*sumc(m2[.,3]);
m2d[i,4]  = 1/(T-a22)*sumc(m2[.,4]);
m2d[i,5]  = 1/(T-a31)*sumc(m2[.,5]);
m2d[i,6]  = 1/(T-a32)*sumc(m2[.,6]);
m2d[i,7]  = 1/(T-a41)*sumc(m2[.,7]);
m2d[i,8]  = 1/(T-a42)*sumc(m2[.,8]);
m2d[i,9]  = 1/(T-a51)*sumc(m2[.,9]);
m2d[i,10] = 1/(T-a52)*sumc(m2[.,10]);
m2d[i,11] = 1/(T-a61)*sumc(m2[.,11]);
m2d[i,12] = 1/(T-a62)*sumc(m2[.,12]);
m2d[i,13] = 1/(T-a71)*sumc(m2[.,13]);
m2d[i,14] = 1/(T-a72)*sumc(m2[.,14]);
m2d[i,15] = 1/(T-a81)*sumc(m2[.,15]);
m2d[i,16] = 1/(T-a82)*sumc(m2[.,16]);
m2d[i,17] = 1/(T-a91)*sumc(m2[.,17]);
m2d[i,18] = 1/(T-a92)*sumc(m2[.,18]);
m2d[i,19] = 1/(T-a101)*sumc(m2[.,19]);
m2d[i,20] = 1/(T-a102)*sumc(m2[.,20]);
m2d[i,21] = 1/(T-a111)*sumc(m2[.,21]);
m2d[i,22] = 1/(T-a112)*sumc(m2[.,22]);
m2d[i,23] = 1/(T-a121)*sumc(m2[.,23]);
m2d[i,24] = 1/(T-a122)*sumc(m2[.,24]);
m2d[i,25] = 1/(T-a131)*sumc(m2[.,25]);
m2d[i,26] = 1/(T-a132)*sumc(m2[.,26]);

/*kernel estimator of the density */
/*kernel estimator of the density function*/
 LVO       = ones(T,1);
 LVO[i]    = 0;
phat[i,1]  = sumc(LVO.*fi(u0i/hs2).*fi(u1i/hs2))/((T-1)*hs2^2);
phat[i,2]  = sumc(LVO.*fi(u0i/hs2).*fi(u2i/hs2))/((T-1)*hs2^2);
phat[i,3]  = sumc(LVO.*fi(u0i/hs2).*fi(u3i/hs2))/((T-1)*hs2^2);
phat[i,4]  = sumc(LVO.*fi(u0i/hs2).*fi(u4i/hs2))/((T-1)*hs2^2);
phat[i,5]  = sumc(LVO.*fi(u0i/hs2).*fi(u5i/hs2))/((T-1)*hs2^2);
phat[i,6]  = sumc(LVO.*fi(u0i/hs2).*fi(u6i/hs2))/((T-1)*hs2^2);
phat[i,7]  = sumc(LVO.*fi(u0i/hs2).*fi(u7i/hs2))/((T-1)*hs2^2);
phat[i,8]  = sumc(LVO.*fi(u0i/hs2).*fi(u8i/hs2))/((T-1)*hs2^2);
phat[i,9]  = sumc(LVO.*fi(u0i/hs2).*fi(u9i/hs2))/((T-1)*hs2^2);
phat[i,10] = sumc(LVO.*fi(u0i/hs2).*fi(u10i/hs2))/((T-1)*hs2^2);
phat[i,11] = sumc(LVO.*fi(u0i/hs2).*fi(u11i/hs2))/((T-1)*hs2^2);
phat[i,12] = sumc(LVO.*fi(u0i/hs2).*fi(u12i/hs2))/((T-1)*hs2^2);
phat[i,13] = sumc(LVO.*fi(u0i/hs2).*fi(u13i/hs2))/((T-1)*hs2^2);

qz2[i,1]  = 1/T*sumc(fi(u0i/h1)./h1);
qz2[i,2]  = 1/T*sumc(fi(u1i/h1)./h1);
qz2[i,3]  = 1/T*sumc(fi(u2i/h1)./h1);
qz2[i,4]  = 1/T*sumc(fi(u3i/h1)./h1);
qz2[i,5]  = 1/T*sumc(fi(u4i/h1)./h1);
qz2[i,6]  = 1/T*sumc(fi(u5i/h1)./h1);
qz2[i,7]  = 1/T*sumc(fi(u6i/h1)./h1);
qz2[i,8]  = 1/T*sumc(fi(u7i/h1)./h1);
qz2[i,9]  = 1/T*sumc(fi(u8i/h1)./h1);
qz2[i,10] = 1/T*sumc(fi(u9i/h1)./h1);
qz2[i,11] = 1/T*sumc(fi(u10i/h1)./h1);
qz2[i,12] = 1/T*sumc(fi(u11i/h1)./h1);
qz2[i,13] = 1/T*sumc(fi(u12i/h1)./h1);
qz2[i,14] = 1/T*sumc(fi(u13i/h1)./h1);
i = i+1;
endo;
yhat1 = y-meanc(y)-(m2d[.,1]-meanc(m2d[.,1])+m2d[.,2]-meanc(m2d[.,2]));
yhat2 = y-meanc(y)-(m2d[.,3]-meanc(m2d[.,3])+m2d[.,4]-meanc(m2d[.,4]));
yhat3 = y-meanc(y)-(m2d[.,5]-meanc(m2d[.,5])+m2d[.,6]-meanc(m2d[.,6]));
yhat4 = y-meanc(y)-(m2d[.,7]-meanc(m2d[.,7])+m2d[.,8]-meanc(m2d[.,8]));
yhat5 = y-meanc(y)-(m2d[.,9]-meanc(m2d[.,9])+m2d[.,10]-meanc(m2d[.,10]));
yhat6 = y-meanc(y)-(m2d[.,11]-meanc(m2d[.,11])+m2d[.,12]-meanc(m2d[.,12]));
yhat7 = y-meanc(y)-(m2d[.,13]-meanc(m2d[.,13])+m2d[.,14]-meanc(m2d[.,14]));
yhat8 = y-meanc(y)-(m2d[.,15]-meanc(m2d[.,15])+m2d[.,16]-meanc(m2d[.,16]));
yhat9 = y-meanc(y)-(m2d[.,17]-meanc(m2d[.,17])+m2d[.,18]-meanc(m2d[.,18]));
yhat10 = y-meanc(y)-(m2d[.,19]-meanc(m2d[.,19])+m2d[.,20]-meanc(m2d[.,20]));
yhat11 = y-meanc(y)-(m2d[.,21]-meanc(m2d[.,21])+m2d[.,22]-meanc(m2d[.,22]));
yhat12 = y-meanc(y)-(m2d[.,23]-meanc(m2d[.,23])+m2d[.,24]-meanc(m2d[.,24]));
yhat13 = y-meanc(y)-(m2d[.,25]-meanc(m2d[.,25])+m2d[.,26]-meanc(m2d[.,26]));

BR1 = (yhat1-y).*(yhat1-y);
BR2 = (yhat2-y).*(yhat2-y);
BR3 = (yhat3-y).*(yhat3-y);
BR4 = (yhat4-y).*(yhat4-y);
BR5 = (yhat5-y).*(yhat5-y);
BR6 = (yhat6-y).*(yhat6-y);
BR7 = (yhat7-y).*(yhat7-y);
BR8 = (yhat8-y).*(yhat8-y);
BR9 = (yhat9-y).*(yhat9-y);
BR10 = (yhat10-y).*(yhat10-y);
BR11 = (yhat11-y).*(yhat11-y);
BR12 = (yhat12-y).*(yhat12-y);
BR13 = (yhat13-y).*(yhat13-y);
quan = quantile(phat,0.05);
index = phat.< = quan;
phat = index*10^5+phat;

/****************************Two Lags Included ******************************************/

       
/****************************Two Lags Included Estimation of A part *******************************************/

kc = 1;
do until kc>nc;
b2 = h2*h[kc,1];
a11 = 0;
a12 = 0;
a21 = 0;
a22 = 0;
a31 = 0;
a32 = 0;
a41 = 0;
a42 = 0;
a51 = 0;
a52 = 0;
a61 = 0;
a62 = 0;
a71 = 0;
a72 = 0;
a81 = 0;
a82 = 0;
a91 = 0;
a92 = 0;
a101 = 0;
a102 = 0;
a111 = 0;
a112 = 0;
a121 = 0;
a122 = 0;
a131 = 0;
a132 = 0;
m2d = zeros(T,26);
m2 = zeros(T,26);
i = 1;
do until i>T;
u0i =  (data[i,lag_1]*ones(T,1) - data[.,lag_1]);
u1i =  (data[i,1]*ones(T,1) - data[.,1]);
u2i =  (data[i,2]*ones(T,1) - data[.,2]);
u3i =  (data[i,3]*ones(T,1) - data[.,3]);
u4i =  (data[i,4]*ones(T,1) - data[.,4]);
u5i =  (data[i,5]*ones(T,1) - data[.,5]);
u6i =  (data[i,6]*ones(T,1) - data[.,6]);
u7i =  (data[i,7]*ones(T,1) - data[.,7]);
u8i =  (data[i,8]*ones(T,1) - data[.,8]);
u9i =  (data[i,9]*ones(T,1) - data[.,9]);
u10i =  (data[i,10]*ones(T,1) - data[.,10]);
u11i =  (data[i,11]*ones(T,1) - data[.,11]);
u12i =  (data[i,12]*ones(T,1) - data[.,12]);
u13i =  (data[i,13]*ones(T,1) - data[.,13]);

j = 1;
do until j>T;
 u0j =  (data[j,lag_1]*ones(T,1) - data[.,lag_1]);
 u1j =  (data[j,1]*ones(T,1) - data[.,1]);
 u2j =  (data[j,2]*ones(T,1) - data[.,2]);
 u3j =  (data[j,3]*ones(T,1) - data[.,3]);
 u4j =  (data[j,4]*ones(T,1) - data[.,4]);
 u5j =  (data[j,5]*ones(T,1) - data[.,5]);
 u6j =  (data[j,5]*ones(T,1) - data[.,6]);
 u7j =  (data[j,7]*ones(T,1) - data[.,7]);
 u8j =  (data[j,8]*ones(T,1) - data[.,8]);
 u9j =  (data[j,9]*ones(T,1) - data[.,9]);
 u10j =  (data[j,10]*ones(T,1) - data[.,10]);
 u11j =  (data[j,11]*ones(T,1) - data[.,11]);
 u12j =  (data[j,12]*ones(T,1) - data[.,12]);
 u13j =  (data[j,13]*ones(T,1) - data[.,13]);

 w11 = (fi(u0i/b2)/b2).*(fi(u1j/b2)/b2);
 w12 = (fi(u0j/b2)/b2).*(fi(u1i/b2)/b2);
 w21  = (fi(u0i/b2)/b2).*(fi(u2j/b2)/b2);
 w22  = (fi(u0j/b2)/b2).*(fi(u2i/b2)/b2);
 w31  = (fi(u0i/b2)/b2).*(fi(u3j/b2)/b2);
 w32  = (fi(u0j/b2)/b2).*(fi(u3i/b2)/b2);
 w41  = (fi(u0i/b2)/b2).*(fi(u4j/b2)/b2);
 w42  = (fi(u0j/b2)/b2).*(fi(u4i/b2)/b2);
 w51  = (fi(u0i/b2)/b2).*(fi(u5j/b2)/b2);
 w52  = (fi(u0j/b2)/b2).*(fi(u5i/b2)/b2);
 w61  = (fi(u0i/b2)/b2).*(fi(u6j/b2)/b2);
 w62  = (fi(u0j/b2)/b2).*(fi(u6i/b2)/b2);
 w71  = (fi(u0i/b2)/b2).*(fi(u7j/b2)/b2);
 w72  = (fi(u0j/b2)/b2).*(fi(u7i/b2)/b2);
 w81  = (fi(u0i/b2)/b2).*(fi(u8j/b2)/b2);
 w82  = (fi(u0j/b2)/b2).*(fi(u8i/b2)/b2);
 w91  = (fi(u0i/b2)/b2).*(fi(u9j/b2)/b2);
 w92  = (fi(u0j/b2)/b2).*(fi(u9i/b2)/b2);
 w101 = (fi(u0i/b2)/b2).*(fi(u10j/b2)/b2);
 w102 = (fi(u0j/b2)/b2).*(fi(u10i/b2)/b2);
 w111 = (fi(u0i/b2)/b2).*(fi(u11j/b2)/b2);
 w112 = (fi(u0j/b2)/b2).*(fi(u11i/b2)/b2);
 w121 = (fi(u0i/b2)/b2).*(fi(u12j/b2)/b2);
 w122 = (fi(u0j/b2)/b2).*(fi(u12i/b2)/b2);
 w131 = (fi(u0i/b2)/b2).*(fi(u13j/b2)/b2);
 w132 = (fi(u0j/b2)/b2).*(fi(u13i/b2)/b2);
 wmat11  = diagrv((eye(T),w11));
 wmat12  = diagrv((eye(T),w12));
 wmat21  = diagrv((eye(T),w21));
 wmat22  = diagrv((eye(T),w22));
 wmat31  = diagrv((eye(T),w31));
 wmat32  = diagrv((eye(T),w32));
 wmat41  = diagrv((eye(T),w41));
 wmat42  = diagrv((eye(T),w42));
 wmat51  = diagrv((eye(T),w51));
 wmat52  = diagrv((eye(T),w52));
 wmat61  = diagrv((eye(T),w61));
 wmat62  = diagrv((eye(T),w62));
 wmat71  = diagrv((eye(T),w71));
 wmat72  = diagrv((eye(T),w72));
 wmat81  = diagrv((eye(T),w81));
 wmat82  = diagrv((eye(T),w82));
 wmat91  = diagrv((eye(T),w91));
 wmat92  = diagrv((eye(T),w92));
 wmat101 = diagrv((eye(T),w101));
 wmat102 = diagrv((eye(T),w102));
 wmat111 = diagrv((eye(T),w111));
 wmat112 = diagrv((eye(T),w112));
 wmat121 = diagrv((eye(T),w121));
 wmat122 = diagrv((eye(T),w122));
 wmat131 = diagrv((eye(T),w131));
 wmat132 = diagrv((eye(T),w132));
 
 xx11  =  ones(T,1)~u0i~u1j ;
 xx12  =  ones(T,1)~u0j~u1i ;
 xx21  =  ones(T,1)~u0i~u2j;
 xx22  =  ones(T,1)~u0j~u2i;
 xx31  =  ones(T,1)~u0i~u3j;
 xx32  =  ones(T,1)~u0j~u3i;
 xx41  =  ones(T,1)~u0i~u4j;
 xx42  =  ones(T,1)~u0j~u4i;
 xx51  =  ones(T,1)~u0i~u5j;
 xx52  =  ones(T,1)~u0j~u5i;
 xx61  =  ones(T,1)~u0i~u6j;
 xx62  =  ones(T,1)~u0j~u6i;
 xx71  =  ones(T,1)~u0i~u7j ;
 xx72  =  ones(T,1)~u0j~u7i ;
 xx81  =  ones(T,1)~u0i~u8j;
 xx82  =  ones(T,1)~u0j~u8i;
 xx91  =  ones(T,1)~u0i~u9j;
 xx92  =  ones(T,1)~u0j~u9i;
 xx101 =  ones(T,1)~u0i~u10j;
 xx102 =  ones(T,1)~u0j~u10i;
 xx111 =  ones(T,1)~u0i~u11j;
 xx112 =  ones(T,1)~u0j~u11i;
 xx121 =  ones(T,1)~u0i~u12j;
 xx122 =  ones(T,1)~u0j~u12i;
 
 /*xx11  =  ones(T,1);
 xx12  =  ones(T,1);
 xx21  =  ones(T,1);
 xx22  =  ones(T,1);
 xx31  =  ones(T,1);
 xx32  =  ones(T,1);
 xx41  =  ones(T,1);
 xx42  =  ones(T,1);
 xx51  =  ones(T,1);
 xx52  =  ones(T,1);
 xx61  =  ones(T,1);
 xx62  =  ones(T,1);
 xx71  =  ones(T,1) ;
 xx72  =  ones(T,1) ;
 xx81  =  ones(T,1);
 xx82  =  ones(T,1);
 xx91  =  ones(T,1);
 xx92  =  ones(T,1);
 xx101  =  ones(T,1);
 xx102  =  ones(T,1);
 xx111  =  ones(T,1);
 xx112  =  ones(T,1);
 xx121  =  ones(T,1);
 xx122  =  ones(T,1);
 xx131  =  ones(T,1);
 xx132  =  ones(T,1);*/

if sumc(w11)> 0;
if det(xx11'wmat11*xx11) < epsln ;
      m2[j,1]  =  w11'y/sumc(w11) ;
else;
 beta =  pinv(xx11'wmat11*xx11)*xx11'wmat11'y ;
 m2[j,1]  =  beta[1,.] ;
endif;
else;
m2[j,1] = 0;
a11 = a11+1;
endif;

if sumc(w12)> 0;
if det(xx12'wmat12*xx12) < epsln ;
      m2[j,1]  =  w12'y/sumc(w12) ;
else;
 beta =  pinv(xx12'wmat12*xx12)*xx12'wmat12'y ;
 m2[j,2]  =  beta[1,.] ;
endif;
else;
m2[j,2] = 0;
a12 = a12+1;
endif;

if sumc(w21)> 0;
if det(xx21'wmat21*xx21) < epsln ;
      m2[j,1]  =  w21'y/sumc(w21) ;
else;
 beta =  pinv(xx21'wmat21*xx21)*xx21'wmat21'y ;
 m2[j,3]  =  beta[1,.] ;
endif;
else;
m2[j,3] = 0;
a21 = a21+1;
endif;

if sumc(w22)> 0;
if det(xx22'wmat22*xx22) < epsln ;
      m2[j,1]  =  w22'y/sumc(w22) ;
else;
 beta =  pinv(xx22'wmat22*xx22)*xx22'wmat22'y ;
 m2[j,4]  =  beta[1,.] ;
endif;
else;
m2[j,4] = 0;
a22 = a22+1;
endif;

if sumc(w31)> 0;
if det(xx31'wmat31*xx31) < epsln ;
      m2[j,5]  =  w31'y/sumc(w31) ;
else;
 beta =  pinv(xx31'wmat31*xx31)*xx31'wmat31'y ;
 m2[j,5]  =  beta[1,.] ;
endif;
else;
m2[j,5] = 0;
a31 = a31+1;
endif;

if sumc(w32)> 0;
if det(xx32'wmat32*xx32) < epsln ;
      m2[j,6]  =  w32'y/sumc(w32) ;
else;
 beta =  pinv(xx32'wmat32*xx32)*xx32'wmat32'y ;
 m2[j,6]  =  beta[1,.] ;
endif;
else;
m2[j,6] = 0;
a32 = a32+1;
endif;

if sumc(w41)> 0;
if det(xx41'wmat41*xx41) < epsln ;
      m2[j,7]  =  w41'y/sumc(w41) ;
else;
 beta =  pinv(xx41'wmat41*xx41)*xx41'wmat41'y ;
 m2[j,7]  =  beta[1,.] ;
endif;
else;
m2[j,7] = 0;
a41 = a41+1;
endif;

if sumc(w42)> 0;
if det(xx42'wmat42*xx42) < epsln ;
      m2[j,8]  =  w42'y/sumc(w42) ;
else;
 beta =  pinv(xx42'wmat42*xx42)*xx42'wmat42'y ;
 m2[j,8]  =  beta[1,.] ;
endif;
else;
m2[j,8] = 0;
a42 = a42+1;
endif;

if sumc(w51)> 0;
if det(xx51'wmat51*xx51) < epsln ;
      m2[j,9]  =  w51'y/sumc(w51) ;
else;
 beta =  pinv(xx51'wmat51*xx51)*xx51'wmat51'y ;
 m2[j,9]  =  beta[1,.] ;
endif;
else;
m2[j,9] = 0;
a51 = a51+1;
endif;

if sumc(w52)> 0;
if det(xx52'wmat52*xx52) < epsln ;
      m2[j,10]  =  w52'y/sumc(w52) ;
else;
 beta =  pinv(xx52'wmat52*xx52)*xx52'wmat52'y ;
 m2[j,10]  =  beta[1,.] ;
endif;
else;
m2[j,10] = 0;
a52 = a52+1;
endif;

if sumc(w61)> 0;
if det(xx61'wmat61*xx61) < epsln ;
      m2[j,11]  =  w61'y/sumc(w61) ;
else;
 beta =  pinv(xx61'wmat61*xx61)*xx61'wmat61'y ;
 m2[j,11]  =  beta[1,.] ;
endif;
else;
m2[j,11] = 0;
a61 = a61+1;
endif;

if sumc(w62)> 0;
if det(xx62'wmat62*xx62) < epsln ;
      m2[j,12]  =  w62'y/sumc(w62) ;
else;
 beta =  pinv(xx62'wmat62*xx62)*xx62'wmat62'y ;
 m2[j,12]  =  beta[1,.] ;
endif;
else;
m2[j,12] = 0;
a62 = a62+1;
endif;

if sumc(w71)> 0;
if det(xx71'wmat71*xx71) < epsln ;
      m2[j,13]  =  w71'y/sumc(w71) ;
else;
 beta =  pinv(xx71'wmat71*xx71)*xx71'wmat71'y ;
 m2[j,13]  =  beta[1,.] ;
endif;
else;
m2[j,13] = 0;
a71 = a71+1;
endif;

if sumc(w72)> 0;
if det(xx72'wmat72*xx72) < epsln ;
      m2[j,14]  =  w72'y/sumc(w72) ;
else;
 beta =  pinv(xx72'wmat72*xx72)*xx72'wmat72'y ;
 m2[j,14]  =  beta[1,.] ;
endif;
else;
m2[j,14] = 0;
a72 = a72+1;
endif;

if sumc(w81)> 0;
if det(xx81'wmat81*xx81) < epsln ;
      m2[j,15]  =  w81'y/sumc(w81) ;
else;
 beta =  pinv(xx81'wmat81*xx81)*xx81'wmat81'y ;
 m2[j,15]  =  beta[1,.] ;
endif;
else;
m2[j,15] = 0;
a81 = a81+1;
endif;

if sumc(w82)> 0;
if det(xx82'wmat82*xx82) < epsln ;
      m2[j,16]  =  w82'y/sumc(w82) ;
else;
 beta =  pinv(xx82'wmat82*xx82)*xx82'wmat82'y ;
 m2[j,16]  =  beta[1,.] ;
endif;
else;
m2[j,16] = 0;
a82 = a82+1;
endif;

if sumc(w91)> 0;
if det(xx91'wmat91*xx91) < epsln ;
      m2[j,17]  =  w91'y/sumc(w91) ;
else;
 beta =  pinv(xx91'wmat91*xx91)*xx91'wmat91'y ;
 m2[j,17]  =  beta[1,.] ;
endif;
else;
m2[j,17] = 0;
a91 = a91+1;
endif;

if sumc(w92)> 0;
if det(xx92'wmat92*xx92) < epsln ;
      m2[j,18]  =  w92'y/sumc(w92) ;
else;
 beta =  pinv(xx92'wmat92*xx92)*xx92'wmat92'y ;
 m2[j,18]  =  beta[1,.] ;
endif;
else;
m2[j,18] = 0;
a92 = a92+1;
endif;

if sumc(w101)> 0;
if det(xx101'wmat101*xx101) < epsln ;
      m2[j,19]  =  w101'y/sumc(w101) ;
else;
 beta =  pinv(xx101'wmat101*xx101)*xx101'wmat101'y ;
 m2[j,19]  =  beta[1,.] ;
endif;
else;
m2[j,19] = 0;
a101 = a101+1;
endif;

if sumc(w102)> 0;
if det(xx102'wmat102*xx102) < epsln ;
      m2[j,20]  =  w102'y/sumc(w102) ;
else;
 beta =  pinv(xx102'wmat102*xx102)*xx102'wmat102'y ;
 m2[j,20]  =  beta[1,.] ;
endif;
else;
m2[j,20] = 0;
a102 = a102+1;
endif;

if sumc(w111)> 0;
if det(xx111'wmat111*xx111) < epsln ;
      m2[j,21]  =  w111'y/sumc(w111) ;
else;
 beta =  pinv(xx111'wmat111*xx111)*xx111'wmat111'y ;
 m2[j,21]  =  beta[1,.] ;
endif;
else;
m2[j,21] = 0;
a111 = a111+1;
endif;

if sumc(w112)> 0;
if det(xx112'wmat112*xx112) < epsln ;
      m2[j,22]  =  w112'y/sumc(w112) ;
else;
 beta =  pinv(xx112'wmat112*xx112)*xx112'wmat112'y ;
 m2[j,22]  =  beta[1,.] ;
endif;
else;
m2[j,22] = 0;
a112 = a112+1;
endif;

if sumc(w121)> 0;
if det(xx121'wmat121*xx121) < epsln ;
      m2[j,23]  =  w121'y/sumc(w121) ;
else;
 beta =  pinv(xx121'wmat121*xx121)*xx121'wmat121'y ;
 m2[j,23]  =  beta[1,.] ;
endif;
else;
m2[j,23] = 0;
a121 = a121+1;
endif;

if sumc(w122)> 0;
if det(xx122'wmat122*xx122) < epsln ;
      m2[j,24]  =  w122'y/sumc(w122) ;
else;
 beta =  pinv(xx122'wmat122*xx122)*xx122'wmat122'y ;
 m2[j,24]  =  beta[1,.] ;
endif;
else;
m2[j,24] = 0;
a122 = a122+1;
endif;

if sumc(w131)> 0;
if det(xx131'wmat131*xx131) < epsln ;
      m2[j,25]  =  w131'y/sumc(w131) ;
else;
 beta =  pinv(xx131'wmat131*xx131)*xx131'wmat131'y ;
 m2[j,25]  =  beta[1,.] ;
endif;
else;
m2[j,25] = 0;
a131 = a131+1;
endif;

if sumc(w132)> 0;
if det(xx132'wmat132*xx132) < epsln ;
      m2[j,26]  =  w132'y/sumc(w132) ;
else;
 beta =  pinv(xx132'wmat132*xx132)*xx132'wmat132'y ;
 m2[j,26]  =  beta[1,.] ;
endif;
else;
m2[j,26] = 0;
a132 = a132+1;
endif;

j = j+1;
endo;

m2d[i,1]  = 1/(T-a11)*sumc(m2[.,1]);
m2d[i,2]  = 1/(T-a12)*sumc(m2[.,2]);
m2d[i,3]  = 1/(T-a21)*sumc(m2[.,3]);
m2d[i,4]  = 1/(T-a22)*sumc(m2[.,4]);
m2d[i,5]  = 1/(T-a31)*sumc(m2[.,5]);
m2d[i,6]  = 1/(T-a32)*sumc(m2[.,6]);
m2d[i,7]  = 1/(T-a41)*sumc(m2[.,7]);
m2d[i,8]  = 1/(T-a42)*sumc(m2[.,8]);
m2d[i,9]  = 1/(T-a51)*sumc(m2[.,9]);
m2d[i,10] = 1/(T-a52)*sumc(m2[.,10]);
m2d[i,11] = 1/(T-a61)*sumc(m2[.,11]);
m2d[i,12] = 1/(T-a62)*sumc(m2[.,12]);
m2d[i,13] = 1/(T-a71)*sumc(m2[.,13]);
m2d[i,14] = 1/(T-a72)*sumc(m2[.,14]);
m2d[i,15] = 1/(T-a81)*sumc(m2[.,15]);
m2d[i,16] = 1/(T-a82)*sumc(m2[.,16]);
m2d[i,17] = 1/(T-a91)*sumc(m2[.,17]);
m2d[i,18] = 1/(T-a92)*sumc(m2[.,18]);
m2d[i,19] = 1/(T-a101)*sumc(m2[.,19]);
m2d[i,20] = 1/(T-a102)*sumc(m2[.,20]);
m2d[i,21] = 1/(T-a111)*sumc(m2[.,21]);
m2d[i,22] = 1/(T-a112)*sumc(m2[.,22]);
m2d[i,23] = 1/(T-a121)*sumc(m2[.,23]);
m2d[i,24] = 1/(T-a122)*sumc(m2[.,24]);
m2d[i,25] = 1/(T-a121)*sumc(m2[.,25]);
m2d[i,26] = 1/(T-a122)*sumc(m2[.,26]);
i = i+1;
endo;

yhat1  = y-meanc(y)-(m2d[.,1]-meanc(m2d[.,1])+m2d[.,2]-meanc(m2d[.,2]));
yhat2  = y-meanc(y)-(m2d[.,3]-meanc(m2d[.,3])+m2d[.,4]-meanc(m2d[.,4]));
yhat3  = y-meanc(y)-(m2d[.,5]-meanc(m2d[.,5])+m2d[.,6]-meanc(m2d[.,6]));
yhat4  = y-meanc(y)-(m2d[.,7]-meanc(m2d[.,7])+m2d[.,8]-meanc(m2d[.,8]));
yhat5  = y-meanc(y)-(m2d[.,9]-meanc(m2d[.,9])+m2d[.,10]-meanc(m2d[.,10]));
yhat6  = y-meanc(y)-(m2d[.,11]-meanc(m2d[.,11])+m2d[.,12]-meanc(m2d[.,12]));
yhat7  = y-meanc(y)-(m2d[.,13]-meanc(m2d[.,13])+m2d[.,14]-meanc(m2d[.,14]));
yhat8  = y-meanc(y)-(m2d[.,15]-meanc(m2d[.,15])+m2d[.,16]-meanc(m2d[.,16]));
yhat9  = y-meanc(y)-(m2d[.,17]-meanc(m2d[.,17])+m2d[.,18]-meanc(m2d[.,18]));
yhat10 = y-meanc(y)-(m2d[.,19]-meanc(m2d[.,19])+m2d[.,20]-meanc(m2d[.,20]));
yhat11 = y-meanc(y)-(m2d[.,21]-meanc(m2d[.,21])+m2d[.,22]-meanc(m2d[.,22]));
yhat12 = y-meanc(y)-(m2d[.,23]-meanc(m2d[.,23])+m2d[.,24]-meanc(m2d[.,24]));
yhat13 = y-meanc(y)-(m2d[.,25]-meanc(m2d[.,25])+m2d[.,26]-meanc(m2d[.,26]));

R1  = yhat1'yhat1;
R2  = yhat2'yhat2;
R3  = yhat3'yhat3;
R4  = yhat4'yhat4;
R5  = yhat5'yhat5;
R6  = yhat6'yhat6;
R7  = yhat7'yhat7;
R8  = yhat8'yhat8;
R9  = yhat9'yhat9;
R10 = yhat10'yhat10;
R11 = yhat11'yhat11;
R12 = yhat12'yhat12;
R13 = yhat13'yhat13;

Ahat1[k,kc] = 1/T*sumc(R1);
Ahat2[k,kc] = 1/T*sumc(R2);
Ahat3[k,kc] = 1/T*sumc(R3);
Ahat4[k,kc] = 1/T*sumc(R4);
Ahat5[k,kc] = 1/T*sumc(R5);
Ahat6[k,kc] = 1/T*sumc(R6);
Ahat7[k,kc] = 1/T*sumc(R7);
Ahat8[k,kc] = 1/T*sumc(R8);
Ahat9[k,kc] = 1/T*sumc(R9);
Ahat10[k,kc] = 1/T*sumc(R10);
Ahat11[k,kc] = 1/T*sumc(R11);
Ahat12[k,kc] = 1/T*sumc(R12);
Ahat13[k,kc] = 1/T*sumc(R13);

Bhat1[k,kc]  =  2/b2*1/sqrt(2*pi)*1/T*sumc((BR1.*qz2[.,1]+BR1.*qz2[.,2])./phat[.,1])*1/T;
Bhat2[k,kc]  =  2/b2*1/sqrt(2*pi)*1/T*sumc((BR2.*qz2[.,1]+BR2.*qz2[.,3])./phat[.,2])*1/T;
Bhat3[k,kc]  =  2/b2*1/sqrt(2*pi)*1/T*sumc((BR3.*qz2[.,1]+BR3.*qz2[.,4])./phat[.,3])*1/T;
Bhat4[k,kc]  =  2/b2*1/sqrt(2*pi)*1/T*sumc((BR4.*qz2[.,1]+BR4.*qz2[.,5])./phat[.,4])*1/T;
Bhat5[k,kc]  =  2/b2*1/sqrt(2*pi)*1/T*sumc((BR5.*qz2[.,1]+BR5.*qz2[.,6])./phat[.,5])*1/T;
Bhat6[k,kc]  =  2/b2*1/sqrt(2*pi)*1/T*sumc((BR6.*qz2[.,1]+BR6.*qz2[.,7])./phat[.,6])*1/T;
Bhat7[k,kc]  =  2/b2*1/sqrt(2*pi)*1/T*sumc((BR7.*qz2[.,1]+BR7.*qz2[.,8])./phat[.,7])*1/T;
Bhat8[k,kc]  =  2/b2*1/sqrt(2*pi)*1/T*sumc((BR8.*qz2[.,1]+BR8.*qz2[.,9])./phat[.,8])*1/T;
Bhat9[k,kc]  =  2/b2*1/sqrt(2*pi)*1/T*sumc((BR9.*qz2[.,1]+BR9.*qz2[.,10])./phat[.,9])*1/T;
Bhat10[k,kc] =  2/b2*1/sqrt(2*pi)*1/T*sumc((BR10.*qz2[.,1]+BR10.*qz2[.,11])./phat[.,10])*1/T;
Bhat11[k,kc] =  2/b2*1/sqrt(2*pi)*1/T*sumc((BR11.*qz2[.,1]+BR11.*qz2[.,12])./phat[.,11])*1/T;
Bhat12[k,kc] =  2/b2*1/sqrt(2*pi)*1/T*sumc((BR12.*qz2[.,1]+BR12.*qz2[.,13])./phat[.,12])*1/T;
Bhat13[k,kc] =  2/b2*1/sqrt(2*pi)*1/T*sumc((BR13.*qz2[.,1]+BR13.*qz2[.,14])./phat[.,13])*1/T;

FPE1[k,kc] = Ahat1[k,kc]+Bhat1[k,kc];
FPE2[k,kc] = Ahat2[k,kc]+Bhat2[k,kc];
FPE3[k,kc] = Ahat3[k,kc]+Bhat3[k,kc];
FPE4[k,kc] = Ahat4[k,kc]+Bhat4[k,kc];
FPE5[k,kc] = Ahat5[k,kc]+Bhat5[k,kc];
FPE6[k,kc] = Ahat6[k,kc]+Bhat6[k,kc];
FPE7[k,kc] = Ahat7[k,kc]+Bhat7[k,kc];
FPE8[k,kc] = Ahat8[k,kc]+Bhat8[k,kc];
FPE9[k,kc] = Ahat9[k,kc]+Bhat9[k,kc];
FPE10[k,kc] = Ahat10[k,kc]+Bhat10[k,kc];
FPE11[k,kc] = Ahat11[k,kc]+Bhat11[k,kc];
FPE12[k,kc] = Ahat12[k,kc]+Bhat12[k,kc];
FPE13[k,kc] = Ahat13[k,kc]+Bhat13[k,kc];
kc = kc+1;
endo;

cmin1 = minc(FPE1[k,.]');
cmin2 = minc(FPE2[k,.]');
cmin3 = minc(FPE3[k,.]');
cmin4 = minc(FPE4[k,.]');
cmin5 = minc(FPE5[k,.]');
cmin6 = minc(FPE6[k,.]');
cmin7 = minc(FPE7[k,.]');
cmin8 = minc(FPE8[k,.]');
cmin9 = minc(FPE9[k,.]');
cmin10 = minc(FPE10[k,.]');
cmin11 = minc(FPE11[k,.]');
cmin12 = minc(FPE12[k,.]');
cmin13 = minc(FPE13[k,.]');
minFPE = cmin1~cmin2~cmin3~cmin4~cmin5~cmin6~cmin7~cmin8~cmin9~cmin10~cmin11~cmin12~cmin13;
minFPE[.,lag_1] = 10^10;
AFPE[k,2] = minc(minFPE');
lag[k,2] = minindc(minFPE');
lag_2 = lag[k,2];

/****************************Three Lags Included ******************************************/

       
/****************************Three Lags Included Estimation of B part *******************************************/

i = 1;
a11 = 0;
a12 = 0;
a13 = 0;
a21 = 0;
a22 = 0;
a23 = 0;
a31 = 0;
a32 = 0;
a33 = 0;
a41 = 0;
a42 = 0;
a43 = 0;
a51 = 0;
a52 = 0;
a53 = 0;
a61 = 0;
a62 = 0;
a63 = 0;
a71 = 0;
a72 = 0;
a73 = 0;
a81 = 0;
a82 = 0;
a83 = 0;
a91 = 0;
a92 = 0;
a93 = 0;
a101 = 0;
a102 = 0;
a103 = 0;
a111 = 0;
a112 = 0;
a113 = 0;
a121 = 0;
a122 = 0;
a123 = 0;
a131 = 0;
a132 = 0;
a133 = 0;
m3 = zeros(T,39);
m3d = zeros(T,39);
qz3 = zeros(T,39);
i = 1;
do until i>T;
u01i = data[i,lag_1]*ones(T,1) - data[.,lag_1];
u02i = data[i,lag_2]*ones(T,1) - data[.,lag_2];
u0i = u01i~u02i;
u1i =  (data[i,1]*ones(T,1) - data[.,1]);
u2i =  (data[i,2]*ones(T,1) - data[.,2]);
u3i =  (data[i,3]*ones(T,1) - data[.,3]);
u4i =  (data[i,4]*ones(T,1) - data[.,4]);
u5i =  (data[i,5]*ones(T,1) - data[.,5]);
u6i =  (data[i,6]*ones(T,1) - data[.,6]);
u7i =  (data[i,7]*ones(T,1) - data[.,7]);
u8i =  (data[i,8]*ones(T,1) - data[.,8]);
u9i =  (data[i,9]*ones(T,1) - data[.,9]);
u10i =  (data[i,10]*ones(T,1) - data[.,10]);
u11i =  (data[i,11]*ones(T,1) - data[.,11]);
u12i =  (data[i,12]*ones(T,1) - data[.,12]);
u13i =  (data[i,13]*ones(T,1) - data[.,13]);
j = 1;
do until j>T;
u01j = data[j,lag_1]*ones(T,1) - data[.,lag_1];
u02j = data[j,lag_2]*ones(T,1) - data[.,lag_2];
 u1j =  (data[j,1]*ones(T,1) - data[.,1]);
 u2j =  (data[j,2]*ones(T,1) - data[.,2]);
 u3j =  (data[j,3]*ones(T,1) - data[.,3]);
 u4j =  (data[j,4]*ones(T,1) - data[.,4]);
 u5j =  (data[j,5]*ones(T,1) - data[.,5]);
 u6j =  (data[j,5]*ones(T,1) - data[.,6]);
 u7j =  (data[j,7]*ones(T,1) - data[.,7]);
 u8j =  (data[j,8]*ones(T,1) - data[.,8]);
 u9j =  (data[j,9]*ones(T,1) - data[.,9]);
 u10j =  (data[j,10]*ones(T,1) - data[.,10]);
 u11j =  (data[j,11]*ones(T,1) - data[.,11]);
 u12j =  (data[j,12]*ones(T,1) - data[.,12]);
 u13j =  (data[j,13]*ones(T,1) - data[.,13]);
 w11 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u1j/h1)/h1);
 w12 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u1j/h1)/h1);
 w13 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u1i/h1)/h1);
 w21 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u2j/h1)/h1);
 w22 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u2j/h1)/h1);
 w23 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u2i/h1)/h1);
 w31 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u3j/h1)/h1);
 w32 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u3j/h1)/h1);
 w33 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u3i/h1)/h1);
 w41 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u4j/h1)/h1);
 w42 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u4j/h1)/h1);
 w43 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u4i/h1)/h1);
 w51 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u5j/h1)/h1);
 w52 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u5j/h1)/h1);
 w53 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u5i/h1)/h1);
 w61 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u6j/h1)/h1);
 w62 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u6j/h1)/h1);
 w63 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u6i/h1)/h1);
 w71 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1.*(fi(u7j/h1)/h1));
 w72 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u7j/h1)/h1);
 w73 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u7i/h1)/h1);
 w81 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u8j/h1)/h1);
 w82 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u8j/h1)/h1);
 w83 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u8i/h1)/h1);
 w91 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u9j/h1)/h1);
 w92 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u9j/h1)/h1);
 w93 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u9i/h1)/h1);
 w101 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u10j/h1)/h1);
 w102 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u10j/h1)/h1);
 w103 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u10i/h1)/h1);
 w111 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u11j/h1)/h1);
 w112 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u11j/h1)/h1);
 w113 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u11i/h1)/h1);
 w121 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u12j/h1)/h1);
 w122 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u12j/h1)/h1);
 w123 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u12i/h1)/h1);
 w131 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u13j/h1)/h1);
 w132 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u13j/h1)/h1);
 w133 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u13i/h1)/h1);
 
 w11[j] = 0;
 w12[j] = 0;
 w13[j] = 0;
 w21[j] = 0;
 w22[j] = 0;
 w23[j] = 0;
 w31[j] = 0;
 w32[j] = 0;
 w33[j] = 0;
 w41[j] = 0;
 w42[j] = 0;
 w43[j] = 0;
 w51[j] = 0;
 w52[j] = 0;
 w53[j] = 0;
 w61[j] = 0;
 w62[j] = 0;
 w63[j] = 0;
 w71[j] = 0;
 w72[j] = 0;
 w73[j] = 0;
 w81[j] = 0;
 w82[j] = 0;
 w83[j] = 0;
 w91[j] = 0;
 w92[j] = 0;
 w93[j] = 0;
 w101[j] = 0;
 w102[j] = 0;
 w103[j] = 0;
 w111[j] = 0;
 w112[j] = 0;
 w113[j] = 0;
 w121[j] = 0;
 w122[j] = 0;
 w123[j] = 0;
 w131[j] = 0;
 w132[j] = 0;
 w133[j] = 0;
 wmat11  = diagrv((eye(T),w11));
 wmat12  = diagrv((eye(T),w12));
 wmat13  = diagrv((eye(T),w13));
 wmat21  = diagrv((eye(T),w21));
 wmat22  = diagrv((eye(T),w22));
 wmat23  = diagrv((eye(T),w23));
 wmat31  = diagrv((eye(T),w31));
 wmat32  = diagrv((eye(T),w32));
 wmat33  = diagrv((eye(T),w33));
 wmat41  = diagrv((eye(T),w41));
 wmat42  = diagrv((eye(T),w42));
 wmat43  = diagrv((eye(T),w43));
 wmat51  = diagrv((eye(T),w51));
 wmat52  = diagrv((eye(T),w52));
 wmat53  = diagrv((eye(T),w53));
 wmat61  = diagrv((eye(T),w61));
 wmat62  = diagrv((eye(T),w62));
 wmat63  = diagrv((eye(T),w63));
 wmat71  = diagrv((eye(T),w71));
 wmat72  = diagrv((eye(T),w72));
 wmat73  = diagrv((eye(T),w73));
 wmat81  = diagrv((eye(T),w81));
 wmat82  = diagrv((eye(T),w82));
 wmat83  = diagrv((eye(T),w83));
 wmat91  = diagrv((eye(T),w91));
 wmat92  = diagrv((eye(T),w92));
 wmat93  = diagrv((eye(T),w93));
 wmat101  = diagrv((eye(T),w101));
 wmat102  = diagrv((eye(T),w102));
 wmat103  = diagrv((eye(T),w103));
 wmat111  = diagrv((eye(T),w111));
 wmat112  = diagrv((eye(T),w112));
 wmat113  = diagrv((eye(T),w113));
 wmat121  = diagrv((eye(T),w121));
 wmat122  = diagrv((eye(T),w122));
 wmat123  = diagrv((eye(T),w123));
 wmat131  = diagrv((eye(T),w131));
 wmat132  = diagrv((eye(T),w132));
 wmat133  = diagrv((eye(T),w133));
 xx11  =  ones(T,1)~u01i~u02j~u1j ;
 xx12  =  ones(T,1)~u01j~u02i~u1j ;
 xx13  =  ones(T,1)~u01j~u02j~u1i;
 xx21  =  ones(T,1)~u01i~u02j~u2j;
 xx22  =  ones(T,1)~u01j~u02i~u2j ;
 xx23  =  ones(T,1)~u01j~u02j~u2i ;
 xx31  =  ones(T,1)~u01i~u02j~u3j ;
 xx32  =  ones(T,1)~u01j~u02i~u3j ;
 xx33  =  ones(T,1)~u01j~u02j~u3i ;
 xx41  =  ones(T,1)~u01i~u02j~u4j ;
 xx42  =  ones(T,1)~u01j~u02i~u4j ;
 xx43  =  ones(T,1)~u01j~u02j~u4i ;
 xx51  =  ones(T,1)~u01i~u02j~u5j ;
 xx52  =  ones(T,1)~u01j~u02i~u5j ;
 xx53  =  ones(T,1)~u01j~u02j~u5i ;
 xx61  =  ones(T,1)~u01i~u02j~u6j ;
 xx62  =  ones(T,1)~u01j~u02i~u6j ;
 xx63  =  ones(T,1)~u01j~u02j~u6i ;
 xx71  =  ones(T,1)~u01i~u02j~u7j ;
 xx72  =  ones(T,1)~u01j~u02i~u7j ;
 xx73  =  ones(T,1)~u01j~u02j~u7i ;
 xx81  =  ones(T,1)~u01i~u02j~u8j ;
 xx82  =  ones(T,1)~u01j~u02i~u8j ;
 xx83  =  ones(T,1)~u01j~u02j~u8i ;
 xx91  =  ones(T,1)~u01i~u02j~u9j ;
 xx92  =  ones(T,1)~u01j~u02i~u9j ;
 xx93  =  ones(T,1)~u01j~u02j~u9i ;
 xx101  =  ones(T,1)~u01i~u02j~u10j;
 xx102  =  ones(T,1)~u01j~u02i~u10j;
 xx103  =  ones(T,1)~u01j~u02j~u10i;
 xx111  =  ones(T,1)~u01i~u02j~u11j;
 xx112  =  ones(T,1)~u01j~u02i~u11j;
 xx113  =  ones(T,1)~u01j~u02j~u11i;
 xx121  =  ones(T,1)~u01i~u02j~u12j;
 xx122  =  ones(T,1)~u01j~u02i~u12j;
 xx123  =  ones(T,1)~u01j~u02j~u12i;
 xx131  =  ones(T,1)~u01i~u02j~u13j;
 xx132  =  ones(T,1)~u01j~u02i~u13j;
 xx133  =  ones(T,1)~u01j~u02j~u13i;
 
/* xx11  =  ones(T,1);
 xx12  =  ones(T,1);
 xx13  =  ones(T,1);
 xx21  =  ones(T,1);
 xx22  =  ones(T,1);
 xx23  =  ones(T,1);
 xx31  =  ones(T,1);
 xx32  =  ones(T,1);
 xx33  =  ones(T,1);
 xx41  =  ones(T,1);
 xx42  =  ones(T,1);
 xx43  =  ones(T,1);
 xx51  =  ones(T,1);
 xx52  =  ones(T,1);
 xx53  =  ones(T,1);
 xx61  =  ones(T,1);
 xx62  =  ones(T,1);
 xx63  =  ones(T,1);
 xx71  =  ones(T,1);
 xx72  =  ones(T,1);
 xx73  =  ones(T,1);
 xx81  =  ones(T,1);
 xx82  =  ones(T,1);
 xx83  =  ones(T,1);
 xx91  =  ones(T,1);
 xx92  =  ones(T,1);
 xx93  =  ones(T,1);
 xx101  =  ones(T,1);
 xx102  =  ones(T,1);
 xx103  =  ones(T,1);
 xx111  =  ones(T,1);
 xx112  =  ones(T,1);
 xx113  =  ones(T,1);
 xx121  =  ones(T,1);
 xx122  =  ones(T,1);
 xx123  =  ones(T,1);
 xx131  =  ones(T,1);
 xx132  =  ones(T,1);
 xx133  =  ones(T,1);*/

if sumc(w11)> 0;
if det(xx11'wmat11*xx11) < epsln ;
      m3[j,1]  =  w11'y/sumc(w11) ;
else;
 beta =  pinv(xx11'wmat11*xx11)*xx11'wmat11'y ;
 m3[j,1]  =  beta[1,.] ;
endif;
else;
m3[j,1] = 0;
a11 = a11+1;
endif;

if sumc(w12)> 0;
if det(xx12'wmat12*xx12) < epsln ;
      m3[j,2]  =  w12'y/sumc(w12) ;
else;
 beta =  pinv(xx12'wmat12*xx12)*xx12'wmat12'y ;
 m3[j,2]  =  beta[1,.] ;
endif;
else;
m3[j,2] = 0;
a12 = a12+1;
endif;

if sumc(w13)> 0;
if det(xx13'wmat13*xx13) < epsln ;
      m3[j,3]  =  w13'y/sumc(w13) ;
else;
 beta =  pinv(xx13'wmat13*xx13)*xx13'wmat13'y ;
 m3[j,3]  =  beta[1,.] ;
endif;
else;
m3[j,3] = 0;
a13 = a13+1;
endif;

if sumc(w21)> 0;
if det(xx21'wmat21*xx21) < epsln ;
      m3[j,4]  =  w21'y/sumc(w21) ;
else;
 beta =  pinv(xx21'wmat21*xx21)*xx21'wmat21'y ;
 m3[j,4]  =  beta[1,.] ;
endif;
else;
m3[j,4] = 0;
a21 = a21+1;
endif;

if sumc(w22)> 0;
if det(xx22'wmat22*xx22) < epsln ;
      m3[j,5]  =  w22'y/sumc(w22) ;
else;
 beta =  pinv(xx22'wmat22*xx22)*xx22'wmat22'y ;
 m3[j,5]  =  beta[1,.] ;
endif;
else;
m3[j,5] = 0;
a22 = a22+1;
endif;

if sumc(w23)> 0;
if det(xx23'wmat23*xx23) < epsln ;
      m3[j,6]  =  w23'y/sumc(w23) ;
else;
 beta =  pinv(xx23'wmat23*xx23)*xx23'wmat23'y ;
 m3[j,6]  =  beta[1,.] ;
endif;
else;
m3[j,6] = 0;
a23 = a23+1;
endif;

if sumc(w31)> 0;
if det(xx31'wmat31*xx31) < epsln ;
      m3[j,7]  =  w31'y/sumc(w31) ;
else;
 beta =  pinv(xx31'wmat31*xx31)*xx31'wmat31'y ;
 m3[j,7]  =  beta[1,.] ;
endif;
else;
m3[j,7] = 0;
a31 = a31+1;
endif;

if sumc(w32)> 0;
if det(xx32'wmat32*xx32) < epsln ;
      m3[j,8]  =  w32'y/sumc(w32) ;
else;
 beta =  pinv(xx32'wmat32*xx32)*xx32'wmat32'y ;
 m3[j,8]  =  beta[1,.] ;
endif;
else;
m3[j,8] = 0;
a32 = a32+1;
endif;

if sumc(w33)> 0;
if det(xx33'wmat33*xx33) < epsln ;
      m3[j,9]  =  w33'y/sumc(w33) ;
else;
 beta =  pinv(xx33'wmat33*xx33)*xx33'wmat33'y ;
 m3[j,9]  =  beta[1,.] ;
endif;
else;
m3[j,9] = 0;
a33 = a33+1;
endif;

if sumc(w41)> 0;
if det(xx41'wmat41*xx41) < epsln ;
      m3[j,10]  =  w41'y/sumc(w41) ;
else;
 beta =  pinv(xx41'wmat41*xx41)*xx41'wmat41'y ;
 m3[j,10]  =  beta[1,.] ;
endif;
else;
m3[j,10] = 0;
a41 = a41+1;
endif;

if sumc(w42)> 0;
if det(xx42'wmat42*xx42) < epsln ;
      m3[j,11]  =  w42'y/sumc(w42) ;
else;
 beta =  pinv(xx42'wmat42*xx42)*xx42'wmat42'y ;
 m3[j,11]  =  beta[1,.] ;
endif;
else;
m3[j,11] = 0;
a42 = a42+1;
endif;

if sumc(w43)> 0;
if det(xx43'wmat43*xx43) < epsln ;
      m3[j,12]  =  w43'y/sumc(w43) ;
else;
 beta =  pinv(xx43'wmat43*xx43)*xx43'wmat43'y ;
 m3[j,12]  =  beta[1,.] ;
endif;
else;
m3[j,12] = 0;
a43 = a43+1;
endif;

if sumc(w51)> 0;
if det(xx51'wmat51*xx51) < epsln ;
      m3[j,13]  =  w51'y/sumc(w51) ;
else;
 beta =  pinv(xx51'wmat51*xx51)*xx51'wmat51'y ;
 m3[j,13]  =  beta[1,.] ;
endif;
else;
m3[j,13] = 0;
a51 = a51+1;
endif;

if sumc(w52)> 0;
if det(xx52'wmat52*xx52) < epsln ;
      m3[j,10]  =  w52'y/sumc(w52) ;
else;
 beta =  pinv(xx52'wmat52*xx52)*xx52'wmat52'y ;
 m3[j,14]  =  beta[1,.] ;
endif;
else;
m3[j,14] = 0;
a52 = a52+1;
endif;

if sumc(w53)> 0;
if det(xx53'wmat53*xx53) < epsln ;
      m3[j,15]  =  w53'y/sumc(w53) ;
else;
 beta =  pinv(xx53'wmat53*xx53)*xx53'wmat53'y ;
 m3[j,15]  =  beta[1,.] ;
endif;
else;
m3[j,15] = 0;
a53 = a53+1;
endif;

if sumc(w61)> 0;
if det(xx61'wmat61*xx61) < epsln ;
      m3[j,16]  =  w61'y/sumc(w61) ;
else;
 beta =  pinv(xx61'wmat61*xx61)*xx61'wmat61'y ;
 m3[j,16]  =  beta[1,.] ;
endif;
else;
m3[j,16] = 0;
a61 = a61+1;
endif;

if sumc(w62)> 0;
if det(xx62'wmat62*xx62) < epsln ;
      m3[j,17]  =  w62'y/sumc(w62) ;
else;
 beta =  pinv(xx62'wmat62*xx62)*xx62'wmat62'y ;
 m3[j,17]  =  beta[1,.] ;
endif;
else;
m3[j,17] = 0;
a62 = a62+1;
endif;

if sumc(w63)> 0;
if det(xx63'wmat63*xx63) < epsln ;
      m3[j,18]  =  w63'y/sumc(w63) ;
else;
 beta =  pinv(xx63'wmat63*xx63)*xx63'wmat63'y ;
 m3[j,18]  =  beta[1,.] ;
endif;
else;
m3[j,18] = 0;
a63 = a63+1;
endif;

if sumc(w71)> 0;
if det(xx71'wmat71*xx71) < epsln ;
      m3[j,19]  =  w71'y/sumc(w71) ;
else;
 beta =  pinv(xx71'wmat71*xx71)*xx71'wmat71'y ;
 m3[j,19]  =  beta[1,.] ;
endif;
else;
m3[j,19] = 0;
a71 = a71+1;
endif;

if sumc(w72)> 0;
if det(xx72'wmat72*xx72) < epsln ;
      m3[j,20]  =  w72'y/sumc(w72) ;
else;
 beta =  pinv(xx72'wmat72*xx72)*xx72'wmat72'y ;
 m3[j,20]  =  beta[1,.] ;
endif;
else;
m3[j,20] = 0;
a72 = a72+1;
endif;

if sumc(w73)> 0;
if det(xx73'wmat73*xx73) < epsln ;
      m3[j,21]  =  w73'y/sumc(w73) ;
else;
 beta =  pinv(xx73'wmat73*xx73)*xx73'wmat73'y ;
 m3[j,21]  =  beta[1,.] ;
endif;
else;
m3[j,21] = 0;
a73 = a73+1;
endif;

if sumc(w81)> 0;
if det(xx81'wmat81*xx81) < epsln ;
      m3[j,22]  =  w81'y/sumc(w81) ;
else;
 beta =  pinv(xx81'wmat81*xx81)*xx81'wmat81'y ;
 m3[j,22]  =  beta[1,.] ;
endif;
else;
m3[j,22] = 0;
a81 = a81+1;
endif;

if sumc(w82)> 0;
if det(xx82'wmat82*xx82) < epsln ;
      m3[j,23]  =  w82'y/sumc(w82) ;
else;
 beta =  pinv(xx82'wmat82*xx82)*xx82'wmat82'y ;
 m3[j,23]  =  beta[1,.] ;
endif;
else;
m3[j,23] = 0;
a82 = a82+1;
endif;

if sumc(w83)> 0;
if det(xx83'wmat83*xx83) < epsln ;
      m3[j,24]  =  w83'y/sumc(w83) ;
else;
 beta =  pinv(xx83'wmat83*xx83)*xx83'wmat83'y ;
 m3[j,24]  =  beta[1,.] ;
endif;
else;
m3[j,24] = 0;
a83 = a83+1;
endif;

if sumc(w91)> 0;
if det(xx91'wmat91*xx91) < epsln ;
      m3[j,25]  =  w91'y/sumc(w91) ;
else;
 beta =  pinv(xx91'wmat91*xx91)*xx91'wmat91'y ;
 m3[j,25]  =  beta[1,.] ;
endif;
else;
m3[j,25] = 0;
a91 = a91+1;
endif;

if sumc(w92)> 0;
if det(xx92'wmat92*xx92) < epsln ;
      m3[j,26]  =  w92'y/sumc(w92) ;
else;
 beta =  pinv(xx92'wmat92*xx92)*xx92'wmat92'y ;
 m3[j,26]  =  beta[1,.] ;
endif;
else;
m3[j,26] = 0;
a92 = a92+1;
endif;

if sumc(w93)> 0;
if det(xx93'wmat93*xx93) < epsln ;
      m3[j,27]  =  w93'y/sumc(w93) ;
else;
 beta =  pinv(xx93'wmat93*xx93)*xx93'wmat93'y ;
 m3[j,27]  =  beta[1,.] ;
endif;
else;
m3[j,27] = 0;
a93 = a93+1;
endif;

if sumc(w101)> 0;
if det(xx101'wmat101*xx101) < epsln ;
      m3[j,28]  =  w101'y/sumc(w101) ;
else;
 beta =  pinv(xx101'wmat101*xx101)*xx101'wmat101'y ;
 m3[j,28]  =  beta[1,.] ;
endif;
else;
m3[j,28] = 0;
a101 = a101+1;
endif;

if sumc(w102)> 0;
if det(xx102'wmat102*xx102) < epsln ;
      m3[j,29]  =  w102'y/sumc(w102) ;
else;
 beta =  pinv(xx102'wmat102*xx102)*xx102'wmat102'y ;
 m3[j,29]  =  beta[1,.] ;
endif;
else;
m3[j,29] = 0;
a102 = a102+1;
endif;

if sumc(w103)> 0;
if det(xx103'wmat103*xx103) < epsln ;
      m3[j,30]  =  w103'y/sumc(w103) ;
else;
 beta =  pinv(xx103'wmat103*xx103)*xx103'wmat103'y ;
 m3[j,30]  =  beta[1,.] ;
endif;
else;
m3[j,30] = 0;
a103 = a103+1;
endif;

if sumc(w111)> 0;
if det(xx111'wmat111*xx111) < epsln ;
      m3[j,31]  =  w111'y/sumc(w111) ;
else;
 beta =  pinv(xx111'wmat111*xx111)*xx111'wmat111'y ;
 m3[j,31]  =  beta[1,.] ;
endif;
else;
m3[j,31] = 0;
a111 = a111+1;
endif;

if sumc(w112)> 0;
if det(xx112'wmat112*xx112) < epsln ;
      m3[j,32]  =  w112'y/sumc(w112) ;
else;
 beta =  pinv(xx112'wmat112*xx112)*xx112'wmat112'y ;
 m3[j,32]  =  beta[1,.] ;
endif;
else;
m3[j,32] = 0;
a112 = a112+1;
endif;

if sumc(w113)> 0;
if det(xx113'wmat113*xx113) < epsln ;
      m3[j,33]  =  w113'y/sumc(w113) ;
else;
 beta =  pinv(xx113'wmat113*xx113)*xx113'wmat113'y ;
 m3[j,33]  =  beta[1,.] ;
endif;
else;
m3[j,33] = 0;
a113 = a113+1;
endif;

if sumc(w121)> 0;
if det(xx121'wmat121*xx121) < epsln ;
      m3[j,34]  =  w121'y/sumc(w121) ;
else;
 beta =  pinv(xx121'wmat121*xx121)*xx121'wmat121'y ;
 m3[j,34]  =  beta[1,.] ;
endif;
else;
m3[j,34] = 0;
a121 = a121+1;
endif;

if sumc(w122)> 0;
if det(xx122'wmat122*xx122) < epsln ;
      m3[j,35]  =  w122'y/sumc(w122) ;
else;
 beta =  pinv(xx122'wmat122*xx122)*xx122'wmat122'y ;
 m3[j,35]  =  beta[1,.] ;
endif;
else;
m3[j,35] = 0;
a122 = a122+1;
endif;

if sumc(w123)> 0;
if det(xx123'wmat123*xx123) < epsln ;
      m3[j,36]  =  w123'y/sumc(w123) ;
else;
 beta =  pinv(xx123'wmat123*xx123)*xx123'wmat123'y ;
 m3[j,36]  =  beta[1,.] ;
endif;
else;
m3[j,36] = 0;
a122 = a122+1;
endif;

if sumc(w131)> 0;
if det(xx131'wmat131*xx131) < epsln ;
      m3[j,37]  =  w131'y/sumc(w131) ;
else;
 beta =  pinv(xx131'wmat131*xx131)*xx131'wmat131'y ;
 m3[j,37]  =  beta[1,.] ;
endif;
else;
m3[j,37] = 0;
a131 = a131+1;
endif;

if sumc(w132)> 0;
if det(xx132'wmat132*xx132) < epsln ;
      m3[j,38]  =  w132'y/sumc(w132) ;
else;
 beta =  pinv(xx132'wmat132*xx132)*xx132'wmat132'y ;
 m3[j,38]  =  beta[1,.] ;
endif;
else;
m3[j,38] = 0;
a132 = a132+1;
endif;

if sumc(w133)> 0;
if det(xx133'wmat133*xx133) < epsln ;
      m3[j,39]  =  w133'y/sumc(w133) ;
else;
 beta =  pinv(xx133'wmat133*xx133)*xx133'wmat133'y ;
 m3[j,39]  =  beta[1,.] ;
endif;
else;
m3[j,39] = 0;
a133 = a133+1;
endif;

j = j+1;
endo;
m3d[i,1] = 1/(T-a11)*sumc(m3[.,1]);
m3d[i,2] = 1/(T-a12)*sumc(m3[.,2]);
m3d[i,3] = 1/(T-a13)*sumc(m3[.,3]);
m3d[i,4] = 1/(T-a21)*sumc(m3[.,4]);
m3d[i,5] = 1/(T-a22)*sumc(m3[.,5]);
m3d[i,6] = 1/(T-a23)*sumc(m3[.,6]);
m3d[i,7] = 1/(T-a31)*sumc(m3[.,7]);
m3d[i,8] = 1/(T-a32)*sumc(m3[.,8]);
m3d[i,9] = 1/(T-a33)*sumc(m3[.,9]);
m3d[i,10] = 1/(T-a41)*sumc(m3[.,10]);
m3d[i,11] = 1/(T-a42)*sumc(m3[.,11]);
m3d[i,12] = 1/(T-a43)*sumc(m3[.,12]);
m3d[i,13] = 1/(T-a51)*sumc(m3[.,13]);
m3d[i,14] = 1/(T-a52)*sumc(m3[.,14]);
m3d[i,15] = 1/(T-a53)*sumc(m3[.,15]);
m3d[i,16] = 1/(T-a61)*sumc(m3[.,16]);
m3d[i,17] = 1/(T-a62)*sumc(m3[.,17]);
m3d[i,18] = 1/(T-a63)*sumc(m3[.,18]);
m3d[i,19] = 1/(T-a71)*sumc(m3[.,19]);
m3d[i,20] = 1/(T-a72)*sumc(m3[.,20]);
m3d[i,21] = 1/(T-a73)*sumc(m3[.,21]);
m3d[i,22] = 1/(T-a81)*sumc(m3[.,22]);
m3d[i,23] = 1/(T-a82)*sumc(m3[.,23]);
m3d[i,24] = 1/(T-a83)*sumc(m3[.,24]);
m3d[i,25] = 1/(T-a91)*sumc(m3[.,25]);
m3d[i,26] = 1/(T-a92)*sumc(m3[.,26]);
m3d[i,27] = 1/(T-a93)*sumc(m3[.,27]);
m3d[i,28] = 1/(T-a101)*sumc(m3[.,28]);
m3d[i,29] = 1/(T-a102)*sumc(m3[.,29]);
m3d[i,30] = 1/(T-a103)*sumc(m3[.,30]);
m3d[i,31] = 1/(T-a111)*sumc(m3[.,31]);
m3d[i,32] = 1/(T-a112)*sumc(m3[.,32]);
m3d[i,33] = 1/(T-a113)*sumc(m3[.,33]);
m3d[i,34] = 1/(T-a121)*sumc(m3[.,34]);
m3d[i,35] = 1/(T-a122)*sumc(m3[.,35]);
m3d[i,36] = 1/(T-a123)*sumc(m3[.,36]);
m3d[i,37] = 1/(T-a131)*sumc(m3[.,37]);
m3d[i,38] = 1/(T-a132)*sumc(m3[.,38]);
m3d[i,39] = 1/(T-a133)*sumc(m3[.,39]);

/*three dimension kernel estimator of the density */
LVO =  ones(T,1);
LVO[i] = 0;
phat[i,1]  = sumc(LVO.*fi(u01i/hs3).*fi(u02i/hs3).*fi(u1i/hs3))/(T*hs3^3);
phat[i,2]  = sumc(LVO.*fi(u01i/hs3).*fi(u02i/hs3).*fi(u2i/hs3))/(T*hs3^3);
phat[i,3]  = sumc(LVO.*fi(u01i/hs3).*fi(u02i/hs3).*fi(u3i/hs3))/(T*hs3^3);
phat[i,4]  = sumc(LVO.*fi(u01i/hs3).*fi(u02i/hs3).*fi(u4i/hs3))/(T*hs3^3);
phat[i,5]  = sumc(LVO.*fi(u01i/hs3).*fi(u02i/hs3).*fi(u5i/hs3))/(T*hs3^3);
phat[i,6]  = sumc(LVO.*fi(u01i/hs3).*fi(u02i/hs3).*fi(u6i/hs3))/(T*hs3^3);
phat[i,7]  = sumc(LVO.*fi(u01i/hs3).*fi(u02i/hs3).*fi(u7i/hs3))/(T*hs3^3);
phat[i,8]  = sumc(LVO.*fi(u01i/hs3).*fi(u02i/hs3).*fi(u8i/hs3))/(T*hs3^3);
phat[i,9]  = sumc(LVO.*fi(u01i/hs3).*fi(u02i/hs3).*fi(u9i/hs3))/(T*hs3^3);
phat[i,10] = sumc(LVO.*fi(u01i/hs3).*fi(u02i/hs3).*fi(u10i/hs3))/(T*hs3^3);
phat[i,11] = sumc(LVO.*fi(u01i/hs3).*fi(u02i/hs3).*fi(u11i/hs3))/(T*hs3^3);
phat[i,12] = sumc(LVO.*fi(u01i/hs3).*fi(u02i/hs3).*fi(u12i/hs3))/(T*hs3^3);
phat[i,13] = sumc(LVO.*fi(u01i/hs3).*fi(u02i/hs3).*fi(u13i/hs3))/(T*hs3^3);

@estimating q(z) @
qz3[i,1]  = 1/T*sumc(fi(u02i/h1)./h1.*fi(u1i/h1)./h1);
qz3[i,2]  = 1/T*sumc(fi(u01i/h1)./h1.*fi(u1i/h1)./h1);
qz3[i,3]  = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1);
qz3[i,4]  = 1/T*sumc(fi(u02i/h1)./h1.*fi(u2i/h1)./h1);
qz3[i,5]  = 1/T*sumc(fi(u01i/h1)./h1.*fi(u2i/h1)./h1);
qz3[i,6]  = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1);
qz3[i,7]  = 1/T*sumc(fi(u02i/h1)./h1.*fi(u3i/h1)./h1);
qz3[i,8]  = 1/T*sumc(fi(u01i/h1)./h1.*fi(u3i/h1)./h1);
qz3[i,9]  = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1);
qz3[i,10] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u4i/h1)./h1);
qz3[i,11] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u4i/h1)./h1);
qz3[i,12] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1);
qz3[i,13] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u5i/h1)./h1);
qz3[i,14] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u5i/h1)./h1);
qz3[i,15] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1);
qz3[i,16] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u6i/h1)./h1);
qz3[i,17] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u6i/h1)./h1);
qz3[i,18] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1);
qz3[i,19] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u7i/h1)./h1);
qz3[i,20] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u7i/h1)./h1);
qz3[i,21] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1);
qz3[i,22] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u8i/h1)./h1);
qz3[i,23] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u8i/h1)./h1);
qz3[i,24] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1);
qz3[i,25] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u9i/h1)./h1);
qz3[i,26] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u9i/h1)./h1);
qz3[i,27] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1);
qz3[i,28] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u10i/h1)./h1);
qz3[i,29] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u10i/h1)./h1);
qz3[i,30] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1);
qz3[i,31] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u11i/h1)./h1);
qz3[i,32] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u11i/h1)./h1);
qz3[i,33] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1);
qz3[i,34] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u12i/h1)./h1);
qz3[i,35] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u12i/h1)./h1);
qz3[i,36] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1);
qz3[i,37] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u13i/h1)./h1);
qz3[i,38] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u13i/h1)./h1);
qz3[i,39] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1);
i = i+1;
endo;

yhat1  = y-meanc(y)-(m3d[.,1]-meanc(m3d[.,1])+m3d[.,2]-meanc(m3d[.,2])+m3d[.,3]-meanc(m3d[.,3]));
yhat2  = y-meanc(y)-(m3d[.,4]-meanc(m3d[.,4])+m3d[.,5]-meanc(m3d[.,5])+m3d[.,6]-meanc(m3d[.,6]));
yhat3  = y-meanc(y)-(m3d[.,7]-meanc(m3d[.,7])+m3d[.,8]-meanc(m3d[.,8])+m3d[.,9]-meanc(m3d[.,9]));
yhat4  = y-meanc(y)-(m3d[.,10]-meanc(m3d[.,10])+m3d[.,11]-meanc(m3d[.,11])+m3d[.,12]-meanc(m3d[.,12]));
yhat5  = y-meanc(y)-(m3d[.,13]-meanc(m3d[.,13])+m3d[.,14]-meanc(m3d[.,14])+m3d[.,15]-meanc(m3d[.,15]));
yhat6  = y-meanc(y)-(m3d[.,16]-meanc(m3d[.,16])+m3d[.,17]-meanc(m3d[.,17])+m3d[.,18]-meanc(m3d[.,18]));
yhat7  = y-meanc(y)-(m3d[.,19]-meanc(m3d[.,19])+m3d[.,20]-meanc(m3d[.,20])+m3d[.,21]-meanc(m3d[.,21]));
yhat8  = y-meanc(y)-(m3d[.,22]-meanc(m3d[.,22])+m3d[.,23]-meanc(m3d[.,23])+m3d[.,24]-meanc(m3d[.,24]));
yhat9  = y-meanc(y)-(m3d[.,25]-meanc(m3d[.,25])+m3d[.,26]-meanc(m3d[.,26])+m3d[.,27]-meanc(m3d[.,27]));
yhat10 = y-meanc(y)-(m3d[.,28]-meanc(m3d[.,28])+m3d[.,29]-meanc(m3d[.,29])+m3d[.,30]-meanc(m3d[.,30]));
yhat11 = y-meanc(y)-(m3d[.,31]-meanc(m3d[.,31])+m3d[.,32]-meanc(m3d[.,32])+m3d[.,33]-meanc(m3d[.,33]));
yhat12 = y-meanc(y)-(m3d[.,34]-meanc(m3d[.,34])+m3d[.,35]-meanc(m3d[.,35])+m3d[.,36]-meanc(m3d[.,36]));
yhat13 = y-meanc(y)-(m3d[.,37]-meanc(m3d[.,37])+m3d[.,38]-meanc(m3d[.,38])+m3d[.,39]-meanc(m3d[.,39]));

BR1 = (yhat1-y).*(yhat1-y);
BR2 = (yhat2-y).*(yhat2-y);
BR3 = (yhat3-y).*(yhat3-y);
BR4 = (yhat4-y).*(yhat4-y);
BR5 = (yhat5-y).*(yhat5-y);
BR6 = (yhat6-y).*(yhat6-y);
BR7 = (yhat7-y).*(yhat7-y);
BR8 = (yhat8-y).*(yhat8-y);
BR9 = (yhat9-y).*(yhat9-y);
BR10 = (yhat10-y).*(yhat10-y);
BR11 = (yhat11-y).*(yhat11-y);
BR12 = (yhat12-y).*(yhat12-y);
BR13 = (yhat13-y).*(yhat13-y);
quan = quantile(phat,0.05);
index = phat.< = quan;
phat = index*10^5+phat;

/****************************Three Lags Included ******************************************/

       
/****************************Three Lags Included Estimation of A part *******************************************/
kc = 1;
do until kc>nc;
b3 = h3*h[kc,1];
a11 = 0;
a12 = 0;
a13 = 0;
a21 = 0;
a22 = 0;
a23 = 0;
a31 = 0;
a32 = 0;
a33 = 0;
a41 = 0;
a42 = 0;
a43 = 0;
a51 = 0;
a52 = 0;
a53 = 0;
a61 = 0;
a62 = 0;
a63 = 0;
a71 = 0;
a72 = 0;
a73 = 0;
a81 = 0;
a82 = 0;
a83 = 0;
a91 = 0;
a92 = 0;
a93 = 0;
a101 = 0;
a102 = 0;
a103 = 0;
a111 = 0;
a112 = 0;
a113 = 0;
a121 = 0;
a122 = 0;
a123 = 0;
a131 = 0;
a132 = 0;
a133 = 0;
m3d = zeros(T,39);
m3 = zeros(T,39);
i = 1;
do until i>T;
u01i = data[i,lag_1]*ones(T,1) - data[.,lag_1];
u02i = data[i,lag_2]*ones(T,1) - data[.,lag_2];
u0i = u01i~u02i;
u1i  =  (data[i,1]*ones(T,1) - data[.,1]);
u2i  =  (data[i,2]*ones(T,1) - data[.,2]);
u3i  =  (data[i,3]*ones(T,1) - data[.,3]);
u4i  =  (data[i,4]*ones(T,1) - data[.,4]);
u5i  =  (data[i,5]*ones(T,1) - data[.,5]);
u6i  =  (data[i,6]*ones(T,1) - data[.,6]);
u7i  =  (data[i,7]*ones(T,1) - data[.,7]);
u8i  =  (data[i,8]*ones(T,1) - data[.,8]);
u9i  =  (data[i,9]*ones(T,1) - data[.,9]);
u10i =  (data[i,10]*ones(T,1) - data[.,10]);
u11i =  (data[i,11]*ones(T,1) - data[.,11]);
u12i =  (data[i,12]*ones(T,1) - data[.,12]);
u13i =  (data[i,13]*ones(T,1) - data[.,13]);
j = 1;

do until j>T;
u01j = data[j,lag_1]*ones(T,1) - data[.,lag_1];
u02j = data[j,lag_2]*ones(T,1) - data[.,lag_2];

 u1j  =  (data[j,1]*ones(T,1) - data[.,1]);
 u2j  =  (data[j,2]*ones(T,1) - data[.,2]);
 u3j  =  (data[j,3]*ones(T,1) - data[.,3]);
 u4j  =  (data[j,4]*ones(T,1) - data[.,4]);
 u5j  =  (data[j,5]*ones(T,1) - data[.,5]);
 u6j  =  (data[j,5]*ones(T,1) - data[.,6]);
 u7j  =  (data[j,7]*ones(T,1) - data[.,7]);
 u8j  =  (data[j,8]*ones(T,1) - data[.,8]);
 u9j  =  (data[j,9]*ones(T,1) - data[.,9]);
 u10j =  (data[j,10]*ones(T,1) - data[.,10]);
 u11j =  (data[j,11]*ones(T,1) - data[.,11]);
 u12j =  (data[j,12]*ones(T,1) - data[.,12]);
 u13j =  (data[j,13]*ones(T,1) - data[.,13]);
 
 w11 = (fi(u01i/b3)/b3).*(fi(u02j/b3)/b3).*(fi(u1j/b3)/b3);
 w12 = (fi(u01j/b3)/b3).*(fi(u02i/b3)/b3).*(fi(u1j/b3)/b3);
 w13 = (fi(u01j/b3)/b3).*(fi(u02j/b3)/b3).*(fi(u1i/b3)/b3);
 w21 = (fi(u01i/b3)/b3).*(fi(u02j/b3)/b3).*(fi(u2j/b3)/b3);
 w22 = (fi(u01j/b3)/b3).*(fi(u02i/b3)/b3).*(fi(u2j/b3)/b3);
 w23 = (fi(u01j/b3)/b3).*(fi(u02j/b3)/b3).*(fi(u2i/b3)/b3);
 w31 = (fi(u01i/b3)/b3).*(fi(u02j/b3)/b3).*(fi(u3j/b3)/b3);
 w32 = (fi(u01j/b3)/b3).*(fi(u02i/b3)/b3).*(fi(u3j/b3)/b3);
 w33 = (fi(u01j/b3)/b3).*(fi(u02j/b3)/b3).*(fi(u3i/b3)/b3);
 w41 = (fi(u01i/b3)/b3).*(fi(u02j/b3)/b3).*(fi(u4j/b3)/b3);
 w42 = (fi(u01j/b3)/b3).*(fi(u02i/b3)/b3).*(fi(u4j/b3)/b3);
 w43 = (fi(u01j/b3)/b3).*(fi(u02j/b3)/b3).*(fi(u4i/b3)/b3);
 w51 = (fi(u01i/b3)/b3).*(fi(u02j/b3)/b3).*(fi(u5j/b3)/b3);
 w52 = (fi(u01j/b3)/b3).*(fi(u02i/b3)/b3).*(fi(u5j/b3)/b3);
 w53 = (fi(u01j/b3)/b3).*(fi(u02j/b3)/b3).*(fi(u5i/b3)/b3);
 w61 = (fi(u01i/b3)/b3).*(fi(u02j/b3)/b3).*(fi(u6j/b3)/b3);
 w62 = (fi(u01j/b3)/b3).*(fi(u02i/b3)/b3).*(fi(u6j/b3)/b3);
 w63 = (fi(u01j/b3)/b3).*(fi(u02j/b3)/b3).*(fi(u6i/b3)/b3);
 w71 = (fi(u01i/b3)/b3).*(fi(u02j/b3)/b3).*(fi(u7j/b3)/b3);
 w72 = (fi(u01j/b3)/b3).*(fi(u02i/b3)/b3).*(fi(u7j/b3)/b3);
 w73 = (fi(u01j/b3)/b3).*(fi(u02j/b3)/b3).*(fi(u7i/b3)/b3);
 w81 = (fi(u01i/b3)/b3).*(fi(u02j/b3)/b3).*(fi(u8j/b3)/b3);
 w82 = (fi(u01j/b3)/b3).*(fi(u02i/b3)/b3).*(fi(u8j/b3)/b3);
 w83 = (fi(u01j/b3)/b3).*(fi(u02j/b3)/b3).*(fi(u8i/b3)/b3);
 w91 = (fi(u01i/b3)/b3).*(fi(u02j/b3)/b3).*(fi(u9j/b3)/b3);
 w92 = (fi(u01j/b3)/b3).*(fi(u02i/b3)/b3).*(fi(u9j/b3)/b3);
 w93 = (fi(u01j/b3)/b3).*(fi(u02j/b3)/b3).*(fi(u9i/b3)/b3);
 w101 = (fi(u01i/b3)/b3).*(fi(u02j/b3)/b3).*(fi(u10j/b3)/b3);
 w102 = (fi(u01j/b3)/b3).*(fi(u02i/b3)/b3).*(fi(u10j/b3)/b3);
 w103 = (fi(u01j/b3)/b3).*(fi(u02j/b3)/b3).*(fi(u10i/b3)/b3);
 w111 = (fi(u01i/b3)/b3).*(fi(u02j/b3)/b3).*(fi(u11j/b3)/b3);
 w112 = (fi(u01j/b3)/b3).*(fi(u02i/b3)/b3).*(fi(u11j/b3)/b3);
 w113 = (fi(u01j/b3)/b3).*(fi(u02j/b3)/b3).*(fi(u11i/b3)/b3);
 w121 = (fi(u01i/b3)/b3).*(fi(u02j/b3)/b3).*(fi(u12j/b3)/b3);
 w122 = (fi(u01j/b3)/b3).*(fi(u02i/b3)/b3).*(fi(u12j/b3)/b3);
 w123 = (fi(u01j/b3)/b3).*(fi(u02j/b3)/b3).*(fi(u12i/b3)/b3);
 w131 = (fi(u01i/b3)/b3).*(fi(u02j/b3)/b3).*(fi(u13j/b3)/b3);
 w132 = (fi(u01j/b3)/b3).*(fi(u02i/b3)/b3).*(fi(u13j/b3)/b3);
 w133 = (fi(u01j/b3)/b3).*(fi(u02j/b3)/b3).*(fi(u13i/b3)/b3);
 
 wmat11  = diagrv((eye(T),w11));
 wmat12  = diagrv((eye(T),w12));
 wmat13  = diagrv((eye(T),w13));
 wmat21  = diagrv((eye(T),w21));
 wmat22  = diagrv((eye(T),w22));
 wmat23  = diagrv((eye(T),w23));
 wmat31  = diagrv((eye(T),w31));
 wmat32  = diagrv((eye(T),w32));
 wmat33  = diagrv((eye(T),w33));
 wmat41  = diagrv((eye(T),w41));
 wmat42  = diagrv((eye(T),w42));
 wmat43  = diagrv((eye(T),w43));
 wmat51  = diagrv((eye(T),w51));
 wmat52  = diagrv((eye(T),w52));
 wmat53  = diagrv((eye(T),w53));
 wmat61  = diagrv((eye(T),w61));
 wmat62  = diagrv((eye(T),w62));
 wmat63  = diagrv((eye(T),w63));
 wmat71  = diagrv((eye(T),w71));
 wmat72  = diagrv((eye(T),w72));
 wmat73  = diagrv((eye(T),w73));
 wmat81  = diagrv((eye(T),w81));
 wmat82  = diagrv((eye(T),w82));
 wmat83  = diagrv((eye(T),w83));
 wmat91  = diagrv((eye(T),w91));
 wmat92  = diagrv((eye(T),w92));
 wmat93  = diagrv((eye(T),w93));
 wmat101  = diagrv((eye(T),w101));
 wmat102  = diagrv((eye(T),w102));
 wmat103  = diagrv((eye(T),w103));
 wmat111  = diagrv((eye(T),w111));
 wmat112  = diagrv((eye(T),w112));
 wmat113  = diagrv((eye(T),w113));
 wmat121  = diagrv((eye(T),w121));
 wmat122  = diagrv((eye(T),w122));
 wmat123  = diagrv((eye(T),w123));
 wmat131  = diagrv((eye(T),w131));
 wmat132  = diagrv((eye(T),w132));
 wmat133  = diagrv((eye(T),w133));
 xx11  =  ones(T,1)~u01i~u02j~u1j ;
 xx12  =  ones(T,1)~u01j~u02i~u1j ;
 xx13  =  ones(T,1)~u01j~u02j~u1i;
 xx21  =  ones(T,1)~u01i~u02j~u2j;
 xx22  =  ones(T,1)~u01j~u02i~u2j ;
 xx23  =  ones(T,1)~u01j~u02j~u2i ;
 xx31  =  ones(T,1)~u01i~u02j~u3j ;
 xx32  =  ones(T,1)~u01j~u02i~u3j ;
 xx33  =  ones(T,1)~u01j~u02j~u3i ;
 xx41  =  ones(T,1)~u01i~u02j~u4j ;
 xx42  =  ones(T,1)~u01j~u02i~u4j ;
 xx43  =  ones(T,1)~u01j~u02j~u4i ;
 xx51  =  ones(T,1)~u01i~u02j~u5j ;
 xx52  =  ones(T,1)~u01j~u02i~u5j ;
 xx53  =  ones(T,1)~u01j~u02j~u5i ;
 xx61  =  ones(T,1)~u01i~u02j~u6j ;
 xx62  =  ones(T,1)~u01j~u02i~u6j ;
 xx63  =  ones(T,1)~u01j~u02j~u6i ;
 xx71  =  ones(T,1)~u01i~u02j~u7j ;
 xx72  =  ones(T,1)~u01j~u02i~u7j ;
 xx73  =  ones(T,1)~u01j~u02j~u7i ;
 xx81  =  ones(T,1)~u01i~u02j~u8j ;
 xx82  =  ones(T,1)~u01j~u02i~u8j ;
 xx83  =  ones(T,1)~u01j~u02j~u8i ;
 xx91  =  ones(T,1)~u01i~u02j~u9j ;
 xx92  =  ones(T,1)~u01j~u02i~u9j ;
 xx93  =  ones(T,1)~u01j~u02j~u9i ;
 xx101  =  ones(T,1)~u01i~u02j~u10j;
 xx102  =  ones(T,1)~u01j~u02i~u10j;
 xx103  =  ones(T,1)~u01j~u02j~u10i;
 xx111  =  ones(T,1)~u01i~u02j~u11j;
 xx112  =  ones(T,1)~u01j~u02i~u11j;
 xx113  =  ones(T,1)~u01j~u02j~u11i;
 xx121  =  ones(T,1)~u01i~u02j~u12j;
 xx122  =  ones(T,1)~u01j~u02i~u12j;
 xx123  =  ones(T,1)~u01j~u02j~u12i;
 xx131  =  ones(T,1)~u01i~u02j~u13j;
 xx132  =  ones(T,1)~u01j~u02i~u13j;
 xx133  =  ones(T,1)~u01j~u02j~u13i;
 /*xx11  =  ones(T,1);
 xx12  =  ones(T,1);
 xx13  =  ones(T,1);
 xx21  =  ones(T,1);
 xx22  =  ones(T,1);
 xx23  =  ones(T,1);
 xx31  =  ones(T,1);
 xx32  =  ones(T,1);
 xx33  =  ones(T,1);
 xx41  =  ones(T,1);
 xx42  =  ones(T,1);
 xx43  =  ones(T,1);
 xx51  =  ones(T,1);
 xx52  =  ones(T,1);
 xx53  =  ones(T,1);
 xx61  =  ones(T,1);
 xx62  =  ones(T,1);
 xx63  =  ones(T,1);
 xx71  =  ones(T,1);
 xx72  =  ones(T,1);
 xx73  =  ones(T,1);
 xx81  =  ones(T,1);
 xx82  =  ones(T,1);
 xx83  =  ones(T,1);
 xx91  =  ones(T,1);
 xx92  =  ones(T,1);
 xx93  =  ones(T,1);
 xx101  =  ones(T,1);
 xx102  =  ones(T,1);
 xx103  =  ones(T,1);
 xx111  =  ones(T,1);
 xx112  =  ones(T,1);
 xx113  =  ones(T,1);
 xx121  =  ones(T,1);
 xx122  =  ones(T,1);
 xx123  =  ones(T,1);
 xx131  =  ones(T,1);
 xx132  =  ones(T,1);
 xx133  =  ones(T,1);*/

if sumc(w11)> 0;
if det(xx11'wmat11*xx11) < epsln ;
      m3[j,1]  =  w11'y/sumc(w11) ;
else;
 beta =  pinv(xx11'wmat11*xx11)*xx11'wmat11'y ;
 m3[j,1]  =  beta[1,.] ;
endif;
else;
m3[j,1] = 0;
a11 = a11+1;
endif;

if sumc(w12)> 0;
if det(xx12'wmat12*xx12) < epsln ;
      m3[j,2]  =  w12'y/sumc(w12) ;
else;
 beta =  pinv(xx12'wmat12*xx12)*xx12'wmat12'y ;
 m3[j,2]  =  beta[1,.] ;
endif;
else;
m3[j,2] = 0;
a12 = a12+1;
endif;

if sumc(w13)> 0;
if det(xx13'wmat13*xx13) < epsln ;
      m3[j,3]  =  w13'y/sumc(w13) ;
else;
 beta =  pinv(xx13'wmat13*xx13)*xx13'wmat13'y ;
 m3[j,3]  =  beta[1,.] ;
endif;
else;
m3[j,3] = 0;
a13 = a13+1;
endif;

if sumc(w21)> 0;
if det(xx21'wmat21*xx21) < epsln ;
      m3[j,4]  =  w21'y/sumc(w21) ;
else;
 beta =  pinv(xx21'wmat21*xx21)*xx21'wmat21'y ;
 m3[j,4]  =  beta[1,.] ;
endif;
else;
m3[j,4] = 0;
a21 = a21+1;
endif;

if sumc(w22)> 0;
if det(xx22'wmat22*xx22) < epsln ;
      m3[j,5]  =  w22'y/sumc(w22) ;
else;
 beta =  pinv(xx22'wmat22*xx22)*xx22'wmat22'y ;
 m3[j,5]  =  beta[1,.] ;
endif;
else;
m3[j,5] = 0;
a22 = a22+1;
endif;

if sumc(w23)> 0;
if det(xx23'wmat23*xx23) < epsln ;
      m3[j,6]  =  w23'y/sumc(w23) ;
else;
 beta =  pinv(xx23'wmat23*xx23)*xx23'wmat23'y ;
 m3[j,6]  =  beta[1,.] ;
endif;
else;
m3[j,6] = 0;
a23 = a23+1;
endif;

if sumc(w31)> 0;
if det(xx31'wmat31*xx31) < epsln ;
      m3[j,7]  =  w31'y/sumc(w31) ;
else;
 beta =  pinv(xx31'wmat31*xx31)*xx31'wmat31'y ;
 m3[j,7]  =  beta[1,.] ;
endif;
else;
m3[j,7] = 0;
a31 = a31+1;
endif;

if sumc(w32)> 0;
if det(xx32'wmat32*xx32) < epsln ;
      m3[j,8]  =  w32'y/sumc(w32) ;
else;
 beta =  pinv(xx32'wmat32*xx32)*xx32'wmat32'y ;
 m3[j,8]  =  beta[1,.] ;
endif;
else;
m3[j,8] = 0;
a32 = a32+1;
endif;

if sumc(w33)> 0;
if det(xx33'wmat33*xx33) < epsln ;
      m3[j,9]  =  w33'y/sumc(w33) ;
else;
 beta =  pinv(xx33'wmat33*xx33)*xx33'wmat33'y ;
 m3[j,9]  =  beta[1,.] ;
endif;
else;
m3[j,9] = 0;
a33 = a33+1;
endif;

if sumc(w41)> 0;
if det(xx41'wmat41*xx41) < epsln ;
      m3[j,10]  =  w41'y/sumc(w41) ;
else;
 beta =  pinv(xx41'wmat41*xx41)*xx41'wmat41'y ;
 m3[j,10]  =  beta[1,.] ;
endif;
else;
m3[j,10] = 0;
a41 = a41+1;
endif;

if sumc(w42)> 0;
if det(xx42'wmat42*xx42) < epsln ;
      m3[j,11]  =  w42'y/sumc(w42) ;
else;
 beta =  pinv(xx42'wmat42*xx42)*xx42'wmat42'y ;
 m3[j,11]  =  beta[1,.] ;
endif;
else;
m3[j,11] = 0;
a42 = a42+1;
endif;

if sumc(w43)> 0;
if det(xx43'wmat43*xx43) < epsln ;
      m3[j,12]  =  w43'y/sumc(w43) ;
else;
 beta =  pinv(xx43'wmat43*xx43)*xx43'wmat43'y ;
 m3[j,12]  =  beta[1,.] ;
endif;
else;
m3[j,12] = 0;
a43 = a43+1;
endif;

if sumc(w51)> 0;
if det(xx51'wmat51*xx51) < epsln ;
      m3[j,13]  =  w51'y/sumc(w51) ;
else;
 beta =  pinv(xx51'wmat51*xx51)*xx51'wmat51'y ;
 m3[j,13]  =  beta[1,.] ;
endif;
else;
m3[j,13] = 0;
a51 = a51+1;
endif;

if sumc(w52)> 0;
if det(xx52'wmat52*xx52) < epsln ;
      m3[j,10]  =  w52'y/sumc(w52) ;
else;
 beta =  pinv(xx52'wmat52*xx52)*xx52'wmat52'y ;
 m3[j,14]  =  beta[1,.] ;
endif;
else;
m3[j,14] = 0;
a52 = a52+1;
endif;

if sumc(w53)> 0;
if det(xx53'wmat53*xx53) < epsln ;
      m3[j,15]  =  w53'y/sumc(w53) ;
else;
 beta =  pinv(xx53'wmat53*xx53)*xx53'wmat53'y ;
 m3[j,15]  =  beta[1,.] ;
endif;
else;
m3[j,15] = 0;
a53 = a53+1;
endif;

/* for lag = 6 and the first significant lag */
if sumc(w61)> 0;
if det(xx61'wmat61*xx61) < epsln ;
      m3[j,16]  =  w61'y/sumc(w61) ;
else;
 beta =  pinv(xx61'wmat61*xx61)*xx61'wmat61'y ;
 m3[j,16]  =  beta[1,.] ;
endif;
else;
m3[j,16] = 0;
a61 = a61+1;
endif;

if sumc(w62)> 0;
if det(xx62'wmat62*xx62) < epsln ;
      m3[j,17]  =  w62'y/sumc(w62) ;
else;
 beta =  pinv(xx62'wmat62*xx62)*xx62'wmat62'y ;
 m3[j,17]  =  beta[1,.] ;
endif;
else;
m3[j,17] = 0;
a62 = a62+1;
endif;

if sumc(w63)> 0;
if det(xx63'wmat63*xx63) < epsln ;
      m3[j,18]  =  w63'y/sumc(w63) ;
else;
 beta =  pinv(xx63'wmat63*xx63)*xx63'wmat63'y ;
 m3[j,18]  =  beta[1,.] ;
endif;
else;
m3[j,18] = 0;
a63 = a63+1;
endif;

if sumc(w71)> 0;
if det(xx71'wmat71*xx71) < epsln ;
      m3[j,19]  =  w71'y/sumc(w71) ;
else;
 beta =  pinv(xx71'wmat71*xx71)*xx71'wmat71'y ;
 m3[j,19]  =  beta[1,.] ;
endif;
else;
m3[j,19] = 0;
a71 = a71+1;
endif;

if sumc(w72)> 0;
if det(xx72'wmat72*xx72) < epsln ;
      m3[j,20]  =  w72'y/sumc(w72) ;
else;
 beta =  pinv(xx72'wmat72*xx72)*xx72'wmat72'y ;
 m3[j,20]  =  beta[1,.] ;
endif;
else;
m3[j,20] = 0;
a72 = a72+1;
endif;

if sumc(w73)> 0;
if det(xx73'wmat73*xx73) < epsln ;
      m3[j,21]  =  w73'y/sumc(w73) ;
else;
 beta =  pinv(xx73'wmat73*xx73)*xx73'wmat73'y ;
 m3[j,21]  =  beta[1,.] ;
endif;
else;
m3[j,21] = 0;
a73 = a73+1;
endif;

if sumc(w81)> 0;
if det(xx81'wmat81*xx81) < epsln ;
      m3[j,22]  =  w81'y/sumc(w81) ;
else;
 beta =  pinv(xx81'wmat81*xx81)*xx81'wmat81'y ;
 m3[j,22]  =  beta[1,.] ;
endif;
else;
m3[j,22] = 0;
a81 = a81+1;
endif;

if sumc(w82)> 0;
if det(xx82'wmat82*xx82) < epsln ;
      m3[j,23]  =  w82'y/sumc(w82) ;
else;
 beta =  pinv(xx82'wmat82*xx82)*xx82'wmat82'y ;
 m3[j,23]  =  beta[1,.] ;
endif;
else;
m3[j,23] = 0;
a82 = a82+1;
endif;

if sumc(w83)> 0;
if det(xx83'wmat83*xx83) < epsln ;
      m3[j,24]  =  w83'y/sumc(w83) ;
else;
 beta =  pinv(xx83'wmat83*xx83)*xx83'wmat83'y ;
 m3[j,24]  =  beta[1,.] ;
endif;
else;
m3[j,24] = 0;
a83 = a83+1;
endif;

if sumc(w91)> 0;
if det(xx91'wmat91*xx91) < epsln ;
      m3[j,25]  =  w91'y/sumc(w91) ;
else;
 beta =  pinv(xx91'wmat91*xx91)*xx91'wmat91'y ;
 m3[j,25]  =  beta[1,.] ;
endif;
else;
m3[j,25] = 0;
a91 = a91+1;
endif;

if sumc(w92)> 0;
if det(xx92'wmat92*xx92) < epsln ;
      m3[j,26]  =  w92'y/sumc(w92) ;
else;
 beta =  pinv(xx92'wmat92*xx92)*xx92'wmat92'y ;
 m3[j,26]  =  beta[1,.] ;
endif;
else;
m3[j,26] = 0;
a92 = a92+1;
endif;

if sumc(w93)> 0;
if det(xx93'wmat93*xx93) < epsln ;
      m3[j,27]  =  w93'y/sumc(w93) ;
else;
 beta =  pinv(xx93'wmat93*xx93)*xx93'wmat93'y ;
 m3[j,27]  =  beta[1,.] ;
endif;
else;
m3[j,27] = 0;
a93 = a93+1;
endif;

if sumc(w101)> 0;
if det(xx101'wmat101*xx101) < epsln ;
      m3[j,28]  =  w101'y/sumc(w101) ;
else;
 beta =  pinv(xx101'wmat101*xx101)*xx101'wmat101'y ;
 m3[j,28]  =  beta[1,.] ;
endif;
else;
m3[j,28] = 0;
a101 = a101+1;
endif;

if sumc(w102)> 0;
if det(xx102'wmat102*xx102) < epsln ;
      m3[j,29]  =  w102'y/sumc(w102) ;
else;
 beta =  pinv(xx102'wmat102*xx102)*xx102'wmat102'y ;
 m3[j,29]  =  beta[1,.] ;
endif;
else;
m3[j,29] = 0;
a102 = a102+1;
endif;

if sumc(w103)> 0;
if det(xx103'wmat103*xx103) < epsln ;
      m3[j,30]  =  w103'y/sumc(w103) ;
else;
 beta =  pinv(xx103'wmat103*xx103)*xx103'wmat103'y ;
 m3[j,30]  =  beta[1,.] ;
endif;
else;
m3[j,30] = 0;
a103 = a103+1;
endif;


if sumc(w111)> 0;
if det(xx111'wmat111*xx111) < epsln ;
      m3[j,31]  =  w111'y/sumc(w111) ;
else;
 beta =  pinv(xx111'wmat111*xx111)*xx111'wmat111'y ;
 m3[j,31]  =  beta[1,.] ;
endif;
else;
m3[j,31] = 0;
a111 = a111+1;
endif;

if sumc(w112)> 0;
if det(xx112'wmat112*xx112) < epsln ;
      m3[j,32]  =  w112'y/sumc(w112) ;
else;
 beta =  pinv(xx112'wmat112*xx112)*xx112'wmat112'y ;
 m3[j,32]  =  beta[1,.] ;
endif;
else;
m3[j,32] = 0;
a112 = a112+1;
endif;

if sumc(w113)> 0;
if det(xx113'wmat113*xx113) < epsln ;
      m3[j,33]  =  w113'y/sumc(w113) ;
else;
 beta =  pinv(xx113'wmat113*xx113)*xx113'wmat113'y ;
 m3[j,33]  =  beta[1,.] ;
endif;
else;
m3[j,33] = 0;
a113 = a113+1;
endif;


if sumc(w121)> 0;
if det(xx121'wmat121*xx121) < epsln ;
      m3[j,34]  =  w121'y/sumc(w121) ;
else;
 beta =  pinv(xx121'wmat121*xx121)*xx121'wmat121'y ;
 m3[j,34]  =  beta[1,.] ;
endif;
else;
m3[j,34] = 0;
a121 = a121+1;
endif;

if sumc(w122)> 0;
if det(xx122'wmat122*xx122) < epsln ;
      m3[j,35]  =  w122'y/sumc(w122) ;
else;
 beta =  pinv(xx122'wmat122*xx122)*xx122'wmat122'y ;
 m3[j,35]  =  beta[1,.] ;
endif;
else;
m3[j,35] = 0;
a122 = a122+1;
endif;

if sumc(w123)> 0;
if det(xx123'wmat123*xx123) < epsln ;
      m3[j,36]  =  w123'y/sumc(w123) ;
else;
 beta =  pinv(xx123'wmat123*xx123)*xx123'wmat123'y ;
 m3[j,36]  =  beta[1,.] ;
endif;
else;
m3[j,36] = 0;
a122 = a122+1;
endif;

if sumc(w131)> 0;
if det(xx131'wmat131*xx131) < epsln ;
      m3[j,37]  =  w131'y/sumc(w131) ;
else;
 beta =  pinv(xx131'wmat131*xx131)*xx131'wmat131'y ;
 m3[j,37]  =  beta[1,.] ;
endif;
else;
m3[j,37] = 0;
a131 = a131+1;
endif;

if sumc(w132)> 0;
if det(xx132'wmat132*xx132) < epsln ;
      m3[j,38]  =  w132'y/sumc(w132) ;
else;
 beta =  pinv(xx132'wmat132*xx132)*xx132'wmat132'y ;
 m3[j,38]  =  beta[1,.] ;
endif;
else;
m3[j,38] = 0;
a132 = a132+1;
endif;

if sumc(w133)> 0;
if det(xx133'wmat133*xx133) < epsln ;
      m3[j,39]  =  w133'y/sumc(w133) ;
else;
 beta =  pinv(xx133'wmat133*xx133)*xx133'wmat133'y ;
 m3[j,39]  =  beta[1,.] ;
endif;
else;
m3[j,39] = 0;
a133 = a133+1;
endif;
j = j+1;
endo;
m3d[i,1] = 1/(T-a11)*sumc(m3[.,1]);
m3d[i,2] = 1/(T-a12)*sumc(m3[.,2]);
m3d[i,3] = 1/(T-a13)*sumc(m3[.,3]);
m3d[i,4] = 1/(T-a21)*sumc(m3[.,4]);
m3d[i,5] = 1/(T-a22)*sumc(m3[.,5]);
m3d[i,6] = 1/(T-a23)*sumc(m3[.,6]);
m3d[i,7] = 1/(T-a31)*sumc(m3[.,7]);
m3d[i,8] = 1/(T-a32)*sumc(m3[.,8]);
m3d[i,9] = 1/(T-a33)*sumc(m3[.,9]);
m3d[i,10] = 1/(T-a41)*sumc(m3[.,10]);
m3d[i,11] = 1/(T-a42)*sumc(m3[.,11]);
m3d[i,12] = 1/(T-a43)*sumc(m3[.,12]);
m3d[i,13] = 1/(T-a51)*sumc(m3[.,13]);
m3d[i,14] = 1/(T-a52)*sumc(m3[.,14]);
m3d[i,15] = 1/(T-a53)*sumc(m3[.,15]);
m3d[i,16] = 1/(T-a61)*sumc(m3[.,16]);
m3d[i,17] = 1/(T-a62)*sumc(m3[.,17]);
m3d[i,18] = 1/(T-a63)*sumc(m3[.,18]);
m3d[i,19] = 1/(T-a71)*sumc(m3[.,19]);
m3d[i,20] = 1/(T-a72)*sumc(m3[.,20]);
m3d[i,21] = 1/(T-a73)*sumc(m3[.,21]);
m3d[i,22] = 1/(T-a81)*sumc(m3[.,22]);
m3d[i,23] = 1/(T-a82)*sumc(m3[.,23]);
m3d[i,24] = 1/(T-a83)*sumc(m3[.,24]);
m3d[i,25] = 1/(T-a91)*sumc(m3[.,25]);
m3d[i,26] = 1/(T-a92)*sumc(m3[.,26]);
m3d[i,27] = 1/(T-a93)*sumc(m3[.,27]);
m3d[i,28] = 1/(T-a101)*sumc(m3[.,28]);
m3d[i,29] = 1/(T-a102)*sumc(m3[.,29]);
m3d[i,30] = 1/(T-a103)*sumc(m3[.,30]);
m3d[i,31] = 1/(T-a111)*sumc(m3[.,31]);
m3d[i,32] = 1/(T-a112)*sumc(m3[.,32]);
m3d[i,33] = 1/(T-a113)*sumc(m3[.,33]);
m3d[i,34] = 1/(T-a121)*sumc(m3[.,34]);
m3d[i,35] = 1/(T-a122)*sumc(m3[.,35]);
m3d[i,36] = 1/(T-a123)*sumc(m3[.,36]);
m3d[i,37] = 1/(T-a131)*sumc(m3[.,37]);
m3d[i,38] = 1/(T-a132)*sumc(m3[.,38]);
m3d[i,39] = 1/(T-a133)*sumc(m3[.,39]);

i = i+1;
endo;
yhat1 = y-meanc(y)-(m3d[.,1]-meanc(m3d[.,1])+m3d[.,2]-meanc(m3d[.,2])+m3d[.,3]-meanc(m3d[.,3]));
yhat2 = y-meanc(y)-(m3d[.,4]-meanc(m3d[.,4])+m3d[.,5]-meanc(m3d[.,5])+m3d[.,6]-meanc(m3d[.,6]));
yhat3 = y-meanc(y)-(m3d[.,7]-meanc(m3d[.,7])+m3d[.,8]-meanc(m3d[.,8])+m3d[.,9]-meanc(m3d[.,9]));
yhat4 = y-meanc(y)-(m3d[.,10]-meanc(m3d[.,10])+m3d[.,11]-meanc(m3d[.,11])+m3d[.,12]-meanc(m3d[.,12]));
yhat5 = y-meanc(y)-(m3d[.,13]-meanc(m3d[.,13])+m3d[.,14]-meanc(m3d[.,14])+m3d[.,15]-meanc(m3d[.,15]));
yhat6 = y-meanc(y)-(m3d[.,16]-meanc(m3d[.,16])+m3d[.,17]-meanc(m3d[.,17])+m3d[.,18]-meanc(m3d[.,18]));
yhat7 = y-meanc(y)-(m3d[.,19]-meanc(m3d[.,19])+m3d[.,20]-meanc(m3d[.,20])+m3d[.,21]-meanc(m3d[.,21]));
yhat8 = y-meanc(y)-(m3d[.,22]-meanc(m3d[.,22])+m3d[.,23]-meanc(m3d[.,23])+m3d[.,24]-meanc(m3d[.,24]));
yhat9 = y-meanc(y)-(m3d[.,25]-meanc(m3d[.,25])+m3d[.,26]-meanc(m3d[.,26])+m3d[.,27]-meanc(m3d[.,27]));
yhat10 = y-meanc(y)-(m3d[.,28]-meanc(m3d[.,28])+m3d[.,29]-meanc(m3d[.,29])+m3d[.,30]-meanc(m3d[.,30]));
yhat11 = y-meanc(y)-(m3d[.,31]-meanc(m3d[.,31])+m3d[.,32]-meanc(m3d[.,32])+m3d[.,33]-meanc(m3d[.,33]));
yhat12 = y-meanc(y)-(m3d[.,34]-meanc(m3d[.,34])+m3d[.,35]-meanc(m3d[.,35])+m3d[.,36]-meanc(m3d[.,36]));
yhat13 = y-meanc(y)-(m3d[.,37]-meanc(m3d[.,37])+m3d[.,38]-meanc(m3d[.,38])+m3d[.,39]-meanc(m3d[.,39]));
R1 = yhat1'yhat1;
R2 = yhat2'yhat2;
R3 = yhat3'yhat3;
R4 = yhat4'yhat4;
R5 = yhat5'yhat5;
R6 = yhat6'yhat6;
R7 = yhat7'yhat7;
R8 = yhat8'yhat8;
R9 = yhat9'yhat9;
R10 = yhat10'yhat10;
R11 = yhat11'yhat11;
R12 = yhat12'yhat12;
R13 = yhat13'yhat13;
Ahat1[k,kc] = 1/T*sumc(R1);
Ahat2[k,kc] = 1/T*sumc(R2);
Ahat3[k,kc] = 1/T*sumc(R3);
Ahat4[k,kc] = 1/T*sumc(R4);
Ahat5[k,kc] = 1/T*sumc(R5);
Ahat6[k,kc] = 1/T*sumc(R6);
Ahat7[k,kc] = 1/T*sumc(R7);
Ahat8[k,kc] = 1/T*sumc(R8);
Ahat9[k,kc] = 1/T*sumc(R9);
Ahat10[k,kc] = 1/T*sumc(R10);
Ahat11[k,kc] = 1/T*sumc(R11);
Ahat12[k,kc] = 1/T*sumc(R12);
Ahat13[k,kc] = 1/T*sumc(R13);
Bhat1[k,kc]  =  2/b3*1/sqrt(2*pi)*1/T*sumc((BR1.*qz3[.,1]+BR1.*qz3[.,2]+BR1.*qz3[.,3])./phat[.,1])*1/T;
Bhat2[k,kc]  =  2/b3*1/sqrt(2*pi)*1/T*sumc((BR2.*qz3[.,4]+BR2.*qz3[.,5]+BR2.*qz3[.,6])./phat[.,2])*1/T;
Bhat3[k,kc]  =  2/b3*1/sqrt(2*pi)*1/T*sumc((BR3.*qz3[.,7]+BR3.*qz3[.,8]+BR3.*qz3[.,9])./phat[.,3])*1/T;
Bhat4[k,kc]  =  2/b3*1/sqrt(2*pi)*1/T*sumc((BR4.*qz3[.,10]+BR4.*qz3[.,11]+BR4.*qz3[.,12])./phat[.,4])*1/T;
Bhat5[k,kc]  =  2/b3*1/sqrt(2*pi)*1/T*sumc((BR5.*qz3[.,13]+BR5.*qz3[.,14]+BR5.*qz3[.,15])./phat[.,5])*1/T;
Bhat6[k,kc]  =  2/b3*1/sqrt(2*pi)*1/T*sumc((BR6.*qz3[.,16]+BR6.*qz3[.,17]+BR6.*qz3[.,18])./phat[.,6])*1/T;
Bhat7[k,kc]  =  2/b3*1/sqrt(2*pi)*1/T*sumc((BR7.*qz3[.,19]+BR7.*qz3[.,20]+BR7.*qz3[.,21])./phat[.,7])*1/T;
Bhat8[k,kc]  =  2/b3*1/sqrt(2*pi)*1/T*sumc((BR8.*qz3[.,22]+BR8.*qz3[.,23]+BR8.*qz3[.,24])./phat[.,8])*1/T;
Bhat9[k,kc]  =  2/b3*1/sqrt(2*pi)*1/T*sumc((BR9.*qz3[.,25]+BR9.*qz3[.,26]+BR9.*qz3[.,27])./phat[.,9])*1/T;
Bhat10[k,kc]  =  2/b3*1/sqrt(2*pi)*1/T*sumc((BR10.*qz3[.,28]+BR10.*qz3[.,29]+BR10.*qz3[.,30])./phat[.,10])*1/T;
Bhat11[k,kc]  =  2/b3*1/sqrt(2*pi)*1/T*sumc((BR11.*qz3[.,31]+BR11.*qz3[.,32]+BR11.*qz3[.,33])./phat[.,11])*1/T;
Bhat12[k,kc]  =  2/b3*1/sqrt(2*pi)*1/T*sumc((BR12.*qz3[.,34]+BR12.*qz3[.,35]+BR12.*qz3[.,36])./phat[.,12])*1/T;
Bhat13[k,kc]  =  2/b3*1/sqrt(2*pi)*1/T*sumc((BR13.*qz3[.,37]+BR13.*qz3[.,38]+BR13.*qz3[.,39])./phat[.,13])*1/T;
FPE1[k,kc] = Ahat1[k,kc]+Bhat1[k,kc];
FPE2[k,kc] = Ahat2[k,kc]+Bhat2[k,kc];
FPE3[k,kc] = Ahat3[k,kc]+Bhat3[k,kc];
FPE4[k,kc] = Ahat4[k,kc]+Bhat4[k,kc];
FPE5[k,kc] = Ahat5[k,kc]+Bhat5[k,kc];
FPE6[k,kc] = Ahat6[k,kc]+Bhat6[k,kc];
FPE7[k,kc] = Ahat7[k,kc]+Bhat7[k,kc];
FPE8[k,kc] = Ahat8[k,kc]+Bhat8[k,kc];
FPE9[k,kc] = Ahat9[k,kc]+Bhat9[k,kc];
FPE10[k,kc] = Ahat10[k,kc]+Bhat10[k,kc];
FPE11[k,kc] = Ahat11[k,kc]+Bhat11[k,kc];
FPE12[k,kc] = Ahat12[k,kc]+Bhat12[k,kc];
FPE13[k,kc] = Ahat13[k,kc]+Bhat13[k,kc];
kc = kc+1;
endo;
cmin1 = minc(FPE1[k,.]');
cmin2 = minc(FPE2[k,.]');
cmin3 = minc(FPE3[k,.]');
cmin4 = minc(FPE4[k,.]');
cmin5 = minc(FPE5[k,.]');
cmin6 = minc(FPE6[k,.]');
cmin7 = minc(FPE7[k,.]');
cmin8 = minc(FPE8[k,.]');
cmin9 = minc(FPE9[k,.]');
cmin10 = minc(FPE10[k,.]');
cmin11 = minc(FPE11[k,.]');
cmin12 = minc(FPE12[k,.]');
cmin13 = minc(FPE13[k,.]');
minFPE = cmin1~cmin2~cmin3~cmin4~cmin5~cmin6~cmin7~cmin8~cmin9~cmin10~cmin11~cmin12~cmin13;
minFPE[.,lag_1] = 10^10;
minFPE[.,lag_2] = 10^10;
AFPE[k,3] = minc(minFPE');
lag[k,3] = minindc(minFPE');
lag_3 = lag[k,3];
/****************************Four Lags Included ******************************************/
       
/****************************Four Lags Included Estimation of B part *******************************************/

i = 1;
a11 = 0;
a12 = 0;
a13 = 0;
a14 = 0;
a21 = 0;
a22 = 0;
a23 = 0;
a24 = 0;
a31 = 0;
a32 = 0;
a33 = 0;
a34 = 0;
a41 = 0;
a42 = 0;
a43 = 0;
a44 = 0;
a51 = 0;
a52 = 0;
a53 = 0;
a54 = 0;
a61 = 0;
a62 = 0;
a63 = 0;
a64 = 0;
a71 = 0;
a72 = 0;
a73 = 0;
a74 = 0;
a81 = 0;
a82 = 0;
a83 = 0;
a84 = 0;
a91 = 0;
a92 = 0;
a93 = 0;
a94 = 0;
a101 = 0;
a102 = 0;
a103 = 0;
a104 = 0;
a111 = 0;
a112 = 0;
a113 = 0;
a114 = 0;
a121 = 0;
a122 = 0;
a123 = 0;
a124 = 0;
a131 = 0;
a132 = 0;
a133 = 0;
a134 = 0;
m4 = zeros(T,52);
m4d = zeros(T,52);
qz4 = zeros(T,52);
i = 1;
do until i>T;
u01i = data[i,lag_1]*ones(T,1) - data[.,lag_1];
u02i = data[i,lag_2]*ones(T,1) - data[.,lag_2];
u03i = data[i,lag_3]*ones(T,1) - data[.,lag_3];
u0i = u01i~u02i~u03i;
u1i =  data[i,1]*ones(T,1) - data[.,1];
u2i =  data[i,2]*ones(T,1) - data[.,2];
u3i =  data[i,3]*ones(T,1) - data[.,3];
u4i =  data[i,4]*ones(T,1) - data[.,4];
u5i =  data[i,5]*ones(T,1) - data[.,5];
u6i =  data[i,6]*ones(T,1) - data[.,6];
u7i =  data[i,7]*ones(T,1) - data[.,7];
u8i =  data[i,8]*ones(T,1) - data[.,8];
u9i =  data[i,9]*ones(T,1) - data[.,9];
u10i =  data[i,10]*ones(T,1) - data[.,10];
u11i =  data[i,11]*ones(T,1) - data[.,11];
u12i =  data[i,12]*ones(T,1) - data[.,12];
u13i =  data[i,13]*ones(T,1) - data[.,13];
j = 1;
do until j>T;
u01j = data[j,lag_1]*ones(T,1) - data[.,lag_1];
u02j = data[j,lag_2]*ones(T,1) - data[.,lag_2];
u03j = data[j,lag_3]*ones(T,1) - data[.,lag_3];
u0j = u01j~u02j~u03j;
 u1j =  (data[j,1]*ones(T,1) - data[.,1]);
 u2j =  (data[j,2]*ones(T,1) - data[.,2]);
 u3j =  (data[j,3]*ones(T,1) - data[.,3]);
 u4j =  (data[j,4]*ones(T,1) - data[.,4]);
 u5j =  (data[j,5]*ones(T,1) - data[.,5]);
 u6j =  (data[j,5]*ones(T,1) - data[.,6]);
 u7j =  (data[j,7]*ones(T,1) - data[.,7]);
 u8j =  (data[j,8]*ones(T,1) - data[.,8]);
 u9j =  (data[j,9]*ones(T,1) - data[.,9]);
 u10j =  (data[j,10]*ones(T,1) - data[.,10]);
 u11j =  (data[j,11]*ones(T,1) - data[.,11]);
 u12j =  (data[j,12]*ones(T,1) - data[.,12]);
 u13j =  (data[j,13]*ones(T,1) - data[.,13]);
 w11 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u1j/h1)/h1);
 w12 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u1j/h1)/h1);
 w13 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03i/h1)/h1).*(fi(u1j/h1)/h1);
 w14 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u1i/h1)/h1);
 w21 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u2j/h1)/h1);
 w22 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u2j/h1)/h1);
 w23 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03i/h1)/h1).*(fi(u2j/h1)/h1);
 w24 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u2i/h1)/h1);
 w31 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u3j/h1)/h1);
 w32 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u3j/h1)/h1);
 w33 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03i/h1)/h1).*(fi(u3j/h1)/h1);
 w34 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u3i/h1)/h1);
 w41 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u4j/h1)/h1);
 w42 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u4j/h1)/h1);
 w43 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03i/h1)/h1).*(fi(u4j/h1)/h1);
 w44 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u4i/h1)/h1);
 w51 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u5j/h1)/h1);
 w52 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u5j/h1)/h1);
 w53 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03i/h1)/h1).*(fi(u5j/h1)/h1);
 w54 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u5i/h1)/h1);
 w61 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u6j/h1)/h1);
 w62 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u6j/h1)/h1);
 w63 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03i/h1)/h1).*(fi(u6j/h1)/h1);
 w64 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u6i/h1)/h1);
 w71 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u7j/h1)/h1);
 w72 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u7j/h1)/h1);
 w73 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03i/h1)/h1).*(fi(u7j/h1)/h1);
 w74 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u7i/h1)/h1);
 w81 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u8j/h1)/h1);
 w82 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u8j/h1)/h1);
 w83 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03i/h1)/h1).*(fi(u8j/h1)/h1);
 w84 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u8i/h1)/h1);
 w91 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u9j/h1)/h1);
 w92 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u9j/h1)/h1);
 w93 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03i/h1)/h1).*(fi(u9j/h1)/h1);
 w94 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u9i/h1)/h1);
 w101 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u10j/h1)/h1);
 w102 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u10j/h1)/h1);
 w103 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03i/h1)/h1).*(fi(u10j/h1)/h1);
 w104 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u10i/h1)/h1);
 w111 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u11j/h1)/h1);
 w112 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u11j/h1)/h1);
 w113 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03i/h1)/h1).*(fi(u11j/h1)/h1);
 w114 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u11i/h1)/h1);
 w121 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u12j/h1)/h1);
 w122 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u12j/h1)/h1);
 w123 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03i/h1)/h1).*(fi(u12j/h1)/h1);
 w124 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u12i/h1)/h1);
 w131 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u13j/h1)/h1);
 w132 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u13j/h1)/h1);
 w133 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03i/h1)/h1).*(fi(u13j/h1)/h1);
 w134 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u13i/h1)/h1);
 w11[j] = 0;
 w12[j] = 0;
 w13[j] = 0;
 w14[j] = 0;
 w21[j] = 0;
 w22[j] = 0;
 w23[j] = 0;
 w24[j] = 0;
 w31[j] = 0;
 w32[j] = 0;
 w33[j] = 0;
 w34[j] = 0;
 w41[j] = 0;
 w42[j] = 0;
 w43[j] = 0;
 w44[j] = 0;
 w51[j] = 0;
 w52[j] = 0;
 w53[j] = 0;
 w54[j] = 0;
 w61[j] = 0;
 w62[j] = 0;
 w63[j] = 0;
 w64[j] = 0;
 w71[j] = 0;
 w72[j] = 0;
 w73[j] = 0;
 w74[j] = 0;
 w81[j] = 0;
 w82[j] = 0;
 w83[j] = 0;
 w84[j] = 0;
 w91[j] = 0;
 w92[j] = 0;
 w93[j] = 0;
 w94[j] = 0;
 w101[j] = 0;
 w102[j] = 0;
 w103[j] = 0;
 w104[j] = 0;
 w111[j] = 0;
 w112[j] = 0;
 w113[j] = 0;
 w114[j] = 0;
 w121[j] = 0;
 w122[j] = 0;
 w123[j] = 0;
 w124[j] = 0;
 w131[j] = 0;
 w132[j] = 0;
 w133[j] = 0;
 w134[j] = 0;
 wmat11  = diagrv((eye(T),w11));
 wmat12  = diagrv((eye(T),w12));
 wmat13  = diagrv((eye(T),w13));
 wmat14  = diagrv((eye(T),w14));
 wmat21  = diagrv((eye(T),w21));
 wmat22  = diagrv((eye(T),w22));
 wmat23  = diagrv((eye(T),w23));
 wmat24  = diagrv((eye(T),w24));
 wmat31  = diagrv((eye(T),w31));
 wmat32  = diagrv((eye(T),w32));
 wmat33  = diagrv((eye(T),w33));
 wmat34  = diagrv((eye(T),w34));
 wmat41  = diagrv((eye(T),w41));
 wmat42  = diagrv((eye(T),w42));
 wmat43  = diagrv((eye(T),w43));
 wmat44  = diagrv((eye(T),w44));
 wmat51  = diagrv((eye(T),w51));
 wmat52  = diagrv((eye(T),w52));
 wmat53  = diagrv((eye(T),w53));
 wmat54  = diagrv((eye(T),w54));
 wmat61  = diagrv((eye(T),w61));
 wmat62  = diagrv((eye(T),w62));
 wmat63  = diagrv((eye(T),w63));
 wmat64  = diagrv((eye(T),w64));
 wmat71  = diagrv((eye(T),w71));
 wmat72  = diagrv((eye(T),w72));
 wmat73  = diagrv((eye(T),w73));
 wmat74  = diagrv((eye(T),w74));
 wmat81  = diagrv((eye(T),w81));
 wmat82  = diagrv((eye(T),w82));
 wmat83  = diagrv((eye(T),w83));
 wmat84  = diagrv((eye(T),w84));
 wmat91  = diagrv((eye(T),w91));
 wmat92  = diagrv((eye(T),w92));
 wmat93  = diagrv((eye(T),w93));
 wmat94  = diagrv((eye(T),w94));
 wmat101  = diagrv((eye(T),w101));
 wmat102  = diagrv((eye(T),w102));
 wmat103  = diagrv((eye(T),w103));
 wmat104  = diagrv((eye(T),w104));
 wmat111  = diagrv((eye(T),w111));
 wmat112  = diagrv((eye(T),w112));
 wmat113  = diagrv((eye(T),w113));
 wmat114  = diagrv((eye(T),w114));
 wmat121  = diagrv((eye(T),w121));
 wmat122  = diagrv((eye(T),w122));
 wmat123  = diagrv((eye(T),w123));
 wmat124  = diagrv((eye(T),w124));
 wmat131  = diagrv((eye(T),w131));
 wmat132  = diagrv((eye(T),w132));
 wmat133  = diagrv((eye(T),w133));
 wmat134  = diagrv((eye(T),w134));
 xx11  =  ones(T,1)~u01i~u02j~u03j~u1j ;
 xx12  =  ones(T,1)~u01j~u02i~u03j~u1j ;
 xx13  =  ones(T,1)~u01j~u02j~u03i~u1j ;
 xx14  =  ones(T,1)~u01j~u02j~u03j~u1i;
 xx21  =  ones(T,1)~u01i~u02j~u03j~u2j ;
 xx22  =  ones(T,1)~u01j~u02i~u03j~u2j ;
 xx23  =  ones(T,1)~u01j~u02j~u03i~u2j ;
 xx24  =  ones(T,1)~u01j~u02j~u03j~u2i;
 xx31  =  ones(T,1)~u01i~u02j~u03j~u3j ;
 xx32  =  ones(T,1)~u01j~u02i~u03j~u3j ;
 xx33  =  ones(T,1)~u01j~u02j~u03i~u3j ;
 xx34  =  ones(T,1)~u01j~u02j~u03j~u3i;
 xx41  =  ones(T,1)~u01i~u02j~u03j~u4j ;
 xx42  =  ones(T,1)~u01j~u02i~u03j~u4j ;
 xx43  =  ones(T,1)~u01j~u02j~u03i~u4j ;
 xx44  =  ones(T,1)~u01j~u02j~u03j~u4i;
 xx51  =  ones(T,1)~u01i~u02j~u03j~u5j ;
 xx52  =  ones(T,1)~u01j~u02i~u03j~u5j ;
 xx53  =  ones(T,1)~u01j~u02j~u03i~u5j ;
 xx54  =  ones(T,1)~u01j~u02j~u03j~u5i;
 xx61  =  ones(T,1)~u01i~u02j~u03j~u6j ;
 xx62  =  ones(T,1)~u01j~u02i~u03j~u6j ;
 xx63  =  ones(T,1)~u01j~u02j~u03i~u6j ;
 xx64  =  ones(T,1)~u01j~u02j~u03j~u6i;
 xx71  =  ones(T,1)~u01i~u02j~u03j~u7j ;
 xx72  =  ones(T,1)~u01j~u02i~u03j~u7j ;
 xx73  =  ones(T,1)~u01j~u02j~u03i~u7j ;
 xx74  =  ones(T,1)~u01j~u02j~u03j~u7i;
 xx81  =  ones(T,1)~u01i~u02j~u03j~u8j ;
 xx82  =  ones(T,1)~u01j~u02i~u03j~u8j ;
 xx83  =  ones(T,1)~u01j~u02j~u03i~u8j ;
 xx84  =  ones(T,1)~u01j~u02j~u03j~u8i;
 xx91  =  ones(T,1)~u01i~u02j~u03j~u9j ;
 xx92  =  ones(T,1)~u01j~u02i~u03j~u9j ;
 xx93  =  ones(T,1)~u01j~u02j~u03i~u9j ;
 xx94  =  ones(T,1)~u01j~u02j~u03j~u9i;
 xx101  =  ones(T,1)~u01i~u02j~u03j~u10j ;
 xx102  =  ones(T,1)~u01j~u02i~u03j~u10j ;
 xx103  =  ones(T,1)~u01j~u02j~u03i~u10j ;
 xx104  =  ones(T,1)~u01j~u02j~u03j~u10i;
 xx111  =  ones(T,1)~u01i~u02j~u03j~u11j ;
 xx112  =  ones(T,1)~u01j~u02i~u03j~u11j ;
 xx113  =  ones(T,1)~u01j~u02j~u03i~u11j ;
 xx114  =  ones(T,1)~u01j~u02j~u03j~u11i;
 xx121  =  ones(T,1)~u01i~u02j~u03j~u12j ;
 xx122  =  ones(T,1)~u01j~u02i~u03j~u12j ;
 xx123  =  ones(T,1)~u01j~u02j~u03i~u12j ;
 xx124  =  ones(T,1)~u01j~u02j~u03j~u12i;
 xx131  =  ones(T,1)~u01i~u02j~u03j~u13j ;
 xx132  =  ones(T,1)~u01j~u02i~u03j~u13j ;
 xx133  =  ones(T,1)~u01j~u02j~u03i~u13j ;
 xx134  =  ones(T,1)~u01j~u02j~u03j~u13i;
 /*xx11  =  ones(T,1);
 xx12  =  ones(T,1);
 xx13  =  ones(T,1);
 xx14  =  ones(T,1);
 xx21  =  ones(T,1) ;
 xx22  =  ones(T,1);
 xx23  =  ones(T,1);
 xx24  =  ones(T,1);
 xx31  =  ones(T,1) ;
 xx32  =  ones(T,1);
 xx33  =  ones(T,1);
 xx34  =  ones(T,1);
 xx41  =  ones(T,1) ;
 xx42  =  ones(T,1);
 xx43  =  ones(T,1);
 xx44  =  ones(T,1);
 xx51  =  ones(T,1);
 xx52  =  ones(T,1);
 xx53  =  ones(T,1);
 xx54  =  ones(T,1);
 xx61  =  ones(T,1);
 xx62  =  ones(T,1);
 xx63  =  ones(T,1);
 xx64  =  ones(T,1);
 xx71  =  ones(T,1) ;
 xx72  =  ones(T,1);
 xx73  =  ones(T,1);
 xx74  =  ones(T,1);
 xx81  =  ones(T,1);
 xx82  =  ones(T,1);
 xx83  =  ones(T,1);
 xx84  =  ones(T,1);
 xx91  =  ones(T,1);
 xx92  =  ones(T,1) ;
 xx93  =  ones(T,1);
 xx94  =  ones(T,1);
 xx101  =  ones(T,1);
 xx102  =  ones(T,1);
 xx103  =  ones(T,1);
 xx104  =  ones(T,1);
 xx111  =  ones(T,1);
 xx112  =  ones(T,1);
 xx113  =  ones(T,1) ;
 xx114  =  ones(T,1);
 xx121  =  ones(T,1);
 xx122  =  ones(T,1);
 xx123  =  ones(T,1) ;
 xx124  =  ones(T,1);
 xx131  =  ones(T,1) ;
 xx132  =  ones(T,1) ;
 xx133  =  ones(T,1);
 xx134  =  ones(T,1);*/

if sumc(w11)> 0;
if det(xx11'wmat11*xx11) < epsln ;
      m4[j,1]  =  w11'y/sumc(w11) ;
else;
 beta =  pinv(xx11'wmat11*xx11)*xx11'wmat11'y ;
 m4[j,1]  =  beta[1,.] ;
endif;
else;
m4[j,1] = 0;
a11 = a11+1;
endif;

if sumc(w12)> 0;
if det(xx12'wmat12*xx12) < epsln ;
      m4[j,2]  =  w12'y/sumc(w12) ;
else;
 beta =  pinv(xx12'wmat12*xx12)*xx12'wmat12'y ;
 m4[j,2]  =  beta[1,.] ;
endif;
else;
m4[j,2] = 0;
a12 = a12+1;
endif;

if sumc(w13)> 0;
if det(xx13'wmat13*xx13) < epsln ;
      m4[j,3]  =  w13'y/sumc(w13) ;
else;
 beta =  pinv(xx13'wmat13*xx13)*xx13'wmat13'y ;
 m4[j,3]  =  beta[1,.] ;
endif;
else;
m4[j,3] = 0;
a13 = a13+1;
endif;

if sumc(w14)> 0;
if det(xx14'wmat14*xx14) < epsln ;
      m4[j,4]  =  w13'y/sumc(w14) ;
else;
 beta =  pinv(xx14'wmat14*xx14)*xx14'wmat14'y ;
 m4[j,4]  =  beta[1,.] ;
endif;
else;
m4[j,4] = 0;
a14 = a14+1;
endif;

if sumc(w21)> 0;
if det(xx21'wmat21*xx21) < epsln ;
      m4[j,5]  =  w21'y/sumc(w21) ;
else;
 beta =  pinv(xx21'wmat21*xx21)*xx21'wmat21'y ;
 m4[j,5]  =  beta[1,.] ;
endif;
else;
m4[j,5] = 0;
a21 = a21+1;
endif;

if sumc(w22)> 0;
if det(xx22'wmat22*xx22) < epsln ;
      m4[j,6]  =  w22'y/sumc(w22) ;
else;
 beta =  pinv(xx22'wmat22*xx22)*xx22'wmat22'y ;
 m4[j,6]  =  beta[1,.] ;
endif;
else;
m4[j,6] = 0;
a22 = a22+1;
endif;

if sumc(w23)> 0;
if det(xx23'wmat23*xx23) < epsln ;
      m4[j,7]  =  w23'y/sumc(w23) ;
else;
 beta =  pinv(xx23'wmat23*xx23)*xx23'wmat23'y ;
 m4[j,7]  =  beta[1,.] ;
endif;
else;
m4[j,7] = 0;
a23 = a23+1;
endif;

if sumc(w24)> 0;
if det(xx24'wmat24*xx24) < epsln ;
      m4[j,8]  =  w24'y/sumc(w24) ;
else;
 beta =  pinv(xx24'wmat24*xx24)*xx24'wmat24'y ;
 m4[j,8]  =  beta[1,.] ;
endif;
else;
m4[j,8] = 0;
a24 = a24+1;
endif;

if sumc(w31)> 0;
if det(xx31'wmat31*xx31) < epsln ;
      m4[j,9]  =  w31'y/sumc(w31) ;
else;
 beta =  pinv(xx31'wmat31*xx31)*xx31'wmat31'y ;
 m4[j,9]  =  beta[1,.] ;
endif;
else;
m4[j,9] = 0;
a31 = a31+1;
endif;

if sumc(w32)> 0;
if det(xx32'wmat32*xx32) < epsln ;
      m4[j,10]  =  w32'y/sumc(w32) ;
else;
 beta =  pinv(xx32'wmat32*xx32)*xx32'wmat32'y ;
 m4[j,10]  =  beta[1,.] ;
endif;
else;
m4[j,10] = 0;
a32 = a32+1;
endif;

if sumc(w33)> 0;
if det(xx33'wmat33*xx33) < epsln ;
      m4[j,11]  =  w33'y/sumc(w33) ;
else;
 beta =  pinv(xx33'wmat33*xx33)*xx33'wmat33'y ;
 m4[j,11]  =  beta[1,.] ;
endif;
else;
m4[j,11] = 0;
a33 = a33+1;
endif;

if sumc(w34)> 0;
if det(xx34'wmat34*xx34) < epsln ;
      m4[j,12]  =  w34'y/sumc(w34) ;
else;
 beta =  pinv(xx34'wmat34*xx34)*xx34'wmat34'y ;
 m4[j,12]  =  beta[1,.] ;
endif;
else;
m4[j,12] = 0;
a34 = a34+1;
endif;

if sumc(w41)> 0;
if det(xx41'wmat41*xx41) < epsln ;
      m4[j,13]  =  w41'y/sumc(w41) ;
else;
 beta =  pinv(xx41'wmat41*xx41)*xx41'wmat41'y ;
 m4[j,13]  =  beta[1,.] ;
endif;
else;
m4[j,13] = 0;
a41 = a41+1;
endif;

if sumc(w42)> 0;
if det(xx42'wmat42*xx42) < epsln ;
      m4[j,14]  =  w42'y/sumc(w42) ;
else;
 beta =  pinv(xx42'wmat42*xx42)*xx42'wmat42'y ;
 m4[j,14]  =  beta[1,.] ;
endif;
else;
m4[j,14] = 0;
a42 = a42+1;
endif;

if sumc(w43)> 0;
if det(xx43'wmat43*xx43) < epsln ;
      m4[j,15]  =  w43'y/sumc(w43) ;
else;
 beta =  pinv(xx43'wmat43*xx43)*xx43'wmat43'y ;
 m4[j,15]  =  beta[1,.] ;
endif;
else;
m4[j,15] = 0;
a43 = a43+1;
endif;

if sumc(w44)> 0;
if det(xx44'wmat44*xx44) < epsln ;
      m4[j,16]  =  w44'y/sumc(w44) ;
else;
 beta =  pinv(xx44'wmat44*xx44)*xx44'wmat44'y ;
 m4[j,16]  =  beta[1,.] ;
endif;
else;
m4[j,16] = 0;
a44 = a44+1;
endif;

if sumc(w51)> 0;
if det(xx51'wmat51*xx51) < epsln ;
      m4[j,17]  =  w51'y/sumc(w51) ;
else;
 beta =  pinv(xx51'wmat51*xx51)*xx51'wmat51'y ;
 m4[j,17]  =  beta[1,.] ;
endif;
else;
m4[j,17] = 0;
a51 = a51+1;
endif;

if sumc(w52)> 0;
if det(xx52'wmat52*xx52) < epsln ;
      m4[j,18]  =  w52'y/sumc(w52) ;
else;
 beta =  pinv(xx52'wmat52*xx52)*xx52'wmat52'y ;
 m4[j,18]  =  beta[1,.] ;
endif;
else;
m4[j,18] = 0;
a52 = a52+1;
endif;

if sumc(w53)> 0;
if det(xx53'wmat53*xx53) < epsln ;
      m4[j,19]  =  w53'y/sumc(w53) ;
else;
 beta =  pinv(xx53'wmat53*xx53)*xx53'wmat53'y ;
 m4[j,19]  =  beta[1,.] ;
endif;
else;
m4[j,19] = 0;
a53 = a53+1;
endif;

if sumc(w54)> 0;
if det(xx54'wmat54*xx54) < epsln ;
      m4[j,20]  =  w54'y/sumc(w54) ;
else;
 beta =  pinv(xx54'wmat54*xx54)*xx54'wmat54'y ;
 m4[j,20]  =  beta[1,.] ;
endif;
else;
m4[j,20] = 0;
a54 = a54+1;
endif;

if sumc(w61)> 0;
if det(xx61'wmat61*xx61) < epsln ;
      m4[j,21]  =  w61'y/sumc(w61) ;
else;
 beta =  pinv(xx61'wmat61*xx61)*xx61'wmat61'y ;
 m4[j,21]  =  beta[1,.] ;
endif;
else;
m4[j,21] = 0;
a61 = a61+1;
endif;

if sumc(w62)> 0;
if det(xx62'wmat62*xx62) < epsln ;
      m4[j,22]  =  w62'y/sumc(w62) ;
else;
 beta =  pinv(xx62'wmat62*xx62)*xx62'wmat62'y ;
 m4[j,22]  =  beta[1,.] ;
endif;
else;
m4[j,22] = 0;
a62 = a62+1;
endif;

if sumc(w63)> 0;
if det(xx63'wmat63*xx63) < epsln ;
      m4[j,23]  =  w63'y/sumc(w63) ;
else;
 beta =  pinv(xx63'wmat63*xx63)*xx63'wmat63'y ;
 m4[j,23]  =  beta[1,.] ;
endif;
else;
m4[j,23] = 0;
a63 = a63+1;
endif;

if sumc(w64)> 0;
if det(xx64'wmat64*xx64) < epsln ;
      m4[j,24]  =  w64'y/sumc(w64) ;
else;
 beta =  pinv(xx64'wmat64*xx64)*xx64'wmat64'y ;
 m4[j,24]  =  beta[1,.] ;
endif;
else;
m4[j,24] = 0;
a64 = a64+1;
endif;

if sumc(w71)> 0;
if det(xx71'wmat71*xx71) < epsln ;
      m4[j,25]  =  w71'y/sumc(w71) ;
else;
 beta =  pinv(xx71'wmat71*xx71)*xx71'wmat71'y ;
 m4[j,25]  =  beta[1,.] ;
endif;
else;
m4[j,25] = 0;
a71 = a71+1;
endif;

if sumc(w72)> 0;
if det(xx72'wmat72*xx72) < epsln ;
      m4[j,20]  =  w72'y/sumc(w72) ;
else;
 beta =  pinv(xx72'wmat72*xx72)*xx72'wmat72'y ;
 m4[j,26]  =  beta[1,.] ;
endif;
else;
m4[j,26] = 0;
a72 = a72+1;
endif;

if sumc(w73)> 0;
if det(xx73'wmat73*xx73) < epsln ;
      m4[j,27]  =  w73'y/sumc(w73) ;
else;
 beta =  pinv(xx73'wmat73*xx73)*xx73'wmat73'y ;
 m4[j,27]  =  beta[1,.] ;
endif;
else;
m4[j,27] = 0;
a73 = a73+1;
endif;

if sumc(w74)> 0;
if det(xx74'wmat74*xx74) < epsln ;
      m4[j,28]  =  w74'y/sumc(w74) ;
else;
 beta =  pinv(xx74'wmat74*xx74)*xx74'wmat74'y ;
 m4[j,28]  =  beta[1,.] ;
endif;
else;
m4[j,28] = 0;
a74 = a74+1;
endif;

if sumc(w81)> 0;
if det(xx81'wmat81*xx81) < epsln ;
      m4[j,29]  =  w81'y/sumc(w81) ;
else;
 beta =  pinv(xx81'wmat81*xx81)*xx81'wmat81'y ;
 m4[j,29]  =  beta[1,.] ;
endif;
else;
m4[j,29] = 0;
a81 = a81+1;
endif;

if sumc(w82)> 0;
if det(xx82'wmat82*xx82) < epsln ;
      m4[j,30]  =  w82'y/sumc(w82) ;
else;
 beta =  pinv(xx82'wmat82*xx82)*xx82'wmat82'y ;
 m4[j,30]  =  beta[1,.] ;
endif;
else;
m4[j,30] = 0;
a82 = a82+1;
endif;

if sumc(w83)> 0;
if det(xx83'wmat83*xx83) < epsln ;
      m4[j,31]  =  w83'y/sumc(w83) ;
else;
 beta =  pinv(xx83'wmat83*xx83)*xx83'wmat83'y ;
 m4[j,31]  =  beta[1,.] ;
endif;
else;
m4[j,31] = 0;
a83 = a83+1;
endif;

if sumc(w84)> 0;
if det(xx84'wmat84*xx84) < epsln ;
      m4[j,32]  =  w84'y/sumc(w84) ;
else;
 beta =  pinv(xx84'wmat84*xx84)*xx84'wmat84'y ;
 m4[j,32]  =  beta[1,.] ;
endif;
else;
m4[j,32] = 0;
a84 = a84+1;
endif;

if sumc(w91)> 0;
if det(xx91'wmat91*xx91) < epsln ;
      m4[j,33]  =  w91'y/sumc(w91) ;
else;
 beta =  pinv(xx91'wmat91*xx91)*xx91'wmat91'y ;
 m4[j,33]  =  beta[1,.] ;
endif;
else;
m4[j,33] = 0;
a91 = a91+1;
endif;

if sumc(w92)> 0;
if det(xx92'wmat92*xx92) < epsln ;
      m4[j,34]  =  w92'y/sumc(w92) ;
else;
 beta =  pinv(xx92'wmat92*xx92)*xx92'wmat92'y ;
 m4[j,34]  =  beta[1,.] ;
endif;
else;
m4[j,34] = 0;
a92 = a92+1;
endif;

if sumc(w93)> 0;
if det(xx93'wmat93*xx93) < epsln ;
      m4[j,35]  =  w93'y/sumc(w93) ;
else;
 beta =  pinv(xx93'wmat93*xx93)*xx93'wmat93'y ;
 m4[j,35]  =  beta[1,.] ;
endif;
else;
m4[j,35] = 0;
a93 = a93+1;
endif;

if sumc(w94)> 0;
if det(xx94'wmat94*xx94) < epsln ;
      m4[j,36]  =  w94'y/sumc(w94) ;
else;
 beta =  pinv(xx94'wmat94*xx94)*xx94'wmat94'y ;
 m4[j,36]  =  beta[1,.] ;
endif;
else;
m4[j,36] = 0;
a94 = a94+1;
endif;

if sumc(w101)> 0;
if det(xx101'wmat101*xx101) < epsln ;
      m4[j,37]  =  w101'y/sumc(w101) ;
else;
 beta =  pinv(xx101'wmat101*xx101)*xx101'wmat101'y ;
 m4[j,37]  =  beta[1,.] ;
endif;
else;
m4[j,37] = 0;
a101 = a101+1;
endif;

if sumc(w102)> 0;
if det(xx102'wmat102*xx102) < epsln ;
      m4[j,38]  =  w102'y/sumc(w102) ;
else;
 beta =  pinv(xx102'wmat102*xx102)*xx102'wmat102'y ;
 m4[j,38]  =  beta[1,.] ;
endif;
else;
m4[j,38] = 0;
a102 = a102+1;
endif;

if sumc(w103)> 0;
if det(xx103'wmat103*xx103) < epsln ;
      m4[j,39]  =  w103'y/sumc(w103) ;
else;
 beta =  pinv(xx103'wmat103*xx103)*xx103'wmat103'y ;
 m4[j,39]  =  beta[1,.] ;
endif;
else;
m4[j,39] = 0;
a103 = a103+1;
endif;

if sumc(w104)> 0;
if det(xx104'wmat104*xx104) < epsln ;
      m4[j,40]  =  w104'y/sumc(w104) ;
else;
 beta =  pinv(xx104'wmat104*xx104)*xx104'wmat104'y ;
 m4[j,40]  =  beta[1,.] ;
endif;
else;
m4[j,40] = 0;
a104 = a104+1;
endif;


if sumc(w111)> 0;
if det(xx111'wmat111*xx111) < epsln ;
      m4[j,41]  =  w111'y/sumc(w111) ;
else;
 beta =  pinv(xx111'wmat111*xx111)*xx111'wmat111'y ;
 m4[j,41]  =  beta[1,.] ;
endif;
else;
m4[j,41] = 0;
a111 = a111+1;
endif;

if sumc(w112)> 0;
if det(xx112'wmat112*xx112) < epsln ;
      m4[j,42]  =  w112'y/sumc(w112) ;
else;
 beta =  pinv(xx112'wmat112*xx112)*xx112'wmat112'y ;
 m4[j,42]  =  beta[1,.] ;
endif;
else;
m4[j,42] = 0;
a112 = a112+1;
endif;

if sumc(w113)> 0;
if det(xx113'wmat113*xx113) < epsln ;
      m4[j,43]  =  w113'y/sumc(w113) ;
else;
 beta =  pinv(xx113'wmat113*xx113)*xx113'wmat113'y ;
 m4[j,43]  =  beta[1,.] ;
endif;
else;
m4[j,43] = 0;
a113 = a113+1;
endif;

if sumc(w114)> 0;
if det(xx114'wmat114*xx114) < epsln ;
      m4[j,44]  =  w114'y/sumc(w114) ;
else;
 beta =  pinv(xx114'wmat114*xx114)*xx114'wmat114'y ;
 m4[j,44]  =  beta[1,.] ;
endif;
else;
m4[j,44] = 0;
a114 = a114+1;
endif;

if sumc(w121)> 0;
if det(xx121'wmat121*xx121) < epsln ;
      m4[j,45]  =  w121'y/sumc(w121) ;
else;
 beta =  pinv(xx121'wmat121*xx121)*xx121'wmat121'y ;
 m4[j,45]  =  beta[1,.] ;
endif;
else;
m4[j,45] = 0;
a121 = a121+1;
endif;

if sumc(w122)> 0;
if det(xx122'wmat122*xx122) < epsln ;
      m4[j,46]  =  w122'y/sumc(w122) ;
else;
 beta =  pinv(xx122'wmat122*xx122)*xx122'wmat122'y ;
 m4[j,46]  =  beta[1,.] ;
endif;
else;
m4[j,46] = 0;
a122 = a122+1;
endif;

if sumc(w123)> 0;
if det(xx123'wmat123*xx123) < epsln ;
      m4[j,47]  =  w123'y/sumc(w123) ;
else;
 beta =  pinv(xx123'wmat123*xx123)*xx123'wmat123'y ;
 m4[j,47]  =  beta[1,.] ;
endif;
else;
m4[j,47] = 0;
a123 = a123+1;
endif;

if sumc(w124)> 0;
if det(xx124'wmat124*xx124) < epsln ;
      m4[j,48]  =  w124'y/sumc(w124) ;
else;
 beta =  pinv(xx124'wmat124*xx124)*xx124'wmat124'y ;
 m4[j,48]  =  beta[1,.] ;
endif;
else;
m4[j,48] = 0;
a124 = a124+1;
endif;

if sumc(w131)> 0;
if det(xx131'wmat131*xx131) < epsln ;
      m4[j,49]  =  w131'y/sumc(w131) ;
else;
 beta =  pinv(xx131'wmat131*xx131)*xx131'wmat131'y ;
 m4[j,49]  =  beta[1,.] ;
endif;
else;
m4[j,49] = 0;
a131 = a131+1;
endif;

if sumc(w132)> 0;
if det(xx132'wmat132*xx132) < epsln ;
      m4[j,50]  =  w132'y/sumc(w132) ;
else;
 beta =  pinv(xx132'wmat132*xx132)*xx132'wmat132'y ;
 m4[j,50]  =  beta[1,.] ;
endif;
else;
m4[j,50] = 0;
a132 = a132+1;
endif;

if sumc(w133)> 0;
if det(xx133'wmat133*xx133) < epsln ;
      m4[j,51]  =  w133'y/sumc(w133) ;
else;
 beta =  pinv(xx133'wmat133*xx133)*xx133'wmat133'y ;
 m4[j,51]  =  beta[1,.] ;
endif;
else;
m4[j,51] = 0;
a133 = a133+1;
endif;

if sumc(w134)> 0;
if det(xx134'wmat134*xx134) < epsln ;
      m4[j,52]  =  w134'y/sumc(w134) ;
else;
 beta =  pinv(xx134'wmat134*xx134)*xx134'wmat134'y ;
 m4[j,52]  =  beta[1,.] ;
endif;
else;
m4[j,52] = 0;
a134 = a134+1;
endif;
j = j+1;
endo;

m4d[i,1] = 1/(T-a11)*sumc(m4[.,1]);
m4d[i,2] = 1/(T-a12)*sumc(m4[.,2]);
m4d[i,3] = 1/(T-a13)*sumc(m4[.,3]);
m4d[i,4] = 1/(T-a14)*sumc(m4[.,4]);
m4d[i,5] = 1/(T-a21)*sumc(m4[.,5]);
m4d[i,6] = 1/(T-a22)*sumc(m4[.,6]);
m4d[i,7] = 1/(T-a23)*sumc(m4[.,7]);
m4d[i,8] = 1/(T-a24)*sumc(m4[.,8]);
m4d[i,9] = 1/(T-a31)*sumc(m4[.,9]);
m4d[i,10] = 1/(T-a32)*sumc(m4[.,10]);
m4d[i,11] = 1/(T-a33)*sumc(m4[.,11]);
m4d[i,12] = 1/(T-a34)*sumc(m4[.,12]);
m4d[i,13] = 1/(T-a41)*sumc(m4[.,13]);
m4d[i,14] = 1/(T-a42)*sumc(m4[.,14]);
m4d[i,15] = 1/(T-a43)*sumc(m4[.,15]);
m4d[i,16] = 1/(T-a44)*sumc(m4[.,16]);
m4d[i,17] = 1/(T-a51)*sumc(m4[.,17]);
m4d[i,18] = 1/(T-a52)*sumc(m4[.,18]);
m4d[i,19] = 1/(T-a53)*sumc(m4[.,19]);
m4d[i,20] = 1/(T-a54)*sumc(m4[.,20]);
m4d[i,21] = 1/(T-a61)*sumc(m4[.,21]);
m4d[i,22] = 1/(T-a62)*sumc(m4[.,22]);
m4d[i,23] = 1/(T-a63)*sumc(m4[.,23]);
m4d[i,24] = 1/(T-a64)*sumc(m4[.,23]);
m4d[i,25] = 1/(T-a71)*sumc(m4[.,25]);
m4d[i,26] = 1/(T-a72)*sumc(m4[.,26]);
m4d[i,27] = 1/(T-a73)*sumc(m4[.,27]);
m4d[i,28] = 1/(T-a73)*sumc(m4[.,28]);
m4d[i,29] = 1/(T-a81)*sumc(m4[.,29]);
m4d[i,30] = 1/(T-a82)*sumc(m4[.,30]);
m4d[i,31] = 1/(T-a83)*sumc(m4[.,31]);
m4d[i,32] = 1/(T-a84)*sumc(m4[.,32]);
m4d[i,33] = 1/(T-a91)*sumc(m4[.,33]);
m4d[i,34] = 1/(T-a92)*sumc(m4[.,34]);
m4d[i,35] = 1/(T-a93)*sumc(m4[.,35]);
m4d[i,36] = 1/(T-a94)*sumc(m4[.,36]);
m4d[i,37] = 1/(T-a101)*sumc(m4[.,37]);
m4d[i,38] = 1/(T-a102)*sumc(m4[.,38]);
m4d[i,39] = 1/(T-a103)*sumc(m4[.,39]);
m4d[i,40] = 1/(T-a104)*sumc(m4[.,40]);
m4d[i,41] = 1/(T-a111)*sumc(m4[.,41]);
m4d[i,42] = 1/(T-a112)*sumc(m4[.,42]);
m4d[i,43] = 1/(T-a113)*sumc(m4[.,43]);
m4d[i,44] = 1/(T-a114)*sumc(m4[.,44]);
m4d[i,45] = 1/(T-a121)*sumc(m4[.,45]);
m4d[i,46] = 1/(T-a122)*sumc(m4[.,46]);
m4d[i,47] = 1/(T-a123)*sumc(m4[.,47]);
m4d[i,48] = 1/(T-a124)*sumc(m4[.,48]);
m4d[i,49] = 1/(T-a131)*sumc(m4[.,49]);
m4d[i,50] = 1/(T-a132)*sumc(m4[.,50]);
m4d[i,51] = 1/(T-a133)*sumc(m4[.,51]);
m4d[i,52] = 1/(T-a134)*sumc(m4[.,52]);

/*Four dimension kernel estimator of the density */
LVO =  ones(T,1);
LVO[i] = 0;
phat[i,1] = sumc(LVO.*fi(u01i/hs4).*fi(u02i/hs4).*fi(u03i/hs4).*fi(u1i/hs4))/((T-1)*hs4^4);
phat[i,2] = sumc(LVO.*fi(u01i/hs4).*fi(u02i/hs4).*fi(u03i/hs4).*fi(u2i/hs4))/((T-1)*hs4^4);
phat[i,3] = sumc(LVO.*fi(u01i/hs4).*fi(u02i/hs4).*fi(u03i/hs4).*fi(u3i/hs4))/((T-1)*hs4^4);
phat[i,4] = sumc(LVO.*fi(u01i/hs4).*fi(u02i/hs4).*fi(u03i/hs4).*fi(u4i/hs4))/((T-1)*hs4^4);
phat[i,5] = sumc(LVO.*fi(u01i/hs4).*fi(u02i/hs4).*fi(u03i/hs4).*fi(u5i/hs4))/((T-1)*hs4^4);
phat[i,6] = sumc(LVO.*fi(u01i/hs4).*fi(u02i/hs4).*fi(u03i/hs4).*fi(u6i/hs4))/((T-1)*hs4^4);
phat[i,7] = sumc(LVO.*fi(u01i/hs4).*fi(u02i/hs4).*fi(u03i/hs4).*fi(u7i/hs4))/((T-1)*hs4^4);
phat[i,8] = sumc(LVO.*fi(u01i/hs4).*fi(u02i/hs4).*fi(u03i/hs4).*fi(u8i/hs4))/((T-1)*hs4^4);
phat[i,9] = sumc(LVO.*fi(u01i/hs4).*fi(u02i/hs4).*fi(u03i/hs4).*fi(u9i/hs4))/((T-1)*hs4^4);
phat[i,10] = sumc(LVO.*fi(u01i/hs4).*fi(u02i/hs4).*fi(u03i/hs4).*fi(u10i/hs4))/((T-1)*hs4^4);
phat[i,11] = sumc(LVO.*fi(u01i/hs4).*fi(u02i/hs4).*fi(u03i/hs4).*fi(u11i/hs4))/((T-1)*hs4^4);
phat[i,12] = sumc(LVO.*fi(u01i/hs4).*fi(u02i/hs4).*fi(u03i/hs4).*fi(u12i/hs4))/((T-1)*hs4^4);
phat[i,13] = sumc(LVO.*fi(u01i/hs4).*fi(u02i/hs4).*fi(u03i/hs4).*fi(u13i/hs4))/((T-1)*hs4^4);
@estimating q(z) @
qz4[i,1] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1);
qz4[i,2] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u1i/h1)./h1);
qz4[i,3] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u1i/h1)./h1);
qz4[i,4] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u1i/h1)./h1);
qz4[i,5] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u2i/h1)./h1);
qz4[i,6] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u2i/h1)./h1);
qz4[i,7] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u2i/h1)./h1);
qz4[i,8] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u3i/h1)./h1);
qz4[i,9] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u3i/h1)./h1);
qz4[i,10] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u3i/h1)./h1);
qz4[i,11] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u4i/h1)./h1);
qz4[i,12] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u4i/h1)./h1);
qz4[i,13] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u4i/h1)./h1);
qz4[i,14] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u5i/h1)./h1);
qz4[i,15] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u5i/h1)./h1);
qz4[i,16] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u5i/h1)./h1);
qz4[i,17] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u6i/h1)./h1);
qz4[i,18] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u6i/h1)./h1);
qz4[i,19] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u6i/h1)./h1);
qz4[i,20] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u7i/h1)./h1);
qz4[i,21] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u7i/h1)./h1);
qz4[i,22] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u7i/h1)./h1);
qz4[i,23] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u8i/h1)./h1);
qz4[i,24] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u8i/h1)./h1);
qz4[i,25] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u8i/h1)./h1);
qz4[i,26] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u9i/h1)./h1);
qz4[i,27] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u9i/h1)./h1);
qz4[i,28] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u9i/h1)./h1);
qz4[i,29] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u10i/h1)./h1);
qz4[i,30] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u10i/h1)./h1);
qz4[i,31] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u10i/h1)./h1);
qz4[i,32] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u11i/h1)./h1);
qz4[i,33] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u11i/h1)./h1);
qz4[i,34] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u11i/h1)./h1);
qz4[i,35] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u12i/h1)./h1);
qz4[i,36] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u12i/h1)./h1);
qz4[i,37] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u12i/h1)./h1);
qz4[i,38] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u13i/h1)./h1);
qz4[i,39] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u13i/h1)./h1);
qz4[i,40] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u13i/h1)./h1);

i = i+1;
endo;
yhat1 = y-meanc(y)-(m4d[.,1]-meanc(m4d[.,1])+m4d[.,2]-meanc(m4d[.,2])+m4d[.,3]-meanc(m4d[.,3])+m4d[.,4]-meanc(m4d[.,4]));
yhat2 = y-meanc(y)-(m4d[.,5]-meanc(m4d[.,5])+m4d[.,6]-meanc(m4d[.,6])+m4d[.,7]-meanc(m4d[.,7])+m4d[.,8]-meanc(m4d[.,8]));
yhat3 = y-meanc(y)-(m4d[.,9]-meanc(m4d[.,9])+m4d[.,10]-meanc(m4d[.,10])+m4d[.,11]-meanc(m4d[.,11])+m4d[.,12]-meanc(m4d[.,12]));
yhat4 = y-meanc(y)-(m4d[.,13]-meanc(m4d[.,13])+m4d[.,14]-meanc(m4d[.,14])+m4d[.,15]-meanc(m4d[.,15])+m4d[.,16]-meanc(m4d[.,16]));
yhat5 = y-meanc(y)-(m4d[.,17]-meanc(m4d[.,17])+m4d[.,18]-meanc(m4d[.,18])+m4d[.,19]-meanc(m4d[.,19])+m4d[.,20]-meanc(m4d[.,20]));
yhat6 = y-meanc(y)-(m4d[.,21]-meanc(m4d[.,21])+m4d[.,22]-meanc(m4d[.,22])+m4d[.,23]-meanc(m4d[.,23])+m4d[.,24]-meanc(m4d[.,24]));
yhat7 = y-meanc(y)-(m4d[.,25]-meanc(m4d[.,25])+m4d[.,26]-meanc(m4d[.,26])+m4d[.,27]-meanc(m4d[.,27])+m4d[.,28]-meanc(m4d[.,28]));
yhat8 = y-meanc(y)-(m4d[.,29]-meanc(m4d[.,29])+m4d[.,30]-meanc(m4d[.,30])+m4d[.,31]-meanc(m4d[.,31])+m4d[.,32]-meanc(m4d[.,32]));
yhat9 = y-meanc(y)-(m4d[.,33]-meanc(m4d[.,33])+m4d[.,34]-meanc(m4d[.,34])+m4d[.,35]-meanc(m4d[.,35])+m4d[.,36]-meanc(m4d[.,36]));
yhat10 = y-meanc(y)-(m4d[.,37]-meanc(m4d[.,37])+m4d[.,38]-meanc(m4d[.,38])+m4d[.,39]-meanc(m4d[.,39])+m4d[.,40]-meanc(m4d[.,40]));
yhat11 = y-meanc(y)-(m4d[.,41]-meanc(m4d[.,41])+m4d[.,42]-meanc(m4d[.,42])+m4d[.,43]-meanc(m4d[.,43])+m4d[.,44]-meanc(m4d[.,44]));
yhat12 = y-meanc(y)-(m4d[.,45]-meanc(m4d[.,45])+m4d[.,46]-meanc(m4d[.,46])+m4d[.,47]-meanc(m4d[.,47])+m4d[.,48]-meanc(m4d[.,48]));
yhat13 = y-meanc(y)-(m4d[.,49]-meanc(m4d[.,49])+m4d[.,50]-meanc(m4d[.,50])+m4d[.,51]-meanc(m4d[.,51])+m4d[.,52]-meanc(m4d[.,52]));
BR1 = (yhat1-y).*(yhat1-y);
BR2 = (yhat2-y).*(yhat2-y);
BR3 = (yhat3-y).*(yhat3-y);
BR4 = (yhat4-y).*(yhat4-y);
BR5 = (yhat5-y).*(yhat5-y);
BR6 = (yhat6-y).*(yhat6-y);
BR7 = (yhat7-y).*(yhat7-y);
BR8 = (yhat8-y).*(yhat8-y);
BR9 = (yhat9-y).*(yhat9-y);
BR10 = (yhat10-y).*(yhat10-y);
BR11 = (yhat11-y).*(yhat11-y);
BR12 = (yhat12-y).*(yhat12-y);
BR13 = (yhat13-y).*(yhat13-y);
quan = quantile(phat,0.05);
index = phat.< = quan;
phat = index*10^5+phat;

/****************************Four Lags Included ******************************************/

       
/****************************Four Lags Included Estimation of A part *******************************************/
kc = 1;
do until kc>nc;
b4 = h4*h[kc,1];
i = 1;
a11 = 0;
a12 = 0;
a13 = 0;
a14 = 0;
a21 = 0;
a22 = 0;
a23 = 0;
a24 = 0;
a31 = 0;
a32 = 0;
a33 = 0;
a34 = 0;
a41 = 0;
a42 = 0;
a43 = 0;
a44 = 0;
a51 = 0;
a52 = 0;
a53 = 0;
a54 = 0;
a61 = 0;
a62 = 0;
a63 = 0;
a64 = 0;
a71 = 0;
a72 = 0;
a73 = 0;
a74 = 0;
a81 = 0;
a82 = 0;
a83 = 0;
a84 = 0;
a91 = 0;
a92 = 0;
a93 = 0;
a94 = 0;
a101 = 0;
a102 = 0;
a103 = 0;
a104 = 0;
a111 = 0;
a112 = 0;
a113 = 0;
a114 = 0;
a121 = 0;
a122 = 0;
a123 = 0;
a124 = 0;
a131 = 0;
a132 = 0;
a133 = 0;
a134 = 0;
m4 = zeros(T,52);
m4d = zeros(T,52);
i = 1;
do until i>T;
u01i = data[i,lag_1]*ones(T,1) - data[.,lag_1];
u02i = data[i,lag_2]*ones(T,1) - data[.,lag_2];
u03i = data[i,lag_3]*ones(T,1) - data[.,lag_3];
u0i = u01i~u02i~u03i;
u1i =  data[i,1]*ones(T,1) - data[.,1];
u2i =  data[i,2]*ones(T,1) - data[.,2];
u3i =  data[i,3]*ones(T,1) - data[.,3];
u4i =  data[i,4]*ones(T,1) - data[.,4];
u5i =  data[i,5]*ones(T,1) - data[.,5];
u6i =  data[i,6]*ones(T,1) - data[.,6];
u7i =  data[i,7]*ones(T,1) - data[.,7];
u8i =  data[i,8]*ones(T,1) - data[.,8];
u9i =  data[i,9]*ones(T,1) - data[.,9];
u10i =  data[i,10]*ones(T,1) - data[.,10];
u11i =  data[i,11]*ones(T,1) - data[.,11];
u12i =  data[i,12]*ones(T,1) - data[.,12];
u13i =  data[i,13]*ones(T,1) - data[.,13];
j = 1;
do until j>T;
u01j = data[j,lag_1]*ones(T,1) - data[.,lag_1];
u02j = data[j,lag_2]*ones(T,1) - data[.,lag_2];
u03j = data[j,lag_3]*ones(T,1) - data[.,lag_3];
u0j = u01j~u02j~u03j;
 u1j =  (data[j,1]*ones(T,1) - data[.,1]);
 u2j =  (data[j,2]*ones(T,1) - data[.,2]);
 u3j =  (data[j,3]*ones(T,1) - data[.,3]);
 u4j =  (data[j,4]*ones(T,1) - data[.,4]);
 u5j =  (data[j,5]*ones(T,1) - data[.,5]);
 u6j =  (data[j,5]*ones(T,1) - data[.,6]);
 u7j =  (data[j,7]*ones(T,1) - data[.,7]);
 u8j =  (data[j,8]*ones(T,1) - data[.,8]);
 u9j =  (data[j,9]*ones(T,1) - data[.,9]);
 u10j =  (data[j,10]*ones(T,1) - data[.,10]);
 u11j =  (data[j,11]*ones(T,1) - data[.,11]);
 u12j =  (data[j,12]*ones(T,1) - data[.,12]);
 u13j =  (data[j,13]*ones(T,1) - data[.,13]);
 w11 = (fi(u01i/b4)/b4).*(fi(u02j/b4)/b4).*(fi(u03j/b4)/b4).*(fi(u1j/b4)/b4);
 w12 = (fi(u01j/b4)/b4).*(fi(u02i/b4)/b4).*(fi(u03j/b4)/b4).*(fi(u1j/b4)/b4);
 w13 = (fi(u01j/b4)/b4).*(fi(u02j/b4)/b4).*(fi(u03i/b4)/b4).*(fi(u1j/b4)/b4);
 w14 = (fi(u01j/b4)/b4).*(fi(u02j/b4)/b4).*(fi(u03j/b4)/b4).*(fi(u1i/b4)/b4);
 w21 = (fi(u01i/b4)/b4).*(fi(u02j/b4)/b4).*(fi(u03j/b4)/b4).*(fi(u2j/b4)/b4);
 w22 = (fi(u01j/b4)/b4).*(fi(u02i/b4)/b4).*(fi(u03j/b4)/b4).*(fi(u2j/b4)/b4);
 w23 = (fi(u01j/b4)/b4).*(fi(u02j/b4)/b4).*(fi(u03i/b4)/b4).*(fi(u2j/b4)/b4);
 w24 = (fi(u01j/b4)/b4).*(fi(u02j/b4)/b4).*(fi(u03j/b4)/b4).*(fi(u2i/b4)/b4);
 w31 = (fi(u01i/b4)/b4).*(fi(u02j/b4)/b4).*(fi(u03j/b4)/b4).*(fi(u3j/b4)/b4);
 w32 = (fi(u01j/b4)/b4).*(fi(u02i/b4)/b4).*(fi(u03j/b4)/b4).*(fi(u3j/b4)/b4);
 w33 = (fi(u01j/b4)/b4).*(fi(u02j/b4)/b4).*(fi(u03i/b4)/b4).*(fi(u3j/b4)/b4);
 w34 = (fi(u01j/b4)/b4).*(fi(u02j/b4)/b4).*(fi(u03j/b4)/b4).*(fi(u3i/b4)/b4);
 w41 = (fi(u01i/b4)/b4).*(fi(u02j/b4)/b4).*(fi(u03j/b4)/b4).*(fi(u4j/b4)/b4);
 w42 = (fi(u01j/b4)/b4).*(fi(u02i/b4)/b4).*(fi(u03j/b4)/b4).*(fi(u4j/b4)/b4);
 w43 = (fi(u01j/b4)/b4).*(fi(u02j/b4)/b4).*(fi(u03i/b4)/b4).*(fi(u4j/b4)/b4);
 w44 = (fi(u01j/b4)/b4).*(fi(u02j/b4)/b4).*(fi(u03j/b4)/b4).*(fi(u4i/b4)/b4);
 w51 = (fi(u01i/b4)/b4).*(fi(u02j/b4)/b4).*(fi(u03j/b4)/b4).*(fi(u5j/b4)/b4);
 w52 = (fi(u01j/b4)/b4).*(fi(u02i/b4)/b4).*(fi(u03j/b4)/b4).*(fi(u5j/b4)/b4);
 w53 = (fi(u01j/b4)/b4).*(fi(u02j/b4)/b4).*(fi(u03i/b4)/b4).*(fi(u5j/b4)/b4);
 w54 = (fi(u01j/b4)/b4).*(fi(u02j/b4)/b4).*(fi(u03j/b4)/b4).*(fi(u5i/b4)/b4);
 w61 = (fi(u01i/b4)/b4).*(fi(u02j/b4)/b4).*(fi(u03j/b4)/b4).*(fi(u6j/b4)/b4);
 w62 = (fi(u01j/b4)/b4).*(fi(u02i/b4)/b4).*(fi(u03j/b4)/b4).*(fi(u6j/b4)/b4);
 w63 = (fi(u01j/b4)/b4).*(fi(u02j/b4)/b4).*(fi(u03i/b4)/b4).*(fi(u6j/b4)/b4);
 w64 = (fi(u01j/b4)/b4).*(fi(u02j/b4)/b4).*(fi(u03j/b4)/b4).*(fi(u6i/b4)/b4);
 w71 = (fi(u01i/b4)/b4).*(fi(u02j/b4)/b4).*(fi(u03j/b4)/b4).*(fi(u7j/b4)/b4);
 w72 = (fi(u01j/b4)/b4).*(fi(u02i/b4)/b4).*(fi(u03j/b4)/b4).*(fi(u7j/b4)/b4);
 w73 = (fi(u01j/b4)/b4).*(fi(u02j/b4)/b4).*(fi(u03i/b4)/b4).*(fi(u7j/b4)/b4);
 w74 = (fi(u01j/b4)/b4).*(fi(u02j/b4)/b4).*(fi(u03j/b4)/b4).*(fi(u7i/b4)/b4);
 w81 = (fi(u01i/b4)/b4).*(fi(u02j/b4)/b4).*(fi(u03j/b4)/b4).*(fi(u8j/b4)/b4);
 w82 = (fi(u01j/b4)/b4).*(fi(u02i/b4)/b4).*(fi(u03j/b4)/b4).*(fi(u8j/b4)/b4);
 w83 = (fi(u01j/b4)/b4).*(fi(u02j/b4)/b4).*(fi(u03i/b4)/b4).*(fi(u8j/b4)/b4);
 w84 = (fi(u01j/b4)/b4).*(fi(u02j/b4)/b4).*(fi(u03j/b4)/b4).*(fi(u8i/b4)/b4);
 w91 = (fi(u01i/b4)/b4).*(fi(u02j/b4)/b4).*(fi(u03j/b4)/b4).*(fi(u9j/b4)/b4);
 w92 = (fi(u01j/b4)/b4).*(fi(u02i/b4)/b4).*(fi(u03j/b4)/b4).*(fi(u9j/b4)/b4);
 w93 = (fi(u01j/b4)/b4).*(fi(u02j/b4)/b4).*(fi(u03i/b4)/b4).*(fi(u9j/b4)/b4);
 w94 = (fi(u01j/b4)/b4).*(fi(u02j/b4)/b4).*(fi(u03j/b4)/b4).*(fi(u9i/b4)/b4);
 w101 = (fi(u01i/b4)/b4).*(fi(u02j/b4)/b4).*(fi(u03j/b4)/b4).*(fi(u10j/b4)/b4);
 w102 = (fi(u01j/b4)/b4).*(fi(u02i/b4)/b4).*(fi(u03j/b4)/b4).*(fi(u10j/b4)/b4);
 w103 = (fi(u01j/b4)/b4).*(fi(u02j/b4)/b4).*(fi(u03i/b4)/b4).*(fi(u10j/b4)/b4);
 w104 = (fi(u01j/b4)/b4).*(fi(u02j/b4)/b4).*(fi(u03j/b4)/b4).*(fi(u10i/b4)/b4);
 w111 = (fi(u01i/b4)/b4).*(fi(u02j/b4)/b4).*(fi(u03j/b4)/b4).*(fi(u11j/b4)/b4);
 w112 = (fi(u01j/b4)/b4).*(fi(u02i/b4)/b4).*(fi(u03j/b4)/b4).*(fi(u11j/b4)/b4);
 w113 = (fi(u01j/b4)/b4).*(fi(u02j/b4)/b4).*(fi(u03i/b4)/b4).*(fi(u11j/b4)/b4);
 w114 = (fi(u01j/b4)/b4).*(fi(u02j/b4)/b4).*(fi(u03j/b4)/b4).*(fi(u11i/b4)/b4);
 w121 = (fi(u01i/b4)/b4).*(fi(u02j/b4)/b4).*(fi(u03j/b4)/b4).*(fi(u12j/b4)/b4);
 w122 = (fi(u01j/b4)/b4).*(fi(u02i/b4)/b4).*(fi(u03j/b4)/b4).*(fi(u12j/b4)/b4);
 w123 = (fi(u01j/b4)/b4).*(fi(u02j/b4)/b4).*(fi(u03i/b4)/b4).*(fi(u12j/b4)/b4);
 w124 = (fi(u01j/b4)/b4).*(fi(u02j/b4)/b4).*(fi(u03j/b4)/b4).*(fi(u12i/b4)/b4);
 w131 = (fi(u01i/b4)/b4).*(fi(u02j/b4)/b4).*(fi(u03j/b4)/b4).*(fi(u13j/b4)/b4);
 w132 = (fi(u01j/b4)/b4).*(fi(u02i/b4)/b4).*(fi(u03j/b4)/b4).*(fi(u13j/b4)/b4);
 w133 = (fi(u01j/b4)/b4).*(fi(u02j/b4)/b4).*(fi(u03i/b4)/b4).*(fi(u13j/b4)/b4);
 w134 = (fi(u01j/b4)/b4).*(fi(u02j/b4)/b4).*(fi(u03j/b4)/b4).*(fi(u13i/b4)/b4);
 wmat11  = diagrv((eye(T),w11));
 wmat12  = diagrv((eye(T),w12));
 wmat13  = diagrv((eye(T),w13));
 wmat14  = diagrv((eye(T),w14));
 wmat21  = diagrv((eye(T),w21));
 wmat22  = diagrv((eye(T),w22));
 wmat23  = diagrv((eye(T),w23));
 wmat24  = diagrv((eye(T),w24));
 wmat31  = diagrv((eye(T),w31));
 wmat32  = diagrv((eye(T),w32));
 wmat33  = diagrv((eye(T),w33));
 wmat34  = diagrv((eye(T),w34));
 wmat41  = diagrv((eye(T),w41));
 wmat42  = diagrv((eye(T),w42));
 wmat43  = diagrv((eye(T),w43));
 wmat44  = diagrv((eye(T),w44));
 wmat51  = diagrv((eye(T),w51));
 wmat52  = diagrv((eye(T),w52));
 wmat53  = diagrv((eye(T),w53));
 wmat54  = diagrv((eye(T),w54));
 wmat61  = diagrv((eye(T),w61));
 wmat62  = diagrv((eye(T),w62));
 wmat63  = diagrv((eye(T),w63));
 wmat64  = diagrv((eye(T),w64));
 wmat71  = diagrv((eye(T),w71));
 wmat72  = diagrv((eye(T),w72));
 wmat73  = diagrv((eye(T),w73));
 wmat74  = diagrv((eye(T),w74));
 wmat81  = diagrv((eye(T),w81));
 wmat82  = diagrv((eye(T),w82));
 wmat83  = diagrv((eye(T),w83));
 wmat84  = diagrv((eye(T),w84));
 wmat91  = diagrv((eye(T),w91));
 wmat92  = diagrv((eye(T),w92));
 wmat93  = diagrv((eye(T),w93));
 wmat94  = diagrv((eye(T),w94));
 wmat101  = diagrv((eye(T),w101));
 wmat102  = diagrv((eye(T),w102));
 wmat103  = diagrv((eye(T),w103));
 wmat104  = diagrv((eye(T),w104));
 wmat111  = diagrv((eye(T),w111));
 wmat112  = diagrv((eye(T),w112));
 wmat113  = diagrv((eye(T),w113));
 wmat114  = diagrv((eye(T),w114));
 wmat121  = diagrv((eye(T),w121));
 wmat122  = diagrv((eye(T),w122));
 wmat123  = diagrv((eye(T),w123));
 wmat124  = diagrv((eye(T),w124));
 wmat131  = diagrv((eye(T),w131));
 wmat132  = diagrv((eye(T),w132));
 wmat133  = diagrv((eye(T),w133));
 wmat134  = diagrv((eye(T),w134));
 xx11  =  ones(T,1)~u01i~u02j~u03j~u1j ;
 xx12  =  ones(T,1)~u01j~u02i~u03j~u1j ;
 xx13  =  ones(T,1)~u01j~u02j~u03i~u1j ;
 xx14  =  ones(T,1)~u01j~u02j~u03j~u1i;
 xx21  =  ones(T,1)~u01i~u02j~u03j~u2j ;
 xx22  =  ones(T,1)~u01j~u02i~u03j~u2j ;
 xx23  =  ones(T,1)~u01j~u02j~u03i~u2j ;
 xx24  =  ones(T,1)~u01j~u02j~u03j~u2i;
 xx31  =  ones(T,1)~u01i~u02j~u03j~u3j ;
 xx32  =  ones(T,1)~u01j~u02i~u03j~u3j ;
 xx33  =  ones(T,1)~u01j~u02j~u03i~u3j ;
 xx34  =  ones(T,1)~u01j~u02j~u03j~u3i;
 xx41  =  ones(T,1)~u01i~u02j~u03j~u4j ;
 xx42  =  ones(T,1)~u01j~u02i~u03j~u4j ;
 xx43  =  ones(T,1)~u01j~u02j~u03i~u4j ;
 xx44  =  ones(T,1)~u01j~u02j~u03j~u4i;
 xx51  =  ones(T,1)~u01i~u02j~u03j~u5j ;
 xx52  =  ones(T,1)~u01j~u02i~u03j~u5j ;
 xx53  =  ones(T,1)~u01j~u02j~u03i~u5j ;
 xx54  =  ones(T,1)~u01j~u02j~u03j~u5i;
 xx61  =  ones(T,1)~u01i~u02j~u03j~u6j ;
 xx62  =  ones(T,1)~u01j~u02i~u03j~u6j ;
 xx63  =  ones(T,1)~u01j~u02j~u03i~u6j ;
 xx64  =  ones(T,1)~u01j~u02j~u03j~u6i;
 xx71  =  ones(T,1)~u01i~u02j~u03j~u7j ;
 xx72  =  ones(T,1)~u01j~u02i~u03j~u7j ;
 xx73  =  ones(T,1)~u01j~u02j~u03i~u7j ;
 xx74  =  ones(T,1)~u01j~u02j~u03j~u7i;
 xx81  =  ones(T,1)~u01i~u02j~u03j~u8j ;
 xx82  =  ones(T,1)~u01j~u02i~u03j~u8j ;
 xx83  =  ones(T,1)~u01j~u02j~u03i~u8j ;
 xx84  =  ones(T,1)~u01j~u02j~u03j~u8i;
 xx91  =  ones(T,1)~u01i~u02j~u03j~u9j ;
 xx92  =  ones(T,1)~u01j~u02i~u03j~u9j ;
 xx93  =  ones(T,1)~u01j~u02j~u03i~u9j ;
 xx94  =  ones(T,1)~u01j~u02j~u03j~u9i;
 xx101  =  ones(T,1)~u01i~u02j~u03j~u10j ;
 xx102  =  ones(T,1)~u01j~u02i~u03j~u10j ;
 xx103  =  ones(T,1)~u01j~u02j~u03i~u10j ;
 xx104  =  ones(T,1)~u01j~u02j~u03j~u10i;
 xx111  =  ones(T,1)~u01i~u02j~u03j~u11j ;
 xx112  =  ones(T,1)~u01j~u02i~u03j~u11j ;
 xx113  =  ones(T,1)~u01j~u02j~u03i~u11j ;
 xx114  =  ones(T,1)~u01j~u02j~u03j~u11i;
 xx121  =  ones(T,1)~u01i~u02j~u03j~u12j ;
 xx122  =  ones(T,1)~u01j~u02i~u03j~u12j ;
 xx123  =  ones(T,1)~u01j~u02j~u03i~u12j ;
 xx124  =  ones(T,1)~u01j~u02j~u03j~u12i;
 xx131  =  ones(T,1)~u01i~u02j~u03j~u13j ;
 xx132  =  ones(T,1)~u01j~u02i~u03j~u13j ;
 xx133  =  ones(T,1)~u01j~u02j~u03i~u13j ;
 xx134  =  ones(T,1)~u01j~u02j~u03j~u13i;
 /*xx11  =  ones(T,1);
 xx12  =  ones(T,1);
 xx13  =  ones(T,1);
 xx14  =  ones(T,1);
 xx21  =  ones(T,1);
 xx22  =  ones(T,1);
 xx23  =  ones(T,1);
 xx24  =  ones(T,1);
 xx31  =  ones(T,1);
 xx32  =  ones(T,1);
 xx33  =  ones(T,1);
 xx34  =  ones(T,1);
 xx41  =  ones(T,1);
 xx42  =  ones(T,1);
 xx43  =  ones(T,1);
 xx44  =  ones(T,1);
 xx51  =  ones(T,1);
 xx52  =  ones(T,1);
 xx53  =  ones(T,1);
 xx54  =  ones(T,1);
 xx61  =  ones(T,1);
 xx62  =  ones(T,1);
 xx63  =  ones(T,1);
 xx64  =  ones(T,1);
 xx71  =  ones(T,1) ;
 xx72  =  ones(T,1);
 xx73  =  ones(T,1);
 xx74  =  ones(T,1);
 xx81  =  ones(T,1);
 xx82  =  ones(T,1);
 xx83  =  ones(T,1);
 xx84  =  ones(T,1);
 xx91  =  ones(T,1);
 xx92  =  ones(T,1) ;
 xx93  =  ones(T,1);
 xx94  =  ones(T,1);
 xx101  =  ones(T,1);
 xx102  =  ones(T,1);
 xx103  =  ones(T,1);
 xx104  =  ones(T,1);
 xx111  =  ones(T,1);
 xx112  =  ones(T,1);
 xx113  =  ones(T,1) ;
 xx114  =  ones(T,1);
 xx121  =  ones(T,1);
 xx122  =  ones(T,1);
 xx123  =  ones(T,1) ;
 xx124  =  ones(T,1);
 xx131  =  ones(T,1) ;
 xx132  =  ones(T,1) ;
 xx133  =  ones(T,1);
 xx134  =  ones(T,1);*/

if sumc(w11)> 0;
if det(xx11'wmat11*xx11) < epsln ;
      m4[j,1]  =  w11'y/sumc(w11) ;
else;
 beta =  pinv(xx11'wmat11*xx11)*xx11'wmat11'y ;
 m4[j,1]  =  beta[1,.] ;
endif;
else;
m4[j,1] = 0;
a11 = a11+1;
endif;

if sumc(w12)> 0;
if det(xx12'wmat12*xx12) < epsln ;
      m4[j,2]  =  w12'y/sumc(w12) ;
else;
 beta =  pinv(xx12'wmat12*xx12)*xx12'wmat12'y ;
 m4[j,2]  =  beta[1,.] ;
endif;
else;
m4[j,2] = 0;
a12 = a12+1;
endif;

if sumc(w13)> 0;
if det(xx13'wmat13*xx13) < epsln ;
      m4[j,3]  =  w13'y/sumc(w13) ;
else;
 beta =  pinv(xx13'wmat13*xx13)*xx13'wmat13'y ;
 m4[j,3]  =  beta[1,.] ;
endif;
else;
m4[j,3] = 0;
a13 = a13+1;
endif;

if sumc(w14)> 0;
if det(xx14'wmat14*xx14) < epsln ;
      m4[j,4]  =  w13'y/sumc(w14) ;
else;
 beta =  pinv(xx14'wmat14*xx14)*xx14'wmat14'y ;
 m4[j,4]  =  beta[1,.] ;
endif;
else;
m4[j,4] = 0;
a14 = a14+1;
endif;

if sumc(w21)> 0;
if det(xx21'wmat21*xx21) < epsln ;
      m4[j,5]  =  w21'y/sumc(w21) ;
else;
 beta =  pinv(xx21'wmat21*xx21)*xx21'wmat21'y ;
 m4[j,5]  =  beta[1,.] ;
endif;
else;
m4[j,5] = 0;
a21 = a21+1;
endif;

if sumc(w22)> 0;
if det(xx22'wmat22*xx22) < epsln ;
      m4[j,6]  =  w22'y/sumc(w22) ;
else;
 beta =  pinv(xx22'wmat22*xx22)*xx22'wmat22'y ;
 m4[j,6]  =  beta[1,.] ;
endif;
else;
m4[j,6] = 0;
a22 = a22+1;
endif;

if sumc(w23)> 0;
if det(xx23'wmat23*xx23) < epsln ;
      m4[j,7]  =  w23'y/sumc(w23) ;
else;
 beta =  pinv(xx23'wmat23*xx23)*xx23'wmat23'y ;
 m4[j,7]  =  beta[1,.] ;
endif;
else;
m4[j,7] = 0;
a23 = a23+1;
endif;

if sumc(w24)> 0;
if det(xx24'wmat24*xx24) < epsln ;
      m4[j,8]  =  w24'y/sumc(w24) ;
else;
 beta =  pinv(xx24'wmat24*xx24)*xx24'wmat24'y ;
 m4[j,8]  =  beta[1,.] ;
endif;
else;
m4[j,8] = 0;
a24 = a24+1;
endif;

if sumc(w31)> 0;
if det(xx31'wmat31*xx31) < epsln ;
      m4[j,9]  =  w31'y/sumc(w31) ;
else;
 beta =  pinv(xx31'wmat31*xx31)*xx31'wmat31'y ;
 m4[j,9]  =  beta[1,.] ;
endif;
else;
m4[j,9] = 0;
a31 = a31+1;
endif;

if sumc(w32)> 0;
if det(xx32'wmat32*xx32) < epsln ;
      m4[j,10]  =  w32'y/sumc(w32) ;
else;
 beta =  pinv(xx32'wmat32*xx32)*xx32'wmat32'y ;
 m4[j,10]  =  beta[1,.] ;
endif;
else;
m4[j,10] = 0;
a32 = a32+1;
endif;

if sumc(w33)> 0;
if det(xx33'wmat33*xx33) < epsln ;
      m4[j,11]  =  w33'y/sumc(w33) ;
else;
 beta =  pinv(xx33'wmat33*xx33)*xx33'wmat33'y ;
 m4[j,11]  =  beta[1,.] ;
endif;
else;
m4[j,11] = 0;
a33 = a33+1;
endif;

if sumc(w34)> 0;
if det(xx34'wmat34*xx34) < epsln ;
      m4[j,12]  =  w34'y/sumc(w34) ;
else;
 beta =  pinv(xx34'wmat34*xx34)*xx34'wmat34'y ;
 m4[j,12]  =  beta[1,.] ;
endif;
else;
m4[j,12] = 0;
a34 = a34+1;
endif;

if sumc(w41)> 0;
if det(xx41'wmat41*xx41) < epsln ;
      m4[j,13]  =  w41'y/sumc(w41) ;
else;
 beta =  pinv(xx41'wmat41*xx41)*xx41'wmat41'y ;
 m4[j,13]  =  beta[1,.] ;
endif;
else;
m4[j,13] = 0;
a41 = a41+1;
endif;

if sumc(w42)> 0;
if det(xx42'wmat42*xx42) < epsln ;
      m4[j,14]  =  w42'y/sumc(w42) ;
else;
 beta =  pinv(xx42'wmat42*xx42)*xx42'wmat42'y ;
 m4[j,14]  =  beta[1,.] ;
endif;
else;
m4[j,14] = 0;
a42 = a42+1;
endif;

if sumc(w43)> 0;
if det(xx43'wmat43*xx43) < epsln ;
      m4[j,15]  =  w43'y/sumc(w43) ;
else;
 beta =  pinv(xx43'wmat43*xx43)*xx43'wmat43'y ;
 m4[j,15]  =  beta[1,.] ;
endif;
else;
m4[j,15] = 0;
a43 = a43+1;
endif;

if sumc(w44)> 0;
if det(xx44'wmat44*xx44) < epsln ;
      m4[j,16]  =  w44'y/sumc(w44) ;
else;
 beta =  pinv(xx44'wmat44*xx44)*xx44'wmat44'y ;
 m4[j,16]  =  beta[1,.] ;
endif;
else;
m4[j,16] = 0;
a44 = a44+1;
endif;

if sumc(w51)> 0;
if det(xx51'wmat51*xx51) < epsln ;
      m4[j,17]  =  w51'y/sumc(w51) ;
else;
 beta =  pinv(xx51'wmat51*xx51)*xx51'wmat51'y ;
 m4[j,17]  =  beta[1,.] ;
endif;
else;
m4[j,17] = 0;
a51 = a51+1;
endif;

if sumc(w52)> 0;
if det(xx52'wmat52*xx52) < epsln ;
      m4[j,18]  =  w52'y/sumc(w52) ;
else;
 beta =  pinv(xx52'wmat52*xx52)*xx52'wmat52'y ;
 m4[j,18]  =  beta[1,.] ;
endif;
else;
m4[j,18] = 0;
a52 = a52+1;
endif;

if sumc(w53)> 0;
if det(xx53'wmat53*xx53) < epsln ;
      m4[j,19]  =  w53'y/sumc(w53) ;
else;
 beta =  pinv(xx53'wmat53*xx53)*xx53'wmat53'y ;
 m4[j,19]  =  beta[1,.] ;
endif;
else;
m4[j,19] = 0;
a53 = a53+1;
endif;

if sumc(w54)> 0;
if det(xx54'wmat54*xx54) < epsln ;
      m4[j,20]  =  w54'y/sumc(w54) ;
else;
 beta =  pinv(xx54'wmat54*xx54)*xx54'wmat54'y ;
 m4[j,20]  =  beta[1,.] ;
endif;
else;
m4[j,20] = 0;
a54 = a54+1;
endif;

if sumc(w61)> 0;
if det(xx61'wmat61*xx61) < epsln ;
      m4[j,21]  =  w61'y/sumc(w61) ;
else;
 beta =  pinv(xx61'wmat61*xx61)*xx61'wmat61'y ;
 m4[j,21]  =  beta[1,.] ;
endif;
else;
m4[j,21] = 0;
a61 = a61+1;
endif;

if sumc(w62)> 0;
if det(xx62'wmat62*xx62) < epsln ;
      m4[j,22]  =  w62'y/sumc(w62) ;
else;
 beta =  pinv(xx62'wmat62*xx62)*xx62'wmat62'y ;
 m4[j,22]  =  beta[1,.] ;
endif;
else;
m4[j,22] = 0;
a62 = a62+1;
endif;

if sumc(w63)> 0;
if det(xx63'wmat63*xx63) < epsln ;
      m4[j,23]  =  w63'y/sumc(w63) ;
else;
 beta =  pinv(xx63'wmat63*xx63)*xx63'wmat63'y ;
 m4[j,23]  =  beta[1,.] ;
endif;
else;
m4[j,23] = 0;
a63 = a63+1;
endif;

if sumc(w64)> 0;
if det(xx64'wmat64*xx64) < epsln ;
      m4[j,24]  =  w64'y/sumc(w64) ;
else;
 beta =  pinv(xx64'wmat64*xx64)*xx64'wmat64'y ;
 m4[j,24]  =  beta[1,.] ;
endif;
else;
m4[j,24] = 0;
a64 = a64+1;
endif;

if sumc(w71)> 0;
if det(xx71'wmat71*xx71) < epsln ;
      m4[j,25]  =  w71'y/sumc(w71) ;
else;
 beta =  pinv(xx71'wmat71*xx71)*xx71'wmat71'y ;
 m4[j,25]  =  beta[1,.] ;
endif;
else;
m4[j,25] = 0;
a71 = a71+1;
endif;

if sumc(w72)> 0;
if det(xx72'wmat72*xx72) < epsln ;
      m4[j,20]  =  w72'y/sumc(w72) ;
else;
 beta =  pinv(xx72'wmat72*xx72)*xx72'wmat72'y ;
 m4[j,26]  =  beta[1,.] ;
endif;
else;
m4[j,26] = 0;
a72 = a72+1;
endif;

if sumc(w73)> 0;
if det(xx73'wmat73*xx73) < epsln ;
      m4[j,27]  =  w73'y/sumc(w73) ;
else;
 beta =  pinv(xx73'wmat73*xx73)*xx73'wmat73'y ;
 m4[j,27]  =  beta[1,.] ;
endif;
else;
m4[j,27] = 0;
a73 = a73+1;
endif;

if sumc(w74)> 0;
if det(xx74'wmat74*xx74) < epsln ;
      m4[j,28]  =  w74'y/sumc(w74) ;
else;
 beta =  pinv(xx74'wmat74*xx74)*xx74'wmat74'y ;
 m4[j,28]  =  beta[1,.] ;
endif;
else;
m4[j,28] = 0;
a74 = a74+1;
endif;

if sumc(w81)> 0;
if det(xx81'wmat81*xx81) < epsln ;
      m4[j,29]  =  w81'y/sumc(w81) ;
else;
 beta =  pinv(xx81'wmat81*xx81)*xx81'wmat81'y ;
 m4[j,29]  =  beta[1,.] ;
endif;
else;
m4[j,29] = 0;
a81 = a81+1;
endif;

if sumc(w82)> 0;
if det(xx82'wmat82*xx82) < epsln ;
      m4[j,30]  =  w82'y/sumc(w82) ;
else;
 beta =  pinv(xx82'wmat82*xx82)*xx82'wmat82'y ;
 m4[j,30]  =  beta[1,.] ;
endif;
else;
m4[j,30] = 0;
a82 = a82+1;
endif;

if sumc(w83)> 0;
if det(xx83'wmat83*xx83) < epsln ;
      m4[j,31]  =  w83'y/sumc(w83) ;
else;
 beta =  pinv(xx83'wmat83*xx83)*xx83'wmat83'y ;
 m4[j,31]  =  beta[1,.] ;
endif;
else;
m4[j,31] = 0;
a83 = a83+1;
endif;

if sumc(w84)> 0;
if det(xx84'wmat84*xx84) < epsln ;
      m4[j,32]  =  w84'y/sumc(w84) ;
else;
 beta =  pinv(xx84'wmat84*xx84)*xx84'wmat84'y ;
 m4[j,32]  =  beta[1,.] ;
endif;
else;
m4[j,32] = 0;
a84 = a84+1;
endif;

if sumc(w91)> 0;
if det(xx91'wmat91*xx91) < epsln ;
      m4[j,33]  =  w91'y/sumc(w91) ;
else;
 beta =  pinv(xx91'wmat91*xx91)*xx91'wmat91'y ;
 m4[j,33]  =  beta[1,.] ;
endif;
else;
m4[j,33] = 0;
a91 = a91+1;
endif;

if sumc(w92)> 0;
if det(xx92'wmat92*xx92) < epsln ;
      m4[j,34]  =  w92'y/sumc(w92) ;
else;
 beta =  pinv(xx92'wmat92*xx92)*xx92'wmat92'y ;
 m4[j,34]  =  beta[1,.] ;
endif;
else;
m4[j,34] = 0;
a92 = a92+1;
endif;

if sumc(w93)> 0;
if det(xx93'wmat93*xx93) < epsln ;
      m4[j,35]  =  w93'y/sumc(w93) ;
else;
 beta =  pinv(xx93'wmat93*xx93)*xx93'wmat93'y ;
 m4[j,35]  =  beta[1,.] ;
endif;
else;
m4[j,35] = 0;
a93 = a93+1;
endif;

if sumc(w94)> 0;
if det(xx94'wmat94*xx94) < epsln ;
      m4[j,36]  =  w94'y/sumc(w94) ;
else;
 beta =  pinv(xx94'wmat94*xx94)*xx94'wmat94'y ;
 m4[j,36]  =  beta[1,.] ;
endif;
else;
m4[j,36] = 0;
a94 = a94+1;
endif;

if sumc(w101)> 0;
if det(xx101'wmat101*xx101) < epsln ;
      m4[j,37]  =  w101'y/sumc(w101) ;
else;
 beta =  pinv(xx101'wmat101*xx101)*xx101'wmat101'y ;
 m4[j,37]  =  beta[1,.] ;
endif;
else;
m4[j,37] = 0;
a101 = a101+1;
endif;

if sumc(w102)> 0;
if det(xx102'wmat102*xx102) < epsln ;
      m4[j,38]  =  w102'y/sumc(w102) ;
else;
 beta =  pinv(xx102'wmat102*xx102)*xx102'wmat102'y ;
 m4[j,38]  =  beta[1,.] ;
endif;
else;
m4[j,38] = 0;
a102 = a102+1;
endif;

if sumc(w103)> 0;
if det(xx103'wmat103*xx103) < epsln ;
      m4[j,39]  =  w103'y/sumc(w103) ;
else;
 beta =  pinv(xx103'wmat103*xx103)*xx103'wmat103'y ;
 m4[j,39]  =  beta[1,.] ;
endif;
else;
m4[j,39] = 0;
a103 = a103+1;
endif;

if sumc(w104)> 0;
if det(xx104'wmat104*xx104) < epsln ;
      m4[j,40]  =  w104'y/sumc(w104) ;
else;
 beta =  pinv(xx104'wmat104*xx104)*xx104'wmat104'y ;
 m4[j,40]  =  beta[1,.] ;
endif;
else;
m4[j,40] = 0;
a104 = a104+1;
endif;


if sumc(w111)> 0;
if det(xx111'wmat111*xx111) < epsln ;
      m4[j,41]  =  w111'y/sumc(w111) ;
else;
 beta =  pinv(xx111'wmat111*xx111)*xx111'wmat111'y ;
 m4[j,41]  =  beta[1,.] ;
endif;
else;
m4[j,41] = 0;
a111 = a111+1;
endif;

if sumc(w112)> 0;
if det(xx112'wmat112*xx112) < epsln ;
      m4[j,42]  =  w112'y/sumc(w112) ;
else;
 beta =  pinv(xx112'wmat112*xx112)*xx112'wmat112'y ;
 m4[j,42]  =  beta[1,.] ;
endif;
else;
m4[j,42] = 0;
a112 = a112+1;
endif;

if sumc(w113)> 0;
if det(xx113'wmat113*xx113) < epsln ;
      m4[j,43]  =  w113'y/sumc(w113) ;
else;
 beta =  pinv(xx113'wmat113*xx113)*xx113'wmat113'y ;
 m4[j,43]  =  beta[1,.] ;
endif;
else;
m4[j,43] = 0;
a113 = a113+1;
endif;

if sumc(w114)> 0;
if det(xx114'wmat114*xx114) < epsln ;
      m4[j,44]  =  w114'y/sumc(w114) ;
else;
 beta =  pinv(xx114'wmat114*xx114)*xx114'wmat114'y ;
 m4[j,44]  =  beta[1,.] ;
endif;
else;
m4[j,44] = 0;
a114 = a114+1;
endif;

if sumc(w121)> 0;
if det(xx121'wmat121*xx121) < epsln ;
      m4[j,45]  =  w121'y/sumc(w121) ;
else;
 beta =  pinv(xx121'wmat121*xx121)*xx121'wmat121'y ;
 m4[j,45]  =  beta[1,.] ;
endif;
else;
m4[j,45] = 0;
a121 = a121+1;
endif;

if sumc(w122)> 0;
if det(xx122'wmat122*xx122) < epsln ;
      m4[j,46]  =  w122'y/sumc(w122) ;
else;
 beta =  pinv(xx122'wmat122*xx122)*xx122'wmat122'y ;
 m4[j,46]  =  beta[1,.] ;
endif;
else;
m4[j,46] = 0;
a122 = a122+1;
endif;

if sumc(w123)> 0;
if det(xx123'wmat123*xx123) < epsln ;
      m4[j,47]  =  w123'y/sumc(w123) ;
else;
 beta =  pinv(xx123'wmat123*xx123)*xx123'wmat123'y ;
 m4[j,47]  =  beta[1,.] ;
endif;
else;
m4[j,47] = 0;
a123 = a123+1;
endif;

if sumc(w124)> 0;
if det(xx124'wmat124*xx124) < epsln ;
      m4[j,48]  =  w124'y/sumc(w124) ;
else;
 beta =  pinv(xx124'wmat124*xx124)*xx124'wmat124'y ;
 m4[j,48]  =  beta[1,.] ;
endif;
else;
m4[j,48] = 0;
a124 = a124+1;
endif;

if sumc(w131)> 0;
if det(xx131'wmat131*xx131) < epsln ;
      m4[j,49]  =  w131'y/sumc(w131) ;
else;
 beta =  pinv(xx131'wmat131*xx131)*xx131'wmat131'y ;
 m4[j,49]  =  beta[1,.] ;
endif;
else;
m4[j,49] = 0;
a131 = a131+1;
endif;

if sumc(w132)> 0;
if det(xx132'wmat132*xx132) < epsln ;
      m4[j,50]  =  w132'y/sumc(w132) ;
else;
 beta =  pinv(xx132'wmat132*xx132)*xx132'wmat132'y ;
 m4[j,50]  =  beta[1,.] ;
endif;
else;
m4[j,50] = 0;
a132 = a132+1;
endif;

if sumc(w133)> 0;
if det(xx133'wmat133*xx133) < epsln ;
      m4[j,51]  =  w133'y/sumc(w133) ;
else;
 beta =  pinv(xx133'wmat133*xx133)*xx133'wmat133'y ;
 m4[j,51]  =  beta[1,.] ;
endif;
else;
m4[j,51] = 0;
a133 = a133+1;
endif;

if sumc(w134)> 0;
if det(xx134'wmat134*xx134) < epsln ;
      m4[j,52]  =  w134'y/sumc(w134) ;
else;
 beta =  pinv(xx134'wmat134*xx134)*xx134'wmat134'y ;
 m4[j,52]  =  beta[1,.] ;
endif;
else;
m4[j,52] = 0;
a134 = a134+1;
endif;
j = j+1;
endo;

m4d[i,1] = 1/(T-a11)*sumc(m4[.,1]);
m4d[i,2] = 1/(T-a12)*sumc(m4[.,2]);
m4d[i,3] = 1/(T-a13)*sumc(m4[.,3]);
m4d[i,4] = 1/(T-a14)*sumc(m4[.,4]);
m4d[i,5] = 1/(T-a21)*sumc(m4[.,5]);
m4d[i,6] = 1/(T-a22)*sumc(m4[.,6]);
m4d[i,7] = 1/(T-a23)*sumc(m4[.,7]);
m4d[i,8] = 1/(T-a24)*sumc(m4[.,8]);
m4d[i,9] = 1/(T-a31)*sumc(m4[.,9]);
m4d[i,10] = 1/(T-a32)*sumc(m4[.,10]);
m4d[i,11] = 1/(T-a33)*sumc(m4[.,11]);
m4d[i,12] = 1/(T-a34)*sumc(m4[.,12]);
m4d[i,13] = 1/(T-a41)*sumc(m4[.,13]);
m4d[i,14] = 1/(T-a42)*sumc(m4[.,14]);
m4d[i,15] = 1/(T-a43)*sumc(m4[.,15]);
m4d[i,16] = 1/(T-a44)*sumc(m4[.,16]);
m4d[i,17] = 1/(T-a51)*sumc(m4[.,17]);
m4d[i,18] = 1/(T-a52)*sumc(m4[.,18]);
m4d[i,19] = 1/(T-a53)*sumc(m4[.,19]);
m4d[i,20] = 1/(T-a54)*sumc(m4[.,20]);
m4d[i,21] = 1/(T-a61)*sumc(m4[.,21]);
m4d[i,22] = 1/(T-a62)*sumc(m4[.,22]);
m4d[i,23] = 1/(T-a63)*sumc(m4[.,23]);
m4d[i,24] = 1/(T-a64)*sumc(m4[.,24]);
m4d[i,25] = 1/(T-a71)*sumc(m4[.,25]);
m4d[i,26] = 1/(T-a72)*sumc(m4[.,26]);
m4d[i,27] = 1/(T-a73)*sumc(m4[.,27]);
m4d[i,28] = 1/(T-a73)*sumc(m4[.,28]);
m4d[i,29] = 1/(T-a81)*sumc(m4[.,29]);
m4d[i,30] = 1/(T-a82)*sumc(m4[.,30]);
m4d[i,31] = 1/(T-a83)*sumc(m4[.,31]);
m4d[i,32] = 1/(T-a84)*sumc(m4[.,32]);
m4d[i,33] = 1/(T-a91)*sumc(m4[.,33]);
m4d[i,34] = 1/(T-a92)*sumc(m4[.,34]);
m4d[i,35] = 1/(T-a93)*sumc(m4[.,35]);
m4d[i,36] = 1/(T-a94)*sumc(m4[.,36]);
m4d[i,37] = 1/(T-a101)*sumc(m4[.,37]);
m4d[i,38] = 1/(T-a102)*sumc(m4[.,38]);
m4d[i,39] = 1/(T-a103)*sumc(m4[.,39]);
m4d[i,40] = 1/(T-a104)*sumc(m4[.,40]);
m4d[i,41] = 1/(T-a111)*sumc(m4[.,41]);
m4d[i,42] = 1/(T-a112)*sumc(m4[.,42]);
m4d[i,43] = 1/(T-a113)*sumc(m4[.,43]);
m4d[i,44] = 1/(T-a114)*sumc(m4[.,44]);
m4d[i,45] = 1/(T-a121)*sumc(m4[.,45]);
m4d[i,46] = 1/(T-a122)*sumc(m4[.,46]);
m4d[i,47] = 1/(T-a123)*sumc(m4[.,47]);
m4d[i,48] = 1/(T-a124)*sumc(m4[.,48]);
m4d[i,49] = 1/(T-a131)*sumc(m4[.,49]);
m4d[i,50] = 1/(T-a132)*sumc(m4[.,50]);
m4d[i,51] = 1/(T-a133)*sumc(m4[.,51]);
m4d[i,52] = 1/(T-a134)*sumc(m4[.,52]);
i = i+1;
endo;
yhat1 = y-meanc(y)-(m4d[.,1]-meanc(m4d[.,1])+m4d[.,2]-meanc(m4d[.,2])+m4d[.,3]-meanc(m4d[.,3])+m4d[.,4]-meanc(m4d[.,4]));
yhat2 = y-meanc(y)-(m4d[.,5]-meanc(m4d[.,5])+m4d[.,6]-meanc(m4d[.,6])+m4d[.,7]-meanc(m4d[.,7])+m4d[.,8]-meanc(m4d[.,8]));
yhat3 = y-meanc(y)-(m4d[.,9]-meanc(m4d[.,9])+m4d[.,10]-meanc(m4d[.,10])+m4d[.,11]-meanc(m4d[.,11])+m4d[.,12]-meanc(m4d[.,12]));
yhat4 = y-meanc(y)-(m4d[.,13]-meanc(m4d[.,13])+m4d[.,14]-meanc(m4d[.,14])+m4d[.,15]-meanc(m4d[.,15])+m4d[.,16]-meanc(m4d[.,16]));
yhat5 = y-meanc(y)-(m4d[.,17]-meanc(m4d[.,17])+m4d[.,18]-meanc(m4d[.,18])+m4d[.,19]-meanc(m4d[.,19])+m4d[.,20]-meanc(m4d[.,20]));
yhat6 = y-meanc(y)-(m4d[.,21]-meanc(m4d[.,21])+m4d[.,22]-meanc(m4d[.,22])+m4d[.,23]-meanc(m4d[.,23])+m4d[.,24]-meanc(m4d[.,24]));
yhat7 = y-meanc(y)-(m4d[.,25]-meanc(m4d[.,25])+m4d[.,26]-meanc(m4d[.,26])+m4d[.,27]-meanc(m4d[.,27])+m4d[.,28]-meanc(m4d[.,28]));
yhat8 = y-meanc(y)-(m4d[.,29]-meanc(m4d[.,29])+m4d[.,30]-meanc(m4d[.,30])+m4d[.,31]-meanc(m4d[.,31])+m4d[.,32]-meanc(m4d[.,32]));
yhat9 = y-meanc(y)-(m4d[.,33]-meanc(m4d[.,33])+m4d[.,34]-meanc(m4d[.,34])+m4d[.,35]-meanc(m4d[.,35])+m4d[.,36]-meanc(m4d[.,36]));
yhat10 = y-meanc(y)-(m4d[.,37]-meanc(m4d[.,37])+m4d[.,38]-meanc(m4d[.,38])+m4d[.,39]-meanc(m4d[.,39])+m4d[.,40]-meanc(m4d[.,40]));
yhat11 = y-meanc(y)-(m4d[.,41]-meanc(m4d[.,41])+m4d[.,42]-meanc(m4d[.,42])+m4d[.,43]-meanc(m4d[.,43])+m4d[.,44]-meanc(m4d[.,44]));
yhat12 = y-meanc(y)-(m4d[.,45]-meanc(m4d[.,45])+m4d[.,46]-meanc(m4d[.,46])+m4d[.,47]-meanc(m4d[.,47])+m4d[.,48]-meanc(m4d[.,48]));
yhat13 = y-meanc(y)-(m4d[.,49]-meanc(m4d[.,49])+m4d[.,50]-meanc(m4d[.,50])+m4d[.,51]-meanc(m4d[.,51])+m4d[.,52]-meanc(m4d[.,52]));
R1 = yhat1'yhat1;
R2 = yhat2'yhat2;
R3 = yhat3'yhat3;
R4 = yhat4'yhat4;
R5 = yhat5'yhat5;
R6 = yhat6'yhat6;
R7 = yhat7'yhat7;
R8 = yhat8'yhat8;
R9 = yhat9'yhat9;
R10 = yhat10'yhat10;
R11 = yhat11'yhat11;
R12 = yhat12'yhat12;
R13 = yhat13'yhat13;
Ahat1[k,kc] = 1/T*sumc(R1);
Ahat2[k,kc] = 1/T*sumc(R2);
Ahat3[k,kc] = 1/T*sumc(R3);
Ahat4[k,kc] = 1/T*sumc(R4);
Ahat5[k,kc] = 1/T*sumc(R5);
Ahat6[k,kc] = 1/T*sumc(R6);
Ahat7[k,kc] = 1/T*sumc(R7);
Ahat8[k,kc] = 1/T*sumc(R8);
Ahat9[k,kc] = 1/T*sumc(R9);
Ahat10[k,kc] = 1/T*sumc(R10);
Ahat11[k,kc] = 1/T*sumc(R11);
Ahat12[k,kc] = 1/T*sumc(R12);
Ahat13[k,kc] = 1/T*sumc(R13);
Bhat1[k,kc]  =  2/b4*1/sqrt(2*pi)*1/T*sumc((BR1.*qz4[.,1]+BR1.*qz4[.,2]+BR1.*qz4[.,3]+BR1.*qz4[.,4])./phat[.,1])*1/T;
Bhat2[k,kc]  =  2/b4*1/sqrt(2*pi)*1/T*sumc((BR2.*qz4[.,1]+BR2.*qz4[.,5]+BR2.*qz4[.,6]+BR2.*qz4[.,7])./phat[.,2])*1/T;
Bhat3[k,kc]  =  2/b4*1/sqrt(2*pi)*1/T*sumc((BR3.*qz4[.,1]+BR3.*qz4[.,8]+BR3.*qz4[.,9]+BR3.*qz4[.,10])./phat[.,3])*1/T;
Bhat4[k,kc]  =  2/b4*1/sqrt(2*pi)*1/T*sumc((BR4.*qz4[.,1]+BR4.*qz4[.,11]+BR4.*qz4[.,12]+BR4.*qz4[.,13])./phat[.,4])*1/T;
Bhat5[k,kc]  =  2/b4*1/sqrt(2*pi)*1/T*sumc((BR5.*qz4[.,1]+BR5.*qz4[.,14]+BR5.*qz4[.,15]+BR5.*qz4[.,16])./phat[.,5])*1/T;
Bhat6[k,kc]  =  2/b4*1/sqrt(2*pi)*1/T*sumc((BR6.*qz4[.,1]+BR6.*qz4[.,17]+BR6.*qz4[.,18]+BR6.*qz4[.,19])./phat[.,6])*1/T;
Bhat7[k,kc]  =  2/b4*1/sqrt(2*pi)*1/T*sumc((BR7.*qz4[.,1]+BR7.*qz4[.,20]+BR7.*qz4[.,21]+BR7.*qz4[.,22])./phat[.,7])*1/T;
Bhat8[k,kc]  =  2/b4*1/sqrt(2*pi)*1/T*sumc((BR8.*qz4[.,1]+BR8.*qz4[.,23]+BR8.*qz4[.,24]+BR8.*qz4[.,25])./phat[.,8])*1/T;
Bhat9[k,kc]  =  2/b4*1/sqrt(2*pi)*1/T*sumc((BR9.*qz4[.,1]+BR9.*qz4[.,26]+BR9.*qz4[.,27]+BR9.*qz4[.,28])./phat[.,9])*1/T;
Bhat10[k,kc]  =  2/b4*1/sqrt(2*pi)*1/T*sumc((BR10.*qz4[.,1]+BR10.*qz4[.,29]+BR10.*qz4[.,30]+BR10.*qz4[.,31])./phat[.,10])*1/T;
Bhat11[k,kc]  =  2/b4*1/sqrt(2*pi)*1/T*sumc((BR11.*qz4[.,1]+BR11.*qz4[.,32]+BR11.*qz4[.,33]+BR11.*qz4[.,34])./phat[.,11])*1/T;
Bhat12[k,kc]  =  2/b4*1/sqrt(2*pi)*1/T*sumc((BR12.*qz4[.,1]+BR12.*qz4[.,35]+BR12.*qz4[.,36]+BR12.*qz4[.,37])./phat[.,12])*1/T;
Bhat13[k,kc]  =  2/b4*1/sqrt(2*pi)*1/T*sumc((BR13.*qz4[.,1]+BR13.*qz4[.,38]+BR13.*qz4[.,39]+BR13.*qz4[.,40])./phat[.,13])*1/T;
FPE1[k,kc] = Ahat1[k,kc]+Bhat1[k,kc];
FPE2[k,kc] = Ahat2[k,kc]+Bhat2[k,kc];
FPE3[k,kc] = Ahat3[k,kc]+Bhat3[k,kc];
FPE4[k,kc] = Ahat4[k,kc]+Bhat4[k,kc];
FPE5[k,kc] = Ahat5[k,kc]+Bhat5[k,kc];
FPE6[k,kc] = Ahat6[k,kc]+Bhat6[k,kc];
FPE7[k,kc] = Ahat7[k,kc]+Bhat7[k,kc];
FPE8[k,kc] = Ahat8[k,kc]+Bhat8[k,kc];
FPE9[k,kc] = Ahat9[k,kc]+Bhat9[k,kc];
FPE10[k,kc] = Ahat10[k,kc]+Bhat10[k,kc];
FPE11[k,kc] = Ahat11[k,kc]+Bhat11[k,kc];
FPE12[k,kc] = Ahat12[k,kc]+Bhat12[k,kc];
FPE13[k,kc] = Ahat13[k,kc]+Bhat13[k,kc];
kc = kc+1;
endo;
cmin1 = minc(FPE1[k,.]');
cmin2 = minc(FPE2[k,.]');
cmin3 = minc(FPE3[k,.]');
cmin4 = minc(FPE4[k,.]');
cmin5 = minc(FPE5[k,.]');
cmin6 = minc(FPE6[k,.]');
cmin7 = minc(FPE7[k,.]');
cmin8 = minc(FPE8[k,.]');
cmin9 = minc(FPE9[k,.]');
cmin10 = minc(FPE10[k,.]');
cmin11 = minc(FPE11[k,.]');
cmin12 = minc(FPE12[k,.]');
cmin13 = minc(FPE13[k,.]');
minFPE = cmin1~cmin2~cmin3~cmin4~cmin5~cmin6~cmin7~cmin8~cmin9~cmin10~cmin11~cmin12~cmin13;
minFPE[.,lag_1] = 10^10;
minFPE[.,lag_2] = 10^10;
minFPE[.,lag_3] = 10^10;
AFPE[k,4] = minc(minFPE');
lag[k,4] = minindc(minFPE');
lag_4 = lag[k,4];


/****************************Five Lags Included ******************************************/
       
/****************************Five Lags Included Estimation of B part *******************************************/

i = 1;
a11 = 0;
a12 = 0;
a13 = 0;
a14 = 0;
a15 = 0;
a21 = 0;
a22 = 0;
a23 = 0;
a24 = 0;
a25 = 0;
a31 = 0;
a32 = 0;
a33 = 0;
a34 = 0;
a35 = 0;
a41 = 0;
a42 = 0;
a43 = 0;
a44 = 0;
a45 = 0;
a51 = 0;
a52 = 0;
a53 = 0;
a54 = 0;
a55 = 0;
a61 = 0;
a62 = 0;
a63 = 0;
a64 = 0;
a65 = 0;
a71 = 0;
a72 = 0;
a73 = 0;
a74 = 0;
a75 = 0;
a81 = 0;
a82 = 0;
a83 = 0;
a84 = 0;
a85 = 0;
a91 = 0;
a92 = 0;
a93 = 0;
a94 = 0;
a95 = 0;
a101 = 0;
a102 = 0;
a103 = 0;
a104 = 0;
a105 = 0;
a111 = 0;
a112 = 0;
a113 = 0;
a114 = 0;
a115 = 0;
a121 = 0;
a122 = 0;
a123 = 0;
a124 = 0;
a125 = 0;
a131 = 0;
a132 = 0;
a133 = 0;
a134 = 0;
a135 = 0;
m5 = zeros(T,65);
m5d = zeros(T,65);
qz5 = zeros(T,65);
i = 1;
do until i>T;
u01i = data[i,lag_1]*ones(T,1) - data[.,lag_1];
u02i = data[i,lag_2]*ones(T,1) - data[.,lag_2];
u03i = data[i,lag_3]*ones(T,1) - data[.,lag_3];
u04i = data[i,lag_4]*ones(T,1) - data[.,lag_4];
u0i = u01i~u02i~u03i~u04i;
u1i =  data[i,1]*ones(T,1) - data[.,1];
u2i =  data[i,2]*ones(T,1) - data[.,2];
u3i =  data[i,3]*ones(T,1) - data[.,3];
u4i =  data[i,4]*ones(T,1) - data[.,4];
u5i =  data[i,5]*ones(T,1) - data[.,5];
u6i =  data[i,6]*ones(T,1) - data[.,6];
u7i =  data[i,7]*ones(T,1) - data[.,7];
u8i =  data[i,8]*ones(T,1) - data[.,8];
u9i =  data[i,9]*ones(T,1) - data[.,9];
u10i =  data[i,10]*ones(T,1) - data[.,10];
u11i =  data[i,11]*ones(T,1) - data[.,11];
u12i =  data[i,12]*ones(T,1) - data[.,12];
u13i =  data[i,13]*ones(T,1) - data[.,13];
j = 1;
do until j>T;
u01j = data[j,lag_1]*ones(T,1) - data[.,lag_1];
u02j = data[j,lag_2]*ones(T,1) - data[.,lag_2];
u03j = data[j,lag_3]*ones(T,1) - data[.,lag_3];
u04j = data[j,lag_4]*ones(T,1) - data[.,lag_4];
u0j = u01j~u02j~u03j~u04j;
 u1j =  (data[j,1]*ones(T,1) - data[.,1]);
 u2j =  (data[j,2]*ones(T,1) - data[.,2]);
 u3j =  (data[j,3]*ones(T,1) - data[.,3]);
 u4j =  (data[j,4]*ones(T,1) - data[.,4]);
 u5j =  (data[j,5]*ones(T,1) - data[.,5]);
 u6j =  (data[j,5]*ones(T,1) - data[.,6]);
 u7j =  (data[j,7]*ones(T,1) - data[.,7]);
 u8j =  (data[j,8]*ones(T,1) - data[.,8]);
 u9j =  (data[j,9]*ones(T,1) - data[.,9]);
 u10j =  (data[j,10]*ones(T,1) - data[.,10]);
 u11j =  (data[j,11]*ones(T,1) - data[.,11]);
 u12j =  (data[j,12]*ones(T,1) - data[.,12]);
 u13j =  (data[j,13]*ones(T,1) - data[.,13]);
 w11 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u1j/h1)/h1);
 w12 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u1j/h1)/h1);
 w13 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03i/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u1j/h1)/h1);
 w14 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04i/h1)/h1).*(fi(u1j/h1)/h1);
 w15 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u1i/h1)/h1);
 w21 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u2j/h1)/h1);
 w22 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u2j/h1)/h1);
 w23 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03i/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u2j/h1)/h1);
 w24 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04i/h1)/h1).*(fi(u2j/h1)/h1);
 w25 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u2i/h1)/h1);
 w31 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u3j/h1)/h1);
 w32 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u3j/h1)/h1);
 w33 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03i/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u3j/h1)/h1);
 w34 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04i/h1)/h1).*(fi(u3j/h1)/h1);
 w35 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u3i/h1)/h1);
 w41 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u4j/h1)/h1);
 w42 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u4j/h1)/h1);
 w43 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03i/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u4j/h1)/h1);
 w44 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04i/h1)/h1).*(fi(u4j/h1)/h1);
 w45 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u4i/h1)/h1);
 w51 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u5j/h1)/h1);
 w52 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u5j/h1)/h1);
 w53 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03i/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u5j/h1)/h1);
 w54 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04i/h1)/h1).*(fi(u5j/h1)/h1);
 w55 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u5i/h1)/h1);
 w61 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u6j/h1)/h1);
 w62 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u6j/h1)/h1);
 w63 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03i/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u6j/h1)/h1);
 w64 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04i/h1)/h1).*(fi(u6j/h1)/h1);
 w65 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u6i/h1)/h1);
 w71 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u7j/h1)/h1);
 w72 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u7j/h1)/h1);
 w73 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03i/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u7j/h1)/h1);
 w74 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04i/h1)/h1).*(fi(u7j/h1)/h1);
 w75 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u7i/h1)/h1);
 w81 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u8j/h1)/h1);
 w82 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u8j/h1)/h1);
 w83 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03i/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u8j/h1)/h1);
 w84 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04i/h1)/h1).*(fi(u8j/h1)/h1);
 w85 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u8i/h1)/h1);
 w91 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u9j/h1)/h1);
 w92 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u9j/h1)/h1);
 w93 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03i/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u9j/h1)/h1);
 w94 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04i/h1)/h1).*(fi(u9j/h1)/h1);
 w95 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u9i/h1)/h1);
 w101 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u10j/h1)/h1);
 w102 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u10j/h1)/h1);
 w103 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03i/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u10j/h1)/h1);
 w104 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04i/h1)/h1).*(fi(u10j/h1)/h1);
 w105 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u10i/h1)/h1);
 w111 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u11j/h1)/h1);
 w112 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u11j/h1)/h1);
 w113 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03i/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u11j/h1)/h1);
 w114 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04i/h1)/h1).*(fi(u11j/h1)/h1);
 w115 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u11i/h1)/h1);
 w121 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u12j/h1)/h1);
 w122 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u12j/h1)/h1);
 w123 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03i/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u12j/h1)/h1);
 w124 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04i/h1)/h1).*(fi(u12j/h1)/h1);
 w125 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u12i/h1)/h1);
 w131 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u13j/h1)/h1);
 w132 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u13j/h1)/h1);
 w133 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03i/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u13j/h1)/h1);
 w134 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04i/h1)/h1).*(fi(u13j/h1)/h1);
 w135 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u13i/h1)/h1);
 w11[j] = 0;
 w12[j] = 0;
 w13[j] = 0;
 w14[j] = 0;
 w15[j] = 0;
 w21[j] = 0;
 w22[j] = 0;
 w23[j] = 0;
 w24[j] = 0;
 w25[j] = 0;
 w31[j] = 0;
 w32[j] = 0;
 w33[j] = 0;
 w34[j] = 0;
 w35[j] = 0;
 w41[j] = 0;
 w42[j] = 0;
 w43[j] = 0;
 w44[j] = 0;
 w45[j] = 0;
 w51[j] = 0;
 w52[j] = 0;
 w53[j] = 0;
 w54[j] = 0;
 w55[j] = 0;
 w61[j] = 0;
 w62[j] = 0;
 w63[j] = 0;
 w64[j] = 0;
 w65[j] = 0;
 w71[j] = 0;
 w72[j] = 0;
 w73[j] = 0;
 w74[j] = 0;
 w75[j] = 0;
 w81[j] = 0;
 w82[j] = 0;
 w83[j] = 0;
 w84[j] = 0;
 w85[j] = 0;
 w91[j] = 0;
 w92[j] = 0;
 w93[j] = 0;
 w94[j] = 0;
 w95[j] = 0;
 w101[j] = 0;
 w102[j] = 0;
 w103[j] = 0;
 w104[j] = 0;
 w105[j] = 0;
 w111[j] = 0;
 w112[j] = 0;
 w113[j] = 0;
 w114[j] = 0;
 w115[j] = 0;
 w121[j] = 0;
 w122[j] = 0;
 w123[j] = 0;
 w124[j] = 0;
 w125[j] = 0;
 w131[j] = 0;
 w132[j] = 0;
 w133[j] = 0;
 w134[j] = 0;
 w135[j] = 0;
 wmat11  = diagrv((eye(T),w11));
 wmat12  = diagrv((eye(T),w12));
 wmat13  = diagrv((eye(T),w13));
 wmat14  = diagrv((eye(T),w14));
 wmat15  = diagrv((eye(T),w15));
 wmat21  = diagrv((eye(T),w21));
 wmat22  = diagrv((eye(T),w22));
 wmat23  = diagrv((eye(T),w23));
 wmat24  = diagrv((eye(T),w24));
 wmat25  = diagrv((eye(T),w25));
 wmat31  = diagrv((eye(T),w31));
 wmat32  = diagrv((eye(T),w32));
 wmat33  = diagrv((eye(T),w33));
 wmat34  = diagrv((eye(T),w34));
 wmat35  = diagrv((eye(T),w35));
 wmat41  = diagrv((eye(T),w41));
 wmat42  = diagrv((eye(T),w42));
 wmat43  = diagrv((eye(T),w43));
 wmat44  = diagrv((eye(T),w44));
 wmat45  = diagrv((eye(T),w45));
 wmat51  = diagrv((eye(T),w51));
 wmat52  = diagrv((eye(T),w52));
 wmat53  = diagrv((eye(T),w53));
 wmat54  = diagrv((eye(T),w54));
 wmat55  = diagrv((eye(T),w55));
 wmat61  = diagrv((eye(T),w61));
 wmat62  = diagrv((eye(T),w62));
 wmat63  = diagrv((eye(T),w63));
 wmat64  = diagrv((eye(T),w64));
 wmat65  = diagrv((eye(T),w65));
 wmat71  = diagrv((eye(T),w71));
 wmat72  = diagrv((eye(T),w72));
 wmat73  = diagrv((eye(T),w73));
 wmat74  = diagrv((eye(T),w74));
 wmat75  = diagrv((eye(T),w75));
 wmat81  = diagrv((eye(T),w81));
 wmat82  = diagrv((eye(T),w82));
 wmat83  = diagrv((eye(T),w83));
 wmat84  = diagrv((eye(T),w84));
 wmat85  = diagrv((eye(T),w85));
 wmat91  = diagrv((eye(T),w91));
 wmat92  = diagrv((eye(T),w92));
 wmat93  = diagrv((eye(T),w93));
 wmat94  = diagrv((eye(T),w94));
 wmat95  = diagrv((eye(T),w95));
 wmat101  = diagrv((eye(T),w101));
 wmat102  = diagrv((eye(T),w102));
 wmat103  = diagrv((eye(T),w103));
 wmat104  = diagrv((eye(T),w104));
 wmat105  = diagrv((eye(T),w105));
 wmat111  = diagrv((eye(T),w111));
 wmat112  = diagrv((eye(T),w112));
 wmat113  = diagrv((eye(T),w113));
 wmat114  = diagrv((eye(T),w114));
 wmat115  = diagrv((eye(T),w115));
 wmat121  = diagrv((eye(T),w121));
 wmat122  = diagrv((eye(T),w122));
 wmat123  = diagrv((eye(T),w123));
 wmat124  = diagrv((eye(T),w124));
 wmat125  = diagrv((eye(T),w125));
 wmat131  = diagrv((eye(T),w131));
 wmat132  = diagrv((eye(T),w132));
 wmat133  = diagrv((eye(T),w133));
 wmat134  = diagrv((eye(T),w134));
 wmat135  = diagrv((eye(T),w135));

xx11  =  ones(T,1)~u01i~u02j~u03j~u04j~u1j ;
xx12  =  ones(T,1)~u01j~u02i~u03j~u04j~u1j ;
xx13  =  ones(T,1)~u01j~u02j~u03i~u04j~u1j ;
xx14  =  ones(T,1)~u01j~u02j~u03j~u04i~u1j ;
xx15  =  ones(T,1)~u01j~u02j~u03j~u04j~u1i;
 xx21  =  ones(T,1)~u01i~u02j~u03j~u04j~u2j ;
 xx22  =  ones(T,1)~u01j~u02i~u03j~u04j~u2j ;
 xx23  =  ones(T,1)~u01j~u02j~u03i~u04j~u2j ;
 xx24  =  ones(T,1)~u01j~u02j~u03j~u04i~u2j ;
 xx25  =  ones(T,1)~u01j~u02j~u03j~u04j~u2i;
 xx31  =  ones(T,1)~u01i~u02j~u03j~u04j~u3j ;
 xx32  =  ones(T,1)~u01j~u02i~u03j~u04j~u3j ;
 xx33  =  ones(T,1)~u01j~u02j~u03i~u04j~u3j ;
 xx34  =  ones(T,1)~u01j~u02j~u03j~u04i~u3j ;
 xx35  =  ones(T,1)~u01j~u02j~u03j~u04j~u3i;
 xx41  =  ones(T,1)~u01i~u02j~u03j~u04j~u4j ;
 xx42  =  ones(T,1)~u01j~u02i~u03j~u04j~u4j ;
 xx43  =  ones(T,1)~u01j~u02j~u03i~u04j~u4j ;
 xx44  =  ones(T,1)~u01j~u02j~u03j~u04i~u4j ;
 xx45  =  ones(T,1)~u01j~u02j~u03j~u04j~u4i;
 xx51  =  ones(T,1)~u01i~u02j~u03j~u04j~u5j ;
 xx52  =  ones(T,1)~u01j~u02i~u03j~u04j~u5j ;
 xx53  =  ones(T,1)~u01j~u02j~u03i~u04j~u5j ;
 xx54  =  ones(T,1)~u01j~u02j~u03j~u04i~u5j ;
 xx55  =  ones(T,1)~u01j~u02j~u03j~u04j~u5i;
 xx61  =  ones(T,1)~u01i~u02j~u03j~u04j~u6j ;
 xx62  =  ones(T,1)~u01j~u02i~u03j~u04j~u6j ;
 xx63  =  ones(T,1)~u01j~u02j~u03i~u04j~u6j ;
 xx64  =  ones(T,1)~u01j~u02j~u03j~u04i~u6j ;
 xx65  =  ones(T,1)~u01j~u02j~u03j~u04j~u6i;
 xx71  =  ones(T,1)~u01i~u02j~u03j~u04j~u7j ;
 xx72  =  ones(T,1)~u01j~u02i~u03j~u04j~u7j ;
 xx73  =  ones(T,1)~u01j~u02j~u03i~u04j~u7j ;
 xx74  =  ones(T,1)~u01j~u02j~u03j~u04i~u7j ;
 xx75  =  ones(T,1)~u01j~u02j~u03j~u04j~u7i;
 xx81  =  ones(T,1)~u01i~u02j~u03j~u04j~u8j ;
 xx82  =  ones(T,1)~u01j~u02i~u03j~u04j~u8j ;
 xx83  =  ones(T,1)~u01j~u02j~u03i~u04j~u8j ;
 xx84  =  ones(T,1)~u01j~u02j~u03j~u04i~u8j ;
 xx85  =  ones(T,1)~u01j~u02j~u03j~u04j~u8i;
 xx91  =  ones(T,1)~u01i~u02j~u03j~u04j~u9j ;
 xx92  =  ones(T,1)~u01j~u02i~u03j~u04j~u9j ;
 xx93  =  ones(T,1)~u01j~u02j~u03i~u04j~u9j ;
 xx94  =  ones(T,1)~u01j~u02j~u03j~u04i~u9j ;
 xx95  =  ones(T,1)~u01j~u02j~u03j~u04j~u9i;
 xx101  =  ones(T,1)~u01i~u02j~u03j~u04j~u10j ;
 xx102  =  ones(T,1)~u01j~u02i~u03j~u04j~u10j ;
 xx103  =  ones(T,1)~u01j~u02j~u03i~u04j~u10j ;
 xx104  =  ones(T,1)~u01j~u02j~u03j~u04i~u10j ;
 xx105  =  ones(T,1)~u01j~u02j~u03j~u04j~u10i;
 xx111  =  ones(T,1)~u01i~u02j~u03j~u04j~u11j ;
 xx112  =  ones(T,1)~u01j~u02i~u03j~u04j~u11j ;
 xx113  =  ones(T,1)~u01j~u02j~u03i~u04j~u11j ;
 xx114  =  ones(T,1)~u01j~u02j~u03j~u04i~u11j ; 
 xx115  =  ones(T,1)~u01j~u02j~u03j~u04j~u11i;
 xx121  =  ones(T,1)~u01i~u02j~u03j~u04j~u12j ;
 xx122  =  ones(T,1)~u01j~u02i~u03j~u04j~u12j ;
 xx123  =  ones(T,1)~u01j~u02j~u03i~u04j~u12j ;
 xx124  =  ones(T,1)~u01j~u02j~u03j~u04i~u12j ;
 xx125  =  ones(T,1)~u01j~u02j~u03j~u04j~u12i;
 xx131  =  ones(T,1)~u01i~u02j~u03j~u04j~u13j ;
 xx132  =  ones(T,1)~u01j~u02i~u03j~u04j~u13j ;
 xx133  =  ones(T,1)~u01j~u02j~u03i~u04j~u13j ;
 xx134  =  ones(T,1)~u01j~u02j~u03j~u04i~u13j ;
 xx135  =  ones(T,1)~u01j~u02j~u03j~u04j~u13i;

if sumc(w11)> 0;
if det(xx11'wmat11*xx11) < epsln ;
      m5[j,1]  =  w11'y/sumc(w11) ;
else;
 beta =  pinv(xx11'wmat11*xx11)*xx11'wmat11'y ;
 m5[j,1]  =  beta[1,.] ;
endif;
else;
m5[j,1] = 0;
a11 = a11+1;
endif;

if sumc(w12)> 0;
if det(xx12'wmat12*xx12) < epsln ;
      m5[j,2]  =  w12'y/sumc(w12) ;
else;
 beta =  pinv(xx12'wmat12*xx12)*xx12'wmat12'y ;
 m5[j,2]  =  beta[1,.] ;
endif;
else;
m5[j,2] = 0;
a12 = a12+1;
endif;

if sumc(w13)> 0;
if det(xx13'wmat13*xx13) < epsln ;
      m5[j,3]  =  w13'y/sumc(w13) ;
else;
 beta =  pinv(xx13'wmat13*xx13)*xx13'wmat13'y ;
 m5[j,3]  =  beta[1,.] ;
endif;
else;
m5[j,3] = 0;
a13 = a13+1;
endif;

if sumc(w14)> 0;
if det(xx14'wmat14*xx14) < epsln ;
      m5[j,4]  =  w14'y/sumc(w14) ;
else;
 beta =  pinv(xx14'wmat14*xx14)*xx14'wmat14'y ;
 m5[j,4]  =  beta[1,.] ;
endif;
else;
m5[j,4] = 0;
a14 = a14+1;
endif;

if sumc(w15)> 0;
if det(xx15'wmat15*xx15) < epsln ;
      m5[j,5]  =  w15'y/sumc(w15) ;
else;
 beta =  pinv(xx15'wmat15*xx15)*xx15'wmat15'y ;
 m5[j,5]  =  beta[1,.] ;
endif;
else;
m5[j,5] = 0;
a15 = a15+1;
endif;

if sumc(w21)> 0;
if det(xx21'wmat21*xx21) < epsln ;
      m5[j,6]  =  w21'y/sumc(w21) ;
else;
 beta =  pinv(xx21'wmat21*xx21)*xx21'wmat21'y ;
 m5[j,6]  =  beta[1,.] ;
endif;
else;
m5[j,6] = 0;
a21 = a21+1;
endif;

if sumc(w22)> 0;
if det(xx22'wmat22*xx22) < epsln ;
      m5[j,7]  =  w22'y/sumc(w22) ;
else;
 beta =  pinv(xx22'wmat22*xx22)*xx22'wmat22'y ;
 m5[j,7]  =  beta[1,.] ;
endif;
else;
m5[j,7] = 0;
a22 = a22+1;
endif;

if sumc(w23)> 0;
if det(xx23'wmat23*xx23) < epsln ;
      m5[j,8]  =  w23'y/sumc(w23) ;
else;
 beta =  pinv(xx23'wmat23*xx23)*xx23'wmat23'y ;
 m5[j,8]  =  beta[1,.] ;
endif;
else;
m5[j,8] = 0;
a23 = a23+1;
endif;

if sumc(w24)> 0;
if det(xx24'wmat24*xx24) < epsln ;
      m5[j,9]  =  w24'y/sumc(w24) ;
else;
 beta =  pinv(xx24'wmat24*xx24)*xx24'wmat24'y ;
 m5[j,9]  =  beta[1,.] ;
endif;
else;
m5[j,9] = 0;
a24 = a24+1;
endif;

if sumc(w25)> 0;
if det(xx25'wmat25*xx25) < epsln ;
      m5[j,10]  =  w25'y/sumc(w25) ;
else;
 beta =  pinv(xx25'wmat25*xx25)*xx25'wmat25'y ;
 m5[j,10]  =  beta[1,.] ;
endif;
else;
m5[j,10] = 0;
a25 = a25+1;
endif;

if sumc(w31)> 0;
if det(xx31'wmat31*xx31) < epsln ;
      m5[j,11]  =  w31'y/sumc(w31) ;
else;
 beta =  pinv(xx31'wmat31*xx31)*xx31'wmat31'y ;
 m5[j,11]  =  beta[1,.] ;
endif;
else;
m5[j,11] = 0;
a31 = a31+1;
endif;

if sumc(w32)> 0;
if det(xx32'wmat32*xx32) < epsln ;
      m5[j,12]  =  w32'y/sumc(w32) ;
else;
 beta =  pinv(xx32'wmat32*xx32)*xx32'wmat32'y ;
 m5[j,12]  =  beta[1,.] ;
endif;
else;
m5[j,12] = 0;
a32 = a32+1;
endif;

if sumc(w33)> 0;
if det(xx33'wmat33*xx33) < epsln ;
      m5[j,13]  =  w33'y/sumc(w33) ;
else;
 beta =  pinv(xx33'wmat33*xx33)*xx33'wmat33'y ;
 m5[j,13]  =  beta[1,.] ;
endif;
else;
m5[j,13] = 0;
a33 = a33+1;
endif;

if sumc(w34)> 0;
if det(xx34'wmat34*xx34) < epsln ;
      m5[j,14]  =  w34'y/sumc(w34) ;
else;
 beta =  pinv(xx34'wmat34*xx34)*xx34'wmat34'y ;
 m5[j,14]  =  beta[1,.] ;
endif;
else;
m5[j,14] = 0;
a34 = a34+1;
endif;

if sumc(w35)> 0;
if det(xx35'wmat35*xx35) < epsln ;
      m5[j,15]  =  w35'y/sumc(w35) ;
else;
 beta =  pinv(xx35'wmat35*xx35)*xx35'wmat35'y ;
 m5[j,15]  =  beta[1,.] ;
endif;
else;
m5[j,15] = 0;
a35 = a35+1;
endif;

if sumc(w41)> 0;
if det(xx41'wmat41*xx41) < epsln ;
      m5[j,16]  =  w41'y/sumc(w41) ;
else;
 beta =  pinv(xx41'wmat41*xx41)*xx41'wmat41'y ;
 m5[j,16]  =  beta[1,.] ;
endif;
else;
m5[j,16] = 0;
a41 = a41+1;
endif;

if sumc(w42)> 0;
if det(xx42'wmat42*xx42) < epsln ;
      m5[j,17]  =  w42'y/sumc(w42) ;
else;
 beta =  pinv(xx42'wmat42*xx42)*xx42'wmat42'y ;
 m5[j,17]  =  beta[1,.] ;
endif;
else;
m5[j,17] = 0;
a42 = a42+1;
endif;

if sumc(w43)> 0;
if det(xx43'wmat43*xx43) < epsln ;
      m5[j,18]  =  w43'y/sumc(w43) ;
else;
 beta =  pinv(xx43'wmat43*xx43)*xx43'wmat43'y ;
 m5[j,18]  =  beta[1,.] ;
endif;
else;
m5[j,18] = 0;
a43 = a43+1;
endif;

if sumc(w44)> 0;
if det(xx44'wmat44*xx44) < epsln ;
      m5[j,19]  =  w44'y/sumc(w44) ;
else;
 beta =  pinv(xx44'wmat44*xx44)*xx44'wmat44'y ;
 m5[j,19]  =  beta[1,.] ;
endif;
else;
m5[j,19] = 0;
a44 = a44+1;
endif;

if sumc(w45)> 0;
if det(xx45'wmat45*xx45) < epsln ;
      m5[j,20]  =  w45'y/sumc(w45) ;
else;
 beta =  pinv(xx45'wmat45*xx45)*xx45'wmat45'y ;
 m5[j,20]  =  beta[1,.] ;
endif;
else;
m5[j,20] = 0;
a45 = a45+1;
endif;

if sumc(w51)> 0;
if det(xx51'wmat51*xx51) < epsln ;
      m5[j,21]  =  w51'y/sumc(w51) ;
else;
 beta =  pinv(xx51'wmat51*xx51)*xx51'wmat51'y ;
 m5[j,21]  =  beta[1,.] ;
endif;
else;
m5[j,21] = 0;
a51 = a51+1;
endif;

if sumc(w52)> 0;
if det(xx52'wmat52*xx52) < epsln ;
      m5[j,22]  =  w52'y/sumc(w52) ;
else;
 beta =  pinv(xx52'wmat52*xx52)*xx52'wmat52'y ;
 m5[j,22]  =  beta[1,.] ;
endif;
else;
m5[j,22] = 0;
a52 = a52+1;
endif;

if sumc(w53)> 0;
if det(xx53'wmat53*xx53) < epsln ;
      m5[j,23]  =  w53'y/sumc(w53) ;
else;
 beta =  pinv(xx53'wmat53*xx53)*xx53'wmat53'y ;
 m5[j,23]  =  beta[1,.] ;
endif;
else;
m5[j,23] = 0;
a53 = a53+1;
endif;

if sumc(w54)> 0;
if det(xx54'wmat54*xx54) < epsln ;
      m5[j,24]  =  w54'y/sumc(w54) ;
else;
 beta =  pinv(xx54'wmat54*xx54)*xx54'wmat54'y ;
 m5[j,24]  =  beta[1,.] ;
endif;
else;
m5[j,24] = 0;
a54 = a54+1;
endif;


if sumc(w55)> 0;
if det(xx55'wmat55*xx55) < epsln ;
      m5[j,25]  =  w55'y/sumc(w55) ;
else;
 beta =  pinv(xx55'wmat55*xx55)*xx55'wmat55'y ;
 m5[j,25]  =  beta[1,.] ;
endif;
else;
m5[j,25] = 0;
a55 = a55+1;
endif;


if sumc(w61)> 0;
if det(xx61'wmat61*xx61) < epsln ;
      m5[j,26]  =  w61'y/sumc(w61) ;
else;
 beta =  pinv(xx61'wmat61*xx61)*xx61'wmat61'y ;
 m5[j,26]  =  beta[1,.] ;
endif;
else;
m5[j,26] = 0;
a61 = a61+1;
endif;

if sumc(w62)> 0;
if det(xx62'wmat62*xx62) < epsln ;
      m5[j,27]  =  w62'y/sumc(w62) ;
else;
 beta =  pinv(xx62'wmat62*xx62)*xx62'wmat62'y ;
 m5[j,27]  =  beta[1,.] ;
endif;
else;
m5[j,27] = 0;
a62 = a62+1;
endif;

if sumc(w63)> 0;
if det(xx63'wmat63*xx63) < epsln ;
      m5[j,28]  =  w63'y/sumc(w63) ;
else;
 beta =  pinv(xx63'wmat63*xx63)*xx63'wmat63'y ;
 m5[j,28]  =  beta[1,.] ;
endif;
else;
m5[j,28] = 0;
a63 = a63+1;
endif;

if sumc(w64)> 0;
if det(xx64'wmat64*xx64) < epsln ;
      m5[j,29]  =  w64'y/sumc(w64) ;
else;
 beta =  pinv(xx64'wmat64*xx64)*xx64'wmat64'y ;
 m5[j,29]  =  beta[1,.] ;
endif;
else;
m5[j,29] = 0;
a64 = a64+1;
endif;

if sumc(w65)> 0;
if det(xx65'wmat65*xx65) < epsln ;
      m5[j,30]  =  w65'y/sumc(w65) ;
else;
 beta =  pinv(xx65'wmat65*xx65)*xx65'wmat65'y ;
 m5[j,30]  =  beta[1,.] ;
endif;
else;
m5[j,30] = 0;
a65 = a65+1;
endif;


if sumc(w71)> 0;
if det(xx71'wmat71*xx71) < epsln ;
      m5[j,31]  =  w71'y/sumc(w71) ;
else;
 beta =  pinv(xx71'wmat71*xx71)*xx71'wmat71'y ;
 m5[j,31]  =  beta[1,.] ;
endif;
else;
m5[j,31] = 0;
a71 = a71+1;
endif;


if sumc(w72)> 0;
if det(xx72'wmat72*xx72) < epsln ;
      m5[j,32]  =  w72'y/sumc(w72) ;
else;
 beta =  pinv(xx72'wmat72*xx72)*xx72'wmat72'y ;
 m5[j,32]  =  beta[1,.] ;
endif;
else;
m5[j,32] = 0;
a72 = a72+1;
endif;

if sumc(w73)> 0;
if det(xx73'wmat73*xx73) < epsln ;
      m5[j,33]  =  w73'y/sumc(w73) ;
else;
 beta =  pinv(xx73'wmat73*xx73)*xx73'wmat73'y ;
 m5[j,33]  =  beta[1,.] ;
endif;
else;
m5[j,33] = 0;
a73 = a73+1;
endif;

if sumc(w74)> 0;
if det(xx74'wmat74*xx74) < epsln ;
      m5[j,34]  =  w74'y/sumc(w74) ;
else;
 beta =  pinv(xx74'wmat74*xx74)*xx74'wmat74'y ;
 m5[j,34]  =  beta[1,.] ;
endif;
else;
m5[j,34] = 0;
a74 = a74+1;
endif;

if sumc(w75)> 0;
if det(xx75'wmat75*xx75) < epsln ;
      m5[j,35]  =  w75'y/sumc(w75) ;
else;
 beta =  pinv(xx75'wmat75*xx75)*xx75'wmat75'y ;
 m5[j,35]  =  beta[1,.] ;
endif;
else;
m5[j,35] = 0;
a75 = a75+1;
endif;

if sumc(w81)> 0;
if det(xx81'wmat81*xx81) < epsln ;
      m5[j,36]  =  w81'y/sumc(w81) ;
else;
 beta =  pinv(xx81'wmat81*xx81)*xx81'wmat81'y ;
 m5[j,36]  =  beta[1,.] ;
endif;
else;
m5[j,36] = 0;
a81 = a81+1;
endif;

if sumc(w82)> 0;
if det(xx82'wmat82*xx82) < epsln ;
      m5[j,37]  =  w82'y/sumc(w82) ;
else;
 beta =  pinv(xx82'wmat82*xx82)*xx82'wmat82'y ;
 m5[j,37]  =  beta[1,.] ;
endif;
else;
m5[j,37] = 0;
a82 = a82+1;
endif;

if sumc(w83)> 0;
if det(xx83'wmat83*xx83) < epsln ;
      m5[j,38]  =  w83'y/sumc(w83) ;
else;
 beta =  pinv(xx83'wmat83*xx83)*xx83'wmat83'y ;
 m5[j,38]  =  beta[1,.] ;
endif;
else;
m5[j,38] = 0;
a83 = a83+1;
endif;

if sumc(w84)> 0;
if det(xx84'wmat84*xx84) < epsln ;
      m5[j,39]  =  w84'y/sumc(w84) ;
else;
 beta =  pinv(xx84'wmat84*xx84)*xx84'wmat84'y ;
 m5[j,39]  =  beta[1,.] ;
endif;
else;
m5[j,39] = 0;
a84 = a84+1;
endif;

if sumc(w85)> 0;
if det(xx85'wmat85*xx85) < epsln ;
      m5[j,40]  =  w85'y/sumc(w85) ;
else;
 beta =  pinv(xx85'wmat85*xx85)*xx85'wmat85'y ;
 m5[j,40]  =  beta[1,.] ;
endif;
else;
m5[j,40] = 0;
a85 = a85+1;
endif;

if sumc(w91)> 0;
if det(xx91'wmat91*xx91) < epsln ;
      m5[j,41]  =  w91'y/sumc(w91) ;
else;
 beta =  pinv(xx91'wmat91*xx91)*xx91'wmat91'y ;
 m5[j,41]  =  beta[1,.] ;
endif;
else;
m5[j,41] = 0;
a91 = a91+1;
endif;

if sumc(w92)> 0;
if det(xx92'wmat92*xx92) < epsln ;
      m5[j,42]  =  w92'y/sumc(w92) ;
else;
 beta =  pinv(xx92'wmat92*xx92)*xx92'wmat92'y ;
 m5[j,42]  =  beta[1,.] ;
endif;
else;
m5[j,42] = 0;
a92 = a92+1;
endif;

if sumc(w93)> 0;
if det(xx93'wmat93*xx93) < epsln ;
      m5[j,43]  =  w93'y/sumc(w93) ;
else;
 beta =  pinv(xx93'wmat93*xx93)*xx93'wmat93'y ;
 m5[j,43]  =  beta[1,.] ;
endif;
else;
m5[j,43] = 0;
a93 = a93+1;
endif;

if sumc(w94)> 0;
if det(xx94'wmat94*xx94) < epsln ;
      m5[j,44]  =  w94'y/sumc(w94) ;
else;
 beta =  pinv(xx94'wmat94*xx94)*xx94'wmat94'y ;
 m5[j,44]  =  beta[1,.] ;
endif;
else;
m5[j,44] = 0;
a94 = a94+1;
endif;


if sumc(w95)> 0;
if det(xx95'wmat95*xx95) < epsln ;
      m5[j,45]  =  w95'y/sumc(w95) ;
else;
 beta =  pinv(xx95'wmat95*xx95)*xx95'wmat95'y ;
 m5[j,45]  =  beta[1,.] ;
endif;
else;
m5[j,45] = 0;
a95 = a95+1;
endif;

if sumc(w101)> 0;
if det(xx101'wmat101*xx101) < epsln ;
      m5[j,46]  =  w101'y/sumc(w101) ;
else;
 beta =  pinv(xx101'wmat101*xx101)*xx101'wmat101'y ;
 m5[j,46]  =  beta[1,.] ;
endif;
else;
m5[j,46] = 0;
a101 = a101+1;
endif;

if sumc(w102)> 0;
if det(xx102'wmat102*xx102) < epsln ;
      m5[j,47]  =  w102'y/sumc(w102) ;
else;
 beta =  pinv(xx102'wmat102*xx102)*xx102'wmat102'y ;
 m5[j,47]  =  beta[1,.] ;
endif;
else;
m5[j,47] = 0;
a102 = a102+1;
endif;

if sumc(w103)> 0;
if det(xx103'wmat103*xx103) < epsln ;
      m5[j,48]  =  w103'y/sumc(w103) ;
else;
 beta =  pinv(xx103'wmat103*xx103)*xx103'wmat103'y ;
 m5[j,48]  =  beta[1,.] ;
endif;
else;
m5[j,48] = 0;
a103 = a103+1;
endif;

if sumc(w104)> 0;
if det(xx104'wmat104*xx104) < epsln ;
      m5[j,49]  =  w104'y/sumc(w104) ;
else;
 beta =  pinv(xx104'wmat104*xx104)*xx104'wmat104'y ;
 m5[j,49]  =  beta[1,.] ;
endif;
else;
m5[j,49] = 0;
a104 = a104+1;
endif;

if sumc(w105)> 0;
if det(xx105'wmat105*xx105) < epsln ;
      m5[j,50]  =  w105'y/sumc(w105) ;
else;
 beta =  pinv(xx105'wmat105*xx105)*xx105'wmat105'y ;
 m5[j,50]  =  beta[1,.] ;
endif;
else;
m5[j,50] = 0;
a105 = a105+1;
endif;



if sumc(w111)> 0;
if det(xx111'wmat111*xx111) < epsln ;
      m5[j,51]  =  w111'y/sumc(w111) ;
else;
 beta =  pinv(xx111'wmat111*xx111)*xx111'wmat111'y ;
 m5[j,51]  =  beta[1,.] ;
endif;
else;
m5[j,51] = 0;
a111 = a111+1;
endif;

if sumc(w112)> 0;
if det(xx112'wmat112*xx112) < epsln ;
      m5[j,52]  =  w112'y/sumc(w112) ;
else;
 beta =  pinv(xx112'wmat112*xx112)*xx112'wmat112'y ;
 m5[j,52]  =  beta[1,.] ;
endif;
else;
m5[j,52] = 0;
a112 = a112+1;
endif;

if sumc(w113)> 0;
if det(xx113'wmat113*xx113) < epsln ;
      m5[j,53]  =  w113'y/sumc(w113) ;
else;
 beta =  pinv(xx113'wmat113*xx113)*xx113'wmat113'y ;
 m5[j,53]  =  beta[1,.] ;
endif;
else;
m5[j,53] = 0;
a113 = a113+1;
endif;

if sumc(w114)> 0;
if det(xx114'wmat114*xx114) < epsln ;
      m5[j,54]  =  w114'y/sumc(w114) ;
else;
 beta =  pinv(xx114'wmat114*xx114)*xx114'wmat114'y ;
 m5[j,54]  =  beta[1,.] ;
endif;
else;
m5[j,54] = 0;
a114 = a114+1;
endif;

if sumc(w115)> 0;
if det(xx115'wmat115*xx115) < epsln ;
      m5[j,55]  =  w115'y/sumc(w115) ;
else;
 beta =  pinv(xx115'wmat115*xx115)*xx115'wmat115'y ;
 m5[j,55]  =  beta[1,.] ;
endif;
else;
m5[j,55] = 0;
a115 = a115+1;
endif;


if sumc(w121)> 0;
if det(xx121'wmat121*xx121) < epsln ;
      m5[j,56]  =  w121'y/sumc(w121) ;
else;
 beta =  pinv(xx121'wmat121*xx121)*xx121'wmat121'y ;
 m5[j,56]  =  beta[1,.] ;
endif;
else;
m5[j,56] = 0;
a121 = a121+1;
endif;

if sumc(w122)> 0;
if det(xx122'wmat122*xx122) < epsln ;
      m5[j,57]  =  w122'y/sumc(w122) ;
else;
 beta =  pinv(xx122'wmat122*xx122)*xx122'wmat122'y ;
 m5[j,57]  =  beta[1,.] ;
endif;
else;
m5[j,57] = 0;
a122 = a122+1;
endif;

if sumc(w123)> 0;
if det(xx123'wmat123*xx123) < epsln ;
      m5[j,58]  =  w123'y/sumc(w123) ;
else;
 beta =  pinv(xx123'wmat123*xx123)*xx123'wmat123'y ;
 m5[j,58]  =  beta[1,.] ;
endif;
else;
m5[j,58] = 0;
a123 = a123+1;
endif;

if sumc(w124)> 0;
if det(xx124'wmat124*xx124) < epsln ;
      m5[j,59]  =  w124'y/sumc(w124) ;
else;
 beta =  pinv(xx124'wmat124*xx124)*xx124'wmat124'y ;
 m5[j,59]  =  beta[1,.] ;
endif;
else;
m5[j,59] = 0;
a124 = a124+1;
endif;

if sumc(w125)> 0;
if det(xx125'wmat125*xx125) < epsln ;
      m5[j,60]  =  w125'y/sumc(w125) ;
else;
 beta =  pinv(xx125'wmat125*xx125)*xx125'wmat125'y ;
 m5[j,60]  =  beta[1,.] ;
endif;
else;
m5[j,60] = 0;
a125 = a125+1;
endif;

if sumc(w131)> 0;
if det(xx131'wmat131*xx131) < epsln ;
      m5[j,61]  =  w131'y/sumc(w131) ;
else;
 beta =  pinv(xx131'wmat131*xx131)*xx131'wmat131'y ;
 m5[j,61]  =  beta[1,.] ;
endif;
else;
m5[j,61] = 0;
a131 = a131+1;
endif;

if sumc(w132)> 0;
if det(xx132'wmat132*xx132) < epsln ;
      m5[j,62]  =  w132'y/sumc(w132) ;
else;
 beta =  pinv(xx132'wmat132*xx132)*xx132'wmat132'y ;
 m5[j,62]  =  beta[1,.] ;
endif;
else;
m5[j,62] = 0;
a132 = a132+1;
endif;

if sumc(w133)> 0;
if det(xx133'wmat133*xx133) < epsln ;
      m5[j,63]  =  w133'y/sumc(w133) ;
else;
 beta =  pinv(xx133'wmat133*xx133)*xx133'wmat133'y ;
 m5[j,63]  =  beta[1,.] ;
endif;
else;
m5[j,63] = 0;
a133 = a133+1;
endif;

if sumc(w134)> 0;
if det(xx134'wmat134*xx134) < epsln ;
      m5[j,64]  =  w134'y/sumc(w134) ;
else;
 beta =  pinv(xx134'wmat134*xx134)*xx134'wmat134'y ;
 m5[j,64]  =  beta[1,.] ;
endif;
else;
m5[j,64] = 0;
a134 = a134+1;
endif;

if sumc(w135)> 0;
if det(xx135'wmat135*xx135) < epsln ;
      m5[j,65]  =  w135'y/sumc(w135) ;
else;
 beta =  pinv(xx135'wmat135*xx135)*xx135'wmat135'y ;
 m5[j,65]  =  beta[1,.] ;
endif;
else;
m5[j,65] = 0;
a135 = a135+1;
endif;

j = j+1;
endo;

m5d[i,1] = 1/(T-a11)*sumc(m5[.,1]);
m5d[i,2] = 1/(T-a12)*sumc(m5[.,2]);
m5d[i,3] = 1/(T-a13)*sumc(m5[.,3]);
m5d[i,4] = 1/(T-a14)*sumc(m5[.,4]);
m5d[i,5] = 1/(T-a15)*sumc(m5[.,5]);
m5d[i,6] = 1/(T-a21)*sumc(m5[.,6]);
m5d[i,7] = 1/(T-a22)*sumc(m5[.,7]);
m5d[i,8] = 1/(T-a23)*sumc(m5[.,8]);
m5d[i,9] = 1/(T-a24)*sumc(m5[.,9]);
m5d[i,10] = 1/(T-a25)*sumc(m5[.,10]);
m5d[i,11] = 1/(T-a31)*sumc(m5[.,11]);
m5d[i,12] = 1/(T-a32)*sumc(m5[.,12]);
m5d[i,13] = 1/(T-a33)*sumc(m5[.,13]);
m5d[i,14] = 1/(T-a34)*sumc(m5[.,14]);
m5d[i,15] = 1/(T-a35)*sumc(m5[.,15]);
m5d[i,16] = 1/(T-a41)*sumc(m5[.,16]);
m5d[i,17] = 1/(T-a42)*sumc(m5[.,17]);
m5d[i,18] = 1/(T-a43)*sumc(m5[.,18]);
m5d[i,19] = 1/(T-a44)*sumc(m5[.,19]);
m5d[i,20] = 1/(T-a45)*sumc(m5[.,20]);
m5d[i,21] = 1/(T-a51)*sumc(m5[.,21]);
m5d[i,22] = 1/(T-a52)*sumc(m5[.,22]);
m5d[i,23] = 1/(T-a53)*sumc(m5[.,23]);
m5d[i,24] = 1/(T-a54)*sumc(m5[.,24]);
m5d[i,25] = 1/(T-a55)*sumc(m5[.,25]);
m5d[i,26] = 1/(T-a61)*sumc(m5[.,26]);
m5d[i,27] = 1/(T-a62)*sumc(m5[.,27]);
m5d[i,28] = 1/(T-a63)*sumc(m5[.,28]);
m5d[i,29] = 1/(T-a64)*sumc(m5[.,29]);
m5d[i,30] = 1/(T-a65)*sumc(m5[.,30]);
m5d[i,31] = 1/(T-a71)*sumc(m5[.,31]);
m5d[i,32] = 1/(T-a72)*sumc(m5[.,32]);
m5d[i,33] = 1/(T-a73)*sumc(m5[.,33]);
m5d[i,34] = 1/(T-a74)*sumc(m5[.,34]);
m5d[i,35] = 1/(T-a75)*sumc(m5[.,35]);
m5d[i,36] = 1/(T-a81)*sumc(m5[.,36]);
m5d[i,37] = 1/(T-a82)*sumc(m5[.,37]);
m5d[i,38] = 1/(T-a83)*sumc(m5[.,38]);
m5d[i,39] = 1/(T-a84)*sumc(m5[.,39]);
m5d[i,40] = 1/(T-a85)*sumc(m5[.,40]);
m5d[i,41] = 1/(T-a91)*sumc(m5[.,41]);
m5d[i,42] = 1/(T-a92)*sumc(m5[.,42]);
m5d[i,43] = 1/(T-a93)*sumc(m5[.,43]);
m5d[i,44] = 1/(T-a94)*sumc(m5[.,44]);
m5d[i,45] = 1/(T-a95)*sumc(m5[.,45]);
m5d[i,46] = 1/(T-a101)*sumc(m5[.,46]);
m5d[i,47] = 1/(T-a102)*sumc(m5[.,47]);
m5d[i,48] = 1/(T-a103)*sumc(m5[.,48]);
m5d[i,49] = 1/(T-a104)*sumc(m5[.,49]);
m5d[i,50] = 1/(T-a105)*sumc(m5[.,50]);
m5d[i,51] = 1/(T-a111)*sumc(m5[.,51]);
m5d[i,52] = 1/(T-a112)*sumc(m5[.,52]);
m5d[i,53] = 1/(T-a113)*sumc(m5[.,53]);
m5d[i,54] = 1/(T-a114)*sumc(m5[.,54]);
m5d[i,55] = 1/(T-a115)*sumc(m5[.,55]);
m5d[i,56] = 1/(T-a121)*sumc(m5[.,56]);
m5d[i,57] = 1/(T-a122)*sumc(m5[.,57]);
m5d[i,58] = 1/(T-a123)*sumc(m5[.,58]);
m5d[i,59] = 1/(T-a124)*sumc(m5[.,59]);
m5d[i,60] = 1/(T-a125)*sumc(m5[.,60]);
m5d[i,61] = 1/(T-a121)*sumc(m5[.,61]);
m5d[i,62] = 1/(T-a122)*sumc(m5[.,62]);
m5d[i,63] = 1/(T-a123)*sumc(m5[.,63]);
m5d[i,64] = 1/(T-a124)*sumc(m5[.,64]);
m5d[i,65] = 1/(T-a125)*sumc(m5[.,65]);

/*Five dimension kernel estimator of the density */
LVO =  ones(T,1);
LVO[i] = 0;
phat[i,1] = sumc(LVO.*fi(u01i/hs5).*fi(u02i/hs5).*fi(u03i/hs5).*fi(u04i/hs5).*fi(u1i/hs5))/((T-1)*hs5^5);
phat[i,2] = sumc(LVO.*fi(u01i/hs5).*fi(u02i/hs5).*fi(u03i/hs5).*fi(u04i/hs5).*fi(u2i/hs5))/((T-1)*hs5^5);
phat[i,3] = sumc(LVO.*fi(u01i/hs5).*fi(u02i/hs5).*fi(u03i/hs5).*fi(u04i/hs5).*fi(u3i/hs5))/((T-1)*hs5^5);
phat[i,4] = sumc(LVO.*fi(u01i/hs5).*fi(u02i/hs5).*fi(u03i/hs5).*fi(u04i/hs5).*fi(u4i/hs5))/((T-1)*hs5^5);
phat[i,5] = sumc(LVO.*fi(u01i/hs5).*fi(u02i/hs5).*fi(u03i/hs5).*fi(u04i/hs5).*fi(u5i/hs5))/((T-1)*hs5^5);
phat[i,6] = sumc(LVO.*fi(u01i/hs5).*fi(u02i/hs5).*fi(u03i/hs5).*fi(u04i/hs5).*fi(u6i/hs5))/((T-1)*hs5^5);
phat[i,7] = sumc(LVO.*fi(u01i/hs5).*fi(u02i/hs5).*fi(u03i/hs5).*fi(u04i/hs5).*fi(u7i/hs5))/((T-1)*hs5^5);
phat[i,8] = sumc(LVO.*fi(u01i/hs5).*fi(u02i/hs5).*fi(u03i/hs5).*fi(u04i/hs5).*fi(u8i/hs5))/((T-1)*hs5^5);
phat[i,9] = sumc(LVO.*fi(u01i/hs5).*fi(u02i/hs5).*fi(u03i/hs5).*fi(u04i/hs5).*fi(u9i/hs5))/((T-1)*hs5^4);
phat[i,10] = sumc(LVO.*fi(u01i/hs5).*fi(u02i/hs5).*fi(u03i/hs5).*fi(u04i/hs5).*fi(u10i/hs5))/((T-1)*hs5^5);
phat[i,11] = sumc(LVO.*fi(u01i/hs5).*fi(u02i/hs5).*fi(u03i/hs5).*fi(u04i/hs5).*fi(u11i/hs5))/((T-1)*hs5^5);
phat[i,12] = sumc(LVO.*fi(u01i/hs5).*fi(u02i/hs5).*fi(u03i/hs5).*fi(u04i/hs5).*fi(u12i/hs5))/((T-1)*hs5^5);
phat[i,13] = sumc(LVO.*fi(u01i/hs5).*fi(u02i/hs5).*fi(u03i/hs5).*fi(u04i/hs5).*fi(u13i/hs5))/((T-1)*hs5^5);
@estimating q(z) @
qz5[i,1] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1);
qz5[i,2] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u1i/h1)./h1);
qz5[i,3] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u1i/h1)./h1);
qz5[i,4] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u04i/h1)./h1.*fi(u1i/h1)./h1);
qz5[i,5] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u1i/h1)./h1);
qz5[i,6] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u2i/h1)./h1);
qz5[i,7] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u2i/h1)./h1);
qz5[i,8] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u04i/h1)./h1.*fi(u2i/h1)./h1);
qz5[i,9] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u2i/h1)./h1);
qz5[i,10] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u3i/h1)./h1);
qz5[i,11] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u3i/h1)./h1);
qz5[i,12] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u04i/h1)./h1.*fi(u3i/h1)./h1);
qz5[i,13] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u3i/h1)./h1);
qz5[i,14] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u4i/h1)./h1);
qz5[i,15] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u4i/h1)./h1);
qz5[i,16] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u04i/h1)./h1.*fi(u4i/h1)./h1);
qz5[i,17] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u4i/h1)./h1);
qz5[i,18] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u5i/h1)./h1);
qz5[i,19] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u5i/h1)./h1);
qz5[i,20] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u04i/h1)./h1.*fi(u5i/h1)./h1);
qz5[i,21] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u5i/h1)./h1);
qz5[i,22] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u5i/h1)./h1);
qz5[i,23] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u5i/h1)./h1);
qz5[i,24] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u04i/h1)./h1.*fi(u5i/h1)./h1);
qz5[i,25] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u5i/h1)./h1);
qz5[i,26] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u6i/h1)./h1);
qz5[i,27] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u6i/h1)./h1);
qz5[i,28] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u04i/h1)./h1.*fi(u6i/h1)./h1);
qz5[i,29] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u6i/h1)./h1);
qz5[i,30] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u7i/h1)./h1);
qz5[i,31] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u7i/h1)./h1);
qz5[i,32] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u04i/h1)./h1.*fi(u7i/h1)./h1);
qz5[i,33] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u7i/h1)./h1);
qz5[i,34] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u8i/h1)./h1);
qz5[i,35] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u8i/h1)./h1);
qz5[i,36] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u04i/h1)./h1.*fi(u8i/h1)./h1);
qz5[i,37] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u8i/h1)./h1);
qz5[i,38] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u9i/h1)./h1);
qz5[i,39] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u9i/h1)./h1);
qz5[i,40] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u04i/h1)./h1.*fi(u9i/h1)./h1);
qz5[i,41] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u9i/h1)./h1);
qz5[i,42] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u10i/h1)./h1);
qz5[i,43] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u10i/h1)./h1);
qz5[i,44] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u04i/h1)./h1.*fi(u10i/h1)./h1);
qz5[i,45] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u10i/h1)./h1);
qz5[i,46] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u11i/h1)./h1);
qz5[i,47] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u11i/h1)./h1);
qz5[i,48] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u04i/h1)./h1.*fi(u11i/h1)./h1);
qz5[i,49] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u11i/h1)./h1);
qz5[i,50] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u12i/h1)./h1);
qz5[i,51] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u12i/h1)./h1);
qz5[i,52] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u04i/h1)./h1.*fi(u12i/h1)./h1);
qz5[i,53] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u12i/h1)./h1);
qz5[i,54] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u13i/h1)./h1);
qz5[i,55] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u13i/h1)./h1);
qz5[i,56] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u04i/h1)./h1.*fi(u13i/h1)./h1);
qz5[i,57] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u13i/h1)./h1);
i = i+1;
endo;
yhat1 = y-meanc(y)-(m5d[.,1]-meanc(m5d[.,1])+m5d[.,2]-meanc(m5d[.,2])+m5d[.,3]-meanc(m5d[.,3])+m5d[.,4]-meanc(m5d[.,4])+m5d[.,5]-meanc(m5d[.,5]));
yhat2 = y-meanc(y)-(m5d[.,6]-meanc(m5d[.,6])+m5d[.,7]-meanc(m5d[.,7])+m5d[.,8]-meanc(m5d[.,8])+m5d[.,9]-meanc(m5d[.,9])+m5d[.,10]-meanc(m5d[.,10]));
yhat3 = y-meanc(y)-(m5d[.,11]-meanc(m5d[.,11])+m5d[.,12]-meanc(m5d[.,12])+m5d[.,13]-meanc(m5d[.,13])+m5d[.,14]-meanc(m5d[.,14])+m5d[.,15]-meanc(m5d[.,15]));
yhat4 = y-meanc(y)-(m5d[.,16]-meanc(m5d[.,16])+m5d[.,17]-meanc(m5d[.,17])+m5d[.,18]-meanc(m5d[.,18])+m5d[.,19]-meanc(m5d[.,19])+m5d[.,20]-meanc(m5d[.,20]));
yhat5 = y-meanc(y)-(m5d[.,21]-meanc(m5d[.,21])+m5d[.,22]-meanc(m5d[.,22])+m5d[.,23]-meanc(m5d[.,23])+m5d[.,24]-meanc(m5d[.,24])+m5d[.,25]-meanc(m5d[.,25]));
yhat6 = y-meanc(y)-(m5d[.,26]-meanc(m5d[.,26])+m5d[.,27]-meanc(m5d[.,27])+m5d[.,28]-meanc(m5d[.,28])+m5d[.,29]-meanc(m5d[.,29])+m5d[.,30]-meanc(m5d[.,30]));
yhat7 = y-meanc(y)-(m5d[.,31]-meanc(m5d[.,31])+m5d[.,32]-meanc(m5d[.,32])+m5d[.,33]-meanc(m5d[.,33])+m5d[.,34]-meanc(m5d[.,34])+m5d[.,35]-meanc(m5d[.,35]));
yhat8 = y-meanc(y)-(m5d[.,36]-meanc(m5d[.,36])+m5d[.,37]-meanc(m5d[.,37])+m5d[.,38]-meanc(m5d[.,38])+m5d[.,39]-meanc(m5d[.,39])+m5d[.,40]-meanc(m5d[.,40]));
yhat9 = y-meanc(y)-(m5d[.,41]-meanc(m5d[.,41])+m5d[.,42]-meanc(m5d[.,42])+m5d[.,43]-meanc(m5d[.,43])+m5d[.,44]-meanc(m5d[.,44])+m5d[.,45]-meanc(m5d[.,45]));
yhat10 = y-meanc(y)-(m5d[.,46]-meanc(m5d[.,46])+m5d[.,47]-meanc(m5d[.,47])+m5d[.,48]-meanc(m5d[.,48])+m5d[.,49]-meanc(m5d[.,49])+m5d[.,50]-meanc(m5d[.,50]));
yhat11 = y-meanc(y)-(m5d[.,51]-meanc(m5d[.,51])+m5d[.,52]-meanc(m5d[.,52])+m5d[.,53]-meanc(m5d[.,53])+m5d[.,54]-meanc(m5d[.,54])+m5d[.,55]-meanc(m5d[.,55]));
yhat12 = y-meanc(y)-(m5d[.,56]-meanc(m5d[.,56])+m5d[.,57]-meanc(m5d[.,57])+m5d[.,58]-meanc(m5d[.,58])+m5d[.,59]-meanc(m5d[.,59])+m5d[.,60]-meanc(m5d[.,60]));
yhat13 = y-meanc(y)-(m5d[.,61]-meanc(m5d[.,61])+m5d[.,62]-meanc(m5d[.,62])+m5d[.,63]-meanc(m5d[.,63])+m5d[.,64]-meanc(m5d[.,64])+m5d[.,65]-meanc(m5d[.,65]));
BR1 = (yhat1-y).*(yhat1-y);
BR2 = (yhat2-y).*(yhat2-y);
BR3 = (yhat3-y).*(yhat3-y);
BR4 = (yhat4-y).*(yhat4-y);
BR5 = (yhat5-y).*(yhat5-y);
BR6 = (yhat6-y).*(yhat6-y);
BR7 = (yhat7-y).*(yhat7-y);
BR8 = (yhat8-y).*(yhat8-y);
BR9 = (yhat9-y).*(yhat9-y);
BR10 = (yhat10-y).*(yhat10-y);
BR11 = (yhat11-y).*(yhat11-y);
BR12 = (yhat12-y).*(yhat12-y);
BR13 = (yhat13-y).*(yhat13-y);
quan = quantile(phat,0.05);
index = phat.< = quan;
phat = index*10^5+phat;

/****************************Five Lags Included ******************************************/

       
/****************************Five Lags Included Estimation of A part *******************************************/
kc = 1;
do until kc>nc;
b5 = h5*h[kc,1];
i = 1;
a11 = 0;
a12 = 0;
a13 = 0;
a14 = 0;
a15 = 0;
a21 = 0;
a22 = 0;
a23 = 0;
a24 = 0;
a25 = 0;
a31 = 0;
a32 = 0;
a33 = 0;
a34 = 0;
a35 = 0;
a41 = 0;
a42 = 0;
a43 = 0;
a44 = 0;
a45 = 0;
a51 = 0;
a52 = 0;
a53 = 0;
a54 = 0;
a55 = 0;
a61 = 0;
a62 = 0;
a63 = 0;
a64 = 0;
a65 = 0;
a71 = 0;
a72 = 0;
a73 = 0;
a74 = 0;
a75 = 0;
a81 = 0;
a82 = 0;
a83 = 0;
a84 = 0;
a85 = 0;
a91 = 0;
a92 = 0;
a93 = 0;
a94 = 0;
a95 = 0;
a101 = 0;
a102 = 0;
a103 = 0;
a104 = 0;
a105 = 0;
a111 = 0;
a112 = 0;
a113 = 0;
a114 = 0;
a115 = 0;
a121 = 0;
a122 = 0;
a123 = 0;
a124 = 0;
a125 = 0;
a131 = 0;
a132 = 0;
a133 = 0;
a134 = 0;
a135 = 0;
m5 = zeros(T,65);
m5d = zeros(T,65);
i = 1;
do until i>T;
u01i = data[i,lag_1]*ones(T,1) - data[.,lag_1];
u02i = data[i,lag_2]*ones(T,1) - data[.,lag_2];
u03i = data[i,lag_3]*ones(T,1) - data[.,lag_3];
u04i = data[i,lag_4]*ones(T,1) - data[.,lag_4];
u0i = u01i~u02i~u03i~u04i;
u1i =  data[i,1]*ones(T,1) - data[.,1];
u2i =  data[i,2]*ones(T,1) - data[.,2];
u3i =  data[i,3]*ones(T,1) - data[.,3];
u4i =  data[i,4]*ones(T,1) - data[.,4];
u5i =  data[i,5]*ones(T,1) - data[.,5];
u6i =  data[i,6]*ones(T,1) - data[.,6];
u7i =  data[i,7]*ones(T,1) - data[.,7];
u8i =  data[i,8]*ones(T,1) - data[.,8];
u9i =  data[i,9]*ones(T,1) - data[.,9];
u10i =  data[i,10]*ones(T,1) - data[.,10];
u11i =  data[i,11]*ones(T,1) - data[.,11];
u12i =  data[i,12]*ones(T,1) - data[.,12];
u13i =  data[i,13]*ones(T,1) - data[.,13];
j = 1;
do until j>T;
u01j = data[j,lag_1]*ones(T,1) - data[.,lag_1];
u02j = data[j,lag_2]*ones(T,1) - data[.,lag_2];
u03j = data[j,lag_3]*ones(T,1) - data[.,lag_3];
u04j = data[j,lag_4]*ones(T,1) - data[.,lag_4];
u0j = u01j~u02j~u03j~u04j;
 u1j =  (data[j,1]*ones(T,1) - data[.,1]);
 u2j =  (data[j,2]*ones(T,1) - data[.,2]);
 u3j =  (data[j,3]*ones(T,1) - data[.,3]);
 u4j =  (data[j,4]*ones(T,1) - data[.,4]);
 u5j =  (data[j,5]*ones(T,1) - data[.,5]);
 u6j =  (data[j,5]*ones(T,1) - data[.,6]);
 u7j =  (data[j,7]*ones(T,1) - data[.,7]);
 u8j =  (data[j,8]*ones(T,1) - data[.,8]);
 u9j =  (data[j,9]*ones(T,1) - data[.,9]);
 u10j =  (data[j,10]*ones(T,1) - data[.,10]);
 u11j =  (data[j,11]*ones(T,1) - data[.,11]);
 u12j =  (data[j,12]*ones(T,1) - data[.,12]);
 u13j =  (data[j,13]*ones(T,1) - data[.,13]);
 w11 = (fi(u01i/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u1j/b5)/b5);
 w12 = (fi(u01j/b5)/b5).*(fi(u02i/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u1j/b5)/b5);
 w13 = (fi(u01j/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03i/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u1j/b5)/b5);
 w14 = (fi(u01j/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04i/b5)/b5).*(fi(u1j/b5)/b5);
 w15 = (fi(u01j/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u1i/b5)/b5);
 w21 = (fi(u01i/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u2j/b5)/b5);
 w22 = (fi(u01j/b5)/b5).*(fi(u02i/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u2j/b5)/b5);
 w23 = (fi(u01j/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03i/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u2j/b5)/b5);
 w24 = (fi(u01j/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04i/b5)/b5).*(fi(u2j/b5)/b5);
 w25 = (fi(u01j/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u2i/b5)/b5);
 w31 = (fi(u01i/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u3j/b5)/b5);
 w32 = (fi(u01j/b5)/b5).*(fi(u02i/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u3j/b5)/b5);
 w33 = (fi(u01j/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03i/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u3j/b5)/b5);
 w34 = (fi(u01j/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04i/b5)/b5).*(fi(u3j/b5)/b5);
 w35 = (fi(u01j/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u3i/b5)/b5);
 w41 = (fi(u01i/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u4j/b5)/b5);
 w42 = (fi(u01j/b5)/b5).*(fi(u02i/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u4j/b5)/b5);
 w43 = (fi(u01j/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03i/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u4j/b5)/b5);
 w44 = (fi(u01j/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04i/b5)/b5).*(fi(u4j/b5)/b5);
 w45 = (fi(u01j/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u4i/b5)/b5);
 w51 = (fi(u01i/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u5j/b5)/b5);
 w52 = (fi(u01j/b5)/b5).*(fi(u02i/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u5j/b5)/b5);
 w53 = (fi(u01j/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03i/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u5j/b5)/b5);
 w54 = (fi(u01j/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04i/b5)/b5).*(fi(u5j/b5)/b5);
 w55 = (fi(u01j/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u5i/b5)/b5);
 w61 = (fi(u01i/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u6j/b5)/b5);
 w62 = (fi(u01j/b5)/b5).*(fi(u02i/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u6j/b5)/b5);
 w63 = (fi(u01j/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03i/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u6j/b5)/b5);
 w64 = (fi(u01j/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04i/b5)/b5).*(fi(u6j/b5)/b5);
 w65 = (fi(u01j/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u6i/b5)/b5);
 w71 = (fi(u01i/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u7j/b5)/b5);
 w72 = (fi(u01j/b5)/b5).*(fi(u02i/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u7j/b5)/b5);
 w73 = (fi(u01j/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03i/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u7j/b5)/b5);
 w74 = (fi(u01j/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04i/b5)/b5).*(fi(u7j/b5)/b5);
 w75 = (fi(u01j/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u7i/b5)/b5);
 w81 = (fi(u01i/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u8j/b5)/b5);
 w82 = (fi(u01j/b5)/b5).*(fi(u02i/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u8j/b5)/b5);
 w83 = (fi(u01j/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03i/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u8j/b5)/b5);
 w84 = (fi(u01j/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04i/b5)/b5).*(fi(u8j/b5)/b5);
 w85 = (fi(u01j/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u8i/b5)/b5);
 w91 = (fi(u01i/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u9j/b5)/b5);
 w92 = (fi(u01j/b5)/b5).*(fi(u02i/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u9j/b5)/b5);
 w93 = (fi(u01j/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03i/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u9j/b5)/b5);
 w94 = (fi(u01j/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04i/b5)/b5).*(fi(u9j/b5)/b5);
 w95 = (fi(u01j/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u9i/b5)/b5);
 w101 = (fi(u01i/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u10j/b5)/b5);
 w102 = (fi(u01j/b5)/b5).*(fi(u02i/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u10j/b5)/b5);
 w103 = (fi(u01j/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03i/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u10j/b5)/b5);
 w104 = (fi(u01j/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04i/b5)/b5).*(fi(u10j/b5)/b5);
 w105 = (fi(u01j/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u10i/b5)/b5);
 w111 = (fi(u01i/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u11j/b5)/b5);
 w112 = (fi(u01j/b5)/b5).*(fi(u02i/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u11j/b5)/b5);
 w113 = (fi(u01j/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03i/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u11j/b5)/b5);
 w114 = (fi(u01j/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04i/b5)/b5).*(fi(u11j/b5)/b5);
 w115 = (fi(u01j/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u11i/b5)/b5);
 w121 = (fi(u01i/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u12j/b5)/b5);
 w122 = (fi(u01j/b5)/b5).*(fi(u02i/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u12j/b5)/b5);
 w123 = (fi(u01j/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03i/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u12j/b5)/b5);
 w124 = (fi(u01j/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04i/b5)/b5).*(fi(u12j/b5)/b5);
 w125 = (fi(u01j/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u12i/b5)/b5);
 w131 = (fi(u01i/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u13j/b5)/b5);
 w132 = (fi(u01j/b5)/b5).*(fi(u02i/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u13j/b5)/b5);
 w133 = (fi(u01j/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03i/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u13j/b5)/b5);
 w134 = (fi(u01j/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04i/b5)/b5).*(fi(u13j/b5)/b5);
 w135 = (fi(u01j/b5)/b5).*(fi(u02j/b5)/b5).*(fi(u03j/b5)/b5).*(fi(u04j/b5)/b5).*(fi(u13i/b5)/b5);
 wmat11  = diagrv((eye(T),w11));
 wmat12  = diagrv((eye(T),w12));
 wmat13  = diagrv((eye(T),w13));
 wmat14  = diagrv((eye(T),w14));
 wmat15  = diagrv((eye(T),w15));
 wmat21  = diagrv((eye(T),w21));
 wmat22  = diagrv((eye(T),w22));
 wmat23  = diagrv((eye(T),w23));
 wmat24  = diagrv((eye(T),w24));
 wmat25  = diagrv((eye(T),w25));
 wmat31  = diagrv((eye(T),w31));
 wmat32  = diagrv((eye(T),w32));
 wmat33  = diagrv((eye(T),w33));
 wmat34  = diagrv((eye(T),w34));
 wmat35  = diagrv((eye(T),w35));
 wmat41  = diagrv((eye(T),w41));
 wmat42  = diagrv((eye(T),w42));
 wmat43  = diagrv((eye(T),w43));
 wmat44  = diagrv((eye(T),w44));
 wmat45  = diagrv((eye(T),w45));
 wmat51  = diagrv((eye(T),w51));
 wmat52  = diagrv((eye(T),w52));
 wmat53  = diagrv((eye(T),w53));
 wmat54  = diagrv((eye(T),w54));
 wmat55  = diagrv((eye(T),w55));
 wmat61  = diagrv((eye(T),w61));
 wmat62  = diagrv((eye(T),w62));
 wmat63  = diagrv((eye(T),w63));
 wmat64  = diagrv((eye(T),w64));
 wmat65  = diagrv((eye(T),w65));
 wmat71  = diagrv((eye(T),w71));
 wmat72  = diagrv((eye(T),w72));
 wmat73  = diagrv((eye(T),w73));
 wmat74  = diagrv((eye(T),w74));
 wmat75  = diagrv((eye(T),w75));
 wmat81  = diagrv((eye(T),w81));
 wmat82  = diagrv((eye(T),w82));
 wmat83  = diagrv((eye(T),w83));
 wmat84  = diagrv((eye(T),w84));
 wmat85  = diagrv((eye(T),w85));
 wmat91  = diagrv((eye(T),w91));
 wmat92  = diagrv((eye(T),w92));
 wmat93  = diagrv((eye(T),w93));
 wmat94  = diagrv((eye(T),w94));
 wmat95  = diagrv((eye(T),w95));
 wmat101  = diagrv((eye(T),w101));
 wmat102  = diagrv((eye(T),w102));
 wmat103  = diagrv((eye(T),w103));
 wmat104  = diagrv((eye(T),w104));
 wmat105  = diagrv((eye(T),w105));
 wmat111  = diagrv((eye(T),w111));
 wmat112  = diagrv((eye(T),w112));
 wmat113  = diagrv((eye(T),w113));
 wmat114  = diagrv((eye(T),w114));
 wmat115  = diagrv((eye(T),w115));
 wmat121  = diagrv((eye(T),w121));
 wmat122  = diagrv((eye(T),w122));
 wmat123  = diagrv((eye(T),w123));
 wmat124  = diagrv((eye(T),w124));
 wmat125  = diagrv((eye(T),w125));
 wmat131  = diagrv((eye(T),w131));
 wmat132  = diagrv((eye(T),w132));
 wmat133  = diagrv((eye(T),w133));
 wmat134  = diagrv((eye(T),w134));
 wmat135  = diagrv((eye(T),w135));
xx11  =  ones(T,1)~u01i~u02j~u03j~u04j~u1j ;
xx12  =  ones(T,1)~u01j~u02i~u03j~u04j~u1j ;
xx13  =  ones(T,1)~u01j~u02j~u03i~u04j~u1j ;
xx14  =  ones(T,1)~u01j~u02j~u03j~u04i~u1j ;
xx15  =  ones(T,1)~u01j~u02j~u03j~u04j~u1i;
xx21  =  ones(T,1)~u01i~u02j~u03j~u04j~u2j ;
 xx22  =  ones(T,1)~u01j~u02i~u03j~u04j~u2j ;
 xx23  =  ones(T,1)~u01j~u02j~u03i~u04j~u2j ;
 xx24  =  ones(T,1)~u01j~u02j~u03j~u04i~u2j ;
 xx25  =  ones(T,1)~u01j~u02j~u03j~u04j~u2i;
 xx31  =  ones(T,1)~u01i~u02j~u03j~u04j~u3j ;
 xx32  =  ones(T,1)~u01j~u02i~u03j~u04j~u3j ;
 xx33  =  ones(T,1)~u01j~u02j~u03i~u04j~u3j ;
 xx34  =  ones(T,1)~u01j~u02j~u03j~u04i~u3j ;
 xx35  =  ones(T,1)~u01j~u02j~u03j~u04j~u3i;
 xx41  =  ones(T,1)~u01i~u02j~u03j~u04j~u4j ;
 xx42  =  ones(T,1)~u01j~u02i~u03j~u04j~u4j ;
 xx43  =  ones(T,1)~u01j~u02j~u03i~u04j~u4j ;
 xx44  =  ones(T,1)~u01j~u02j~u03j~u04i~u4j ;
 xx45  =  ones(T,1)~u01j~u02j~u03j~u04j~u4i;
 xx51  =  ones(T,1)~u01i~u02j~u03j~u04j~u5j ;
 xx52  =  ones(T,1)~u01j~u02i~u03j~u04j~u5j ;
 xx53  =  ones(T,1)~u01j~u02j~u03i~u04j~u5j ;
 xx54  =  ones(T,1)~u01j~u02j~u03j~u04i~u5j ;
 xx55  =  ones(T,1)~u01j~u02j~u03j~u04j~u5i;
 xx61  =  ones(T,1)~u01i~u02j~u03j~u04j~u6j ;
 xx62  =  ones(T,1)~u01j~u02i~u03j~u04j~u6j ;
 xx63  =  ones(T,1)~u01j~u02j~u03i~u04j~u6j ;
 xx64  =  ones(T,1)~u01j~u02j~u03j~u04i~u6j ;
 xx65  =  ones(T,1)~u01j~u02j~u03j~u04j~u6i;
 xx71  =  ones(T,1)~u01i~u02j~u03j~u04j~u7j ;
 xx72  =  ones(T,1)~u01j~u02i~u03j~u04j~u7j ;
 xx73  =  ones(T,1)~u01j~u02j~u03i~u04j~u7j ;
 xx74  =  ones(T,1)~u01j~u02j~u03j~u04i~u7j ;
 xx75  =  ones(T,1)~u01j~u02j~u03j~u04j~u7i;
 xx81  =  ones(T,1)~u01i~u02j~u03j~u04j~u8j ;
 xx82  =  ones(T,1)~u01j~u02i~u03j~u04j~u8j ;
 xx83  =  ones(T,1)~u01j~u02j~u03i~u04j~u8j ;
 xx84  =  ones(T,1)~u01j~u02j~u03j~u04i~u8j ;
 xx85  =  ones(T,1)~u01j~u02j~u03j~u04j~u8i;
 xx91  =  ones(T,1)~u01i~u02j~u03j~u04j~u9j ;
 xx92  =  ones(T,1)~u01j~u02i~u03j~u04j~u9j ;
 xx93  =  ones(T,1)~u01j~u02j~u03i~u04j~u9j ;
 xx94  =  ones(T,1)~u01j~u02j~u03j~u04i~u9j ;
 xx95  =  ones(T,1)~u01j~u02j~u03j~u04j~u9i;
 xx101  =  ones(T,1)~u01i~u02j~u03j~u04j~u10j ;
 xx102  =  ones(T,1)~u01j~u02i~u03j~u04j~u10j ;
 xx103  =  ones(T,1)~u01j~u02j~u03i~u04j~u10j ;
 xx104  =  ones(T,1)~u01j~u02j~u03j~u04i~u10j ;
 xx105  =  ones(T,1)~u01j~u02j~u03j~u04j~u10i;
 xx111  =  ones(T,1)~u01i~u02j~u03j~u04j~u11j ;
 xx112  =  ones(T,1)~u01j~u02i~u03j~u04j~u11j ;
 xx113  =  ones(T,1)~u01j~u02j~u03i~u04j~u11j ;
 xx114  =  ones(T,1)~u01j~u02j~u03j~u04i~u11j ; 
 xx115  =  ones(T,1)~u01j~u02j~u03j~u04j~u11i;
 xx121  =  ones(T,1)~u01i~u02j~u03j~u04j~u12j ;
 xx122  =  ones(T,1)~u01j~u02i~u03j~u04j~u12j ;
 xx123  =  ones(T,1)~u01j~u02j~u03i~u04j~u12j ;
 xx124  =  ones(T,1)~u01j~u02j~u03j~u04i~u12j ;
 xx125  =  ones(T,1)~u01j~u02j~u03j~u04j~u12i;
 xx131  =  ones(T,1)~u01i~u02j~u03j~u04j~u13j ;
 xx132  =  ones(T,1)~u01j~u02i~u03j~u04j~u13j ;
 xx133  =  ones(T,1)~u01j~u02j~u03i~u04j~u13j ;
 xx134  =  ones(T,1)~u01j~u02j~u03j~u04i~u13j ;
 xx135  =  ones(T,1)~u01j~u02j~u03j~u04j~u13i;

if sumc(w11)> 0;
if det(xx11'wmat11*xx11) < epsln ;
      m5[j,1]  =  w11'y/sumc(w11) ;
else;
 beta =  pinv(xx11'wmat11*xx11)*xx11'wmat11'y ;
 m5[j,1]  =  beta[1,.] ;
endif;
else;
m5[j,1] = 0;
a11 = a11+1;
endif;

if sumc(w12)> 0;
if det(xx12'wmat12*xx12) < epsln ;
      m5[j,2]  =  w12'y/sumc(w12) ;
else;
 beta =  pinv(xx12'wmat12*xx12)*xx12'wmat12'y ;
 m5[j,2]  =  beta[1,.] ;
endif;
else;
m5[j,2] = 0;
a12 = a12+1;
endif;

if sumc(w13)> 0;
if det(xx13'wmat13*xx13) < epsln ;
      m5[j,3]  =  w13'y/sumc(w13) ;
else;
 beta =  pinv(xx13'wmat13*xx13)*xx13'wmat13'y ;
 m5[j,3]  =  beta[1,.] ;
endif;
else;
m5[j,3] = 0;
a13 = a13+1;
endif;

if sumc(w14)> 0;
if det(xx14'wmat14*xx14) < epsln ;
      m5[j,4]  =  w14'y/sumc(w14) ;
else;
 beta =  pinv(xx14'wmat14*xx14)*xx14'wmat14'y ;
 m5[j,4]  =  beta[1,.] ;
endif;
else;
m5[j,4] = 0;
a14 = a14+1;
endif;

if sumc(w15)> 0;
if det(xx15'wmat15*xx15) < epsln ;
      m5[j,5]  =  w15'y/sumc(w15) ;
else;
 beta =  pinv(xx15'wmat15*xx15)*xx15'wmat15'y ;
 m5[j,5]  =  beta[1,.] ;
endif;
else;
m5[j,5] = 0;
a15 = a15+1;
endif;

if sumc(w21)> 0;
if det(xx21'wmat21*xx21) < epsln ;
      m5[j,6]  =  w21'y/sumc(w21) ;
else;
 beta =  pinv(xx21'wmat21*xx21)*xx21'wmat21'y ;
 m5[j,6]  =  beta[1,.] ;
endif;
else;
m5[j,6] = 0;
a21 = a21+1;
endif;

if sumc(w22)> 0;
if det(xx22'wmat22*xx22) < epsln ;
      m5[j,7]  =  w22'y/sumc(w22) ;
else;
 beta =  pinv(xx22'wmat22*xx22)*xx22'wmat22'y ;
 m5[j,7]  =  beta[1,.] ;
endif;
else;
m5[j,7] = 0;
a22 = a22+1;
endif;

if sumc(w23)> 0;
if det(xx23'wmat23*xx23) < epsln ;
      m5[j,8]  =  w23'y/sumc(w23) ;
else;
 beta =  pinv(xx23'wmat23*xx23)*xx23'wmat23'y ;
 m5[j,8]  =  beta[1,.] ;
endif;
else;
m5[j,8] = 0;
a23 = a23+1;
endif;

if sumc(w24)> 0;
if det(xx24'wmat24*xx24) < epsln ;
      m5[j,9]  =  w24'y/sumc(w24) ;
else;
 beta =  pinv(xx24'wmat24*xx24)*xx24'wmat24'y ;
 m5[j,9]  =  beta[1,.] ;
endif;
else;
m5[j,9] = 0;
a24 = a24+1;
endif;

if sumc(w25)> 0;
if det(xx25'wmat25*xx25) < epsln ;
      m5[j,10]  =  w25'y/sumc(w25) ;
else;
 beta =  pinv(xx25'wmat25*xx25)*xx25'wmat25'y ;
 m5[j,10]  =  beta[1,.] ;
endif;
else;
m5[j,10] = 0;
a25 = a25+1;
endif;

if sumc(w31)> 0;
if det(xx31'wmat31*xx31) < epsln ;
      m5[j,11]  =  w31'y/sumc(w31) ;
else;
 beta =  pinv(xx31'wmat31*xx31)*xx31'wmat31'y ;
 m5[j,11]  =  beta[1,.] ;
endif;
else;
m5[j,11] = 0;
a31 = a31+1;
endif;

if sumc(w32)> 0;
if det(xx32'wmat32*xx32) < epsln ;
      m5[j,12]  =  w32'y/sumc(w32) ;
else;
 beta =  pinv(xx32'wmat32*xx32)*xx32'wmat32'y ;
 m5[j,12]  =  beta[1,.] ;
endif;
else;
m5[j,12] = 0;
a32 = a32+1;
endif;

if sumc(w33)> 0;
if det(xx33'wmat33*xx33) < epsln ;
      m5[j,13]  =  w33'y/sumc(w33) ;
else;
 beta =  pinv(xx33'wmat33*xx33)*xx33'wmat33'y ;
 m5[j,13]  =  beta[1,.] ;
endif;
else;
m5[j,13] = 0;
a33 = a33+1;
endif;

if sumc(w34)> 0;
if det(xx34'wmat34*xx34) < epsln ;
      m5[j,14]  =  w34'y/sumc(w34) ;
else;
 beta =  pinv(xx34'wmat34*xx34)*xx34'wmat34'y ;
 m5[j,14]  =  beta[1,.] ;
endif;
else;
m5[j,14] = 0;
a34 = a34+1;
endif;

if sumc(w35)> 0;
if det(xx35'wmat35*xx35) < epsln ;
      m5[j,15]  =  w35'y/sumc(w35) ;
else;
 beta =  pinv(xx35'wmat35*xx35)*xx35'wmat35'y ;
 m5[j,15]  =  beta[1,.] ;
endif;
else;
m5[j,15] = 0;
a35 = a35+1;
endif;

if sumc(w41)> 0;
if det(xx41'wmat41*xx41) < epsln ;
      m5[j,16]  =  w41'y/sumc(w41) ;
else;
 beta =  pinv(xx41'wmat41*xx41)*xx41'wmat41'y ;
 m5[j,16]  =  beta[1,.] ;
endif;
else;
m5[j,16] = 0;
a41 = a41+1;
endif;

if sumc(w42)> 0;
if det(xx42'wmat42*xx42) < epsln ;
      m5[j,17]  =  w42'y/sumc(w42) ;
else;
 beta =  pinv(xx42'wmat42*xx42)*xx42'wmat42'y ;
 m5[j,17]  =  beta[1,.] ;
endif;
else;
m5[j,17] = 0;
a42 = a42+1;
endif;

if sumc(w43)> 0;
if det(xx43'wmat43*xx43) < epsln ;
      m5[j,18]  =  w43'y/sumc(w43) ;
else;
 beta =  pinv(xx43'wmat43*xx43)*xx43'wmat43'y ;
 m5[j,18]  =  beta[1,.] ;
endif;
else;
m5[j,18] = 0;
a43 = a43+1;
endif;

if sumc(w44)> 0;
if det(xx44'wmat44*xx44) < epsln ;
      m5[j,19]  =  w44'y/sumc(w44) ;
else;
 beta =  pinv(xx44'wmat44*xx44)*xx44'wmat44'y ;
 m5[j,19]  =  beta[1,.] ;
endif;
else;
m5[j,19] = 0;
a44 = a44+1;
endif;

if sumc(w45)> 0;
if det(xx45'wmat45*xx45) < epsln ;
      m5[j,20]  =  w45'y/sumc(w45) ;
else;
 beta =  pinv(xx45'wmat45*xx45)*xx45'wmat45'y ;
 m5[j,20]  =  beta[1,.] ;
endif;
else;
m5[j,20] = 0;
a45 = a45+1;
endif;



if sumc(w51)> 0;
if det(xx51'wmat51*xx51) < epsln ;
      m5[j,21]  =  w51'y/sumc(w51) ;
else;
 beta =  pinv(xx51'wmat51*xx51)*xx51'wmat51'y ;
 m5[j,21]  =  beta[1,.] ;
endif;
else;
m5[j,21] = 0;
a51 = a51+1;
endif;

if sumc(w52)> 0;
if det(xx52'wmat52*xx52) < epsln ;
      m5[j,22]  =  w52'y/sumc(w52) ;
else;
 beta =  pinv(xx52'wmat52*xx52)*xx52'wmat52'y ;
 m5[j,22]  =  beta[1,.] ;
endif;
else;
m5[j,22] = 0;
a52 = a52+1;
endif;

if sumc(w53)> 0;
if det(xx53'wmat53*xx53) < epsln ;
      m5[j,23]  =  w53'y/sumc(w53) ;
else;
 beta =  pinv(xx53'wmat53*xx53)*xx53'wmat53'y ;
 m5[j,23]  =  beta[1,.] ;
endif;
else;
m5[j,23] = 0;
a53 = a53+1;
endif;

if sumc(w54)> 0;
if det(xx54'wmat54*xx54) < epsln ;
      m5[j,24]  =  w54'y/sumc(w54) ;
else;
 beta =  pinv(xx54'wmat54*xx54)*xx54'wmat54'y ;
 m5[j,24]  =  beta[1,.] ;
endif;
else;
m5[j,24] = 0;
a54 = a54+1;
endif;


if sumc(w55)> 0;
if det(xx55'wmat55*xx55) < epsln ;
      m5[j,25]  =  w55'y/sumc(w55) ;
else;
 beta =  pinv(xx55'wmat55*xx55)*xx55'wmat55'y ;
 m5[j,25]  =  beta[1,.] ;
endif;
else;
m5[j,25] = 0;
a55 = a55+1;
endif;


if sumc(w61)> 0;
if det(xx61'wmat61*xx61) < epsln ;
      m5[j,26]  =  w61'y/sumc(w61) ;
else;
 beta =  pinv(xx61'wmat61*xx61)*xx61'wmat61'y ;
 m5[j,26]  =  beta[1,.] ;
endif;
else;
m5[j,26] = 0;
a61 = a61+1;
endif;

if sumc(w62)> 0;
if det(xx62'wmat62*xx62) < epsln ;
      m5[j,27]  =  w62'y/sumc(w62) ;
else;
 beta =  pinv(xx62'wmat62*xx62)*xx62'wmat62'y ;
 m5[j,27]  =  beta[1,.] ;
endif;
else;
m5[j,27] = 0;
a62 = a62+1;
endif;

if sumc(w63)> 0;
if det(xx63'wmat63*xx63) < epsln ;
      m5[j,28]  =  w63'y/sumc(w63) ;
else;
 beta =  pinv(xx63'wmat63*xx63)*xx63'wmat63'y ;
 m5[j,28]  =  beta[1,.] ;
endif;
else;
m5[j,28] = 0;
a63 = a63+1;
endif;

if sumc(w64)> 0;
if det(xx64'wmat64*xx64) < epsln ;
      m5[j,29]  =  w64'y/sumc(w64) ;
else;
 beta =  pinv(xx64'wmat64*xx64)*xx64'wmat64'y ;
 m5[j,29]  =  beta[1,.] ;
endif;
else;
m5[j,29] = 0;
a64 = a64+1;
endif;

if sumc(w65)> 0;
if det(xx65'wmat65*xx65) < epsln ;
      m5[j,30]  =  w65'y/sumc(w65) ;
else;
 beta =  pinv(xx65'wmat65*xx65)*xx65'wmat65'y ;
 m5[j,30]  =  beta[1,.] ;
endif;
else;
m5[j,30] = 0;
a65 = a65+1;
endif;


if sumc(w71)> 0;
if det(xx71'wmat71*xx71) < epsln ;
      m5[j,31]  =  w71'y/sumc(w71) ;
else;
 beta =  pinv(xx71'wmat71*xx71)*xx71'wmat71'y ;
 m5[j,31]  =  beta[1,.] ;
endif;
else;
m5[j,31] = 0;
a71 = a71+1;
endif;


if sumc(w72)> 0;
if det(xx72'wmat72*xx72) < epsln ;
      m5[j,32]  =  w72'y/sumc(w72) ;
else;
 beta =  pinv(xx72'wmat72*xx72)*xx72'wmat72'y ;
 m5[j,32]  =  beta[1,.] ;
endif;
else;
m5[j,32] = 0;
a72 = a72+1;
endif;

if sumc(w73)> 0;
if det(xx73'wmat73*xx73) < epsln ;
      m5[j,33]  =  w73'y/sumc(w73) ;
else;
 beta =  pinv(xx73'wmat73*xx73)*xx73'wmat73'y ;
 m5[j,33]  =  beta[1,.] ;
endif;
else;
m5[j,33] = 0;
a73 = a73+1;
endif;

if sumc(w74)> 0;
if det(xx74'wmat74*xx74) < epsln ;
      m5[j,34]  =  w74'y/sumc(w74) ;
else;
 beta =  pinv(xx74'wmat74*xx74)*xx74'wmat74'y ;
 m5[j,34]  =  beta[1,.] ;
endif;
else;
m5[j,34] = 0;
a74 = a74+1;
endif;

if sumc(w75)> 0;
if det(xx75'wmat75*xx75) < epsln ;
      m5[j,35]  =  w75'y/sumc(w75) ;
else;
 beta =  pinv(xx75'wmat75*xx75)*xx75'wmat75'y ;
 m5[j,35]  =  beta[1,.] ;
endif;
else;
m5[j,35] = 0;
a75 = a75+1;
endif;



if sumc(w81)> 0;
if det(xx81'wmat81*xx81) < epsln ;
      m5[j,36]  =  w81'y/sumc(w81) ;
else;
 beta =  pinv(xx81'wmat81*xx81)*xx81'wmat81'y ;
 m5[j,36]  =  beta[1,.] ;
endif;
else;
m5[j,36] = 0;
a81 = a81+1;
endif;

if sumc(w82)> 0;
if det(xx82'wmat82*xx82) < epsln ;
      m5[j,37]  =  w82'y/sumc(w82) ;
else;
 beta =  pinv(xx82'wmat82*xx82)*xx82'wmat82'y ;
 m5[j,37]  =  beta[1,.] ;
endif;
else;
m5[j,37] = 0;
a82 = a82+1;
endif;

if sumc(w83)> 0;
if det(xx83'wmat83*xx83) < epsln ;
      m5[j,38]  =  w83'y/sumc(w83) ;
else;
 beta =  pinv(xx83'wmat83*xx83)*xx83'wmat83'y ;
 m5[j,38]  =  beta[1,.] ;
endif;
else;
m5[j,38] = 0;
a83 = a83+1;
endif;

if sumc(w84)> 0;
if det(xx84'wmat84*xx84) < epsln ;
      m5[j,39]  =  w84'y/sumc(w84) ;
else;
 beta =  pinv(xx84'wmat84*xx84)*xx84'wmat84'y ;
 m5[j,39]  =  beta[1,.] ;
endif;
else;
m5[j,39] = 0;
a84 = a84+1;
endif;

if sumc(w85)> 0;
if det(xx85'wmat85*xx85) < epsln ;
      m5[j,40]  =  w85'y/sumc(w85) ;
else;
 beta =  pinv(xx85'wmat85*xx85)*xx85'wmat85'y ;
 m5[j,40]  =  beta[1,.] ;
endif;
else;
m5[j,40] = 0;
a85 = a85+1;
endif;

if sumc(w91)> 0;
if det(xx91'wmat91*xx91) < epsln ;
      m5[j,41]  =  w91'y/sumc(w91) ;
else;
 beta =  pinv(xx91'wmat91*xx91)*xx91'wmat91'y ;
 m5[j,41]  =  beta[1,.] ;
endif;
else;
m5[j,41] = 0;
a91 = a91+1;
endif;

if sumc(w92)> 0;
if det(xx92'wmat92*xx92) < epsln ;
      m5[j,42]  =  w92'y/sumc(w92) ;
else;
 beta =  pinv(xx92'wmat92*xx92)*xx92'wmat92'y ;
 m5[j,42]  =  beta[1,.] ;
endif;
else;
m5[j,42] = 0;
a92 = a92+1;
endif;

if sumc(w93)> 0;
if det(xx93'wmat93*xx93) < epsln ;
      m5[j,43]  =  w93'y/sumc(w93) ;
else;
 beta =  pinv(xx93'wmat93*xx93)*xx93'wmat93'y ;
 m5[j,43]  =  beta[1,.] ;
endif;
else;
m5[j,43] = 0;
a93 = a93+1;
endif;

if sumc(w94)> 0;
if det(xx94'wmat94*xx94) < epsln ;
      m5[j,44]  =  w94'y/sumc(w94) ;
else;
 beta =  pinv(xx94'wmat94*xx94)*xx94'wmat94'y ;
 m5[j,44]  =  beta[1,.] ;
endif;
else;
m5[j,44] = 0;
a94 = a94+1;
endif;


if sumc(w95)> 0;
if det(xx95'wmat95*xx95) < epsln ;
      m5[j,45]  =  w95'y/sumc(w95) ;
else;
 beta =  pinv(xx95'wmat95*xx95)*xx95'wmat95'y ;
 m5[j,45]  =  beta[1,.] ;
endif;
else;
m5[j,45] = 0;
a95 = a95+1;
endif;

if sumc(w101)> 0;
if det(xx101'wmat101*xx101) < epsln ;
      m5[j,46]  =  w101'y/sumc(w101) ;
else;
 beta =  pinv(xx101'wmat101*xx101)*xx101'wmat101'y ;
 m5[j,46]  =  beta[1,.] ;
endif;
else;
m5[j,46] = 0;
a101 = a101+1;
endif;

if sumc(w102)> 0;
if det(xx102'wmat102*xx102) < epsln ;
      m5[j,47]  =  w102'y/sumc(w102) ;
else;
 beta =  pinv(xx102'wmat102*xx102)*xx102'wmat102'y ;
 m5[j,47]  =  beta[1,.] ;
endif;
else;
m5[j,47] = 0;
a102 = a102+1;
endif;

if sumc(w103)> 0;
if det(xx103'wmat103*xx103) < epsln ;
      m5[j,48]  =  w103'y/sumc(w103) ;
else;
 beta =  pinv(xx103'wmat103*xx103)*xx103'wmat103'y ;
 m5[j,48]  =  beta[1,.] ;
endif;
else;
m5[j,48] = 0;
a103 = a103+1;
endif;

if sumc(w104)> 0;
if det(xx104'wmat104*xx104) < epsln ;
      m5[j,49]  =  w104'y/sumc(w104) ;
else;
 beta =  pinv(xx104'wmat104*xx104)*xx104'wmat104'y ;
 m5[j,49]  =  beta[1,.] ;
endif;
else;
m5[j,49] = 0;
a104 = a104+1;
endif;

if sumc(w105)> 0;
if det(xx105'wmat105*xx105) < epsln ;
      m5[j,50]  =  w105'y/sumc(w105) ;
else;
 beta =  pinv(xx105'wmat105*xx105)*xx105'wmat105'y ;
 m5[j,50]  =  beta[1,.] ;
endif;
else;
m5[j,50] = 0;
a105 = a105+1;
endif;



if sumc(w111)> 0;
if det(xx111'wmat111*xx111) < epsln ;
      m5[j,51]  =  w111'y/sumc(w111) ;
else;
 beta =  pinv(xx111'wmat111*xx111)*xx111'wmat111'y ;
 m5[j,51]  =  beta[1,.] ;
endif;
else;
m5[j,51] = 0;
a111 = a111+1;
endif;

if sumc(w112)> 0;
if det(xx112'wmat112*xx112) < epsln ;
      m5[j,52]  =  w112'y/sumc(w112) ;
else;
 beta =  pinv(xx112'wmat112*xx112)*xx112'wmat112'y ;
 m5[j,52]  =  beta[1,.] ;
endif;
else;
m5[j,52] = 0;
a112 = a112+1;
endif;

if sumc(w113)> 0;
if det(xx113'wmat113*xx113) < epsln ;
      m5[j,53]  =  w113'y/sumc(w113) ;
else;
 beta =  pinv(xx113'wmat113*xx113)*xx113'wmat113'y ;
 m5[j,53]  =  beta[1,.] ;
endif;
else;
m5[j,53] = 0;
a113 = a113+1;
endif;

if sumc(w114)> 0;
if det(xx114'wmat114*xx114) < epsln ;
      m5[j,54]  =  w114'y/sumc(w114) ;
else;
 beta =  pinv(xx114'wmat114*xx114)*xx114'wmat114'y ;
 m5[j,54]  =  beta[1,.] ;
endif;
else;
m5[j,54] = 0;
a114 = a114+1;
endif;

if sumc(w115)> 0;
if det(xx115'wmat115*xx115) < epsln ;
      m5[j,55]  =  w115'y/sumc(w115) ;
else;
 beta =  pinv(xx115'wmat115*xx115)*xx115'wmat115'y ;
 m5[j,55]  =  beta[1,.] ;
endif;
else;
m5[j,55] = 0;
a115 = a115+1;
endif;


if sumc(w121)> 0;
if det(xx121'wmat121*xx121) < epsln ;
      m5[j,56]  =  w121'y/sumc(w121) ;
else;
 beta =  pinv(xx121'wmat121*xx121)*xx121'wmat121'y ;
 m5[j,56]  =  beta[1,.] ;
endif;
else;
m5[j,56] = 0;
a121 = a121+1;
endif;

if sumc(w122)> 0;
if det(xx122'wmat122*xx122) < epsln ;
      m5[j,57]  =  w122'y/sumc(w122) ;
else;
 beta =  pinv(xx122'wmat122*xx122)*xx122'wmat122'y ;
 m5[j,57]  =  beta[1,.] ;
endif;
else;
m5[j,57] = 0;
a122 = a122+1;
endif;

if sumc(w123)> 0;
if det(xx123'wmat123*xx123) < epsln ;
      m5[j,58]  =  w123'y/sumc(w123) ;
else;
 beta =  pinv(xx123'wmat123*xx123)*xx123'wmat123'y ;
 m5[j,58]  =  beta[1,.] ;
endif;
else;
m5[j,58] = 0;
a123 = a123+1;
endif;

if sumc(w124)> 0;
if det(xx124'wmat124*xx124) < epsln ;
      m5[j,59]  =  w124'y/sumc(w124) ;
else;
 beta =  pinv(xx124'wmat124*xx124)*xx124'wmat124'y ;
 m5[j,59]  =  beta[1,.] ;
endif;
else;
m5[j,59] = 0;
a124 = a124+1;
endif;

if sumc(w125)> 0;
if det(xx125'wmat125*xx125) < epsln ;
      m5[j,60]  =  w125'y/sumc(w125) ;
else;
 beta =  pinv(xx125'wmat125*xx125)*xx125'wmat125'y ;
 m5[j,60]  =  beta[1,.] ;
endif;
else;
m5[j,60] = 0;
a125 = a125+1;
endif;

if sumc(w131)> 0;
if det(xx131'wmat131*xx131) < epsln ;
      m5[j,61]  =  w131'y/sumc(w131) ;
else;
 beta =  pinv(xx131'wmat131*xx131)*xx131'wmat131'y ;
 m5[j,61]  =  beta[1,.] ;
endif;
else;
m5[j,61] = 0;
a131 = a131+1;
endif;

if sumc(w132)> 0;
if det(xx132'wmat132*xx132) < epsln ;
      m5[j,62]  =  w132'y/sumc(w132) ;
else;
 beta =  pinv(xx132'wmat132*xx132)*xx132'wmat132'y ;
 m5[j,62]  =  beta[1,.] ;
endif;
else;
m5[j,62] = 0;
a132 = a132+1;
endif;

if sumc(w133)> 0;
if det(xx133'wmat133*xx133) < epsln ;
      m5[j,63]  =  w133'y/sumc(w133) ;
else;
 beta =  pinv(xx133'wmat133*xx133)*xx133'wmat133'y ;
 m5[j,63]  =  beta[1,.] ;
endif;
else;
m5[j,63] = 0;
a133 = a133+1;
endif;

if sumc(w134)> 0;
if det(xx134'wmat134*xx134) < epsln ;
      m5[j,64]  =  w134'y/sumc(w134) ;
else;
 beta =  pinv(xx134'wmat134*xx134)*xx134'wmat134'y ;
 m5[j,64]  =  beta[1,.] ;
endif;
else;
m5[j,64] = 0;
a134 = a134+1;
endif;

if sumc(w135)> 0;
if det(xx135'wmat135*xx135) < epsln ;
      m5[j,65]  =  w135'y/sumc(w135) ;
else;
 beta =  pinv(xx135'wmat135*xx135)*xx135'wmat135'y ;
 m5[j,65]  =  beta[1,.] ;
endif;
else;
m5[j,65] = 0;
a135 = a135+1;
endif;

j = j+1;
endo;

m5d[i,1] = 1/(T-a11)*sumc(m5[.,1]);
m5d[i,2] = 1/(T-a12)*sumc(m5[.,2]);
m5d[i,3] = 1/(T-a13)*sumc(m5[.,3]);
m5d[i,4] = 1/(T-a14)*sumc(m5[.,4]);
m5d[i,5] = 1/(T-a15)*sumc(m5[.,5]);
m5d[i,6] = 1/(T-a21)*sumc(m5[.,6]);
m5d[i,7] = 1/(T-a22)*sumc(m5[.,7]);
m5d[i,8] = 1/(T-a23)*sumc(m5[.,8]);
m5d[i,9] = 1/(T-a24)*sumc(m5[.,9]);
m5d[i,10] = 1/(T-a25)*sumc(m5[.,10]);
m5d[i,11] = 1/(T-a31)*sumc(m5[.,11]);
m5d[i,12] = 1/(T-a32)*sumc(m5[.,12]);
m5d[i,13] = 1/(T-a33)*sumc(m5[.,13]);
m5d[i,14] = 1/(T-a34)*sumc(m5[.,14]);
m5d[i,15] = 1/(T-a35)*sumc(m5[.,15]);
m5d[i,16] = 1/(T-a41)*sumc(m5[.,16]);
m5d[i,17] = 1/(T-a42)*sumc(m5[.,17]);
m5d[i,18] = 1/(T-a43)*sumc(m5[.,18]);
m5d[i,19] = 1/(T-a44)*sumc(m5[.,19]);
m5d[i,20] = 1/(T-a45)*sumc(m5[.,20]);
m5d[i,21] = 1/(T-a51)*sumc(m5[.,21]);
m5d[i,22] = 1/(T-a52)*sumc(m5[.,22]);
m5d[i,23] = 1/(T-a53)*sumc(m5[.,23]);
m5d[i,24] = 1/(T-a54)*sumc(m5[.,24]);
m5d[i,25] = 1/(T-a55)*sumc(m5[.,25]);
m5d[i,26] = 1/(T-a61)*sumc(m5[.,26]);
m5d[i,27] = 1/(T-a62)*sumc(m5[.,27]);
m5d[i,28] = 1/(T-a63)*sumc(m5[.,28]);
m5d[i,29] = 1/(T-a64)*sumc(m5[.,29]);
m5d[i,30] = 1/(T-a65)*sumc(m5[.,30]);
m5d[i,31] = 1/(T-a71)*sumc(m5[.,31]);
m5d[i,32] = 1/(T-a72)*sumc(m5[.,32]);
m5d[i,33] = 1/(T-a73)*sumc(m5[.,33]);
m5d[i,34] = 1/(T-a74)*sumc(m5[.,34]);
m5d[i,35] = 1/(T-a75)*sumc(m5[.,35]);
m5d[i,36] = 1/(T-a81)*sumc(m5[.,36]);
m5d[i,37] = 1/(T-a82)*sumc(m5[.,37]);
m5d[i,38] = 1/(T-a83)*sumc(m5[.,38]);
m5d[i,39] = 1/(T-a84)*sumc(m5[.,39]);
m5d[i,40] = 1/(T-a85)*sumc(m5[.,40]);
m5d[i,41] = 1/(T-a91)*sumc(m5[.,41]);
m5d[i,42] = 1/(T-a92)*sumc(m5[.,42]);
m5d[i,43] = 1/(T-a93)*sumc(m5[.,43]);
m5d[i,44] = 1/(T-a94)*sumc(m5[.,44]);
m5d[i,45] = 1/(T-a95)*sumc(m5[.,45]);
m5d[i,46] = 1/(T-a101)*sumc(m5[.,46]);
m5d[i,47] = 1/(T-a102)*sumc(m5[.,47]);
m5d[i,48] = 1/(T-a103)*sumc(m5[.,48]);
m5d[i,49] = 1/(T-a104)*sumc(m5[.,49]);
m5d[i,50] = 1/(T-a105)*sumc(m5[.,50]);
m5d[i,51] = 1/(T-a111)*sumc(m5[.,51]);
m5d[i,52] = 1/(T-a112)*sumc(m5[.,52]);
m5d[i,53] = 1/(T-a113)*sumc(m5[.,53]);
m5d[i,54] = 1/(T-a114)*sumc(m5[.,54]);
m5d[i,55] = 1/(T-a115)*sumc(m5[.,55]);
m5d[i,56] = 1/(T-a121)*sumc(m5[.,56]);
m5d[i,57] = 1/(T-a122)*sumc(m5[.,57]);
m5d[i,58] = 1/(T-a123)*sumc(m5[.,58]);
m5d[i,59] = 1/(T-a124)*sumc(m5[.,59]);
m5d[i,60] = 1/(T-a125)*sumc(m5[.,60]);
m5d[i,61] = 1/(T-a121)*sumc(m5[.,61]);
m5d[i,62] = 1/(T-a122)*sumc(m5[.,62]);
m5d[i,63] = 1/(T-a123)*sumc(m5[.,63]);
m5d[i,64] = 1/(T-a124)*sumc(m5[.,64]);
m5d[i,65] = 1/(T-a125)*sumc(m5[.,65]);
i = i+1;
endo;
yhat1 = y-meanc(y)-(m5d[.,1]-meanc(m5d[.,1])+m5d[.,2]-meanc(m5d[.,2])+m5d[.,3]-meanc(m5d[.,3])+m5d[.,4]-meanc(m5d[.,4])+m5d[.,5]-meanc(m5d[.,5]));
yhat2 = y-meanc(y)-(m5d[.,6]-meanc(m5d[.,6])+m5d[.,7]-meanc(m5d[.,7])+m5d[.,8]-meanc(m5d[.,8])+m5d[.,9]-meanc(m5d[.,9])+m5d[.,10]-meanc(m5d[.,10]));
yhat3 = y-meanc(y)-(m5d[.,11]-meanc(m5d[.,11])+m5d[.,12]-meanc(m5d[.,12])+m5d[.,13]-meanc(m5d[.,13])+m5d[.,14]-meanc(m5d[.,14])+m5d[.,15]-meanc(m5d[.,15]));
yhat4 = y-meanc(y)-(m5d[.,16]-meanc(m5d[.,16])+m5d[.,17]-meanc(m5d[.,17])+m5d[.,18]-meanc(m5d[.,18])+m5d[.,19]-meanc(m5d[.,19])+m5d[.,20]-meanc(m5d[.,20]));
yhat5 = y-meanc(y)-(m5d[.,21]-meanc(m5d[.,21])+m5d[.,22]-meanc(m5d[.,22])+m5d[.,23]-meanc(m5d[.,23])+m5d[.,24]-meanc(m5d[.,24])+m5d[.,25]-meanc(m5d[.,25]));
yhat6 = y-meanc(y)-(m5d[.,26]-meanc(m5d[.,26])+m5d[.,27]-meanc(m5d[.,27])+m5d[.,28]-meanc(m5d[.,28])+m5d[.,29]-meanc(m5d[.,29])+m5d[.,30]-meanc(m5d[.,30]));
yhat7 = y-meanc(y)-(m5d[.,31]-meanc(m5d[.,31])+m5d[.,32]-meanc(m5d[.,32])+m5d[.,33]-meanc(m5d[.,33])+m5d[.,34]-meanc(m5d[.,34])+m5d[.,35]-meanc(m5d[.,35]));
yhat8 = y-meanc(y)-(m5d[.,36]-meanc(m5d[.,36])+m5d[.,37]-meanc(m5d[.,37])+m5d[.,38]-meanc(m5d[.,38])+m5d[.,39]-meanc(m5d[.,39])+m5d[.,40]-meanc(m5d[.,40]));
yhat9 = y-meanc(y)-(m5d[.,41]-meanc(m5d[.,41])+m5d[.,42]-meanc(m5d[.,42])+m5d[.,43]-meanc(m5d[.,43])+m5d[.,44]-meanc(m5d[.,44])+m5d[.,45]-meanc(m5d[.,45]));
yhat10 = y-meanc(y)-(m5d[.,46]-meanc(m5d[.,46])+m5d[.,47]-meanc(m5d[.,47])+m5d[.,48]-meanc(m5d[.,48])+m5d[.,49]-meanc(m5d[.,49])+m5d[.,50]-meanc(m5d[.,50]));
yhat11 = y-meanc(y)-(m5d[.,51]-meanc(m5d[.,51])+m5d[.,52]-meanc(m5d[.,52])+m5d[.,53]-meanc(m5d[.,53])+m5d[.,54]-meanc(m5d[.,54])+m5d[.,55]-meanc(m5d[.,55]));
yhat12 = y-meanc(y)-(m5d[.,56]-meanc(m5d[.,56])+m5d[.,57]-meanc(m5d[.,57])+m5d[.,58]-meanc(m5d[.,58])+m5d[.,59]-meanc(m5d[.,59])+m5d[.,60]-meanc(m5d[.,60]));
yhat13 = y-meanc(y)-(m5d[.,61]-meanc(m5d[.,61])+m5d[.,62]-meanc(m5d[.,62])+m5d[.,63]-meanc(m5d[.,63])+m5d[.,64]-meanc(m5d[.,64])+m5d[.,65]-meanc(m5d[.,65]));
R1 = yhat1'yhat1;
R2 = yhat2'yhat2;
R3 = yhat3'yhat3;
R4 = yhat4'yhat4;
R5 = yhat5'yhat5;
R6 = yhat6'yhat6;
R7 = yhat7'yhat7;
R8 = yhat8'yhat8;
R9 = yhat9'yhat9;
R10 = yhat10'yhat10;
R11 = yhat11'yhat11;
R12 = yhat12'yhat12;
R13 = yhat13'yhat13;
Ahat1[k,kc] = 1/T*sumc(R1);
Ahat2[k,kc] = 1/T*sumc(R2);
Ahat3[k,kc] = 1/T*sumc(R3);
Ahat4[k,kc] = 1/T*sumc(R4);
Ahat5[k,kc] = 1/T*sumc(R5);
Ahat6[k,kc] = 1/T*sumc(R6);
Ahat7[k,kc] = 1/T*sumc(R7);
Ahat8[k,kc] = 1/T*sumc(R8);
Ahat9[k,kc] = 1/T*sumc(R9);
Ahat10[k,kc] = 1/T*sumc(R10);
Ahat11[k,kc] = 1/T*sumc(R11);
Ahat12[k,kc] = 1/T*sumc(R12);
Ahat13[k,kc] = 1/T*sumc(R13);
Bhat1[k,kc]  =  2/b5*1/sqrt(2*pi)*1/T*sumc((BR1.*qz5[.,1]+BR1.*qz5[.,2]+BR1.*qz5[.,3]+BR1.*qz5[.,4]+BR1.*qz5[.,5])./phat[.,1])*1/T;
Bhat2[k,kc]  =  2/b5*1/sqrt(2*pi)*1/T*sumc((BR2.*qz5[.,1]+BR2.*qz5[.,6]+BR2.*qz5[.,7]+BR2.*qz5[.,8]+BR2.*qz5[.,9])./phat[.,2])*1/T;
Bhat3[k,kc]  =  2/b5*1/sqrt(2*pi)*1/T*sumc((BR3.*qz5[.,1]+BR3.*qz5[.,10]+BR3.*qz5[.,11]+BR3.*qz5[.,12]+BR3.*qz5[.,13])./phat[.,3])*1/T;
Bhat4[k,kc]  =  2/b5*1/sqrt(2*pi)*1/T*sumc((BR4.*qz5[.,1]+BR4.*qz5[.,14]+BR4.*qz5[.,15]+BR4.*qz5[.,16]+BR4.*qz5[.,17])./phat[.,4])*1/T;
Bhat5[k,kc]  =  2/b5*1/sqrt(2*pi)*1/T*sumc((BR5.*qz5[.,1]+BR5.*qz5[.,18]+BR5.*qz5[.,19]+BR5.*qz5[.,20]+BR5.*qz5[.,21])./phat[.,5])*1/T;
Bhat6[k,kc]  =  2/b5*1/sqrt(2*pi)*1/T*sumc((BR6.*qz5[.,1]+BR6.*qz5[.,22]+BR6.*qz5[.,23]+BR6.*qz5[.,24]+BR6.*qz5[.,25])./phat[.,6])*1/T;
Bhat7[k,kc]  =  2/b5*1/sqrt(2*pi)*1/T*sumc((BR7.*qz5[.,1]+BR7.*qz5[.,26]+BR7.*qz5[.,27]+BR7.*qz5[.,28]+BR7.*qz5[.,29])./phat[.,7])*1/T;
Bhat8[k,kc]  =  2/b5*1/sqrt(2*pi)*1/T*sumc((BR8.*qz5[.,1]+BR8.*qz5[.,30]+BR8.*qz5[.,31]+BR8.*qz5[.,32]+BR8.*qz5[.,33])./phat[.,8])*1/T;
Bhat9[k,kc]  =  2/b5*1/sqrt(2*pi)*1/T*sumc((BR9.*qz5[.,1]+BR9.*qz5[.,34]+BR9.*qz5[.,35]+BR9.*qz5[.,36]+BR9.*qz5[.,37])./phat[.,9])*1/T;
Bhat10[k,kc] =  2/b5*1/sqrt(2*pi)*1/T*sumc((BR10.*qz5[.,1]+BR10.*qz5[.,38]+BR10.*qz5[.,39]+BR10.*qz5[.,40]+BR10.*qz5[.,41])./phat[.,10])*1/T;
Bhat11[k,kc]  =  2/b5*1/sqrt(2*pi)*1/T*sumc((BR11.*qz5[.,1]+BR11.*qz5[.,42]+BR11.*qz5[.,43]+BR11.*qz5[.,44]+BR11.*qz5[.,45])./phat[.,11])*1/T;
Bhat12[k,kc]  =  2/b5*1/sqrt(2*pi)*1/T*sumc((BR12.*qz5[.,1]+BR12.*qz5[.,46]+BR12.*qz5[.,47]+BR12.*qz5[.,48]+BR12.*qz5[.,49])./phat[.,12])*1/T;
Bhat13[k,kc]  =  2/b5*1/sqrt(2*pi)*1/T*sumc((BR13.*qz5[.,1]+BR13.*qz5[.,50]+BR13.*qz5[.,51]+BR13.*qz5[.,52]+BR13.*qz5[.,53])./phat[.,13])*1/T;
FPE1[k,kc] = Ahat1[k,kc]+Bhat1[k,kc];
FPE2[k,kc] = Ahat2[k,kc]+Bhat2[k,kc];
FPE3[k,kc] = Ahat3[k,kc]+Bhat3[k,kc];
FPE4[k,kc] = Ahat4[k,kc]+Bhat4[k,kc];
FPE5[k,kc] = Ahat5[k,kc]+Bhat5[k,kc];
FPE6[k,kc] = Ahat6[k,kc]+Bhat6[k,kc];
FPE7[k,kc] = Ahat7[k,kc]+Bhat7[k,kc];
FPE8[k,kc] = Ahat8[k,kc]+Bhat8[k,kc];
FPE9[k,kc] = Ahat9[k,kc]+Bhat9[k,kc];
FPE10[k,kc] = Ahat10[k,kc]+Bhat10[k,kc];
FPE11[k,kc] = Ahat11[k,kc]+Bhat11[k,kc];
FPE12[k,kc] = Ahat12[k,kc]+Bhat12[k,kc];
FPE13[k,kc] = Ahat13[k,kc]+Bhat13[k,kc];
kc = kc+1;
endo;
cmin1 = minc(FPE1[k,.]');
cmin2 = minc(FPE2[k,.]');
cmin3 = minc(FPE3[k,.]');
cmin4 = minc(FPE4[k,.]');
cmin5 = minc(FPE5[k,.]');
cmin6 = minc(FPE6[k,.]');
cmin7 = minc(FPE7[k,.]');
cmin8 = minc(FPE8[k,.]');
cmin9 = minc(FPE9[k,.]');
cmin10 = minc(FPE10[k,.]');
cmin11 = minc(FPE11[k,.]');
cmin12 = minc(FPE12[k,.]');
cmin13 = minc(FPE13[k,.]');
minFPE = cmin1~cmin2~cmin3~cmin4~cmin5~cmin6~cmin7~cmin8~cmin9~cmin10~cmin11~cmin12~cmin13;
minFPE[.,lag_1] = 10^10;
minFPE[.,lag_2] = 10^10;
minFPE[.,lag_3] = 10^10;
minFPE[.,lag_4] = 10^10;
AFPE[k,5] = minc(minFPE');
lag[k,5] = minindc(minFPE');
lag_5 = lag[k,5];

/****************************Six Lags Included ******************************************/
       
/****************************Six Lags Included Estimation of B part *******************************************/

i = 1;
a11 = 0;
a12 = 0;
a13 = 0;
a14 = 0;
a15 = 0;
a16 = 0;
a21 = 0;
a22 = 0;
a23 = 0;
a24 = 0;
a25 = 0;
a26 = 0;
a31 = 0;
a32 = 0;
a33 = 0;
a34 = 0;
a35 = 0;
a36 = 0;
a41 = 0;
a42 = 0;
a43 = 0;
a44 = 0;
a45 = 0;
a46 = 0;
a51 = 0;
a52 = 0;
a53 = 0;
a54 = 0;
a55 = 0;
a56 = 0;
a61 = 0;
a62 = 0;
a63 = 0;
a64 = 0;
a65 = 0;
a66 = 0;
a71 = 0;
a72 = 0;
a73 = 0;
a74 = 0;
a75 = 0;
a76 = 0;
a81 = 0;
a82 = 0;
a83 = 0;
a84 = 0;
a85 = 0;
a86 = 0;
a91 = 0;
a92 = 0;
a93 = 0;
a94 = 0;
a95 = 0;
a96 = 0;
a101 = 0;
a102 = 0;
a103 = 0;
a104 = 0;
a105 = 0;
a106 = 0;
a111 = 0;
a112 = 0;
a113 = 0;
a114 = 0;
a115 = 0;
a116 = 0;
a121 = 0;
a122 = 0;
a123 = 0;
a124 = 0;
a125 = 0;
a126 = 0;
a131 = 0;
a132 = 0;
a133 = 0;
a134 = 0;
a135 = 0;
a136 = 0;
m6 = zeros(T,78);
m6d = zeros(T,78);
qz6 = zeros(T,78);
i = 1;
do until i>T;
u01i = data[i,lag_1]*ones(T,1) - data[.,lag_1];
u02i = data[i,lag_2]*ones(T,1) - data[.,lag_2];
u03i = data[i,lag_3]*ones(T,1) - data[.,lag_3];
u04i = data[i,lag_4]*ones(T,1) - data[.,lag_4];
u05i = data[i,lag_5]*ones(T,1) - data[.,lag_5];
u0i = u01i~u02i~u03i~u04i~u05i;
u1i =  data[i,1]*ones(T,1) - data[.,1];
u2i =  data[i,2]*ones(T,1) - data[.,2];
u3i =  data[i,3]*ones(T,1) - data[.,3];
u4i =  data[i,4]*ones(T,1) - data[.,4];
u5i =  data[i,5]*ones(T,1) - data[.,5];
u6i =  data[i,6]*ones(T,1) - data[.,6];
u7i =  data[i,7]*ones(T,1) - data[.,7];
u8i =  data[i,8]*ones(T,1) - data[.,8];
u9i =  data[i,9]*ones(T,1) - data[.,9];
u10i =  data[i,10]*ones(T,1) - data[.,10];
u11i =  data[i,11]*ones(T,1) - data[.,11];
u12i =  data[i,12]*ones(T,1) - data[.,12];
u13i =  data[i,13]*ones(T,1) - data[.,13];
j = 1;
do until j>T;
u01j = data[j,lag_1]*ones(T,1) - data[.,lag_1];
u02j = data[j,lag_2]*ones(T,1) - data[.,lag_2];
u03j = data[j,lag_3]*ones(T,1) - data[.,lag_3];
u04j = data[j,lag_4]*ones(T,1) - data[.,lag_4];
u05j = data[j,lag_5]*ones(T,1) - data[.,lag_5];
u0j = u01j~u02j~u03j~u04j~u05j;
 u1j =  (data[j,1]*ones(T,1) - data[.,1]);
 u2j =  (data[j,2]*ones(T,1) - data[.,2]);
 u3j =  (data[j,3]*ones(T,1) - data[.,3]);
 u4j =  (data[j,4]*ones(T,1) - data[.,4]);
 u5j =  (data[j,5]*ones(T,1) - data[.,5]);
 u6j =  (data[j,6]*ones(T,1) - data[.,6]);
 u7j =  (data[j,7]*ones(T,1) - data[.,7]);
 u8j =  (data[j,8]*ones(T,1) - data[.,8]);
 u9j =  (data[j,9]*ones(T,1) - data[.,9]);
 u10j =  (data[j,10]*ones(T,1) - data[.,10]);
 u11j =  (data[j,11]*ones(T,1) - data[.,11]);
 u12j =  (data[j,12]*ones(T,1) - data[.,12]);
 u13j =  (data[j,13]*ones(T,1) - data[.,13]);
 w11 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u1j/h1)/h1);
 w12 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u1j/h1)/h1);
 w13 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03i/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u1j/h1)/h1);
 w14 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04i/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u1j/h1)/h1);
 w15 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05i/h1)/h1).*(fi(u1j/h1)/h1);
 w16 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u1i/h1)/h1);
 w21 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u2j/h1)/h1);
 w22 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u2j/h1)/h1);
 w23 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03i/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u2j/h1)/h1);
 w24 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04i/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u2j/h1)/h1);
 w25 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05i/h1)/h1).*(fi(u2j/h1)/h1);
 w26 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u2i/h1)/h1);
 w31 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u3j/h1)/h1);
 w32 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u3j/h1)/h1);
 w33 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03i/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u3j/h1)/h1);
 w34 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04i/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u3j/h1)/h1);
 w35 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05i/h1)/h1).*(fi(u3j/h1)/h1);
 w36 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u3i/h1)/h1);
 w41 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u4j/h1)/h1);
 w42 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u4j/h1)/h1);
 w43 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03i/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u4j/h1)/h1);
 w44 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04i/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u4j/h1)/h1);
 w45 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05i/h1)/h1).*(fi(u4j/h1)/h1);
 w46 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u4i/h1)/h1);
 w51 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u5j/h1)/h1);
 w52 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u5j/h1)/h1);
 w53 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03i/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u5j/h1)/h1);
 w54 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04i/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u5j/h1)/h1);
 w55 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05i/h1)/h1).*(fi(u5j/h1)/h1);
 w56 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u5i/h1)/h1);
 w61 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u6j/h1)/h1);
 w62 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u6j/h1)/h1);
 w63 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03i/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u6j/h1)/h1);
 w64 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04i/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u6j/h1)/h1);
 w65 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05i/h1)/h1).*(fi(u6j/h1)/h1);
 w66 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u6i/h1)/h1);
 w71 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u7j/h1)/h1);
 w72 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u7j/h1)/h1);
 w73 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03i/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u7j/h1)/h1);
 w74 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04i/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u7j/h1)/h1);
 w75 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05i/h1)/h1).*(fi(u7j/h1)/h1);
 w76 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u7i/h1)/h1);
 w81 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u8j/h1)/h1);
 w82 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u8j/h1)/h1);
 w83 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03i/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u8j/h1)/h1);
 w84 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04i/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u8j/h1)/h1);
 w85 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05i/h1)/h1).*(fi(u8j/h1)/h1);
 w86 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u8i/h1)/h1);
 w91 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u9j/h1)/h1);
 w92 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u9j/h1)/h1);
 w93 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03i/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u9j/h1)/h1);
 w94 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04i/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u9j/h1)/h1);
 w95 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05i/h1)/h1).*(fi(u9j/h1)/h1);
 w96 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u9i/h1)/h1);
 w101 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u10j/h1)/h1);
 w102 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u10j/h1)/h1);
 w103 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03i/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u10j/h1)/h1);
 w104 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04i/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u10j/h1)/h1);
 w105 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05i/h1)/h1).*(fi(u10j/h1)/h1);
 w106 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u10i/h1)/h1);
 w111 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u11j/h1)/h1);
 w112 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u11j/h1)/h1);
 w113 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03i/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u11j/h1)/h1);
 w114 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04i/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u11j/h1)/h1);
 w115 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05i/h1)/h1).*(fi(u11j/h1)/h1);
 w116 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u11i/h1)/h1);
 w121 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u12j/h1)/h1);
 w122 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u12j/h1)/h1);
 w123 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03i/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u12j/h1)/h1);
 w124 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04i/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u12j/h1)/h1);
 w125 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05i/h1)/h1).*(fi(u12j/h1)/h1);
 w126 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u12i/h1)/h1);
 w131 = (fi(u01i/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u13j/h1)/h1);
 w132 = (fi(u01j/h1)/h1).*(fi(u02i/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u13j/h1)/h1);
 w133 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03i/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u13j/h1)/h1);
 w134 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04i/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u13j/h1)/h1);
 w135 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05i/h1)/h1).*(fi(u13j/h1)/h1);
 w136 = (fi(u01j/h1)/h1).*(fi(u02j/h1)/h1).*(fi(u03j/h1)/h1).*(fi(u04j/h1)/h1).*(fi(u05j/h1)/h1).*(fi(u13i/h1)/h1);
 w11[j] = 0;
 w12[j] = 0;
 w13[j] = 0;
 w14[j] = 0;
 w15[j] = 0;
 w16[j] = 0;
 w21[j] = 0;
 w22[j] = 0;
 w23[j] = 0;
 w24[j] = 0;
 w25[j] = 0;
 w26[j] = 0;
 w31[j] = 0;
 w32[j] = 0;
 w33[j] = 0;
 w34[j] = 0;
 w35[j] = 0;
 w36[j] = 0;
 w41[j] = 0;
 w42[j] = 0;
 w43[j] = 0;
 w44[j] = 0;
 w45[j] = 0;
 w46[j] = 0;
 w51[j] = 0;
 w52[j] = 0;
 w53[j] = 0;
 w54[j] = 0;
 w55[j] = 0;
 w56[j] = 0;
 w61[j] = 0;
 w62[j] = 0;
 w63[j] = 0;
 w64[j] = 0;
 w65[j] = 0;
 w66[j] = 0;
 w71[j] = 0;
 w72[j] = 0;
 w73[j] = 0;
 w74[j] = 0;
 w75[j] = 0;
 w76[j] = 0;
 w81[j] = 0;
 w82[j] = 0;
 w83[j] = 0;
 w84[j] = 0;
 w85[j] = 0;
 w86[j] = 0;
 w91[j] = 0;
 w92[j] = 0;
 w93[j] = 0;
 w94[j] = 0;
 w95[j] = 0;
 w96[j] = 0;
 w101[j] = 0;
 w102[j] = 0;
 w103[j] = 0;
 w104[j] = 0;
 w105[j] = 0;
 w106[j] = 0;
 w111[j] = 0;
 w112[j] = 0;
 w113[j] = 0;
 w114[j] = 0;
 w115[j] = 0;
 w116[j] = 0;
 w121[j] = 0;
 w122[j] = 0;
 w123[j] = 0;
 w124[j] = 0;
 w125[j] = 0;
 w126[j] = 0;
 w131[j] = 0;
 w132[j] = 0;
 w133[j] = 0;
 w134[j] = 0;
 w135[j] = 0;
 w136[j] = 0;
 wmat11  = diagrv((eye(T),w11));
 wmat12  = diagrv((eye(T),w12));
 wmat13  = diagrv((eye(T),w13));
 wmat14  = diagrv((eye(T),w14));
 wmat15  = diagrv((eye(T),w15));
 wmat16  = diagrv((eye(T),w16));
 wmat21  = diagrv((eye(T),w21));
 wmat22  = diagrv((eye(T),w22));
 wmat23  = diagrv((eye(T),w23));
 wmat24  = diagrv((eye(T),w24));
 wmat25  = diagrv((eye(T),w25));
 wmat26  = diagrv((eye(T),w26));
 wmat31  = diagrv((eye(T),w31));
 wmat32  = diagrv((eye(T),w32));
 wmat33  = diagrv((eye(T),w33));
 wmat34  = diagrv((eye(T),w34));
 wmat35  = diagrv((eye(T),w35));
 wmat36  = diagrv((eye(T),w36));
 wmat41  = diagrv((eye(T),w41));
 wmat42  = diagrv((eye(T),w42));
 wmat43  = diagrv((eye(T),w43));
 wmat44  = diagrv((eye(T),w44));
 wmat45  = diagrv((eye(T),w45));
 wmat46  = diagrv((eye(T),w46));
 wmat51  = diagrv((eye(T),w51));
 wmat52  = diagrv((eye(T),w52));
 wmat53  = diagrv((eye(T),w53));
 wmat54  = diagrv((eye(T),w54));
 wmat55  = diagrv((eye(T),w55));
 wmat56  = diagrv((eye(T),w56));
 wmat61  = diagrv((eye(T),w61));
 wmat62  = diagrv((eye(T),w62));
 wmat63  = diagrv((eye(T),w63));
 wmat64  = diagrv((eye(T),w64));
 wmat65  = diagrv((eye(T),w65));
 wmat66  = diagrv((eye(T),w66));
 wmat71  = diagrv((eye(T),w71));
 wmat72  = diagrv((eye(T),w72));
 wmat73  = diagrv((eye(T),w73));
 wmat74  = diagrv((eye(T),w74));
 wmat75  = diagrv((eye(T),w75));
 wmat76  = diagrv((eye(T),w76));
 wmat81  = diagrv((eye(T),w81));
 wmat82  = diagrv((eye(T),w82));
 wmat83  = diagrv((eye(T),w83));
 wmat84  = diagrv((eye(T),w84));
 wmat85  = diagrv((eye(T),w85));
 wmat86  = diagrv((eye(T),w86));
 wmat91  = diagrv((eye(T),w91));
 wmat92  = diagrv((eye(T),w92));
 wmat93  = diagrv((eye(T),w93));
 wmat94  = diagrv((eye(T),w94));
 wmat95  = diagrv((eye(T),w95));
 wmat96  = diagrv((eye(T),w96));
 wmat101  = diagrv((eye(T),w101));
 wmat102  = diagrv((eye(T),w102));
 wmat103  = diagrv((eye(T),w103));
 wmat104  = diagrv((eye(T),w104));
 wmat105  = diagrv((eye(T),w105));
 wmat106  = diagrv((eye(T),w106));
 wmat111  = diagrv((eye(T),w111));
 wmat112  = diagrv((eye(T),w112));
 wmat113  = diagrv((eye(T),w113));
 wmat114  = diagrv((eye(T),w114));
 wmat115  = diagrv((eye(T),w115));
 wmat116  = diagrv((eye(T),w116));
 wmat121  = diagrv((eye(T),w121));
 wmat122  = diagrv((eye(T),w122));
 wmat123  = diagrv((eye(T),w123));
 wmat124  = diagrv((eye(T),w124));
 wmat125  = diagrv((eye(T),w125));
 wmat126  = diagrv((eye(T),w126));
 wmat131  = diagrv((eye(T),w131));
 wmat132  = diagrv((eye(T),w132));
 wmat133  = diagrv((eye(T),w133));
 wmat134  = diagrv((eye(T),w134));
 wmat135  = diagrv((eye(T),w135));
 wmat136  = diagrv((eye(T),w136));
 xx11  =  ones(T,1)~u01i~u02j~u03j~u04j~u05j~u1j ;
 xx12  =  ones(T,1)~u01j~u02i~u03j~u04j~u05j~u1j ;
 xx13  =  ones(T,1)~u01j~u02j~u03i~u04j~u05j~u1j ;
 xx14  =  ones(T,1)~u01j~u02j~u03j~u04i~u05j~u1j ;
 xx15  =  ones(T,1)~u01j~u02j~u03j~u04j~u05i~u1j;
 xx16  =  ones(T,1)~u01j~u02j~u03j~u04j~u05j~u1i;
 xx21  =  ones(T,1)~u01i~u02j~u03j~u04j~u05j~u2j ;
 xx22  =  ones(T,1)~u01j~u02i~u03j~u04j~u05j~u2j ;
 xx23  =  ones(T,1)~u01j~u02j~u03i~u04j~u05j~u2j ;
 xx24  =  ones(T,1)~u01j~u02j~u03j~u04i~u05j~u2j ;
 xx25  =  ones(T,1)~u01j~u02j~u03j~u04j~u05i~u2j;
 xx26  =  ones(T,1)~u01j~u02j~u03j~u04j~u05j~u2i;
 xx31  =  ones(T,1)~u01i~u02j~u03j~u04j~u05j~u3j ;
 xx32  =  ones(T,1)~u01j~u02i~u03j~u04j~u05j~u3j ;
 xx33  =  ones(T,1)~u01j~u02j~u03i~u04j~u05j~u3j ;
 xx34  =  ones(T,1)~u01j~u02j~u03j~u04i~u05j~u3j ;
 xx35  =  ones(T,1)~u01j~u02j~u03j~u04j~u05i~u3j;
 xx36  =  ones(T,1)~u01j~u02j~u03j~u04j~u05j~u3i;
 xx41  =  ones(T,1)~u01i~u02j~u03j~u04j~u05j~u4j ;
 xx42  =  ones(T,1)~u01j~u02i~u03j~u04j~u05j~u4j ;
 xx43  =  ones(T,1)~u01j~u02j~u03i~u04j~u05j~u4j ;
 xx44  =  ones(T,1)~u01j~u02j~u03j~u04i~u05j~u4j ;
 xx45  =  ones(T,1)~u01j~u02j~u03j~u04j~u05i~u4j;
 xx46  =  ones(T,1)~u01j~u02j~u03j~u04j~u05j~u4i;
 xx51  =  ones(T,1)~u01i~u02j~u03j~u04j~u05j~u5j ;
 xx52  =  ones(T,1)~u01j~u02i~u03j~u04j~u05j~u5j ;
 xx53  =  ones(T,1)~u01j~u02j~u03i~u04j~u05j~u5j ;
 xx54  =  ones(T,1)~u01j~u02j~u03j~u04i~u05j~u5j ;
 xx55  =  ones(T,1)~u01j~u02j~u03j~u04j~u05i~u5j;
 xx56  =  ones(T,1)~u01j~u02j~u03j~u04j~u05j~u5i;
 xx61  =  ones(T,1)~u01i~u02j~u03j~u04j~u05j~u6j ;
 xx62  =  ones(T,1)~u01j~u02i~u03j~u04j~u05j~u6j ;
 xx63  =  ones(T,1)~u01j~u02j~u03i~u04j~u05j~u6j ;
 xx64  =  ones(T,1)~u01j~u02j~u03j~u04i~u05j~u6j ;
 xx65  =  ones(T,1)~u01j~u02j~u03j~u04j~u05i~u6j;
 xx66  =  ones(T,1)~u01j~u02j~u03j~u04j~u05j~u6i;
 xx71  =  ones(T,1)~u01i~u02j~u03j~u04j~u05j~u7j ;
 xx72  =  ones(T,1)~u01j~u02i~u03j~u04j~u05j~u7j ;
 xx73  =  ones(T,1)~u01j~u02j~u03i~u04j~u05j~u7j ;
 xx74  =  ones(T,1)~u01j~u02j~u03j~u04i~u05j~u7j ;
 xx75  =  ones(T,1)~u01j~u02j~u03j~u04j~u05i~u7j;
 xx76  =  ones(T,1)~u01j~u02j~u03j~u04j~u05j~u7i;
 xx81  =  ones(T,1)~u01i~u02j~u03j~u04j~u05j~u8j ;
 xx82  =  ones(T,1)~u01j~u02i~u03j~u04j~u05j~u8j ;
 xx83  =  ones(T,1)~u01j~u02j~u03i~u04j~u05j~u8j ;
 xx84  =  ones(T,1)~u01j~u02j~u03j~u04i~u05j~u8j ;
 xx85  =  ones(T,1)~u01j~u02j~u03j~u04j~u05i~u8j;
 xx86  =  ones(T,1)~u01j~u02j~u03j~u04j~u05j~u8i;
 xx91  =  ones(T,1)~u01i~u02j~u03j~u04j~u05j~u9j ;
 xx92  =  ones(T,1)~u01j~u02i~u03j~u04j~u05j~u9j ;
 xx93  =  ones(T,1)~u01j~u02j~u03i~u04j~u05j~u9j ;
 xx94  =  ones(T,1)~u01j~u02j~u03j~u04i~u05j~u9j ;
 xx95  =  ones(T,1)~u01j~u02j~u03j~u04j~u05i~u9j;
 xx96  =  ones(T,1)~u01j~u02j~u03j~u04j~u05j~u9i;
 xx101  =  ones(T,1)~u01i~u02j~u03j~u04j~u05j~u10j ;
 xx102  =  ones(T,1)~u01j~u02i~u03j~u04j~u05j~u10j ;
 xx103  =  ones(T,1)~u01j~u02j~u03i~u04j~u05j~u10j ;
 xx104  =  ones(T,1)~u01j~u02j~u03j~u04i~u05j~u10j ;
 xx105  =  ones(T,1)~u01j~u02j~u03j~u04j~u05i~u10j;
 xx106  =  ones(T,1)~u01j~u02j~u03j~u04j~u05j~u10i;
 xx111  =  ones(T,1)~u01i~u02j~u03j~u04j~u05j~u11j ;
 xx112  =  ones(T,1)~u01j~u02i~u03j~u04j~u05j~u11j ;
 xx113  =  ones(T,1)~u01j~u02j~u03i~u04j~u05j~u11j ;
 xx114  =  ones(T,1)~u01j~u02j~u03j~u04i~u05j~u11j ;
 xx115  =  ones(T,1)~u01j~u02j~u03j~u04j~u05i~u11j;
 xx116  =  ones(T,1)~u01j~u02j~u03j~u04j~u05j~u11i;
 xx121  =  ones(T,1)~u01i~u02j~u03j~u04j~u05j~u12j ;
 xx122  =  ones(T,1)~u01j~u02i~u03j~u04j~u05j~u12j ;
 xx123  =  ones(T,1)~u01j~u02j~u03i~u04j~u05j~u12j ;
 xx124  =  ones(T,1)~u01j~u02j~u03j~u04i~u05j~u12j ;
 xx125  =  ones(T,1)~u01j~u02j~u03j~u04j~u05i~u12j;
 xx126  =  ones(T,1)~u01j~u02j~u03j~u04j~u05j~u12i;
 xx131  =  ones(T,1)~u01i~u02j~u03j~u04j~u05j~u13j ;
 xx132  =  ones(T,1)~u01j~u02i~u03j~u04j~u05j~u13j ;
 xx133  =  ones(T,1)~u01j~u02j~u03i~u04j~u05j~u13j ;
 xx134  =  ones(T,1)~u01j~u02j~u03j~u04i~u05j~u13j ;
 xx135  =  ones(T,1)~u01j~u02j~u03j~u04j~u05i~u13j;
 xx136  =  ones(T,1)~u01j~u02j~u03j~u04j~u05j~u13i;

if sumc(w11)> 0;
if det(xx11'wmat11*xx11) < epsln ;
      m6[j,1]  =  w11'y/sumc(w11) ;
else;
 beta =  pinv(xx11'wmat11*xx11)*xx11'wmat11'y ;
 m6[j,1]  =  beta[1,.] ;
endif;
else;
m6[j,1] = 0;
a11 = a11+1;
endif;

if sumc(w12)> 0;
if det(xx12'wmat12*xx12) < epsln ;
      m6[j,2]  =  w12'y/sumc(w12) ;
else;
 beta =  pinv(xx12'wmat12*xx12)*xx12'wmat12'y ;
 m6[j,2]  =  beta[1,.] ;
endif;
else;
m6[j,2] = 0;
a12 = a12+1;
endif;

if sumc(w13)> 0;
if det(xx13'wmat13*xx13) < epsln ;
      m6[j,3]  =  w13'y/sumc(w13) ;
else;
 beta =  pinv(xx13'wmat13*xx13)*xx13'wmat13'y ;
 m6[j,3]  =  beta[1,.] ;
endif;
else;
m6[j,3] = 0;
a13 = a13+1;
endif;

if sumc(w14)> 0;
if det(xx14'wmat14*xx14) < epsln ;
      m6[j,4]  =  w14'y/sumc(w14) ;
else;
 beta =  pinv(xx14'wmat14*xx14)*xx14'wmat14'y ;
 m6[j,4]  =  beta[1,.] ;
endif;
else;
m6[j,4] = 0;
a14 = a14+1;
endif;

if sumc(w15)> 0;
if det(xx15'wmat15*xx15) < epsln ;
      m6[j,5]  =  w15'y/sumc(w15) ;
else;
 beta =  pinv(xx15'wmat15*xx15)*xx15'wmat15'y ;
 m6[j,5]  =  beta[1,.] ;
endif;
else;
m6[j,5] = 0;
a15 = a15+1;
endif;

if sumc(w16)> 0;
if det(xx16'wmat16*xx16) < epsln ;
      m6[j,6]  =  w16'y/sumc(w16) ;
else;
 beta =  pinv(xx16'wmat16*xx16)*xx16'wmat16'y ;
 m6[j,6]  =  beta[1,.] ;
endif;
else;
m6[j,6] = 0;
a16 = a16+1;
endif;

if sumc(w21)> 0;
if det(xx21'wmat21*xx21) < epsln ;
      m6[j,7]  =  w21'y/sumc(w21) ;
else;
 beta =  pinv(xx21'wmat21*xx21)*xx21'wmat21'y ;
 m6[j,7]  =  beta[1,.] ;
endif;
else;
m6[j,7] = 0;
a21 = a21+1;
endif;

if sumc(w22)> 0;
if det(xx22'wmat22*xx22) < epsln ;
      m6[j,8]  =  w22'y/sumc(w22) ;
else;
 beta =  pinv(xx22'wmat22*xx22)*xx22'wmat22'y ;
 m6[j,8]  =  beta[1,.] ;
endif;
else;
m6[j,8] = 0;
a22 = a22+1;
endif;

if sumc(w23)> 0;
if det(xx23'wmat23*xx23) < epsln ;
      m6[j,9]  =  w23'y/sumc(w23) ;
else;
 beta =  pinv(xx23'wmat23*xx23)*xx23'wmat23'y ;
 m6[j,9]  =  beta[1,.] ;
endif;
else;
m6[j,9] = 0;
a23 = a23+1;
endif;

if sumc(w24)> 0;
if det(xx24'wmat24*xx24) < epsln ;
      m6[j,10]  =  w24'y/sumc(w24) ;
else;
 beta =  pinv(xx24'wmat24*xx24)*xx24'wmat24'y ;
 m6[j,10]  =  beta[1,.] ;
endif;
else;
m6[j,10] = 0;
a24 = a24+1;
endif;

if sumc(w25)> 0;
if det(xx25'wmat25*xx25) < epsln ;
      m6[j,11]  =  w25'y/sumc(w25) ;
else;
 beta =  pinv(xx25'wmat25*xx25)*xx25'wmat25'y ;
 m6[j,11]  =  beta[1,.] ;
endif;
else;
m6[j,11] = 0;
a25 = a25+1;
endif;

if sumc(w26)> 0;
if det(xx26'wmat26*xx26) < epsln ;
      m6[j,12]  =  w26'y/sumc(w26) ;
else;
 beta =  pinv(xx26'wmat26*xx26)*xx26'wmat26'y ;
 m6[j,12]  =  beta[1,.] ;
endif;
else;
m6[j,12] = 0;
a26 = a26+1;
endif;



if sumc(w31)> 0;
if det(xx31'wmat31*xx31) < epsln ;
      m6[j,13]  =  w31'y/sumc(w31) ;
else;
 beta =  pinv(xx31'wmat31*xx31)*xx31'wmat31'y ;
 m6[j,13]  =  beta[1,.] ;
endif;
else;
m6[j,13] = 0;
a31 = a31+1;
endif;

if sumc(w32)> 0;
if det(xx32'wmat32*xx32) < epsln ;
      m6[j,14]  =  w32'y/sumc(w32) ;
else;
 beta =  pinv(xx32'wmat32*xx32)*xx32'wmat32'y ;
 m6[j,14]  =  beta[1,.] ;
endif;
else;
m6[j,14] = 0;
a32 = a32+1;
endif;

if sumc(w33)> 0;
if det(xx33'wmat33*xx33) < epsln ;
      m6[j,15]  =  w33'y/sumc(w33) ;
else;
 beta =  pinv(xx33'wmat33*xx33)*xx33'wmat33'y ;
 m6[j,15]  =  beta[1,.] ;
endif;
else;
m6[j,15] = 0;
a33 = a33+1;
endif;

if sumc(w34)> 0;
if det(xx34'wmat34*xx34) < epsln ;
      m6[j,16]  =  w34'y/sumc(w34) ;
else;
 beta =  pinv(xx34'wmat34*xx34)*xx34'wmat34'y ;
 m6[j,16]  =  beta[1,.] ;
endif;
else;
m6[j,16] = 0;
a34 = a34+1;
endif;

if sumc(w35)> 0;
if det(xx35'wmat35*xx35) < epsln ;
      m6[j,17]  =  w35'y/sumc(w35) ;
else;
 beta =  pinv(xx35'wmat35*xx35)*xx35'wmat35'y ;
 m6[j,17]  =  beta[1,.] ;
endif;
else;
m6[j,17] = 0;
a35 = a35+1;
endif;

if sumc(w36)> 0;
if det(xx36'wmat36*xx36) < epsln ;
      m6[j,18]  =  w36'y/sumc(w36) ;
else;
 beta =  pinv(xx36'wmat36*xx36)*xx36'wmat36'y ;
 m6[j,18]  =  beta[1,.] ;
endif;
else;
m6[j,18] = 0;
a36 = a36+1;
endif;

if sumc(w41)> 0;
if det(xx41'wmat41*xx41) < epsln ;
      m6[j,19]  =  w41'y/sumc(w41) ;
else;
 beta =  pinv(xx41'wmat41*xx41)*xx41'wmat41'y ;
 m6[j,19]  =  beta[1,.] ;
endif;
else;
m6[j,19] = 0;
a41 = a41+1;
endif;

if sumc(w42)> 0;
if det(xx42'wmat42*xx42) < epsln ;
      m6[j,20]  =  w42'y/sumc(w42) ;
else;
 beta =  pinv(xx42'wmat42*xx42)*xx42'wmat42'y ;
 m6[j,20]  =  beta[1,.] ;
endif;
else;
m6[j,20] = 0;
a42 = a42+1;
endif;

if sumc(w43)> 0;
if det(xx43'wmat43*xx43) < epsln ;
      m6[j,21]  =  w43'y/sumc(w43) ;
else;
 beta =  pinv(xx43'wmat43*xx43)*xx43'wmat43'y ;
 m6[j,21]  =  beta[1,.] ;
endif;
else;
m6[j,21] = 0;
a43 = a43+1;
endif;

if sumc(w44)> 0;
if det(xx44'wmat44*xx44) < epsln ;
      m6[j,22]  =  w44'y/sumc(w44) ;
else;
 beta =  pinv(xx44'wmat44*xx44)*xx44'wmat44'y ;
 m6[j,22]  =  beta[1,.] ;
endif;
else;
m6[j,19] = 0;
a44 = a44+1;
endif;

if sumc(w45)> 0;
if det(xx45'wmat45*xx45) < epsln ;
      m6[j,23]  =  w45'y/sumc(w45) ;
else;
 beta =  pinv(xx45'wmat45*xx45)*xx45'wmat45'y ;
 m6[j,23]  =  beta[1,.] ;
endif;
else;
m6[j,23] = 0;
a45 = a45+1;
endif;

if sumc(w46)> 0;
if det(xx46'wmat46*xx46) < epsln ;
      m6[j,24]  =  w46'y/sumc(w46) ;
else;
 beta =  pinv(xx46'wmat46*xx46)*xx46'wmat46'y ;
 m6[j,24]  =  beta[1,.] ;
endif;
else;
m6[j,24] = 0;
a46 = a46+1;
endif;

if sumc(w51)> 0;
if det(xx51'wmat51*xx51) < epsln ;
      m6[j,25]  =  w51'y/sumc(w51) ;
else;
 beta =  pinv(xx51'wmat51*xx51)*xx51'wmat51'y ;
 m6[j,25]  =  beta[1,.] ;
endif;
else;
m6[j,25] = 0;
a51 = a51+1;
endif;

if sumc(w52)> 0;
if det(xx52'wmat52*xx52) < epsln ;
      m6[j,26]  =  w52'y/sumc(w52) ;
else;
 beta =  pinv(xx52'wmat52*xx52)*xx52'wmat52'y ;
 m6[j,26]  =  beta[1,.] ;
endif;
else;
m6[j,26] = 0;
a52 = a52+1;
endif;

if sumc(w53)> 0;
if det(xx53'wmat53*xx53) < epsln ;
      m6[j,27]  =  w53'y/sumc(w53) ;
else;
 beta =  pinv(xx53'wmat53*xx53)*xx53'wmat53'y ;
 m6[j,27]  =  beta[1,.] ;
endif;
else;
m6[j,27] = 0;
a53 = a53+1;
endif;

if sumc(w54)> 0;
if det(xx54'wmat54*xx54) < epsln ;
      m6[j,28]  =  w54'y/sumc(w54) ;
else;
 beta =  pinv(xx54'wmat54*xx54)*xx54'wmat54'y ;
 m6[j,28]  =  beta[1,.] ;
endif;
else;
m6[j,28] = 0;
a54 = a54+1;
endif;


if sumc(w55)> 0;
if det(xx55'wmat55*xx55) < epsln ;
      m6[j,29]  =  w55'y/sumc(w55) ;
else;
 beta =  pinv(xx55'wmat55*xx55)*xx55'wmat55'y ;
 m6[j,29]  =  beta[1,.] ;
endif;
else;
m6[j,29] = 0;
a55 = a55+1;
endif;

if sumc(w56)> 0;
if det(xx56'wmat56*xx56) < epsln ;
      m6[j,30]  =  w56'y/sumc(w56) ;
else;
 beta =  pinv(xx56'wmat56*xx56)*xx56'wmat56'y ;
 m6[j,30]  =  beta[1,.] ;
endif;
else;
m6[j,30] = 0;
a56 = a56+1;
endif;


if sumc(w61)> 0;
if det(xx61'wmat61*xx61) < epsln ;
      m6[j,31]  =  w61'y/sumc(w61) ;
else;
 beta =  pinv(xx61'wmat61*xx61)*xx61'wmat61'y ;
 m6[j,31]  =  beta[1,.] ;
endif;
else;
m6[j,31] = 0;
a61 = a61+1;
endif;

if sumc(w62)> 0;
if det(xx62'wmat62*xx62) < epsln ;
      m6[j,32]  =  w62'y/sumc(w62) ;
else;
 beta =  pinv(xx62'wmat62*xx62)*xx62'wmat62'y ;
 m6[j,32]  =  beta[1,.] ;
endif;
else;
m6[j,32] = 0;
a62 = a62+1;
endif;

if sumc(w63)> 0;
if det(xx63'wmat63*xx63) < epsln ;
      m6[j,33]  =  w63'y/sumc(w63) ;
else;
 beta =  pinv(xx63'wmat63*xx63)*xx63'wmat63'y ;
 m6[j,33]  =  beta[1,.] ;
endif;
else;
m6[j,33] = 0;
a63 = a63+1;
endif;

if sumc(w64)> 0;
if det(xx64'wmat64*xx64) < epsln ;
      m6[j,34]  =  w64'y/sumc(w64) ;
else;
 beta =  pinv(xx64'wmat64*xx64)*xx64'wmat64'y ;
 m6[j,34]  =  beta[1,.] ;
endif;
else;
m6[j,34] = 0;
a64 = a64+1;
endif;

if sumc(w65)> 0;
if det(xx65'wmat65*xx65) < epsln ;
      m6[j,35]  =  w65'y/sumc(w65) ;
else;
 beta =  pinv(xx65'wmat65*xx65)*xx65'wmat65'y ;
 m6[j,35]  =  beta[1,.] ;
endif;
else;
m6[j,35] = 0;
a65 = a65+1;
endif;

if sumc(w66)> 0;
if det(xx66'wmat66*xx66) < epsln ;
      m6[j,36]  =  w66'y/sumc(w66) ;
else;
 beta =  pinv(xx66'wmat66*xx66)*xx66'wmat66'y ;
 m6[j,36]  =  beta[1,.] ;
endif;
else;
m6[j,36] = 0;
a66 = a66+1;
endif;

if sumc(w71)> 0;
if det(xx71'wmat71*xx71) < epsln ;
      m6[j,37]  =  w71'y/sumc(w71) ;
else;
 beta =  pinv(xx71'wmat71*xx71)*xx71'wmat71'y ;
 m6[j,37]  =  beta[1,.] ;
endif;
else;
m6[j,37] = 0;
a71 = a71+1;
endif;


if sumc(w72)> 0;
if det(xx72'wmat72*xx72) < epsln ;
      m6[j,38]  =  w72'y/sumc(w72) ;
else;
 beta =  pinv(xx72'wmat72*xx72)*xx72'wmat72'y ;
 m6[j,38]  =  beta[1,.] ;
endif;
else;
m6[j,38] = 0;
a72 = a72+1;
endif;

if sumc(w73)> 0;
if det(xx73'wmat73*xx73) < epsln ;
      m6[j,39]  =  w73'y/sumc(w73) ;
else;
 beta =  pinv(xx73'wmat73*xx73)*xx73'wmat73'y ;
 m6[j,39]  =  beta[1,.] ;
endif;
else;
m6[j,39] = 0;
a73 = a73+1;
endif;

if sumc(w74)> 0;
if det(xx74'wmat74*xx74) < epsln ;
      m6[j,40]  =  w74'y/sumc(w74) ;
else;
 beta =  pinv(xx74'wmat74*xx74)*xx74'wmat74'y ;
 m6[j,40]  =  beta[1,.] ;
endif;
else;
m6[j,40] = 0;
a74 = a74+1;
endif;

if sumc(w75)> 0;
if det(xx75'wmat75*xx75) < epsln ;
      m6[j,41]  =  w75'y/sumc(w75) ;
else;
 beta =  pinv(xx75'wmat75*xx75)*xx75'wmat75'y ;
 m6[j,41]  =  beta[1,.] ;
endif;
else;
m6[j,41] = 0;
a75 = a75+1;
endif;

if sumc(w76)> 0;
if det(xx76'wmat76*xx76) < epsln ;
      m6[j,42]  =  w76'y/sumc(w76) ;
else;
 beta =  pinv(xx76'wmat76*xx76)*xx76'wmat76'y ;
 m6[j,42]  =  beta[1,.] ;
endif;
else;
m6[j,42] = 0;
a76 = a76+1;
endif;


if sumc(w81)> 0;
if det(xx81'wmat81*xx81) < epsln ;
      m6[j,43]  =  w81'y/sumc(w81) ;
else;
 beta =  pinv(xx81'wmat81*xx81)*xx81'wmat81'y ;
 m6[j,43]  =  beta[1,.] ;
endif;
else;
m6[j,43] = 0;
a81 = a81+1;
endif;

if sumc(w82)> 0;
if det(xx82'wmat82*xx82) < epsln ;
      m6[j,44]  =  w82'y/sumc(w82) ;
else;
 beta =  pinv(xx82'wmat82*xx82)*xx82'wmat82'y ;
 m6[j,44]  =  beta[1,.] ;
endif;
else;
m6[j,44] = 0;
a82 = a82+1;
endif;

if sumc(w83)> 0;
if det(xx83'wmat83*xx83) < epsln ;
      m6[j,45]  =  w83'y/sumc(w83) ;
else;
 beta =  pinv(xx83'wmat83*xx83)*xx83'wmat83'y ;
 m6[j,45]  =  beta[1,.] ;
endif;
else;
m6[j,45] = 0;
a83 = a83+1;
endif;

if sumc(w84)> 0;
if det(xx84'wmat84*xx84) < epsln ;
      m6[j,46]  =  w84'y/sumc(w84) ;
else;
 beta =  pinv(xx84'wmat84*xx84)*xx84'wmat84'y ;
 m6[j,46]  =  beta[1,.] ;
endif;
else;
m6[j,46] = 0;
a84 = a84+1;
endif;

if sumc(w85)> 0;
if det(xx85'wmat85*xx85) < epsln ;
      m6[j,47]  =  w85'y/sumc(w85) ;
else;
 beta =  pinv(xx85'wmat85*xx85)*xx85'wmat85'y ;
 m6[j,47]  =  beta[1,.] ;
endif;
else;
m6[j,47] = 0;
a85 = a85+1;
endif;

if sumc(w86)> 0;
if det(xx86'wmat86*xx86) < epsln ;
      m6[j,48]  =  w86'y/sumc(w86) ;
else;
 beta =  pinv(xx86'wmat86*xx86)*xx86'wmat86'y ;
 m6[j,48]  =  beta[1,.] ;
endif;
else;
m6[j,48] = 0;
a86 = a86+1;
endif;

if sumc(w91)> 0;
if det(xx91'wmat91*xx91) < epsln ;
      m6[j,49]  =  w91'y/sumc(w91) ;
else;
 beta =  pinv(xx91'wmat91*xx91)*xx91'wmat91'y ;
 m6[j,49]  =  beta[1,.] ;
endif;
else;
m6[j,49] = 0;
a91 = a91+1;
endif;

if sumc(w92)> 0;
if det(xx92'wmat92*xx92) < epsln ;
      m6[j,50]  =  w92'y/sumc(w92) ;
else;
 beta =  pinv(xx92'wmat92*xx92)*xx92'wmat92'y ;
 m6[j,50]  =  beta[1,.] ;
endif;
else;
m6[j,50] = 0;
a92 = a92+1;
endif;

if sumc(w93)> 0;
if det(xx93'wmat93*xx93) < epsln ;
      m6[j,51]  =  w93'y/sumc(w93) ;
else;
 beta =  pinv(xx93'wmat93*xx93)*xx93'wmat93'y ;
 m6[j,51]  =  beta[1,.] ;
endif;
else;
m6[j,51] = 0;
a93 = a93+1;
endif;

if sumc(w94)> 0;
if det(xx94'wmat94*xx94) < epsln ;
      m6[j,52]  =  w94'y/sumc(w94) ;
else;
 beta =  pinv(xx94'wmat94*xx94)*xx94'wmat94'y ;
 m6[j,52]  =  beta[1,.] ;
endif;
else;
m6[j,52] = 0;
a94 = a94+1;
endif;


if sumc(w95)> 0;
if det(xx95'wmat95*xx95) < epsln ;
      m6[j,53]  =  w95'y/sumc(w95) ;
else;
 beta =  pinv(xx95'wmat95*xx95)*xx95'wmat95'y ;
 m6[j,53]  =  beta[1,.] ;
endif;
else;
m6[j,53] = 0;
a95 = a95+1;
endif;

if sumc(w96)> 0;
if det(xx96'wmat96*xx96) < epsln ;
      m6[j,54]  =  w96'y/sumc(w96) ;
else;
 beta =  pinv(xx96'wmat96*xx96)*xx96'wmat96'y ;
 m6[j,54]  =  beta[1,.] ;
endif;
else;
m6[j,54] = 0;
a96 = a96+1;
endif;

if sumc(w101)> 0;
if det(xx101'wmat101*xx101) < epsln ;
      m6[j,55]  =  w101'y/sumc(w101) ;
else;
 beta =  pinv(xx101'wmat101*xx101)*xx101'wmat101'y ;
 m6[j,55]  =  beta[1,.] ;
endif;
else;
m6[j,55] = 0;
a101 = a101+1;
endif;

if sumc(w102)> 0;
if det(xx102'wmat102*xx102) < epsln ;
      m6[j,56]  =  w102'y/sumc(w102) ;
else;
 beta =  pinv(xx102'wmat102*xx102)*xx102'wmat102'y ;
 m6[j,56]  =  beta[1,.] ;
endif;
else;
m6[j,56] = 0;
a102 = a102+1;
endif;

if sumc(w103)> 0;
if det(xx103'wmat103*xx103) < epsln ;
      m6[j,57]  =  w103'y/sumc(w103) ;
else;
 beta =  pinv(xx103'wmat103*xx103)*xx103'wmat103'y ;
 m6[j,57]  =  beta[1,.] ;
endif;
else;
m6[j,57] = 0;
a103 = a103+1;
endif;

if sumc(w104)> 0;
if det(xx104'wmat104*xx104) < epsln ;
      m6[j,58]  =  w104'y/sumc(w104) ;
else;
 beta =  pinv(xx104'wmat104*xx104)*xx104'wmat104'y ;
 m6[j,58]  =  beta[1,.] ;
endif;
else;
m6[j,58] = 0;
a104 = a104+1;
endif;

if sumc(w105)> 0;
if det(xx105'wmat105*xx105) < epsln ;
      m6[j,59]  =  w105'y/sumc(w105) ;
else;
 beta =  pinv(xx105'wmat105*xx105)*xx105'wmat105'y ;
 m6[j,59]  =  beta[1,.] ;
endif;
else;
m6[j,59] = 0;
a105 = a105+1;
endif;

if sumc(w106)> 0;
if det(xx106'wmat106*xx106) < epsln ;
      m6[j,60]  =  w106'y/sumc(w106) ;
else;
 beta =  pinv(xx106'wmat106*xx106)*xx106'wmat106'y ;
 m6[j,60]  =  beta[1,.] ;
endif;
else;
m6[j,60] = 0;
a106 = a106+1;
endif;


if sumc(w111)> 0;
if det(xx111'wmat111*xx111) < epsln ;
      m6[j,61]  =  w111'y/sumc(w111) ;
else;
 beta =  pinv(xx111'wmat111*xx111)*xx111'wmat111'y ;
 m6[j,61]  =  beta[1,.] ;
endif;
else;
m6[j,61] = 0;
a111 = a111+1;
endif;

if sumc(w112)> 0;
if det(xx112'wmat112*xx112) < epsln ;
      m6[j,62]  =  w112'y/sumc(w112) ;
else;
 beta =  pinv(xx112'wmat112*xx112)*xx112'wmat112'y ;
 m6[j,62]  =  beta[1,.] ;
endif;
else;
m6[j,62] = 0;
a112 = a112+1;
endif;

if sumc(w113)> 0;
if det(xx113'wmat113*xx113) < epsln ;
      m6[j,63]  =  w113'y/sumc(w113) ;
else;
 beta =  pinv(xx113'wmat113*xx113)*xx113'wmat113'y ;
 m6[j,63]  =  beta[1,.] ;
endif;
else;
m6[j,63] = 0;
a113 = a113+1;
endif;

if sumc(w114)> 0;
if det(xx114'wmat114*xx114) < epsln ;
      m6[j,64]  =  w114'y/sumc(w114) ;
else;
 beta =  pinv(xx114'wmat114*xx114)*xx114'wmat114'y ;
 m6[j,64]  =  beta[1,.] ;
endif;
else;
m6[j,64] = 0;
a114 = a114+1;
endif;

if sumc(w115)> 0;
if det(xx115'wmat115*xx115) < epsln ;
      m6[j,65]  =  w115'y/sumc(w115) ;
else;
 beta =  pinv(xx115'wmat115*xx115)*xx115'wmat115'y ;
 m6[j,65]  =  beta[1,.] ;
endif;
else;
m6[j,65] = 0;
a115 = a115+1;
endif;

if sumc(w116)> 0;
if det(xx116'wmat116*xx116) < epsln ;
      m6[j,66]  =  w116'y/sumc(w116) ;
else;
 beta =  pinv(xx116'wmat116*xx116)*xx116'wmat116'y ;
 m6[j,66]  =  beta[1,.] ;
endif;
else;
m6[j,66] = 0;
a116 = a116+1;
endif;


if sumc(w121)> 0;
if det(xx121'wmat121*xx121) < epsln ;
      m6[j,67]  =  w121'y/sumc(w121) ;
else;
 beta =  pinv(xx121'wmat121*xx121)*xx121'wmat121'y ;
 m6[j,67]  =  beta[1,.] ;
endif;
else;
m6[j,67] = 0;
a121 = a121+1;
endif;

if sumc(w122)> 0;
if det(xx122'wmat122*xx122) < epsln ;
      m6[j,68]  =  w122'y/sumc(w122) ;
else;
 beta =  pinv(xx122'wmat122*xx122)*xx122'wmat122'y ;
 m6[j,68]  =  beta[1,.] ;
endif;
else;
m6[j,68] = 0;
a122 = a122+1;
endif;

if sumc(w123)> 0;
if det(xx123'wmat123*xx123) < epsln ;
      m6[j,69]  =  w123'y/sumc(w123) ;
else;
 beta =  pinv(xx123'wmat123*xx123)*xx123'wmat123'y ;
 m6[j,69]  =  beta[1,.] ;
endif;
else;
m6[j,69] = 0;
a123 = a123+1;
endif;

if sumc(w124)> 0;
if det(xx124'wmat124*xx124) < epsln ;
      m6[j,70]  =  w124'y/sumc(w124) ;
else;
 beta =  pinv(xx124'wmat124*xx124)*xx124'wmat124'y ;
 m6[j,70]  =  beta[1,.] ;
endif;
else;
m6[j,70] = 0;
a124 = a124+1;
endif;

if sumc(w125)> 0;
if det(xx125'wmat125*xx125) < epsln ;
      m6[j,71]  =  w125'y/sumc(w125) ;
else;
 beta =  pinv(xx125'wmat125*xx125)*xx125'wmat125'y ;
 m6[j,71]  =  beta[1,.] ;
endif;
else;
m6[j,71] = 0;
a125 = a125+1;
endif;

if sumc(w126)> 0;
if det(xx126'wmat126*xx126) < epsln ;
      m6[j,72]  =  w126'y/sumc(w126) ;
else;
 beta =  pinv(xx126'wmat126*xx126)*xx126'wmat126'y ;
 m6[j,72]  =  beta[1,.] ;
endif;
else;
m6[j,72] = 0;
a126 = a126+1;
endif;

if sumc(w131)> 0;
if det(xx131'wmat131*xx131) < epsln ;
      m6[j,73]  =  w131'y/sumc(w131) ;
else;
 beta =  pinv(xx131'wmat131*xx131)*xx131'wmat131'y ;
 m6[j,73]  =  beta[1,.] ;
endif;
else;
m6[j,73] = 0;
a131 = a131+1;
endif;

if sumc(w132)> 0;
if det(xx132'wmat132*xx132) < epsln ;
      m6[j,74]  =  w132'y/sumc(w132) ;
else;
 beta =  pinv(xx132'wmat132*xx132)*xx132'wmat132'y ;
 m6[j,74]  =  beta[1,.] ;
endif;
else;
m6[j,74] = 0;
a132 = a132+1;
endif;

if sumc(w133)> 0;
if det(xx133'wmat133*xx133) < epsln ;
      m6[j,75]  =  w133'y/sumc(w133) ;
else;
 beta =  pinv(xx133'wmat133*xx133)*xx133'wmat133'y ;
 m6[j,75]  =  beta[1,.] ;
endif;
else;
m6[j,75] = 0;
a133 = a133+1;
endif;

if sumc(w134)> 0;
if det(xx134'wmat134*xx134) < epsln ;
      m6[j,76]  =  w134'y/sumc(w134) ;
else;
 beta =  pinv(xx134'wmat134*xx134)*xx134'wmat134'y ;
 m6[j,76]  =  beta[1,.] ;
endif;
else;
m6[j,76] = 0;
a134 = a134+1;
endif;

if sumc(w135)> 0;
if det(xx135'wmat135*xx135) < epsln ;
      m6[j,77]  =  w135'y/sumc(w135) ;
else;
 beta =  pinv(xx135'wmat135*xx135)*xx135'wmat135'y ;
 m6[j,77]  =  beta[1,.] ;
endif;
else;
m6[j,77] = 0;
a135 = a135+1;
endif;

if sumc(w136)> 0;
if det(xx136'wmat136*xx136) < epsln ;
      m6[j,78]  =  w136'y/sumc(w136) ;
else;
 beta =  pinv(xx136'wmat136*xx136)*xx136'wmat136'y ;
 m6[j,78]  =  beta[1,.] ;
endif;
else;
m6[j,78] = 0;
a136 = a136+1;
endif;

j = j+1;
endo;

m6d[i,1] = 1/(T-a11)*sumc(m6[.,1]);
m6d[i,2] = 1/(T-a12)*sumc(m6[.,2]);
m6d[i,3] = 1/(T-a13)*sumc(m6[.,3]);
m6d[i,4] = 1/(T-a14)*sumc(m6[.,4]);
m6d[i,5] = 1/(T-a15)*sumc(m6[.,5]);
m6d[i,6] = 1/(T-a16)*sumc(m6[.,6]);
m6d[i,7] = 1/(T-a21)*sumc(m6[.,7]);
m6d[i,8] = 1/(T-a22)*sumc(m6[.,8]);
m6d[i,9] = 1/(T-a23)*sumc(m6[.,9]);
m6d[i,10] = 1/(T-a24)*sumc(m6[.,10]);
m6d[i,11] = 1/(T-a25)*sumc(m6[.,11]);
m6d[i,12] = 1/(T-a26)*sumc(m6[.,12]);
m6d[i,13] = 1/(T-a31)*sumc(m6[.,13]);
m6d[i,14] = 1/(T-a32)*sumc(m6[.,14]);
m6d[i,15] = 1/(T-a33)*sumc(m6[.,15]);
m6d[i,16] = 1/(T-a34)*sumc(m6[.,16]);
m6d[i,17] = 1/(T-a35)*sumc(m6[.,17]);
m6d[i,18] = 1/(T-a36)*sumc(m6[.,18]);
m6d[i,19] = 1/(T-a41)*sumc(m6[.,19]);
m6d[i,20] = 1/(T-a42)*sumc(m6[.,20]);
m6d[i,21] = 1/(T-a43)*sumc(m6[.,21]);
m6d[i,22] = 1/(T-a44)*sumc(m6[.,22]);
m6d[i,23] = 1/(T-a45)*sumc(m6[.,23]);
m6d[i,24] = 1/(T-a46)*sumc(m6[.,24]);
m6d[i,25] = 1/(T-a51)*sumc(m6[.,25]);
m6d[i,26] = 1/(T-a52)*sumc(m6[.,26]);
m6d[i,27] = 1/(T-a53)*sumc(m6[.,27]);
m6d[i,28] = 1/(T-a54)*sumc(m6[.,28]);
m6d[i,29] = 1/(T-a55)*sumc(m6[.,29]);
m6d[i,30] = 1/(T-a56)*sumc(m6[.,30]);
m6d[i,31] = 1/(T-a61)*sumc(m6[.,31]);
m6d[i,32] = 1/(T-a62)*sumc(m6[.,32]);
m6d[i,33] = 1/(T-a63)*sumc(m6[.,33]);
m6d[i,34] = 1/(T-a64)*sumc(m6[.,34]);
m6d[i,35] = 1/(T-a65)*sumc(m6[.,35]);
m6d[i,36] = 1/(T-a66)*sumc(m6[.,36]);
m6d[i,37] = 1/(T-a71)*sumc(m6[.,37]);
m6d[i,38] = 1/(T-a72)*sumc(m6[.,38]);
m6d[i,39] = 1/(T-a73)*sumc(m6[.,39]);
m6d[i,40] = 1/(T-a74)*sumc(m6[.,40]);
m6d[i,41] = 1/(T-a75)*sumc(m6[.,41]);
m6d[i,42] = 1/(T-a76)*sumc(m6[.,42]);
m6d[i,43] = 1/(T-a81)*sumc(m6[.,43]);
m6d[i,44] = 1/(T-a82)*sumc(m6[.,44]);
m6d[i,45] = 1/(T-a83)*sumc(m6[.,45]);
m6d[i,46] = 1/(T-a84)*sumc(m6[.,46]);
m6d[i,47] = 1/(T-a85)*sumc(m6[.,47]);
m6d[i,48] = 1/(T-a86)*sumc(m6[.,48]);
m6d[i,49] = 1/(T-a91)*sumc(m6[.,49]);
m6d[i,50] = 1/(T-a92)*sumc(m6[.,50]);
m6d[i,51] = 1/(T-a93)*sumc(m6[.,51]);
m6d[i,52] = 1/(T-a94)*sumc(m6[.,52]);
m6d[i,53] = 1/(T-a95)*sumc(m6[.,53]);
m6d[i,54] = 1/(T-a96)*sumc(m6[.,54]);
m6d[i,55] = 1/(T-a101)*sumc(m6[.,55]);
m6d[i,56] = 1/(T-a102)*sumc(m6[.,56]);
m6d[i,57] = 1/(T-a103)*sumc(m6[.,57]);
m6d[i,58] = 1/(T-a104)*sumc(m6[.,58]);
m6d[i,59] = 1/(T-a105)*sumc(m6[.,59]);
m6d[i,60] = 1/(T-a106)*sumc(m6[.,60]);
m6d[i,61] = 1/(T-a111)*sumc(m6[.,61]);
m6d[i,62] = 1/(T-a112)*sumc(m6[.,62]);
m6d[i,63] = 1/(T-a113)*sumc(m6[.,63]);
m6d[i,64] = 1/(T-a114)*sumc(m6[.,64]);
m6d[i,65] = 1/(T-a115)*sumc(m6[.,65]);
m6d[i,66] = 1/(T-a116)*sumc(m6[.,66]);
m6d[i,67] = 1/(T-a121)*sumc(m6[.,67]);
m6d[i,68] = 1/(T-a122)*sumc(m6[.,68]);
m6d[i,69] = 1/(T-a123)*sumc(m6[.,69]);
m6d[i,70] = 1/(T-a124)*sumc(m6[.,70]);
m6d[i,71] = 1/(T-a125)*sumc(m6[.,71]);
m6d[i,72] = 1/(T-a126)*sumc(m6[.,72]);
m6d[i,73] = 1/(T-a131)*sumc(m6[.,73]);
m6d[i,74] = 1/(T-a132)*sumc(m6[.,74]);
m6d[i,75] = 1/(T-a133)*sumc(m6[.,75]);
m6d[i,76] = 1/(T-a134)*sumc(m6[.,76]);
m6d[i,77] = 1/(T-a135)*sumc(m6[.,77]);
m6d[i,78] = 1/(T-a136)*sumc(m6[.,78]);
/*Six dimension kernel estimator of the density */
LVO =  ones(T,1);
LVO[i] = 0;
phat[i,1] = sumc(LVO.*fi(u01i/hs6).*fi(u02i/hs6).*fi(u03i/hs6).*fi(u04i/hs6).*fi(u05i/hs6).*fi(u1i/hs6))/((T-1)*hs6^6);
phat[i,2] = sumc(LVO.*fi(u01i/hs6).*fi(u02i/hs6).*fi(u03i/hs6).*fi(u04i/hs6).*fi(u05i/hs6).*fi(u2i/hs6))/((T-1)*hs6^6);
phat[i,3] = sumc(LVO.*fi(u01i/hs6).*fi(u02i/hs6).*fi(u03i/hs6).*fi(u04i/hs6).*fi(u05i/hs6).*fi(u3i/hs6))/((T-1)*hs6^6);
phat[i,4] = sumc(LVO.*fi(u01i/hs6).*fi(u02i/hs6).*fi(u03i/hs6).*fi(u04i/hs6).*fi(u05i/hs6).*fi(u4i/hs6))/((T-1)*hs6^6);
phat[i,5] = sumc(LVO.*fi(u01i/hs6).*fi(u02i/hs6).*fi(u03i/hs6).*fi(u04i/hs6).*fi(u05i/hs6).*fi(u5i/hs6))/((T-1)*hs6^6);
phat[i,6] = sumc(LVO.*fi(u01i/hs6).*fi(u02i/hs6).*fi(u03i/hs6).*fi(u04i/hs6).*fi(u05i/hs6).*fi(u6i/hs6))/((T-1)*hs6^6);
phat[i,7] = sumc(LVO.*fi(u01i/hs6).*fi(u02i/hs6).*fi(u03i/hs6).*fi(u04i/hs6).*fi(u05i/hs6).*fi(u7i/hs6))/((T-1)*hs6^6);
phat[i,8] = sumc(LVO.*fi(u01i/hs6).*fi(u02i/hs6).*fi(u03i/hs6).*fi(u04i/hs6).*fi(u05i/hs6).*fi(u8i/hs6))/((T-1)*hs6^6);
phat[i,9] = sumc(LVO.*fi(u01i/hs6).*fi(u02i/hs6).*fi(u03i/hs6).*fi(u04i/hs6).*fi(u05i/hs6).*fi(u9i/hs6))/((T-1)*hs6^6);
phat[i,10] = sumc(LVO.*fi(u01i/hs6).*fi(u02i/hs6).*fi(u03i/hs6).*fi(u04i/hs6).*fi(u05i/hs6).*fi(u10i/hs6))/((T-1)*hs6^6);
phat[i,11] = sumc(LVO.*fi(u01i/hs6).*fi(u02i/hs6).*fi(u03i/hs6).*fi(u04i/hs6).*fi(u05i/hs6).*fi(u11i/hs6))/((T-1)*hs6^6);
phat[i,12] = sumc(LVO.*fi(u01i/hs6).*fi(u02i/hs6).*fi(u03i/hs6).*fi(u04i/hs6).*fi(u05i/hs6).*fi(u12i/hs6))/((T-1)*hs6^6);
phat[i,13] = sumc(LVO.*fi(u01i/hs6).*fi(u02i/hs6).*fi(u03i/hs6).*fi(u04i/hs6).*fi(u05i/hs6).*fi(u13i/hs6))/((T-1)*hs6^6);
@estimating q(z) @
qz6[i,1] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1).*fi(u04i/h1)./h1.*fi(u05i/h1)./h1);
qz6[i,2] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u1i/h1)./h1);
qz6[i,3] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u05i/h1)./h1.*fi(u1i/h1)./h1);
qz6[i,4] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u04i/h1)./h1.*fi(u05i/h1)./h1.*fi(u1i/h1)./h1);
qz6[i,5] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u05i/h1)./h1.*fi(u1i/h1)./h1);
qz6[i,6] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u05i/h1)./h1.*fi(u1i/h1)./h1);
qz6[i,7] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u2i/h1)./h1);
qz6[i,8] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u05i/h1)./h1.*fi(u2i/h1)./h1);
qz6[i,9] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u04i/h1)./h1.*fi(u05i/h1)./h1.*fi(u2i/h1)./h1);
qz6[i,10] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u05i/h1)./h1.*fi(u2i/h1)./h1);
qz6[i,11] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u05i/h1)./h1.*fi(u2i/h1)./h1);
qz6[i,12] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u3i/h1)./h1);
qz6[i,13] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u05i/h1)./h1.*fi(u3i/h1)./h1);
qz6[i,14] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u04i/h1)./h1.*fi(u05i/h1)./h1.*fi(u3i/h1)./h1);
qz6[i,15] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u05i/h1)./h1.*fi(u3i/h1)./h1);
qz6[i,16] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u05i/h1)./h1.*fi(u3i/h1)./h1);
qz6[i,17] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u4i/h1)./h1);
qz6[i,18] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u05i/h1)./h1.*fi(u4i/h1)./h1);
qz6[i,19] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u04i/h1)./h1.*fi(u05i/h1)./h1.*fi(u4i/h1)./h1);
qz6[i,20] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u05i/h1)./h1.*fi(u4i/h1)./h1);
qz6[i,21] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u05i/h1)./h1.*fi(u4i/h1)./h1);
qz6[i,22] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u5i/h1)./h1);
qz6[i,23] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u05i/h1)./h1.*fi(u5i/h1)./h1);
qz6[i,24] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u04i/h1)./h1.*fi(u05i/h1)./h1.*fi(u5i/h1)./h1);
qz6[i,25] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u05i/h1)./h1.*fi(u5i/h1)./h1);
qz6[i,26] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u05i/h1)./h1.*fi(u5i/h1)./h1);
qz6[i,27] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u6i/h1)./h1);
qz6[i,28] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u05i/h1)./h1.*fi(u6i/h1)./h1);
qz6[i,29] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u04i/h1)./h1.*fi(u05i/h1)./h1.*fi(u6i/h1)./h1);
qz6[i,30] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u05i/h1)./h1.*fi(u6i/h1)./h1);
qz6[i,31] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u05i/h1)./h1.*fi(u6i/h1)./h1);
qz6[i,32] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u7i/h1)./h1);
qz6[i,33] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u05i/h1)./h1.*fi(u7i/h1)./h1);
qz6[i,34] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u04i/h1)./h1.*fi(u05i/h1)./h1.*fi(u7i/h1)./h1);
qz6[i,35] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u05i/h1)./h1.*fi(u7i/h1)./h1);
qz6[i,36] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u05i/h1)./h1.*fi(u7i/h1)./h1);
qz6[i,37] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u8i/h1)./h1);
qz6[i,38] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u05i/h1)./h1.*fi(u8i/h1)./h1);
qz6[i,39] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u04i/h1)./h1.*fi(u05i/h1)./h1.*fi(u8i/h1)./h1);
qz6[i,40] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u05i/h1)./h1.*fi(u8i/h1)./h1);
qz6[i,41] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u05i/h1)./h1.*fi(u8i/h1)./h1);
qz6[i,42] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u9i/h1)./h1);
qz6[i,43] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u05i/h1)./h1.*fi(u9i/h1)./h1);
qz6[i,44] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u04i/h1)./h1.*fi(u05i/h1)./h1.*fi(u9i/h1)./h1);
qz6[i,45] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u05i/h1)./h1.*fi(u9i/h1)./h1);
qz6[i,46] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u05i/h1)./h1.*fi(u9i/h1)./h1);
qz6[i,47] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u10i/h1)./h1);
qz6[i,48] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u05i/h1)./h1.*fi(u10i/h1)./h1);
qz6[i,49] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u04i/h1)./h1.*fi(u05i/h1)./h1.*fi(u10i/h1)./h1);
qz6[i,50] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u05i/h1)./h1.*fi(u10i/h1)./h1);
qz6[i,51] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u05i/h1)./h1.*fi(u10i/h1)./h1);
qz6[i,52] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u11i/h1)./h1);
qz6[i,53] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u05i/h1)./h1.*fi(u11i/h1)./h1);
qz6[i,54] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u04i/h1)./h1.*fi(u05i/h1)./h1.*fi(u11i/h1)./h1);
qz6[i,55] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u05i/h1)./h1.*fi(u11i/h1)./h1);
qz6[i,56] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u05i/h1)./h1.*fi(u11i/h1)./h1);
qz6[i,57] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u12i/h1)./h1);
qz6[i,58] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u05i/h1)./h1.*fi(u12i/h1)./h1);
qz6[i,59] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u04i/h1)./h1.*fi(u05i/h1)./h1.*fi(u12i/h1)./h1);
qz6[i,60] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u05i/h1)./h1.*fi(u12i/h1)./h1);
qz6[i,61] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u05i/h1)./h1.*fi(u12i/h1)./h1);
qz6[i,62] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u13i/h1)./h1);
qz6[i,63] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u05i/h1)./h1.*fi(u13i/h1)./h1);
qz6[i,64] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u02i/h1)./h1.*fi(u04i/h1)./h1.*fi(u05i/h1)./h1.*fi(u13i/h1)./h1);
qz6[i,65] = 1/T*sumc(fi(u01i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u05i/h1)./h1.*fi(u13i/h1)./h1);
qz6[i,66] = 1/T*sumc(fi(u02i/h1)./h1.*fi(u03i/h1)./h1.*fi(u04i/h1)./h1.*fi(u05i/h1)./h1.*fi(u13i/h1)./h1);
i = i+1;
endo;
yhat1 = y-meanc(y)-(m6d[.,1]-meanc(m6d[.,1])+m6d[.,2]-meanc(m6d[.,2])+m6d[.,3]-meanc(m6d[.,3])+m6d[.,4]-meanc(m6d[.,4])+m6d[.,5]-meanc(m6d[.,5])+m6d[.,6]-meanc(m6d[.,6]));
yhat2 = y-meanc(y)-(m6d[.,7]-meanc(m6d[.,7])+m6d[.,8]-meanc(m6d[.,8])+m6d[.,9]-meanc(m6d[.,9])+m6d[.,10]-meanc(m6d[.,10])+m6d[.,11]-meanc(m6d[.,11])+m6d[.,12]-meanc(m6d[.,12]));
yhat3 = y-meanc(y)-(m6d[.,13]-meanc(m6d[.,13])+m6d[.,14]-meanc(m6d[.,14])+m6d[.,15]-meanc(m6d[.,15])+m6d[.,16]-meanc(m6d[.,16])+m6d[.,17]-meanc(m6d[.,17])+m6d[.,18]-meanc(m6d[.,18]));
yhat4 = y-meanc(y)-(m6d[.,19]-meanc(m6d[.,19])+m6d[.,20]-meanc(m6d[.,20])+m6d[.,21]-meanc(m6d[.,21])+m6d[.,22]-meanc(m6d[.,22])+m6d[.,23]-meanc(m6d[.,23])+m6d[.,24]-meanc(m6d[.,24]));
yhat5 = y-meanc(y)-(m6d[.,25]-meanc(m6d[.,25])+m6d[.,26]-meanc(m6d[.,26])+m6d[.,27]-meanc(m6d[.,27])+m6d[.,28]-meanc(m6d[.,28])+m6d[.,29]-meanc(m6d[.,29])+m6d[.,30]-meanc(m6d[.,30]));
yhat6 = y-meanc(y)-(m6d[.,31]-meanc(m6d[.,31])+m6d[.,32]-meanc(m6d[.,32])+m6d[.,33]-meanc(m6d[.,33])+m6d[.,34]-meanc(m6d[.,34])+m6d[.,35]-meanc(m6d[.,35])+m6d[.,36]-meanc(m6d[.,36]));
yhat7 = y-meanc(y)-(m6d[.,37]-meanc(m6d[.,37])+m6d[.,38]-meanc(m6d[.,38])+m6d[.,39]-meanc(m6d[.,39])+m6d[.,40]-meanc(m6d[.,40])+m6d[.,41]-meanc(m6d[.,41])+m6d[.,42]-meanc(m6d[.,42]));
yhat8 = y-meanc(y)-(m6d[.,43]-meanc(m6d[.,43])+m6d[.,44]-meanc(m6d[.,44])+m6d[.,45]-meanc(m6d[.,45])+m6d[.,46]-meanc(m6d[.,46])+m6d[.,47]-meanc(m6d[.,47])+m6d[.,48]-meanc(m6d[.,48]));
yhat9 = y-meanc(y)-(m6d[.,49]-meanc(m6d[.,49])+m6d[.,50]-meanc(m6d[.,50])+m6d[.,51]-meanc(m6d[.,51])+m6d[.,52]-meanc(m6d[.,52])+m6d[.,53]-meanc(m6d[.,53])+m6d[.,54]-meanc(m6d[.,54]));
yhat10 = y-meanc(y)-(m6d[.,55]-meanc(m6d[.,55])+m6d[.,56]-meanc(m6d[.,56])+m6d[.,57]-meanc(m6d[.,57])+m6d[.,58]-meanc(m6d[.,58])+m6d[.,59]-meanc(m6d[.,59])+m6d[.,60]-meanc(m6d[.,60]));
yhat11 = y-meanc(y)-(m6d[.,61]-meanc(m6d[.,61])+m6d[.,62]-meanc(m6d[.,62])+m6d[.,63]-meanc(m6d[.,63])+m6d[.,64]-meanc(m6d[.,64])+m6d[.,65]-meanc(m6d[.,65])+m6d[.,66]-meanc(m6d[.,66]));
yhat12 = y-meanc(y)-(m6d[.,67]-meanc(m6d[.,67])+m6d[.,68]-meanc(m6d[.,68])+m6d[.,69]-meanc(m6d[.,69])+m6d[.,70]-meanc(m6d[.,70])+m6d[.,71]-meanc(m6d[.,71])+m6d[.,72]-meanc(m6d[.,72]));
yhat13 = y-meanc(y)-(m6d[.,73]-meanc(m6d[.,73])+m6d[.,74]-meanc(m6d[.,74])+m6d[.,75]-meanc(m6d[.,75])+m6d[.,76]-meanc(m6d[.,76])+m6d[.,77]-meanc(m6d[.,77])+m6d[.,78]-meanc(m6d[.,78]));
BR1 = (yhat1-y).*(yhat1-y);
BR2 = (yhat2-y).*(yhat2-y);
BR3 = (yhat3-y).*(yhat3-y);
BR4 = (yhat4-y).*(yhat4-y);
BR5 = (yhat5-y).*(yhat5-y);
BR6 = (yhat6-y).*(yhat6-y);
BR7 = (yhat7-y).*(yhat7-y);
BR8 = (yhat8-y).*(yhat8-y);
BR9 = (yhat9-y).*(yhat9-y);
BR10 = (yhat10-y).*(yhat10-y);
BR11 = (yhat11-y).*(yhat11-y);
BR12 = (yhat12-y).*(yhat12-y);
BR13 = (yhat13-y).*(yhat13-y);
quan = quantile(phat,0.05);
index = phat.< = quan;
phat = index*10^5+phat;

/****************************Six Lags Included ******************************************/

       
/****************************Six Lags Included Estimation of A part *******************************************/
kc = 1;
do until kc>nc;
b6 = h6*h[kc,1];
i = 1;
a11 = 0;
a12 = 0;
a13 = 0;
a14 = 0;
a15 = 0;
a16 = 0;
a21 = 0;
a22 = 0;
a23 = 0;
a24 = 0;
a25 = 0;
a26 = 0;
a31 = 0;
a32 = 0;
a33 = 0;
a34 = 0;
a35 = 0;
a36 = 0;
a41 = 0;
a42 = 0;
a43 = 0;
a44 = 0;
a45 = 0;
a46 = 0;
a51 = 0;
a52 = 0;
a53 = 0;
a54 = 0;
a55 = 0;
a56 = 0;
a61 = 0;
a62 = 0;
a63 = 0;
a64 = 0;
a65 = 0;
a66 = 0;
a71 = 0;
a72 = 0;
a73 = 0;
a74 = 0;
a75 = 0;
a76 = 0;
a81 = 0;
a82 = 0;
a83 = 0;
a84 = 0;
a85 = 0;
a86 = 0;
a91 = 0;
a92 = 0;
a93 = 0;
a94 = 0;
a95 = 0;
a96 = 0;
a101 = 0;
a102 = 0;
a103 = 0;
a104 = 0;
a105 = 0;
a106 = 0;
a111 = 0;
a112 = 0;
a113 = 0;
a114 = 0;
a115 = 0;
a116 = 0;
a121 = 0;
a122 = 0;
a123 = 0;
a124 = 0;
a125 = 0;
a126 = 0;
a131 = 0;
a132 = 0;
a133 = 0;
a134 = 0;
a135 = 0;
a136 = 0;
m6 = zeros(T,78);
m6d = zeros(T,78);
i = 1;
do until i>T;
u01i = data[i,lag_1]*ones(T,1) - data[.,lag_1];
u02i = data[i,lag_2]*ones(T,1) - data[.,lag_2];
u03i = data[i,lag_3]*ones(T,1) - data[.,lag_3];
u04i = data[i,lag_4]*ones(T,1) - data[.,lag_4];
u05i = data[i,lag_5]*ones(T,1) - data[.,lag_5];
u0i = u01i~u02i~u03i~u04i~u05i;
u1i =  data[i,1]*ones(T,1) - data[.,1];
u2i =  data[i,2]*ones(T,1) - data[.,2];
u3i =  data[i,3]*ones(T,1) - data[.,3];
u4i =  data[i,4]*ones(T,1) - data[.,4];
u5i =  data[i,5]*ones(T,1) - data[.,5];
u6i =  data[i,6]*ones(T,1) - data[.,6];
u7i =  data[i,7]*ones(T,1) - data[.,7];
u8i =  data[i,8]*ones(T,1) - data[.,8];
u9i =  data[i,9]*ones(T,1) - data[.,9];
u10i =  data[i,10]*ones(T,1) - data[.,10];
u11i =  data[i,11]*ones(T,1) - data[.,11];
u12i =  data[i,12]*ones(T,1) - data[.,12];
u13i =  data[i,13]*ones(T,1) - data[.,13];
j = 1;
do until j>T;
u01j = data[j,lag_1]*ones(T,1) - data[.,lag_1];
u02j = data[j,lag_2]*ones(T,1) - data[.,lag_2];
u03j = data[j,lag_3]*ones(T,1) - data[.,lag_3];
u04j = data[j,lag_4]*ones(T,1) - data[.,lag_4];
u05j = data[j,lag_5]*ones(T,1) - data[.,lag_5];
u0j = u01j~u02j~u03j~u04j~u05j;
 u1j =  (data[j,1]*ones(T,1) - data[.,1]);
 u2j =  (data[j,2]*ones(T,1) - data[.,2]);
 u3j =  (data[j,3]*ones(T,1) - data[.,3]);
 u4j =  (data[j,4]*ones(T,1) - data[.,4]);
 u5j =  (data[j,5]*ones(T,1) - data[.,5]);
 u6j =  (data[j,6]*ones(T,1) - data[.,6]);
 u7j =  (data[j,7]*ones(T,1) - data[.,7]);
 u8j =  (data[j,8]*ones(T,1) - data[.,8]);
 u9j =  (data[j,9]*ones(T,1) - data[.,9]);
 u10j =  (data[j,10]*ones(T,1) - data[.,10]);
 u11j =  (data[j,11]*ones(T,1) - data[.,11]);
 u12j =  (data[j,12]*ones(T,1) - data[.,12]);
 u13j =  (data[j,13]*ones(T,1) - data[.,13]);
 w11 = (fi(u01i/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u1j/b6)/b6);
 w12 = (fi(u01j/b6)/b6).*(fi(u02i/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u1j/b6)/b6);
 w13 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03i/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u1j/b6)/b6);
 w14 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04i/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u1j/b6)/b6);
 w15 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05i/b6)/b6).*(fi(u1j/b6)/b6);
 w16 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u1i/b6)/b6);
 w21 = (fi(u01i/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u2j/b6)/b6);
 w22 = (fi(u01j/b6)/b6).*(fi(u02i/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u2j/b6)/b6);
 w23 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03i/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u2j/b6)/b6);
 w24 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04i/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u2j/b6)/b6);
 w25 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05i/b6)/b6).*(fi(u2j/b6)/b6);
 w26 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u2i/b6)/b6);
 w31 = (fi(u01i/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u3j/b6)/b6);
 w32 = (fi(u01j/b6)/b6).*(fi(u02i/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u3j/b6)/b6);
 w33 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03i/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u3j/b6)/b6);
 w34 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04i/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u3j/b6)/b6);
 w35 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05i/b6)/b6).*(fi(u3j/b6)/b6);
 w36 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u3i/b6)/b6);
 w41 = (fi(u01i/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u4j/b6)/b6);
 w42 = (fi(u01j/b6)/b6).*(fi(u02i/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u4j/b6)/b6);
 w43 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03i/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u4j/b6)/b6);
 w44 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04i/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u4j/b6)/b6);
 w45 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05i/b6)/b6).*(fi(u4j/b6)/b6);
 w46 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u4i/b6)/b6);
 w51 = (fi(u01i/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u5j/b6)/b6);
 w52 = (fi(u01j/b6)/b6).*(fi(u02i/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u5j/b6)/b6);
 w53 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03i/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u5j/b6)/b6);
 w54 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04i/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u5j/b6)/b6);
 w55 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05i/b6)/b6).*(fi(u5j/b6)/b6);
 w56 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u5i/b6)/b6);
 w61 = (fi(u01i/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u6j/b6)/b6);
 w62 = (fi(u01j/b6)/b6).*(fi(u02i/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u6j/b6)/b6);
 w63 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03i/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u6j/b6)/b6);
 w64 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04i/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u6j/b6)/b6);
 w65 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05i/b6)/b6).*(fi(u6j/b6)/b6);
 w66 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u6i/b6)/b6);
 w71 = (fi(u01i/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u7j/b6)/b6);
 w72 = (fi(u01j/b6)/b6).*(fi(u02i/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u7j/b6)/b6);
 w73 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03i/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u7j/b6)/b6);
 w74 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04i/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u7j/b6)/b6);
 w75 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05i/b6)/b6).*(fi(u7j/b6)/b6);
 w76 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u7i/b6)/b6);
 w81 = (fi(u01i/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u8j/b6)/b6);
 w82 = (fi(u01j/b6)/b6).*(fi(u02i/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u8j/b6)/b6);
 w83 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03i/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u8j/b6)/b6);
 w84 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04i/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u8j/b6)/b6);
 w85 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05i/b6)/b6).*(fi(u8j/b6)/b6);
 w86 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u8i/b6)/b6);
 w91 = (fi(u01i/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u9j/b6)/b6);
 w92 = (fi(u01j/b6)/b6).*(fi(u02i/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u9j/b6)/b6);
 w93 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03i/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u9j/b6)/b6);
 w94 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04i/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u9j/b6)/b6);
 w95 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05i/b6)/b6).*(fi(u9j/b6)/b6);
 w96 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u9i/b6)/b6);
 w101 = (fi(u01i/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u10j/b6)/b6);
 w102 = (fi(u01j/b6)/b6).*(fi(u02i/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u10j/b6)/b6);
 w103 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03i/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u10j/b6)/b6);
 w104 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04i/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u10j/b6)/b6);
 w105 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05i/b6)/b6).*(fi(u10j/b6)/b6);
 w106 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u10i/b6)/b6);
 w111 = (fi(u01i/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u11j/b6)/b6);
 w112 = (fi(u01j/b6)/b6).*(fi(u02i/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u11j/b6)/b6);
 w113 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03i/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u11j/b6)/b6);
 w114 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04i/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u11j/b6)/b6);
 w115 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05i/b6)/b6).*(fi(u11j/b6)/b6);
 w116 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u11i/b6)/b6);
 w121 = (fi(u01i/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u12j/b6)/b6);
 w122 = (fi(u01j/b6)/b6).*(fi(u02i/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u12j/b6)/b6);
 w123 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03i/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u12j/b6)/b6);
 w124 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04i/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u12j/b6)/b6);
 w125 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05i/b6)/b6).*(fi(u12j/b6)/b6);
 w126 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u12i/b6)/b6);
 w131 = (fi(u01i/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u13j/b6)/b6);
 w132 = (fi(u01j/b6)/b6).*(fi(u02i/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u13j/b6)/b6);
 w133 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03i/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u13j/b6)/b6);
 w134 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04i/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u13j/b6)/b6);
 w135 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05i/b6)/b6).*(fi(u13j/b6)/b6);
 w136 = (fi(u01j/b6)/b6).*(fi(u02j/b6)/b6).*(fi(u03j/b6)/b6).*(fi(u04j/b6)/b6).*(fi(u05j/b6)/b6).*(fi(u13i/b6)/b6);
 wmat11  = diagrv((eye(T),w11));
 wmat12  = diagrv((eye(T),w12));
 wmat13  = diagrv((eye(T),w13));
 wmat14  = diagrv((eye(T),w14));
 wmat15  = diagrv((eye(T),w15));
 wmat16  = diagrv((eye(T),w16));
 wmat21  = diagrv((eye(T),w21));
 wmat22  = diagrv((eye(T),w22));
 wmat23  = diagrv((eye(T),w23));
 wmat24  = diagrv((eye(T),w24));
 wmat25  = diagrv((eye(T),w25));
 wmat26  = diagrv((eye(T),w26));
 wmat31  = diagrv((eye(T),w31));
 wmat32  = diagrv((eye(T),w32));
 wmat33  = diagrv((eye(T),w33));
 wmat34  = diagrv((eye(T),w34));
 wmat35  = diagrv((eye(T),w35));
 wmat36  = diagrv((eye(T),w36));
 wmat41  = diagrv((eye(T),w41));
 wmat42  = diagrv((eye(T),w42));
 wmat43  = diagrv((eye(T),w43));
 wmat44  = diagrv((eye(T),w44));
 wmat45  = diagrv((eye(T),w45));
 wmat46  = diagrv((eye(T),w46));
 wmat51  = diagrv((eye(T),w51));
 wmat52  = diagrv((eye(T),w52));
 wmat53  = diagrv((eye(T),w53));
 wmat54  = diagrv((eye(T),w54));
 wmat55  = diagrv((eye(T),w55));
 wmat56  = diagrv((eye(T),w56));
 wmat61  = diagrv((eye(T),w61));
 wmat62  = diagrv((eye(T),w62));
 wmat63  = diagrv((eye(T),w63));
 wmat64  = diagrv((eye(T),w64));
 wmat65  = diagrv((eye(T),w65));
 wmat66  = diagrv((eye(T),w66));
 wmat71  = diagrv((eye(T),w71));
 wmat72  = diagrv((eye(T),w72));
 wmat73  = diagrv((eye(T),w73));
 wmat74  = diagrv((eye(T),w74));
 wmat75  = diagrv((eye(T),w75));
 wmat76  = diagrv((eye(T),w76));
 wmat81  = diagrv((eye(T),w81));
 wmat82  = diagrv((eye(T),w82));
 wmat83  = diagrv((eye(T),w83));
 wmat84  = diagrv((eye(T),w84));
 wmat85  = diagrv((eye(T),w85));
 wmat86  = diagrv((eye(T),w86));
 wmat91  = diagrv((eye(T),w91));
 wmat92  = diagrv((eye(T),w92));
 wmat93  = diagrv((eye(T),w93));
 wmat94  = diagrv((eye(T),w94));
 wmat95  = diagrv((eye(T),w95));
 wmat96  = diagrv((eye(T),w96));
 wmat101  = diagrv((eye(T),w101));
 wmat102  = diagrv((eye(T),w102));
 wmat103  = diagrv((eye(T),w103));
 wmat104  = diagrv((eye(T),w104));
 wmat105  = diagrv((eye(T),w105));
 wmat106  = diagrv((eye(T),w106));
 wmat111  = diagrv((eye(T),w111));
 wmat112  = diagrv((eye(T),w112));
 wmat113  = diagrv((eye(T),w113));
 wmat114  = diagrv((eye(T),w114));
 wmat115  = diagrv((eye(T),w115));
 wmat116  = diagrv((eye(T),w116));
 wmat121  = diagrv((eye(T),w121));
 wmat122  = diagrv((eye(T),w122));
 wmat123  = diagrv((eye(T),w123));
 wmat124  = diagrv((eye(T),w124));
 wmat125  = diagrv((eye(T),w125));
 wmat126  = diagrv((eye(T),w126));
 wmat131  = diagrv((eye(T),w131));
 wmat132  = diagrv((eye(T),w132));
 wmat133  = diagrv((eye(T),w133));
 wmat134  = diagrv((eye(T),w134));
 wmat135  = diagrv((eye(T),w135));
 wmat136  = diagrv((eye(T),w136));
 xx11  =  ones(T,1)~u01i~u02j~u03j~u04j~u05j~u1j ;
 xx12  =  ones(T,1)~u01j~u02i~u03j~u04j~u05j~u1j ;
 xx13  =  ones(T,1)~u01j~u02j~u03i~u04j~u05j~u1j ;
 xx14  =  ones(T,1)~u01j~u02j~u03j~u04i~u05j~u1j ;
 xx15  =  ones(T,1)~u01j~u02j~u03j~u04j~u05i~u1j;
 xx16  =  ones(T,1)~u01j~u02j~u03j~u04j~u05j~u1i;
 xx21  =  ones(T,1)~u01i~u02j~u03j~u04j~u05j~u2j ;
 xx22  =  ones(T,1)~u01j~u02i~u03j~u04j~u05j~u2j ;
 xx23  =  ones(T,1)~u01j~u02j~u03i~u04j~u05j~u2j ;
 xx24  =  ones(T,1)~u01j~u02j~u03j~u04i~u05j~u2j ;
 xx25  =  ones(T,1)~u01j~u02j~u03j~u04j~u05i~u2j;
 xx26  =  ones(T,1)~u01j~u02j~u03j~u04j~u05j~u2i;
 xx31  =  ones(T,1)~u01i~u02j~u03j~u04j~u05j~u3j ;
 xx32  =  ones(T,1)~u01j~u02i~u03j~u04j~u05j~u3j ;
 xx33  =  ones(T,1)~u01j~u02j~u03i~u04j~u05j~u3j ;
 xx34  =  ones(T,1)~u01j~u02j~u03j~u04i~u05j~u3j ;
 xx35  =  ones(T,1)~u01j~u02j~u03j~u04j~u05i~u3j;
 xx36  =  ones(T,1)~u01j~u02j~u03j~u04j~u05j~u3i;
 xx41  =  ones(T,1)~u01i~u02j~u03j~u04j~u05j~u4j ;
 xx42  =  ones(T,1)~u01j~u02i~u03j~u04j~u05j~u4j ;
 xx43  =  ones(T,1)~u01j~u02j~u03i~u04j~u05j~u4j ;
 xx44  =  ones(T,1)~u01j~u02j~u03j~u04i~u05j~u4j ;
 xx45  =  ones(T,1)~u01j~u02j~u03j~u04j~u05i~u4j;
 xx46  =  ones(T,1)~u01j~u02j~u03j~u04j~u05j~u4i;
 xx51  =  ones(T,1)~u01i~u02j~u03j~u04j~u05j~u5j ;
 xx52  =  ones(T,1)~u01j~u02i~u03j~u04j~u05j~u5j ;
 xx53  =  ones(T,1)~u01j~u02j~u03i~u04j~u05j~u5j ;
 xx54  =  ones(T,1)~u01j~u02j~u03j~u04i~u05j~u5j ;
 xx55  =  ones(T,1)~u01j~u02j~u03j~u04j~u05i~u5j;
 xx56  =  ones(T,1)~u01j~u02j~u03j~u04j~u05j~u5i;
 xx61  =  ones(T,1)~u01i~u02j~u03j~u04j~u05j~u6j ;
 xx62  =  ones(T,1)~u01j~u02i~u03j~u04j~u05j~u6j ;
 xx63  =  ones(T,1)~u01j~u02j~u03i~u04j~u05j~u6j ;
 xx64  =  ones(T,1)~u01j~u02j~u03j~u04i~u05j~u6j ;
 xx65  =  ones(T,1)~u01j~u02j~u03j~u04j~u05i~u6j;
 xx66  =  ones(T,1)~u01j~u02j~u03j~u04j~u05j~u6i;
 xx71  =  ones(T,1)~u01i~u02j~u03j~u04j~u05j~u7j ;
 xx72  =  ones(T,1)~u01j~u02i~u03j~u04j~u05j~u7j ;
 xx73  =  ones(T,1)~u01j~u02j~u03i~u04j~u05j~u7j ;
 xx74  =  ones(T,1)~u01j~u02j~u03j~u04i~u05j~u7j ;
 xx75  =  ones(T,1)~u01j~u02j~u03j~u04j~u05i~u7j;
 xx76  =  ones(T,1)~u01j~u02j~u03j~u04j~u05j~u7i;
 xx81  =  ones(T,1)~u01i~u02j~u03j~u04j~u05j~u8j ;
 xx82  =  ones(T,1)~u01j~u02i~u03j~u04j~u05j~u8j ;
 xx83  =  ones(T,1)~u01j~u02j~u03i~u04j~u05j~u8j ;
 xx84  =  ones(T,1)~u01j~u02j~u03j~u04i~u05j~u8j ;
 xx85  =  ones(T,1)~u01j~u02j~u03j~u04j~u05i~u8j;
 xx86  =  ones(T,1)~u01j~u02j~u03j~u04j~u05j~u8i;
 xx91  =  ones(T,1)~u01i~u02j~u03j~u04j~u05j~u9j ;
 xx92  =  ones(T,1)~u01j~u02i~u03j~u04j~u05j~u9j ;
 xx93  =  ones(T,1)~u01j~u02j~u03i~u04j~u05j~u9j ;
 xx94  =  ones(T,1)~u01j~u02j~u03j~u04i~u05j~u9j ;
 xx95  =  ones(T,1)~u01j~u02j~u03j~u04j~u05i~u9j;
 xx96  =  ones(T,1)~u01j~u02j~u03j~u04j~u05j~u9i;
 xx101  =  ones(T,1)~u01i~u02j~u03j~u04j~u05j~u10j ;
 xx102  =  ones(T,1)~u01j~u02i~u03j~u04j~u05j~u10j ;
 xx103  =  ones(T,1)~u01j~u02j~u03i~u04j~u05j~u10j ;
 xx104  =  ones(T,1)~u01j~u02j~u03j~u04i~u05j~u10j ;
 xx105  =  ones(T,1)~u01j~u02j~u03j~u04j~u05i~u10j;
 xx106  =  ones(T,1)~u01j~u02j~u03j~u04j~u05j~u10i;
 xx111  =  ones(T,1)~u01i~u02j~u03j~u04j~u05j~u11j ;
 xx112  =  ones(T,1)~u01j~u02i~u03j~u04j~u05j~u11j ;
 xx113  =  ones(T,1)~u01j~u02j~u03i~u04j~u05j~u11j ;
 xx114  =  ones(T,1)~u01j~u02j~u03j~u04i~u05j~u11j ;
 xx115  =  ones(T,1)~u01j~u02j~u03j~u04j~u05i~u11j;
 xx116  =  ones(T,1)~u01j~u02j~u03j~u04j~u05j~u11i;
 xx121  =  ones(T,1)~u01i~u02j~u03j~u04j~u05j~u12j ;
 xx122  =  ones(T,1)~u01j~u02i~u03j~u04j~u05j~u12j ;
 xx123  =  ones(T,1)~u01j~u02j~u03i~u04j~u05j~u12j ;
 xx124  =  ones(T,1)~u01j~u02j~u03j~u04i~u05j~u12j ;
 xx125  =  ones(T,1)~u01j~u02j~u03j~u04j~u05i~u12j;
 xx126  =  ones(T,1)~u01j~u02j~u03j~u04j~u05j~u12i;
 xx131  =  ones(T,1)~u01i~u02j~u03j~u04j~u05j~u13j ;
 xx132  =  ones(T,1)~u01j~u02i~u03j~u04j~u05j~u13j ;
 xx133  =  ones(T,1)~u01j~u02j~u03i~u04j~u05j~u13j ;
 xx134  =  ones(T,1)~u01j~u02j~u03j~u04i~u05j~u13j ;
 xx135  =  ones(T,1)~u01j~u02j~u03j~u04j~u05i~u13j;
 xx136  =  ones(T,1)~u01j~u02j~u03j~u04j~u05j~u13i;

if sumc(w11)> 0;
if det(xx11'wmat11*xx11) < epsln ;
      m6[j,1]  =  w11'y/sumc(w11) ;
else;
 beta =  pinv(xx11'wmat11*xx11)*xx11'wmat11'y ;
 m6[j,1]  =  beta[1,.] ;
endif;
else;
m6[j,1] = 0;
a11 = a11+1;
endif;

if sumc(w12)> 0;
if det(xx12'wmat12*xx12) < epsln ;
      m6[j,2]  =  w12'y/sumc(w12) ;
else;
 beta =  pinv(xx12'wmat12*xx12)*xx12'wmat12'y ;
 m6[j,2]  =  beta[1,.] ;
endif;
else;
m6[j,2] = 0;
a12 = a12+1;
endif;

if sumc(w13)> 0;
if det(xx13'wmat13*xx13) < epsln ;
      m6[j,3]  =  w13'y/sumc(w13) ;
else;
 beta =  pinv(xx13'wmat13*xx13)*xx13'wmat13'y ;
 m6[j,3]  =  beta[1,.] ;
endif;
else;
m6[j,3] = 0;
a13 = a13+1;
endif;

if sumc(w14)> 0;
if det(xx14'wmat14*xx14) < epsln ;
      m6[j,4]  =  w14'y/sumc(w14) ;
else;
 beta =  pinv(xx14'wmat14*xx14)*xx14'wmat14'y ;
 m6[j,4]  =  beta[1,.] ;
endif;
else;
m6[j,4] = 0;
a14 = a14+1;
endif;

if sumc(w15)> 0;
if det(xx15'wmat15*xx15) < epsln ;
      m6[j,5]  =  w15'y/sumc(w15) ;
else;
 beta =  pinv(xx15'wmat15*xx15)*xx15'wmat15'y ;
 m6[j,5]  =  beta[1,.] ;
endif;
else;
m6[j,5] = 0;
a15 = a15+1;
endif;

if sumc(w16)> 0;
if det(xx16'wmat16*xx16) < epsln ;
      m6[j,6]  =  w16'y/sumc(w16) ;
else;
 beta =  pinv(xx16'wmat16*xx16)*xx16'wmat16'y ;
 m6[j,6]  =  beta[1,.] ;
endif;
else;
m6[j,6] = 0;
a16 = a16+1;
endif;

if sumc(w21)> 0;
if det(xx21'wmat21*xx21) < epsln ;
      m6[j,7]  =  w21'y/sumc(w21) ;
else;
 beta =  pinv(xx21'wmat21*xx21)*xx21'wmat21'y ;
 m6[j,7]  =  beta[1,.] ;
endif;
else;
m6[j,7] = 0;
a21 = a21+1;
endif;

if sumc(w22)> 0;
if det(xx22'wmat22*xx22) < epsln ;
      m6[j,8]  =  w22'y/sumc(w22) ;
else;
 beta =  pinv(xx22'wmat22*xx22)*xx22'wmat22'y ;
 m6[j,8]  =  beta[1,.] ;
endif;
else;
m6[j,8] = 0;
a22 = a22+1;
endif;

if sumc(w23)> 0;
if det(xx23'wmat23*xx23) < epsln ;
      m6[j,9]  =  w23'y/sumc(w23) ;
else;
 beta =  pinv(xx23'wmat23*xx23)*xx23'wmat23'y ;
 m6[j,9]  =  beta[1,.] ;
endif;
else;
m6[j,9] = 0;
a23 = a23+1;
endif;

if sumc(w24)> 0;
if det(xx24'wmat24*xx24) < epsln ;
      m6[j,10]  =  w24'y/sumc(w24) ;
else;
 beta =  pinv(xx24'wmat24*xx24)*xx24'wmat24'y ;
 m6[j,10]  =  beta[1,.] ;
endif;
else;
m6[j,10] = 0;
a24 = a24+1;
endif;

if sumc(w25)> 0;
if det(xx25'wmat25*xx25) < epsln ;
      m6[j,11]  =  w25'y/sumc(w25) ;
else;
 beta =  pinv(xx25'wmat25*xx25)*xx25'wmat25'y ;
 m6[j,11]  =  beta[1,.] ;
endif;
else;
m6[j,11] = 0;
a25 = a25+1;
endif;

if sumc(w26)> 0;
if det(xx26'wmat26*xx26) < epsln ;
      m6[j,12]  =  w26'y/sumc(w26) ;
else;
 beta =  pinv(xx26'wmat26*xx26)*xx26'wmat26'y ;
 m6[j,12]  =  beta[1,.] ;
endif;
else;
m6[j,12] = 0;
a26 = a26+1;
endif;

if sumc(w31)> 0;
if det(xx31'wmat31*xx31) < epsln ;
      m6[j,13]  =  w31'y/sumc(w31) ;
else;
 beta =  pinv(xx31'wmat31*xx31)*xx31'wmat31'y ;
 m6[j,13]  =  beta[1,.] ;
endif;
else;
m6[j,13] = 0;
a31 = a31+1;
endif;

if sumc(w32)> 0;
if det(xx32'wmat32*xx32) < epsln ;
      m6[j,14]  =  w32'y/sumc(w32) ;
else;
 beta =  pinv(xx32'wmat32*xx32)*xx32'wmat32'y ;
 m6[j,14]  =  beta[1,.] ;
endif;
else;
m6[j,14] = 0;
a32 = a32+1;
endif;

if sumc(w33)> 0;
if det(xx33'wmat33*xx33) < epsln ;
      m6[j,15]  =  w33'y/sumc(w33) ;
else;
 beta =  pinv(xx33'wmat33*xx33)*xx33'wmat33'y ;
 m6[j,15]  =  beta[1,.] ;
endif;
else;
m6[j,15] = 0;
a33 = a33+1;
endif;

if sumc(w34)> 0;
if det(xx34'wmat34*xx34) < epsln ;
      m6[j,16]  =  w34'y/sumc(w34) ;
else;
 beta =  pinv(xx34'wmat34*xx34)*xx34'wmat34'y ;
 m6[j,16]  =  beta[1,.] ;
endif;
else;
m6[j,16] = 0;
a34 = a34+1;
endif;

if sumc(w35)> 0;
if det(xx35'wmat35*xx35) < epsln ;
      m6[j,17]  =  w35'y/sumc(w35) ;
else;
 beta =  pinv(xx35'wmat35*xx35)*xx35'wmat35'y ;
 m6[j,17]  =  beta[1,.] ;
endif;
else;
m6[j,17] = 0;
a35 = a35+1;
endif;

if sumc(w36)> 0;
if det(xx36'wmat36*xx36) < epsln ;
      m6[j,18]  =  w36'y/sumc(w36) ;
else;
 beta =  pinv(xx36'wmat36*xx36)*xx36'wmat36'y ;
 m6[j,18]  =  beta[1,.] ;
endif;
else;
m6[j,18] = 0;
a36 = a36+1;
endif;

if sumc(w41)> 0;
if det(xx41'wmat41*xx41) < epsln ;
      m6[j,19]  =  w41'y/sumc(w41) ;
else;
 beta =  pinv(xx41'wmat41*xx41)*xx41'wmat41'y ;
 m6[j,19]  =  beta[1,.] ;
endif;
else;
m6[j,19] = 0;
a41 = a41+1;
endif;

if sumc(w42)> 0;
if det(xx42'wmat42*xx42) < epsln ;
      m6[j,20]  =  w42'y/sumc(w42) ;
else;
 beta =  pinv(xx42'wmat42*xx42)*xx42'wmat42'y ;
 m6[j,20]  =  beta[1,.] ;
endif;
else;
m6[j,20] = 0;
a42 = a42+1;
endif;

if sumc(w43)> 0;
if det(xx43'wmat43*xx43) < epsln ;
      m6[j,21]  =  w43'y/sumc(w43) ;
else;
 beta =  pinv(xx43'wmat43*xx43)*xx43'wmat43'y ;
 m6[j,21]  =  beta[1,.] ;
endif;
else;
m6[j,21] = 0;
a43 = a43+1;
endif;

if sumc(w44)> 0;
if det(xx44'wmat44*xx44) < epsln ;
      m6[j,22]  =  w44'y/sumc(w44) ;
else;
 beta =  pinv(xx44'wmat44*xx44)*xx44'wmat44'y ;
 m6[j,22]  =  beta[1,.] ;
endif;
else;
m6[j,19] = 0;
a44 = a44+1;
endif;

if sumc(w45)> 0;
if det(xx45'wmat45*xx45) < epsln ;
      m6[j,23]  =  w45'y/sumc(w45) ;
else;
 beta =  pinv(xx45'wmat45*xx45)*xx45'wmat45'y ;
 m6[j,23]  =  beta[1,.] ;
endif;
else;
m6[j,23] = 0;
a45 = a45+1;
endif;

if sumc(w46)> 0;
if det(xx46'wmat46*xx46) < epsln ;
      m6[j,24]  =  w46'y/sumc(w46) ;
else;
 beta =  pinv(xx46'wmat46*xx46)*xx46'wmat46'y ;
 m6[j,24]  =  beta[1,.] ;
endif;
else;
m6[j,24] = 0;
a46 = a46+1;
endif;

if sumc(w51)> 0;
if det(xx51'wmat51*xx51) < epsln ;
      m6[j,25]  =  w51'y/sumc(w51) ;
else;
 beta =  pinv(xx51'wmat51*xx51)*xx51'wmat51'y ;
 m6[j,25]  =  beta[1,.] ;
endif;
else;
m6[j,25] = 0;
a51 = a51+1;
endif;

if sumc(w52)> 0;
if det(xx52'wmat52*xx52) < epsln ;
      m6[j,26]  =  w52'y/sumc(w52) ;
else;
 beta =  pinv(xx52'wmat52*xx52)*xx52'wmat52'y ;
 m6[j,26]  =  beta[1,.] ;
endif;
else;
m6[j,26] = 0;
a52 = a52+1;
endif;

if sumc(w53)> 0;
if det(xx53'wmat53*xx53) < epsln ;
      m6[j,27]  =  w53'y/sumc(w53) ;
else;
 beta =  pinv(xx53'wmat53*xx53)*xx53'wmat53'y ;
 m6[j,27]  =  beta[1,.] ;
endif;
else;
m6[j,27] = 0;
a53 = a53+1;
endif;

if sumc(w54)> 0;
if det(xx54'wmat54*xx54) < epsln ;
      m6[j,28]  =  w54'y/sumc(w54) ;
else;
 beta =  pinv(xx54'wmat54*xx54)*xx54'wmat54'y ;
 m6[j,28]  =  beta[1,.] ;
endif;
else;
m6[j,28] = 0;
a54 = a54+1;
endif;


if sumc(w55)> 0;
if det(xx55'wmat55*xx55) < epsln ;
      m6[j,29]  =  w55'y/sumc(w55) ;
else;
 beta =  pinv(xx55'wmat55*xx55)*xx55'wmat55'y ;
 m6[j,29]  =  beta[1,.] ;
endif;
else;
m6[j,29] = 0;
a55 = a55+1;
endif;

if sumc(w56)> 0;
if det(xx56'wmat56*xx56) < epsln ;
      m6[j,30]  =  w56'y/sumc(w56) ;
else;
 beta =  pinv(xx56'wmat56*xx56)*xx56'wmat56'y ;
 m6[j,30]  =  beta[1,.] ;
endif;
else;
m6[j,30] = 0;
a56 = a56+1;
endif;


if sumc(w61)> 0;
if det(xx61'wmat61*xx61) < epsln ;
      m6[j,31]  =  w61'y/sumc(w61) ;
else;
 beta =  pinv(xx61'wmat61*xx61)*xx61'wmat61'y ;
 m6[j,31]  =  beta[1,.] ;
endif;
else;
m6[j,31] = 0;
a61 = a61+1;
endif;

if sumc(w62)> 0;
if det(xx62'wmat62*xx62) < epsln ;
      m6[j,32]  =  w62'y/sumc(w62) ;
else;
 beta =  pinv(xx62'wmat62*xx62)*xx62'wmat62'y ;
 m6[j,32]  =  beta[1,.] ;
endif;
else;
m6[j,32] = 0;
a62 = a62+1;
endif;

if sumc(w63)> 0;
if det(xx63'wmat63*xx63) < epsln ;
      m6[j,33]  =  w63'y/sumc(w63) ;
else;
 beta =  pinv(xx63'wmat63*xx63)*xx63'wmat63'y ;
 m6[j,33]  =  beta[1,.] ;
endif;
else;
m6[j,33] = 0;
a63 = a63+1;
endif;

if sumc(w64)> 0;
if det(xx64'wmat64*xx64) < epsln ;
      m6[j,34]  =  w64'y/sumc(w64) ;
else;
 beta =  pinv(xx64'wmat64*xx64)*xx64'wmat64'y ;
 m6[j,34]  =  beta[1,.] ;
endif;
else;
m6[j,34] = 0;
a64 = a64+1;
endif;

if sumc(w65)> 0;
if det(xx65'wmat65*xx65) < epsln ;
      m6[j,35]  =  w65'y/sumc(w65) ;
else;
 beta =  pinv(xx65'wmat65*xx65)*xx65'wmat65'y ;
 m6[j,35]  =  beta[1,.] ;
endif;
else;
m6[j,35] = 0;
a65 = a65+1;
endif;

if sumc(w66)> 0;
if det(xx66'wmat66*xx66) < epsln ;
      m6[j,36]  =  w66'y/sumc(w66) ;
else;
 beta =  pinv(xx66'wmat66*xx66)*xx66'wmat66'y ;
 m6[j,36]  =  beta[1,.] ;
endif;
else;
m6[j,36] = 0;
a66 = a66+1;
endif;

if sumc(w71)> 0;
if det(xx71'wmat71*xx71) < epsln ;
      m6[j,37]  =  w71'y/sumc(w71) ;
else;
 beta =  pinv(xx71'wmat71*xx71)*xx71'wmat71'y ;
 m6[j,37]  =  beta[1,.] ;
endif;
else;
m6[j,37] = 0;
a71 = a71+1;
endif;


if sumc(w72)> 0;
if det(xx72'wmat72*xx72) < epsln ;
      m6[j,38]  =  w72'y/sumc(w72) ;
else;
 beta =  pinv(xx72'wmat72*xx72)*xx72'wmat72'y ;
 m6[j,38]  =  beta[1,.] ;
endif;
else;
m6[j,38] = 0;
a72 = a72+1;
endif;

if sumc(w73)> 0;
if det(xx73'wmat73*xx73) < epsln ;
      m6[j,39]  =  w73'y/sumc(w73) ;
else;
 beta =  pinv(xx73'wmat73*xx73)*xx73'wmat73'y ;
 m6[j,39]  =  beta[1,.] ;
endif;
else;
m6[j,39] = 0;
a73 = a73+1;
endif;

if sumc(w74)> 0;
if det(xx74'wmat74*xx74) < epsln ;
      m6[j,40]  =  w74'y/sumc(w74) ;
else;
 beta =  pinv(xx74'wmat74*xx74)*xx74'wmat74'y ;
 m6[j,40]  =  beta[1,.] ;
endif;
else;
m6[j,40] = 0;
a74 = a74+1;
endif;

if sumc(w75)> 0;
if det(xx75'wmat75*xx75) < epsln ;
      m6[j,41]  =  w75'y/sumc(w75) ;
else;
 beta =  pinv(xx75'wmat75*xx75)*xx75'wmat75'y ;
 m6[j,41]  =  beta[1,.] ;
endif;
else;
m6[j,41] = 0;
a75 = a75+1;
endif;

if sumc(w76)> 0;
if det(xx76'wmat76*xx76) < epsln ;
      m6[j,42]  =  w76'y/sumc(w76) ;
else;
 beta =  pinv(xx76'wmat76*xx76)*xx76'wmat76'y ;
 m6[j,42]  =  beta[1,.] ;
endif;
else;
m6[j,42] = 0;
a76 = a76+1;
endif;


if sumc(w81)> 0;
if det(xx81'wmat81*xx81) < epsln ;
      m6[j,43]  =  w81'y/sumc(w81) ;
else;
 beta =  pinv(xx81'wmat81*xx81)*xx81'wmat81'y ;
 m6[j,43]  =  beta[1,.] ;
endif;
else;
m6[j,43] = 0;
a81 = a81+1;
endif;

if sumc(w82)> 0;
if det(xx82'wmat82*xx82) < epsln ;
      m6[j,44]  =  w82'y/sumc(w82) ;
else;
 beta =  pinv(xx82'wmat82*xx82)*xx82'wmat82'y ;
 m6[j,44]  =  beta[1,.] ;
endif;
else;
m6[j,44] = 0;
a82 = a82+1;
endif;

if sumc(w83)> 0;
if det(xx83'wmat83*xx83) < epsln ;
      m6[j,45]  =  w83'y/sumc(w83) ;
else;
 beta =  pinv(xx83'wmat83*xx83)*xx83'wmat83'y ;
 m6[j,45]  =  beta[1,.] ;
endif;
else;
m6[j,45] = 0;
a83 = a83+1;
endif;

if sumc(w84)> 0;
if det(xx84'wmat84*xx84) < epsln ;
      m6[j,46]  =  w84'y/sumc(w84) ;
else;
 beta =  pinv(xx84'wmat84*xx84)*xx84'wmat84'y ;
 m6[j,46]  =  beta[1,.] ;
endif;
else;
m6[j,46] = 0;
a84 = a84+1;
endif;

if sumc(w85)> 0;
if det(xx85'wmat85*xx85) < epsln ;
      m6[j,47]  =  w85'y/sumc(w85) ;
else;
 beta =  pinv(xx85'wmat85*xx85)*xx85'wmat85'y ;
 m6[j,47]  =  beta[1,.] ;
endif;
else;
m6[j,47] = 0;
a85 = a85+1;
endif;

if sumc(w86)> 0;
if det(xx86'wmat86*xx86) < epsln ;
      m6[j,48]  =  w86'y/sumc(w86) ;
else;
 beta =  pinv(xx86'wmat86*xx86)*xx86'wmat86'y ;
 m6[j,48]  =  beta[1,.] ;
endif;
else;
m6[j,48] = 0;
a86 = a86+1;
endif;

if sumc(w91)> 0;
if det(xx91'wmat91*xx91) < epsln ;
      m6[j,49]  =  w91'y/sumc(w91) ;
else;
 beta =  pinv(xx91'wmat91*xx91)*xx91'wmat91'y ;
 m6[j,49]  =  beta[1,.] ;
endif;
else;
m6[j,49] = 0;
a91 = a91+1;
endif;

if sumc(w92)> 0;
if det(xx92'wmat92*xx92) < epsln ;
      m6[j,50]  =  w92'y/sumc(w92) ;
else;
 beta =  pinv(xx92'wmat92*xx92)*xx92'wmat92'y ;
 m6[j,50]  =  beta[1,.] ;
endif;
else;
m6[j,50] = 0;
a92 = a92+1;
endif;

if sumc(w93)> 0;
if det(xx93'wmat93*xx93) < epsln ;
      m6[j,51]  =  w93'y/sumc(w93) ;
else;
 beta =  pinv(xx93'wmat93*xx93)*xx93'wmat93'y ;
 m6[j,51]  =  beta[1,.] ;
endif;
else;
m6[j,51] = 0;
a93 = a93+1;
endif;

if sumc(w94)> 0;
if det(xx94'wmat94*xx94) < epsln ;
      m6[j,52]  =  w94'y/sumc(w94) ;
else;
 beta =  pinv(xx94'wmat94*xx94)*xx94'wmat94'y ;
 m6[j,52]  =  beta[1,.] ;
endif;
else;
m6[j,52] = 0;
a94 = a94+1;
endif;


if sumc(w95)> 0;
if det(xx95'wmat95*xx95) < epsln ;
      m6[j,53]  =  w95'y/sumc(w95) ;
else;
 beta =  pinv(xx95'wmat95*xx95)*xx95'wmat95'y ;
 m6[j,53]  =  beta[1,.] ;
endif;
else;
m6[j,53] = 0;
a95 = a95+1;
endif;

if sumc(w96)> 0;
if det(xx96'wmat96*xx96) < epsln ;
      m6[j,54]  =  w96'y/sumc(w96) ;
else;
 beta =  pinv(xx96'wmat96*xx96)*xx96'wmat96'y ;
 m6[j,54]  =  beta[1,.] ;
endif;
else;
m6[j,54] = 0;
a96 = a96+1;
endif;

if sumc(w101)> 0;
if det(xx101'wmat101*xx101) < epsln ;
      m6[j,55]  =  w101'y/sumc(w101) ;
else;
 beta =  pinv(xx101'wmat101*xx101)*xx101'wmat101'y ;
 m6[j,55]  =  beta[1,.] ;
endif;
else;
m6[j,55] = 0;
a101 = a101+1;
endif;

if sumc(w102)> 0;
if det(xx102'wmat102*xx102) < epsln ;
      m6[j,56]  =  w102'y/sumc(w102) ;
else;
 beta =  pinv(xx102'wmat102*xx102)*xx102'wmat102'y ;
 m6[j,56]  =  beta[1,.] ;
endif;
else;
m6[j,56] = 0;
a102 = a102+1;
endif;

if sumc(w103)> 0;
if det(xx103'wmat103*xx103) < epsln ;
      m6[j,57]  =  w103'y/sumc(w103) ;
else;
 beta =  pinv(xx103'wmat103*xx103)*xx103'wmat103'y ;
 m6[j,57]  =  beta[1,.] ;
endif;
else;
m6[j,57] = 0;
a103 = a103+1;
endif;

if sumc(w104)> 0;
if det(xx104'wmat104*xx104) < epsln ;
      m6[j,58]  =  w104'y/sumc(w104) ;
else;
 beta =  pinv(xx104'wmat104*xx104)*xx104'wmat104'y ;
 m6[j,58]  =  beta[1,.] ;
endif;
else;
m6[j,58] = 0;
a104 = a104+1;
endif;

if sumc(w105)> 0;
if det(xx105'wmat105*xx105) < epsln ;
      m6[j,59]  =  w105'y/sumc(w105) ;
else;
 beta =  pinv(xx105'wmat105*xx105)*xx105'wmat105'y ;
 m6[j,59]  =  beta[1,.] ;
endif;
else;
m6[j,59] = 0;
a105 = a105+1;
endif;

if sumc(w106)> 0;
if det(xx106'wmat106*xx106) < epsln ;
      m6[j,60]  =  w106'y/sumc(w106) ;
else;
 beta =  pinv(xx106'wmat106*xx106)*xx106'wmat106'y ;
 m6[j,60]  =  beta[1,.] ;
endif;
else;
m6[j,60] = 0;
a106 = a106+1;
endif;


if sumc(w111)> 0;
if det(xx111'wmat111*xx111) < epsln ;
      m6[j,61]  =  w111'y/sumc(w111) ;
else;
 beta =  pinv(xx111'wmat111*xx111)*xx111'wmat111'y ;
 m6[j,61]  =  beta[1,.] ;
endif;
else;
m6[j,61] = 0;
a111 = a111+1;
endif;

if sumc(w112)> 0;
if det(xx112'wmat112*xx112) < epsln ;
      m6[j,62]  =  w112'y/sumc(w112) ;
else;
 beta =  pinv(xx112'wmat112*xx112)*xx112'wmat112'y ;
 m6[j,62]  =  beta[1,.] ;
endif;
else;
m6[j,62] = 0;
a112 = a112+1;
endif;

if sumc(w113)> 0;
if det(xx113'wmat113*xx113) < epsln ;
      m6[j,63]  =  w113'y/sumc(w113) ;
else;
 beta =  pinv(xx113'wmat113*xx113)*xx113'wmat113'y ;
 m6[j,63]  =  beta[1,.] ;
endif;
else;
m6[j,63] = 0;
a113 = a113+1;
endif;

if sumc(w114)> 0;
if det(xx114'wmat114*xx114) < epsln ;
      m6[j,64]  =  w114'y/sumc(w114) ;
else;
 beta =  pinv(xx114'wmat114*xx114)*xx114'wmat114'y ;
 m6[j,64]  =  beta[1,.] ;
endif;
else;
m6[j,64] = 0;
a114 = a114+1;
endif;

if sumc(w115)> 0;
if det(xx115'wmat115*xx115) < epsln ;
      m6[j,65]  =  w115'y/sumc(w115) ;
else;
 beta =  pinv(xx115'wmat115*xx115)*xx115'wmat115'y ;
 m6[j,65]  =  beta[1,.] ;
endif;
else;
m6[j,65] = 0;
a115 = a115+1;
endif;

if sumc(w116)> 0;
if det(xx116'wmat116*xx116) < epsln ;
      m6[j,66]  =  w116'y/sumc(w116) ;
else;
 beta =  pinv(xx116'wmat116*xx116)*xx116'wmat116'y ;
 m6[j,66]  =  beta[1,.] ;
endif;
else;
m6[j,66] = 0;
a116 = a116+1;
endif;


if sumc(w121)> 0;
if det(xx121'wmat121*xx121) < epsln ;
      m6[j,67]  =  w121'y/sumc(w121) ;
else;
 beta =  pinv(xx121'wmat121*xx121)*xx121'wmat121'y ;
 m6[j,67]  =  beta[1,.] ;
endif;
else;
m6[j,67] = 0;
a121 = a121+1;
endif;

if sumc(w122)> 0;
if det(xx122'wmat122*xx122) < epsln ;
      m6[j,68]  =  w122'y/sumc(w122) ;
else;
 beta =  pinv(xx122'wmat122*xx122)*xx122'wmat122'y ;
 m6[j,68]  =  beta[1,.] ;
endif;
else;
m6[j,68] = 0;
a122 = a122+1;
endif;

if sumc(w123)> 0;
if det(xx123'wmat123*xx123) < epsln ;
      m6[j,69]  =  w123'y/sumc(w123) ;
else;
 beta =  pinv(xx123'wmat123*xx123)*xx123'wmat123'y ;
 m6[j,69]  =  beta[1,.] ;
endif;
else;
m6[j,69] = 0;
a123 = a123+1;
endif;

if sumc(w124)> 0;
if det(xx124'wmat124*xx124) < epsln ;
      m6[j,70]  =  w124'y/sumc(w124) ;
else;
 beta =  pinv(xx124'wmat124*xx124)*xx124'wmat124'y ;
 m6[j,70]  =  beta[1,.] ;
endif;
else;
m6[j,70] = 0;
a124 = a124+1;
endif;

if sumc(w125)> 0;
if det(xx125'wmat125*xx125) < epsln ;
      m6[j,71]  =  w125'y/sumc(w125) ;
else;
 beta =  pinv(xx125'wmat125*xx125)*xx125'wmat125'y ;
 m6[j,71]  =  beta[1,.] ;
endif;
else;
m6[j,71] = 0;
a125 = a125+1;
endif;

if sumc(w126)> 0;
if det(xx126'wmat126*xx126) < epsln ;
      m6[j,72]  =  w126'y/sumc(w126) ;
else;
 beta =  pinv(xx126'wmat126*xx126)*xx126'wmat126'y ;
 m6[j,72]  =  beta[1,.] ;
endif;
else;
m6[j,72] = 0;
a126 = a126+1;
endif;

if sumc(w131)> 0;
if det(xx131'wmat131*xx131) < epsln ;
      m6[j,73]  =  w131'y/sumc(w131) ;
else;
 beta =  pinv(xx131'wmat131*xx131)*xx131'wmat131'y ;
 m6[j,73]  =  beta[1,.] ;
endif;
else;
m6[j,73] = 0;
a131 = a131+1;
endif;

if sumc(w132)> 0;
if det(xx132'wmat132*xx132) < epsln ;
      m6[j,74]  =  w132'y/sumc(w132) ;
else;
 beta =  pinv(xx132'wmat132*xx132)*xx132'wmat132'y ;
 m6[j,74]  =  beta[1,.] ;
endif;
else;
m6[j,74] = 0;
a132 = a132+1;
endif;

if sumc(w133)> 0;
if det(xx133'wmat133*xx133) < epsln ;
      m6[j,75]  =  w133'y/sumc(w133) ;
else;
 beta =  pinv(xx133'wmat133*xx133)*xx133'wmat133'y ;
 m6[j,75]  =  beta[1,.] ;
endif;
else;
m6[j,75] = 0;
a133 = a133+1;
endif;

if sumc(w134)> 0;
if det(xx134'wmat134*xx134) < epsln ;
      m6[j,76]  =  w134'y/sumc(w134) ;
else;
 beta =  pinv(xx134'wmat134*xx134)*xx134'wmat134'y ;
 m6[j,76]  =  beta[1,.] ;
endif;
else;
m6[j,76] = 0;
a134 = a134+1;
endif;

if sumc(w135)> 0;
if det(xx135'wmat135*xx135) < epsln ;
      m6[j,77]  =  w135'y/sumc(w135) ;
else;
 beta =  pinv(xx135'wmat135*xx135)*xx135'wmat135'y ;
 m6[j,77]  =  beta[1,.] ;
endif;
else;
m6[j,77] = 0;
a135 = a135+1;
endif;

if sumc(w136)> 0;
if det(xx136'wmat136*xx136) < epsln ;
      m6[j,78]  =  w136'y/sumc(w136) ;
else;
 beta =  pinv(xx136'wmat136*xx136)*xx136'wmat136'y ;
 m6[j,78]  =  beta[1,.] ;
endif;
else;
m6[j,78] = 0;
a136 = a136+1;
endif;
j = j+1;
endo;
m6d[i,1] = 1/(T-a11)*sumc(m6[.,1]);
m6d[i,2] = 1/(T-a12)*sumc(m6[.,2]);
m6d[i,3] = 1/(T-a13)*sumc(m6[.,3]);
m6d[i,4] = 1/(T-a14)*sumc(m6[.,4]);
m6d[i,5] = 1/(T-a15)*sumc(m6[.,5]);
m6d[i,6] = 1/(T-a16)*sumc(m6[.,6]);
m6d[i,7] = 1/(T-a21)*sumc(m6[.,7]);
m6d[i,8] = 1/(T-a22)*sumc(m6[.,8]);
m6d[i,9] = 1/(T-a23)*sumc(m6[.,9]);
m6d[i,10] = 1/(T-a24)*sumc(m6[.,10]);
m6d[i,11] = 1/(T-a25)*sumc(m6[.,11]);
m6d[i,12] = 1/(T-a26)*sumc(m6[.,12]);
m6d[i,13] = 1/(T-a31)*sumc(m6[.,13]);
m6d[i,14] = 1/(T-a32)*sumc(m6[.,14]);
m6d[i,15] = 1/(T-a33)*sumc(m6[.,15]);
m6d[i,16] = 1/(T-a34)*sumc(m6[.,16]);
m6d[i,17] = 1/(T-a35)*sumc(m6[.,17]);
m6d[i,18] = 1/(T-a36)*sumc(m6[.,18]);
m6d[i,19] = 1/(T-a41)*sumc(m6[.,19]);
m6d[i,20] = 1/(T-a42)*sumc(m6[.,20]);
m6d[i,21] = 1/(T-a43)*sumc(m6[.,21]);
m6d[i,22] = 1/(T-a44)*sumc(m6[.,22]);
m6d[i,23] = 1/(T-a45)*sumc(m6[.,23]);
m6d[i,24] = 1/(T-a46)*sumc(m6[.,24]);
m6d[i,25] = 1/(T-a51)*sumc(m6[.,25]);
m6d[i,26] = 1/(T-a52)*sumc(m6[.,26]);
m6d[i,27] = 1/(T-a53)*sumc(m6[.,27]);
m6d[i,28] = 1/(T-a54)*sumc(m6[.,28]);
m6d[i,29] = 1/(T-a55)*sumc(m6[.,29]);
m6d[i,30] = 1/(T-a56)*sumc(m6[.,30]);
m6d[i,31] = 1/(T-a61)*sumc(m6[.,31]);
m6d[i,32] = 1/(T-a62)*sumc(m6[.,32]);
m6d[i,33] = 1/(T-a63)*sumc(m6[.,33]);
m6d[i,34] = 1/(T-a64)*sumc(m6[.,34]);
m6d[i,35] = 1/(T-a65)*sumc(m6[.,35]);
m6d[i,36] = 1/(T-a66)*sumc(m6[.,36]);
m6d[i,37] = 1/(T-a71)*sumc(m6[.,37]);
m6d[i,38] = 1/(T-a72)*sumc(m6[.,38]);
m6d[i,39] = 1/(T-a73)*sumc(m6[.,39]);
m6d[i,40] = 1/(T-a74)*sumc(m6[.,40]);
m6d[i,41] = 1/(T-a75)*sumc(m6[.,41]);
m6d[i,42] = 1/(T-a76)*sumc(m6[.,42]);
m6d[i,43] = 1/(T-a81)*sumc(m6[.,43]);
m6d[i,44] = 1/(T-a82)*sumc(m6[.,44]);
m6d[i,45] = 1/(T-a83)*sumc(m6[.,45]);
m6d[i,46] = 1/(T-a84)*sumc(m6[.,46]);
m6d[i,47] = 1/(T-a85)*sumc(m6[.,47]);
m6d[i,48] = 1/(T-a86)*sumc(m6[.,48]);
m6d[i,49] = 1/(T-a91)*sumc(m6[.,49]);
m6d[i,50] = 1/(T-a92)*sumc(m6[.,50]);
m6d[i,51] = 1/(T-a93)*sumc(m6[.,51]);
m6d[i,52] = 1/(T-a94)*sumc(m6[.,52]);
m6d[i,53] = 1/(T-a95)*sumc(m6[.,53]);
m6d[i,54] = 1/(T-a96)*sumc(m6[.,54]);
m6d[i,55] = 1/(T-a101)*sumc(m6[.,55]);
m6d[i,56] = 1/(T-a102)*sumc(m6[.,56]);
m6d[i,57] = 1/(T-a103)*sumc(m6[.,57]);
m6d[i,58] = 1/(T-a104)*sumc(m6[.,58]);
m6d[i,59] = 1/(T-a105)*sumc(m6[.,59]);
m6d[i,60] = 1/(T-a106)*sumc(m6[.,60]);
m6d[i,61] = 1/(T-a111)*sumc(m6[.,61]);
m6d[i,62] = 1/(T-a112)*sumc(m6[.,62]);
m6d[i,63] = 1/(T-a113)*sumc(m6[.,63]);
m6d[i,64] = 1/(T-a114)*sumc(m6[.,64]);
m6d[i,65] = 1/(T-a115)*sumc(m6[.,65]);
m6d[i,66] = 1/(T-a116)*sumc(m6[.,66]);
m6d[i,67] = 1/(T-a121)*sumc(m6[.,67]);
m6d[i,68] = 1/(T-a122)*sumc(m6[.,68]);
m6d[i,69] = 1/(T-a123)*sumc(m6[.,69]);
m6d[i,70] = 1/(T-a124)*sumc(m6[.,70]);
m6d[i,71] = 1/(T-a125)*sumc(m6[.,71]);
m6d[i,72] = 1/(T-a126)*sumc(m6[.,72]);
m6d[i,73] = 1/(T-a131)*sumc(m6[.,73]);
m6d[i,74] = 1/(T-a132)*sumc(m6[.,74]);
m6d[i,75] = 1/(T-a133)*sumc(m6[.,75]);
m6d[i,76] = 1/(T-a134)*sumc(m6[.,76]);
m6d[i,77] = 1/(T-a135)*sumc(m6[.,77]);
m6d[i,78] = 1/(T-a136)*sumc(m6[.,78]);
i = i+1;
endo;
yhat1 = y-meanc(y)-(m6d[.,1]-meanc(m6d[.,1])+m6d[.,2]-meanc(m6d[.,2])+m6d[.,3]-meanc(m6d[.,3])+m6d[.,4]-meanc(m6d[.,4])+m6d[.,5]-meanc(m6d[.,5])+m6d[.,6]-meanc(m6d[.,6]));
yhat2 = y-meanc(y)-(m6d[.,7]-meanc(m6d[.,7])+m6d[.,8]-meanc(m6d[.,8])+m6d[.,9]-meanc(m6d[.,9])+m6d[.,10]-meanc(m6d[.,10])+m6d[.,11]-meanc(m6d[.,11])+m6d[.,12]-meanc(m6d[.,12]));
yhat3 = y-meanc(y)-(m6d[.,13]-meanc(m6d[.,13])+m6d[.,14]-meanc(m6d[.,14])+m6d[.,15]-meanc(m6d[.,15])+m6d[.,16]-meanc(m6d[.,16])+m6d[.,17]-meanc(m6d[.,17])+m6d[.,18]-meanc(m6d[.,18]));
yhat4 = y-meanc(y)-(m6d[.,19]-meanc(m6d[.,19])+m6d[.,20]-meanc(m6d[.,20])+m6d[.,21]-meanc(m6d[.,21])+m6d[.,22]-meanc(m6d[.,22])+m6d[.,23]-meanc(m6d[.,23])+m6d[.,24]-meanc(m6d[.,24]));
yhat5 = y-meanc(y)-(m6d[.,25]-meanc(m6d[.,25])+m6d[.,26]-meanc(m6d[.,26])+m6d[.,27]-meanc(m6d[.,27])+m6d[.,28]-meanc(m6d[.,28])+m6d[.,29]-meanc(m6d[.,29])+m6d[.,30]-meanc(m6d[.,30]));
yhat6 = y-meanc(y)-(m6d[.,31]-meanc(m6d[.,31])+m6d[.,32]-meanc(m6d[.,32])+m6d[.,33]-meanc(m6d[.,33])+m6d[.,34]-meanc(m6d[.,34])+m6d[.,35]-meanc(m6d[.,35])+m6d[.,36]-meanc(m6d[.,36]));
yhat7 = y-meanc(y)-(m6d[.,37]-meanc(m6d[.,37])+m6d[.,38]-meanc(m6d[.,38])+m6d[.,39]-meanc(m6d[.,39])+m6d[.,40]-meanc(m6d[.,40])+m6d[.,41]-meanc(m6d[.,41])+m6d[.,42]-meanc(m6d[.,42]));
yhat8 = y-meanc(y)-(m6d[.,43]-meanc(m6d[.,43])+m6d[.,44]-meanc(m6d[.,44])+m6d[.,45]-meanc(m6d[.,45])+m6d[.,46]-meanc(m6d[.,46])+m6d[.,47]-meanc(m6d[.,47])+m6d[.,48]-meanc(m6d[.,48]));
yhat9 = y-meanc(y)-(m6d[.,49]-meanc(m6d[.,49])+m6d[.,50]-meanc(m6d[.,50])+m6d[.,51]-meanc(m6d[.,51])+m6d[.,52]-meanc(m6d[.,52])+m6d[.,53]-meanc(m6d[.,53])+m6d[.,54]-meanc(m6d[.,54]));
yhat10 = y-meanc(y)-(m6d[.,55]-meanc(m6d[.,55])+m6d[.,56]-meanc(m6d[.,56])+m6d[.,57]-meanc(m6d[.,57])+m6d[.,58]-meanc(m6d[.,58])+m6d[.,59]-meanc(m6d[.,59])+m6d[.,60]-meanc(m6d[.,60]));
yhat11 = y-meanc(y)-(m6d[.,61]-meanc(m6d[.,61])+m6d[.,62]-meanc(m6d[.,62])+m6d[.,63]-meanc(m6d[.,63])+m6d[.,64]-meanc(m6d[.,64])+m6d[.,65]-meanc(m6d[.,65])+m6d[.,66]-meanc(m6d[.,66]));
yhat12 = y-meanc(y)-(m6d[.,67]-meanc(m6d[.,67])+m6d[.,68]-meanc(m6d[.,68])+m6d[.,69]-meanc(m6d[.,69])+m6d[.,70]-meanc(m6d[.,70])+m6d[.,71]-meanc(m6d[.,71])+m6d[.,72]-meanc(m6d[.,72]));
yhat13 = y-meanc(y)-(m6d[.,73]-meanc(m6d[.,73])+m6d[.,74]-meanc(m6d[.,74])+m6d[.,75]-meanc(m6d[.,75])+m6d[.,76]-meanc(m6d[.,76])+m6d[.,77]-meanc(m6d[.,77])+m6d[.,78]-meanc(m6d[.,78]));
R1 = yhat1'yhat1;
R2 = yhat2'yhat2;
R3 = yhat3'yhat3;
R4 = yhat4'yhat4;
R5 = yhat5'yhat5;
R6 = yhat6'yhat6;
R7 = yhat7'yhat7;
R8 = yhat8'yhat8;
R9 = yhat9'yhat9;
R10 = yhat10'yhat10;
R11 = yhat11'yhat11;
R12 = yhat12'yhat12;
R13 = yhat13'yhat13;
Ahat1[k,kc] = 1/T*sumc(R1);
Ahat2[k,kc] = 1/T*sumc(R2);
Ahat3[k,kc] = 1/T*sumc(R3);
Ahat4[k,kc] = 1/T*sumc(R4);
Ahat5[k,kc] = 1/T*sumc(R5);
Ahat6[k,kc] = 1/T*sumc(R6);
Ahat7[k,kc] = 1/T*sumc(R7);
Ahat8[k,kc] = 1/T*sumc(R8);
Ahat9[k,kc] = 1/T*sumc(R9);
Ahat10[k,kc] = 1/T*sumc(R10);
Ahat11[k,kc] = 1/T*sumc(R11);
Ahat12[k,kc] = 1/T*sumc(R12);
Ahat13[k,kc] = 1/T*sumc(R13);
Bhat1[k,kc]  =  2/b6*1/sqrt(2*pi)*1/T*sumc((BR1.*qz6[.,1]+BR1.*qz6[.,2]+BR1.*qz6[.,3]+BR1.*qz6[.,4]+BR1.*qz6[.,5]+BR1.*qz6[.,6])./phat[.,1])*1/T;
Bhat2[k,kc]  =  2/b6*1/sqrt(2*pi)*1/T*sumc((BR2.*qz6[.,1]+BR2.*qz6[.,7]+BR2.*qz6[.,8]+BR2.*qz6[.,9]+BR2.*qz6[.,10]+BR2.*qz6[.,11])./phat[.,2])*1/T;
Bhat3[k,kc]  =  2/b6*1/sqrt(2*pi)*1/T*sumc((BR3.*qz6[.,1]+BR3.*qz6[.,12]+BR3.*qz6[.,13]+BR3.*qz6[.,14]+BR3.*qz6[.,15]+BR3.*qz6[.,16])./phat[.,3])*1/T;
Bhat4[k,kc]  =  2/b6*1/sqrt(2*pi)*1/T*sumc((BR4.*qz6[.,1]+BR4.*qz6[.,17]+BR4.*qz6[.,18]+BR4.*qz6[.,19]+BR4.*qz6[.,20]+BR4.*qz6[.,21])./phat[.,4])*1/T;
Bhat5[k,kc]  =  2/b6*1/sqrt(2*pi)*1/T*sumc((BR5.*qz6[.,1]+BR5.*qz6[.,22]+BR5.*qz6[.,23]+BR5.*qz6[.,24]+BR5.*qz6[.,25]+BR5.*qz6[.,26])./phat[.,5])*1/T;
Bhat6[k,kc]  =  2/b6*1/sqrt(2*pi)*1/T*sumc((BR6.*qz6[.,1]+BR6.*qz6[.,27]+BR6.*qz6[.,28]+BR6.*qz6[.,29]+BR6.*qz6[.,30]+BR6.*qz6[.,31])./phat[.,6])*1/T;
Bhat7[k,kc]  =  2/b6*1/sqrt(2*pi)*1/T*sumc((BR7.*qz6[.,1]+BR7.*qz6[.,32]+BR7.*qz6[.,33]+BR7.*qz6[.,34]+BR7.*qz6[.,35]+BR7.*qz6[.,36])./phat[.,7])*1/T;
Bhat8[k,kc]  =  2/b6*1/sqrt(2*pi)*1/T*sumc((BR8.*qz6[.,1]+BR8.*qz6[.,37]+BR8.*qz6[.,38]+BR8.*qz6[.,39]+BR8.*qz6[.,40]+BR8.*qz6[.,41])./phat[.,8])*1/T;
Bhat9[k,kc]  =  2/b6*1/sqrt(2*pi)*1/T*sumc((BR9.*qz6[.,1]+BR9.*qz6[.,42]+BR9.*qz6[.,43]+BR9.*qz6[.,44]+BR9.*qz6[.,45]+BR9.*qz6[.,46])./phat[.,9])*1/T;
Bhat10[k,kc] =  2/b6*1/sqrt(2*pi)*1/T*sumc((BR10.*qz6[.,1]+BR10.*qz6[.,47]+BR10.*qz6[.,48]+BR10.*qz6[.,49]+BR10.*qz6[.,50]+BR10.*qz6[.,51])./phat[.,10])*1/T;
Bhat11[k,kc]  =  2/b6*1/sqrt(2*pi)*1/T*sumc((BR11.*qz6[.,1]+BR11.*qz6[.,52]+BR11.*qz6[.,53]+BR11.*qz6[.,54]+BR11.*qz6[.,55]+BR11.*qz6[.,56])./phat[.,11])*1/T;
Bhat12[k,kc]  =  2/b6*1/sqrt(2*pi)*1/T*sumc((BR12.*qz6[.,1]+BR12.*qz6[.,47]+BR12.*qz6[.,48]+BR12.*qz6[.,49]+BR12.*qz6[.,50]+BR12.*qz6[.,51])./phat[.,12])*1/T;
Bhat13[k,kc]  =  2/b6*1/sqrt(2*pi)*1/T*sumc((BR13.*qz6[.,1]+BR13.*qz6[.,52]+BR13.*qz6[.,53]+BR13.*qz6[.,54]+BR13.*qz6[.,55]+BR13.*qz6[.,56])./phat[.,13])*1/T;
FPE1[k,kc] = Ahat1[k,kc]+Bhat1[k,kc];
FPE2[k,kc] = Ahat2[k,kc]+Bhat2[k,kc];
FPE3[k,kc] = Ahat3[k,kc]+Bhat3[k,kc];
FPE4[k,kc] = Ahat4[k,kc]+Bhat4[k,kc];
FPE5[k,kc] = Ahat5[k,kc]+Bhat5[k,kc];
FPE6[k,kc] = Ahat6[k,kc]+Bhat6[k,kc];
FPE7[k,kc] = Ahat7[k,kc]+Bhat7[k,kc];
FPE8[k,kc] = Ahat8[k,kc]+Bhat8[k,kc];
FPE9[k,kc] = Ahat9[k,kc]+Bhat9[k,kc];
FPE10[k,kc] = Ahat10[k,kc]+Bhat10[k,kc];
FPE11[k,kc] = Ahat11[k,kc]+Bhat11[k,kc];
FPE12[k,kc] = Ahat12[k,kc]+Bhat12[k,kc];
FPE13[k,kc] = Ahat13[k,kc]+Bhat13[k,kc];
kc = kc+1;
endo;
cmin1 = minc(FPE1[k,.]');
cmin2 = minc(FPE2[k,.]');
cmin3 = minc(FPE3[k,.]');
cmin4 = minc(FPE4[k,.]');
cmin5 = minc(FPE5[k,.]');
cmin6 = minc(FPE6[k,.]');
cmin7 = minc(FPE7[k,.]');
cmin8 = minc(FPE8[k,.]');
cmin9 = minc(FPE9[k,.]');
cmin10 = minc(FPE10[k,.]');
cmin11 = minc(FPE11[k,.]');
cmin12 = minc(FPE12[k,.]');
cmin13 = minc(FPE13[k,.]');
minFPE = cmin1~cmin2~cmin3~cmin4~cmin5~cmin6~cmin7~cmin8~cmin9~cmin10~cmin11~cmin12~cmin13;
minFPE[.,lag_1] = 10^10;
minFPE[.,lag_2] = 10^10;
minFPE[.,lag_3] = 10^10;
minFPE[.,lag_4] = 10^10;
minFPE[.,lag_5] = 10^10;
AFPE[k,6] = minc(minFPE');
lag[k,6] = minindc(minFPE');
lag_6 = lag[k,6];
k = k+1;
endo;

Final_lag = lag;
/*underfitting correct fitting and overfitting*/
minlag = minindc(AFPE');
lags = lag;
i = 1;
do until i>itr;
if minlag[i]<6;
 in = 6-minlag[i];
 b = minlag[i]+1;
 lags[i,b:6] = zeros(1,in);
endif;
i = i+1;
endo;
in1 = lags. =  = para1;
in2 = lags. =  = para2;
lags = lags+in1*100+in2*100;
/*correct fitting */
n0 = minlag. =  = 2;
cc = sumc(lags').> = 200;
aa1 = (n0+cc). =  = 2;
correct = sumc(aa1)/itr;
/*underfitting */
n1 = minlag. =  = 1;
cc = sumc(lags').> = 100;
aa2 = (n1+cc). =  = 2;
underfit = sumc(aa2)/itr;
/*overfitting */
n3 = minlag.>2;
cc = sumc(lags').> = 200;
aa3 = n3+cc.> = 2;
overfit = sumc(aa3)/itr;
FA = underfit~correct~overfit;
print FA;
/*calculate underfitting, overfitting and correct fitting by FPE with correction term*/
ct1 = 1+T^(-4/5);
ct2 = 1+2*T^(-4/6);
ct3 = 1+3*T^(-4/7);
ct4 = 1+4*T^(-4/8);
ct5 = 1+5*T^(-4/9);
ct6 = 1+6*T^(-4/10);
ct = ct1~ct2~ct3~ct4~ct5~ct6;
CFPE = ct.*AFPE;
/*underfitting correct fitting and overfitting */
minlag = minindc(CFPE');
lags = lag;
i = 1;
do until i>itr;
if minlag[i]<6;
 in = 6-minlag[i];
 b = minlag[i]+1;
 lags[i,b:6] = zeros(1,in);
endif;
i = i+1;
endo;
in1 = lags. =  = para1;
in2 = lags. =  = para2;
lags = lags+in1*100+in2*100;
/*correct fitting */
n0 = minlag. =  = 2;
cc = sumc(lags').> = 200;
aa1 = (n0+cc). =  = 2;
ccorrect = sumc(aa1)/itr;
/*underfitting */
n1 = minlag. =  = 1;
cc = sumc(lags').> = 100;
aa2 = (n1+cc). =  = 2;
cunderfit = sumc(aa2)/itr;
/*overfitting */
n3 = minlag.>2;
cc = sumc(lags').> = 200;
aa3 = n3+cc.> = 2;
coverfit = sumc(aa3)/itr;
CFA = cunderfit~ccorrect~coverfit;
tend  =  time;
t2  =  tvec*tend;
th  =  floor((t2-tt)/3600);
tm  =  floor((t2-tt-3600*th)/60);
tsec  =  floor((t2-tt-3600*th-60*tm));
total  =  timestr(th~tm~tsec~0);
locate 20,1;
format /rd 5,5;
output on;
"Total time used in computation:";
print total;
print "nonparametric FPE result";
print "T" ;t;
print "lag" ;final_lag;
print "the true lag";
para1; para2;
print "AFPE" ;
AFPE;
print "CFPE";
CFPE;
print "underfitting,correct fitting and overfitting by FPE";
print underfit correct  overfit;
print "underfitting,correct fitting and overfitting by FPE with correction term";
print  cunderfit ccorrect coverfit;

/* (iii) Gaussian kernel */
fn fi(z)  =  exp(-(z.^2)/2)./sqrt(2*pi) ;

/*
** (i) Epanechnikov kernel
** fn fi(z)  =  (abs(z).<1)*(3/4).*(ones(T,1)-(z.^2)) ;
**
** (ii) Quartic kernel
** fn fi(z)  =  (abs(z).<1)*(15/16).*((ones(T,1)-(z.^2)).^2) ;
**
** (iii) Gaussian kernel
** fn fi(z)  =  exp(-(z.^2)/2)./sqrt(2*pi) ;
*/
/*** Procedures ***/

proc lagn(x,n);
    local y;
    y  =  shiftr(x', n, (miss(0, 0))');
    retp(y');
endp;

